package com.goodix.fingerprint.service;

import android.app.AppOpsManager;
import android.content.ContentResolver;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Binder;
import android.os.PowerManager;
import android.os.RemoteException;
import android.provider.Settings;
import android.util.Log;
import com.goodix.fingerprint.GFConfig;
import com.goodix.fingerprint.GFDevice;
import com.goodix.fingerprint.GFShenzhenConfig;
import com.goodix.fingerprint.ShenzhenConstants;
import com.goodix.fingerprint.proxy.DaemonManager;
import com.goodix.fingerprint.proxy.IGFDaemon;
import com.goodix.fingerprint.proxy.IGFDaemonCallback;
import com.goodix.fingerprint.utils.BaikalTestResultParser;
import com.goodix.fingerprint.utils.ShenzhenTestResultParser;
import com.goodix.fingerprint.utils.TestParamEncoder;
import com.goodix.fingerprint.utils.TestResultParser;
import java.io.File;
import java.io.FileOutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;

public class GFService {
    public static final boolean DEBUG = true;
    protected static final int ERROR_CLIENT_DEFINED = 100;
    protected static final int ERROR_ESRCH = 3;
    public static final String MANAGE_FINGERPRINT = "android.permission.MANAGE_FINGERPRINT";
    private static final String TAG = "GFService";
    private static final int TEST_GET_CONFIG_MAX_RETRY_TIMES = 10;
    private static final int TEST_GET_VERSION_MAX_RETRY_TIMES = 10;
    public static final String USE_FINGERPRINT = "android.permission.USE_FINGERPRINT";
    private IGFDaemon daemon;
    private AppOpsManager mAppOps;
    private GFConfig mConfig = new GFConfig();
    private ContentResolver mContentResolver;
    private Context mContext;
    private IGFDaemonCallback mDaemonCallback = new IGFDaemonCallback() {
        /* class com.goodix.fingerprint.service.GFService.AnonymousClass1 */

        @Override // com.goodix.fingerprint.proxy.IGFDaemonCallback
        public void onDaemonMessage(long devId, int msgId, int cmdId, byte[] data) {
            Log.d(GFService.TAG, "onDaemonMessage devId = " + devId + " msgId=" + msgId + " cmdId=" + cmdId + " data=" + data);
            if (cmdId == 5) {
                GFService.this.onConfigCmd(data);
            } else if (1549 == cmdId) {
                GFService.this.onShenzhenConfigCmd(data);
            } else {
                Log.d(GFService.TAG, "DaemonMessageHandler.handleMessage devId = " + devId + " msgId=" + msgId + " cmdId=" + cmdId + " data=" + data);
                GFService.this.mDaemonMessageHandler.handleMessage(devId, msgId, cmdId, data);
            }
        }

        @Override // com.goodix.fingerprint.proxy.IDaemonDiedCallback
        public void onDaemonDied() {
            GFService.this.mDaemonMessageHandler.handleError(GoodixFingerprintManager.FINGERPRINT_ERROR_HW_UNAVAILABLE);
        }
    };
    private DaemonMessageHandlerBase mDaemonMessageHandler;
    private DaemonManager mDaemonMgr;
    private GFDevice mDevice = new GFDevice();
    int mGetSDKVersionRetryCount = 0;
    int mGetTestNumCount = 0;
    int mGetVersionRetryCount = 0;
    private String mOpPackageName;
    private int mOpUseFingerprint = -1;
    private GFShenzhenConfig mShenzhenConfig = new GFShenzhenConfig();
    private int mUserId;
    private PowerManager pm;

    public GFService(Context context) {
        this.mContext = context;
        this.mAppOps = (AppOpsManager) context.getSystemService("appops");
        this.mUserId = getMyUserId();
        this.mOpPackageName = getAppOpPackageName();
        this.mOpUseFingerprint = getOpUseFingerprint();
        this.mDaemonMgr = DaemonManager.getInstance();
        this.mContentResolver = this.mContext.getContentResolver();
        this.mDaemonMessageHandler = new DaemonMessageHandlerBase();
        this.daemon = getFingerprintDaemon();
        startGetShenzhenConfigTask();
    }

    private void startGetShenzhenConfigTask() {
        new Thread(new GetShenzhenConfigTask()).start();
    }

    /* access modifiers changed from: private */
    public class GetShenzhenConfigTask implements Runnable {
        private GetShenzhenConfigTask() {
        }

        public void run() {
            GFService.this.loadShenzhenConfig();
            GFService gFService = GFService.this;
            gFService.testCmd(ShenzhenConstants.CMD_TEST_SZ_GET_CONFIG, null, gFService.mOpPackageName);
        }
    }

    public void registerFingerprintCallback(IGoodixFingerprintCallback fingerprintCallback) {
        Log.d(TAG, "GFService registerFingerprintCallback = " + fingerprintCallback);
        this.mDaemonMessageHandler.registerFingerprintCallback(fingerprintCallback);
    }

    public void unRegisterFingerprintCallback() {
        this.mDaemonMessageHandler.unRegisterFingerprintCallback();
    }

    private IGFDaemon getFingerprintDaemon() {
        Log.d(TAG, "getFingerprintDaemon()");
        return this.mDaemonMgr.getGoodixFingerprintDaemon(this.mDaemonCallback);
    }

    public void testCmd(int cmdId, byte[] param, String opPackageName) {
        Log.d(TAG, "testCmd " + cmdId);
        if (!canUseFingerprint(opPackageName)) {
            Log.w(TAG, "Calling not granted permission to use fingerprint");
            return;
        }
        if (this.daemon == null) {
            this.daemon = getFingerprintDaemon();
        }
        IGFDaemon iGFDaemon = this.daemon;
        if (iGFDaemon == null) {
            Log.w(TAG, "stopTestCmd: no goodixfingeprintd!");
            this.mDaemonMessageHandler.handleMessage(-1, GoodixFingerprintManager.FINGERPRINT_ERROR_HW_UNAVAILABLE, -1, null);
            return;
        }
        try {
            if (iGFDaemon.sendCommand(cmdId, param).mResultCode != 0) {
                this.mDaemonMessageHandler.handleMessage(-1, GoodixFingerprintManager.FINGERPRINT_ERROR_HW_UNAVAILABLE, -1, null);
            }
        } catch (RemoteException e) {
            e.printStackTrace();
        }
    }

    public String getLibaryVersion() {
        while (true) {
            if (this.mShenzhenConfig.mLibaryVersion != null && !this.mShenzhenConfig.mLibaryVersion.isEmpty()) {
                break;
            }
            this.mGetVersionRetryCount++;
            Log.i(TAG, "getLibaryVersion retry count : " + this.mGetVersionRetryCount);
            if (this.mGetVersionRetryCount >= 10) {
                break;
            }
            try {
                Thread.sleep(40);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        if (this.mShenzhenConfig.mLibaryVersion != null) {
            Log.i(TAG, "getLibaryVersion: " + this.mShenzhenConfig.mLibaryVersion);
        }
        return this.mShenzhenConfig.mLibaryVersion;
    }

    public String getTestNum() {
        while (true) {
            if (this.mShenzhenConfig.mSDKVersion != null && !this.mShenzhenConfig.mSDKVersion.isEmpty()) {
                break;
            }
            this.mGetSDKVersionRetryCount++;
            Log.i(TAG, "getTestNum retry count : " + this.mGetSDKVersionRetryCount);
            if (this.mGetSDKVersionRetryCount >= 10) {
                break;
            }
            try {
                Thread.sleep(40);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        if (this.mShenzhenConfig.mSDKVersion != null) {
            Log.i(TAG, "getTestNum: " + this.mShenzhenConfig.mSDKVersion);
        }
        return this.mShenzhenConfig.mSDKVersion;
    }

    public int getHightBrighness() {
        return this.mShenzhenConfig.mHighBrightness;
    }

    public int getMiddleBrightness() {
        return this.mShenzhenConfig.mMiddleBrightness;
    }

    public int getLowBrightness() {
        return this.mShenzhenConfig.mLowBrightness;
    }

    public void untrustedAuthenticate(String opPackageName) {
        Log.e(TAG, "untrustedAuthenticate:" + opPackageName);
        testCmd(ShenzhenConstants.CMD_TEST_SZ_UNTRUSTED_AUTHENTICATE, null, opPackageName);
    }

    public void untrustedAuthenticate2(String userDesc, String opPackageName) {
        Log.e(TAG, "untrustedAuthenticate2:" + opPackageName);
        byte[] dataBytes = new byte[TestParamEncoder.testEncodeSizeOfArray(userDesc.getBytes().length)];
        TestParamEncoder.encodeArray(dataBytes, 0, 1210, userDesc.getBytes(), userDesc.getBytes().length);
        testCmd(ShenzhenConstants.CMD_TEST_SZ_UNTRUSTED_AUTHENTICATE, dataBytes, opPackageName);
    }

    public void cancelUntrustedAuthentication(String opPackageName) {
        cancelTestCmd(opPackageName);
    }

    public void untrustedEnroll(String opPackageName) {
        Log.e(TAG, "untrustedEnroll:" + opPackageName);
        testCmd(ShenzhenConstants.CMD_TEST_SZ_UNTRUSTED_ENROLL, null, opPackageName);
    }

    public void untrustedEnroll2(String userDesc, String opPackageName) {
        Log.e(TAG, "untrustedEnroll2:" + opPackageName);
        byte[] dataBytes = new byte[TestParamEncoder.testEncodeSizeOfArray(userDesc.getBytes().length)];
        TestParamEncoder.encodeArray(dataBytes, 0, 1210, userDesc.getBytes(), userDesc.getBytes().length);
        testCmd(ShenzhenConstants.CMD_TEST_SZ_UNTRUSTED_ENROLL, dataBytes, opPackageName);
    }

    public void untrustedRemove(IGoodixFingerprintCallback receiver, String opPackageName) {
    }

    public void cancelUntrustedEnrollment(String opPackageName) {
        cancelTestCmd(opPackageName);
    }

    public void cancelTestCmd(String opPackageName) {
        testCmd(4, null, opPackageName);
    }

    public void navigate(int navMode, String opPackageName) {
        Log.d(TAG, "navigate");
        canUseFingerprint(opPackageName);
    }

    public void stopNavigation(String opPackageName) {
        Log.d(TAG, "stopNavigation");
        canUseFingerprint(opPackageName);
    }

    public GFConfig getConfig(String opPackageName) {
        if (!canUseFingerprint(opPackageName)) {
            return null;
        }
        return new GFConfig(this.mConfig);
    }

    public GFDevice getDevice(String opPackageName) {
        if (!canUseFingerprint(opPackageName)) {
            return null;
        }
        return new GFDevice(this.mDevice);
    }

    private int getMyUserId() {
        try {
            return ((Integer) Class.forName("android.os.UserHandle").getMethod("myUserId", new Class[0]).invoke(null, new Object[0])).intValue();
        } catch (ClassNotFoundException e) {
            Log.e(TAG, "getMyUserId ClassNotFoundException: ", e);
            return 0;
        } catch (NoSuchMethodException e2) {
            Log.e(TAG, "getMyUserId NoSuchMethodException: ", e2);
            return 0;
        } catch (IllegalAccessException e3) {
            Log.e(TAG, "getMyUserId IllegalAccessException: ", e3);
            return 0;
        } catch (InvocationTargetException e4) {
            Log.e(TAG, "getMyUserId InvocationTargetException: ", e4);
            return 0;
        } catch (IllegalArgumentException e5) {
            Log.e(TAG, "getMyUserId IllegalArgumentException: ", e5);
            return 0;
        }
    }

    private void checkPermission(String permission) {
        Context context = this.mContext;
        context.enforceCallingOrSelfPermission(permission, "Must have " + permission + " permission.");
    }

    private boolean canUseFingerprint(String opPackageName) {
        checkPermission("android.permission.USE_FINGERPRINT");
        try {
            Method noteOp = this.mAppOps.getClass().getMethod("noteOp", Integer.TYPE, Integer.TYPE, String.class);
            noteOp.setAccessible(true);
            return ((Integer) noteOp.invoke(this.mAppOps, new Object[]{Integer.valueOf(this.mOpUseFingerprint), Integer.valueOf(Binder.getCallingUid()), opPackageName})).intValue() == 0;
        } catch (NoSuchMethodException e) {
            Log.e(TAG, "canUseFingerprint NoSuchMethodException: ", e);
            return false;
        } catch (InvocationTargetException e2) {
            Log.e(TAG, "canUseFingerprint InvocationTargetException: ", e2);
            return false;
        } catch (IllegalAccessException e3) {
            Log.e(TAG, "canUseFingerprint IllegalAccessException: ", e3);
            return false;
        }
    }

    private String getAppOpPackageName() {
        Context context = this.mContext;
        if (context == null) {
            return null;
        }
        try {
            Method getOpPackageName = context.getClass().getMethod("getOpPackageName", new Class[0]);
            getOpPackageName.setAccessible(true);
            return (String) getOpPackageName.invoke(this.mContext, new Object[0]);
        } catch (NoSuchMethodException e) {
            Log.e(TAG, "getAppOpPackageName NoSuchMethodException: ", e);
            return null;
        } catch (IllegalAccessException e2) {
            Log.e(TAG, "getAppOpPackageName IllegalAccessException: ", e2);
            return null;
        } catch (IllegalArgumentException e3) {
            Log.e(TAG, "getAppOpPackageName IllegalArgumentException: ", e3);
            return null;
        } catch (InvocationTargetException e4) {
            Log.e(TAG, "getAppOpPackageName InvocationTargetException: ", e4);
            return null;
        }
    }

    private int getOpUseFingerprint() {
        try {
            return AppOpsManager.class.getDeclaredField("OP_USE_FINGERPRINT").getInt(null);
        } catch (NoSuchFieldException e) {
            Log.e(TAG, "getOpUseFingerprint NoSuchFieldException: ", e);
            return -1;
        } catch (IllegalAccessException e2) {
            Log.e(TAG, "getOpUseFingerprint IllegalAccessException: ", e2);
            return -1;
        } catch (IllegalArgumentException e3) {
            Log.e(TAG, "getOpUseFingerprint IllegalArgumentException: ", e3);
            return -1;
        }
    }

    public int onConfigCmd(byte[] result) {
        int errCode;
        HashMap<Integer, Object> testResult = TestResultParser.parseConfig(result);
        if (testResult == null) {
            return -1;
        }
        if (testResult.containsKey(100) && (errCode = ((Integer) testResult.get(100)).intValue()) > 0) {
            return errCode;
        }
        if (testResult.containsKey(101)) {
            this.mConfig.mChipType = ((Integer) testResult.get(101)).intValue();
        }
        if (testResult.containsKey(102)) {
            this.mConfig.mChipSeries = ((Integer) testResult.get(102)).intValue();
        }
        if (testResult.containsKey(800)) {
            this.mConfig.mMaxFingers = ((Integer) testResult.get(800)).intValue();
        }
        if (testResult.containsKey(801)) {
            this.mConfig.mMaxFingersPerUser = ((Integer) testResult.get(801)).intValue();
        }
        if (testResult.containsKey(802)) {
            this.mConfig.mSupportKeyMode = ((Integer) testResult.get(802)).intValue();
        }
        if (testResult.containsKey(803)) {
            this.mConfig.mSupportFFMode = ((Integer) testResult.get(803)).intValue();
        }
        if (testResult.containsKey(804)) {
            this.mConfig.mSupportPowerKeyFeature = ((Integer) testResult.get(804)).intValue();
        }
        if (testResult.containsKey(805)) {
            this.mConfig.mForbiddenUntrustedEnroll = ((Integer) testResult.get(805)).intValue();
        }
        if (testResult.containsKey(806)) {
            this.mConfig.mForbiddenEnrollDuplicateFingers = ((Integer) testResult.get(806)).intValue();
        }
        if (testResult.containsKey(807)) {
            this.mConfig.mSupportBioAssay = ((Integer) testResult.get(807)).intValue();
        }
        if (testResult.containsKey(808)) {
            this.mConfig.mSupportPerformanceDump = ((Integer) testResult.get(808)).intValue();
        }
        if (testResult.containsKey(809)) {
            this.mConfig.mSupportNavMode = ((Integer) testResult.get(809)).intValue();
        }
        if (testResult.containsKey(810)) {
            this.mConfig.mNavDoubleClickTime = ((Integer) testResult.get(810)).intValue();
        }
        if (testResult.containsKey(811)) {
            this.mConfig.mNavLongPressTime = ((Integer) testResult.get(811)).intValue();
        }
        if (testResult.containsKey(812)) {
            this.mConfig.mEnrollingMinTemplates = ((Integer) testResult.get(812)).intValue();
        }
        if (testResult.containsKey(813)) {
            this.mConfig.mValidImageQualityThreshold = ((Integer) testResult.get(813)).intValue();
        }
        if (testResult.containsKey(814)) {
            this.mConfig.mValidImageAreaThreshold = ((Integer) testResult.get(814)).intValue();
        }
        if (testResult.containsKey(815)) {
            this.mConfig.mDuplicateFingerOverlayScore = ((Integer) testResult.get(815)).intValue();
        }
        if (testResult.containsKey(816)) {
            this.mConfig.mIncreaseRateBetweenStitchInfo = ((Integer) testResult.get(816)).intValue();
        }
        if (testResult.containsKey(817)) {
            this.mConfig.mScreenOnAuthenticateFailRetryCount = ((Integer) testResult.get(817)).intValue();
        }
        if (testResult.containsKey(818)) {
            this.mConfig.mScreenOffAuthenticateFailRetryCount = ((Integer) testResult.get(818)).intValue();
        }
        if (testResult.containsKey(819)) {
            this.mConfig.mScreenOnValidTouchFrameThreshold = ((Integer) testResult.get(819)).intValue();
        }
        if (testResult.containsKey(820)) {
            this.mConfig.mScreenOffValidTouchFrameThreshold = ((Integer) testResult.get(820)).intValue();
        }
        if (testResult.containsKey(821)) {
            this.mConfig.mImageQualityThresholdForMistakeTouch = ((Integer) testResult.get(821)).intValue();
        }
        if (testResult.containsKey(822)) {
            this.mConfig.mAuthenticateOrder = ((Integer) testResult.get(822)).intValue();
        }
        if (testResult.containsKey(823)) {
            this.mConfig.mReissueKeyDownWhenEntryFfMode = ((Integer) testResult.get(823)).intValue();
        }
        if (testResult.containsKey(824)) {
            this.mConfig.mReissueKeyDownWhenEntryImageMode = ((Integer) testResult.get(824)).intValue();
        }
        if (testResult.containsKey(825)) {
            this.mConfig.mSupportSensorBrokenCheck = ((Integer) testResult.get(825)).intValue();
        }
        if (testResult.containsKey(826)) {
            this.mConfig.mBrokenPixelThresholdForDisableSensor = ((Integer) testResult.get(826)).intValue();
        }
        if (testResult.containsKey(827)) {
            this.mConfig.mBrokenPixelThresholdForDisableStudy = ((Integer) testResult.get(827)).intValue();
        }
        if (testResult.containsKey(828)) {
            this.mConfig.mBadPointTestMaxFrameNumber = ((Integer) testResult.get(828)).intValue();
        }
        if (testResult.containsKey(829)) {
            this.mConfig.mReportKeyEventOnlyEnrollAuthenticate = ((Integer) testResult.get(829)).intValue();
        }
        if (testResult.containsKey(830)) {
            this.mConfig.mRequireDownAndUpInPairsForImageMode = ((Integer) testResult.get(830)).intValue();
        }
        if (testResult.containsKey(831)) {
            this.mConfig.mRequireDownAndUpInPairsForFFMode = ((Integer) testResult.get(831)).intValue();
        }
        if (testResult.containsKey(832)) {
            this.mConfig.mRequireDownAndUpInPairsForKeyMode = ((Integer) testResult.get(832)).intValue();
        }
        if (testResult.containsKey(833)) {
            this.mConfig.mRequireDownAndUpInPairsForNavMode = ((Integer) testResult.get(833)).intValue();
        }
        if (testResult.containsKey(834)) {
            this.mConfig.mSupportSetSpiSpeedInTEE = ((Integer) testResult.get(834)).intValue();
        }
        if (testResult.containsKey(835)) {
            this.mConfig.mSupportFrrAnalysis = ((Integer) testResult.get(835)).intValue();
        }
        if (testResult.containsKey(5200)) {
            this.mConfig.mTemplateUpateSaveThreshold = ((Integer) testResult.get(5200)).intValue();
        }
        if (testResult.containsKey(Integer.valueOf((int) BaikalTestResultParser.TEST_TOKEN_SUPPORT_IMAGE_SEGMENT))) {
            this.mConfig.mSupportImageSegment = ((Integer) testResult.get(Integer.valueOf((int) BaikalTestResultParser.TEST_TOKEN_SUPPORT_IMAGE_SEGMENT))).intValue();
        }
        if (testResult.containsKey(Integer.valueOf((int) BaikalTestResultParser.TEST_TOKEN_SUPPORT_BAIKAL_CONTINUOUS_SAMPLING))) {
            this.mConfig.mSupportBaikalContinuousSampling = ((Integer) testResult.get(Integer.valueOf((int) BaikalTestResultParser.TEST_TOKEN_SUPPORT_BAIKAL_CONTINUOUS_SAMPLING))).intValue();
        }
        if (testResult.containsKey(Integer.valueOf((int) BaikalTestResultParser.TEST_TOKEN_SUPPORT_BAIKAL_CONTINUOUS_SAMPLING_NUM))) {
            this.mConfig.mContinuousSamplingNumber = ((Integer) testResult.get(Integer.valueOf((int) BaikalTestResultParser.TEST_TOKEN_SUPPORT_BAIKAL_CONTINUOUS_SAMPLING_NUM))).intValue();
        }
        if (this.mConfig.mMaxFingers >= this.mConfig.mMaxFingersPerUser) {
            return 0;
        }
        GFConfig gFConfig = this.mConfig;
        gFConfig.mMaxFingers = gFConfig.mMaxFingersPerUser;
        return 0;
    }

    public int onShenzhenConfigCmd(byte[] result) {
        int errCode;
        Log.d(TAG, "onShenzhenConfigCmd " + this.mShenzhenConfig.mLibaryVersion);
        HashMap<Integer, Object> testResult = TestResultParser.parseConfig(result);
        if (testResult == null) {
            return -1;
        }
        if (testResult.containsKey(100) && (errCode = ((Integer) testResult.get(100)).intValue()) > 0) {
            return errCode;
        }
        if (testResult.containsKey(6003)) {
            this.mShenzhenConfig.mShortExposureTime = ((Integer) testResult.get(6003)).intValue();
        }
        if (testResult.containsKey(6022)) {
            this.mShenzhenConfig.mRect_bmp_col = ((Integer) testResult.get(6022)).intValue();
        }
        if (testResult.containsKey(6023)) {
            this.mShenzhenConfig.mRect_bmp_row = ((Integer) testResult.get(6023)).intValue();
        }
        if (testResult.containsKey(6028)) {
            this.mShenzhenConfig.mSensorRow = ((Integer) testResult.get(6028)).intValue();
            Log.d(TAG, "mSensorRow " + this.mShenzhenConfig.mSensorRow);
        }
        if (testResult.containsKey(6027)) {
            this.mShenzhenConfig.mSensorCol = ((Integer) testResult.get(6027)).intValue();
        }
        if (testResult.containsKey(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_VERSION))) {
            this.mShenzhenConfig.mLibaryVersion = (String) testResult.get(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_VERSION));
            Log.d(TAG, "mLibaryVersion " + this.mShenzhenConfig.mLibaryVersion);
        }
        if (testResult.containsKey(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_HIGH_BRIGHTNESS))) {
            this.mShenzhenConfig.mHighBrightness = ((Integer) testResult.get(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_HIGH_BRIGHTNESS))).intValue();
            Log.d(TAG, "mHighBrightness " + this.mShenzhenConfig.mHighBrightness);
        }
        if (testResult.containsKey(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_MID_BRIGHTNESS))) {
            this.mShenzhenConfig.mMiddleBrightness = ((Integer) testResult.get(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_MID_BRIGHTNESS))).intValue();
            Log.d(TAG, "mMiddleBrightness " + this.mShenzhenConfig.mMiddleBrightness);
        }
        if (testResult.containsKey(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_LOW_BRIGHTNESS))) {
            this.mShenzhenConfig.mLowBrightness = ((Integer) testResult.get(Integer.valueOf((int) ShenzhenTestResultParser.TEST_TOKEN_SS_IN_SYSTEM_LOW_BRIGHTNESS))).intValue();
            Log.d(TAG, "mLowBrightness " + this.mShenzhenConfig.mLowBrightness);
        }
        if (testResult.containsKey(812)) {
            this.mShenzhenConfig.mEnrollMinTemplateCount = ((Integer) testResult.get(812)).intValue();
            Log.d(TAG, "mEnrollMinTemplateCount " + this.mShenzhenConfig.mEnrollMinTemplateCount);
        }
        if (testResult.containsKey(515)) {
            this.mShenzhenConfig.mTestNum = ((Integer) testResult.get(515)).intValue();
        } else {
            this.mShenzhenConfig.mTestNum = 15;
        }
        Log.d(TAG, "mTestNum " + this.mShenzhenConfig.mTestNum);
        if (testResult.containsKey(200)) {
            this.mShenzhenConfig.mSDKVersion = (String) testResult.get(200);
            Log.d(TAG, "getTestNum from HAL: " + this.mShenzhenConfig.mSDKVersion);
        } else {
            this.mShenzhenConfig.mSDKVersion = "1.2.06,20,1.2.06,17";
        }
        Log.d(TAG, "getTestNum config: " + this.mShenzhenConfig.mSDKVersion);
        saveShenzhenConfig();
        return 0;
    }

    private void saveShenzhenConfig() {
        Log.d(TAG, "saveShenzhenConfig");
        Log.e(TAG, "saveShenzhenConfig mSensorX : " + this.mShenzhenConfig.mSensorY);
        SharedPreferences.Editor editor = this.mContext.getSharedPreferences(ShenzhenConstants.PREFERENCE_SENSOR_CONFIG_INFO, 0).edit();
        editor.putInt(ShenzhenConstants.PREFERENCE_SHORT_EXPOSURE_TIME, this.mShenzhenConfig.mShortExposureTime);
        editor.putInt(ShenzhenConstants.PREFERENCE_RECT_BMP_COL, this.mShenzhenConfig.mRect_bmp_col);
        editor.putInt(ShenzhenConstants.PREFERENCE_RECT_BMP_ROW, this.mShenzhenConfig.mRect_bmp_row);
        editor.putInt("sensor_x", this.mShenzhenConfig.mSensorX);
        editor.putInt("sensor_y", this.mShenzhenConfig.mSensorY);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_ROW, this.mShenzhenConfig.mSensorRow);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_COL, this.mShenzhenConfig.mSensorCol);
        editor.putInt("sensor_width", this.mShenzhenConfig.mSensorWidth);
        editor.putInt("sensor_height", this.mShenzhenConfig.mSensorHeight);
        editor.putBoolean("lock_aspect_ratio", this.mShenzhenConfig.mSensorLockAspectRatio);
        editor.putInt("aspect_ratio_width", this.mShenzhenConfig.mSensorAspectRatioWidth);
        editor.putInt("aspect_ratio_height", this.mShenzhenConfig.mSensorAspectRatioHeight);
        editor.putInt("preview_scale_ratio", this.mShenzhenConfig.mSensorPreviewScaleRatio);
        editor.putInt("sensor_area_background_color", this.mShenzhenConfig.mSensorAreaBackgroundColor);
        editor.putBoolean(ShenzhenConstants.SUPPORT_GRADIENT, this.mShenzhenConfig.mIsSupportGradient);
        editor.putInt(ShenzhenConstants.SUPPORT_GRADIENT_COLOR, this.mShenzhenConfig.mGradientColor);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_TOUCH_SENSITIVE, this.mShenzhenConfig.mTouchSensitive);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_RADIUS, this.mShenzhenConfig.mCircleRadius);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_STROKE_WIDTH, this.mShenzhenConfig.mCircleStokeWidth);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_COLOR, this.mShenzhenConfig.mCircleColor);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_AUTO_COLOR, this.mShenzhenConfig.mAutoBackgroundColor);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_RATE, this.mShenzhenConfig.mCircleRate);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_IN_SYSTEM_H_BRIGHTNESS, this.mShenzhenConfig.mHighBrightness);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_IN_SYSTEM_M_BRIGHTNESS, this.mShenzhenConfig.mMiddleBrightness);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_IN_SYSTEM_L_BRIGHTNESS, this.mShenzhenConfig.mLowBrightness);
        editor.putInt(ShenzhenConstants.PREFERENCE_KEY_ENROLLING_MIN_TEMPLATES, this.mShenzhenConfig.mEnrollMinTemplateCount);
        editor.commit();
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private void loadShenzhenConfig() {
        SharedPreferences pref = this.mContext.getSharedPreferences(ShenzhenConstants.PREFERENCE_SENSOR_CONFIG_INFO, 0);
        this.mShenzhenConfig.mShortExposureTime = pref.getInt(ShenzhenConstants.PREFERENCE_SHORT_EXPOSURE_TIME, 0);
        this.mShenzhenConfig.mRect_bmp_col = pref.getInt(ShenzhenConstants.PREFERENCE_RECT_BMP_COL, 0);
        this.mShenzhenConfig.mRect_bmp_row = pref.getInt(ShenzhenConstants.PREFERENCE_RECT_BMP_ROW, 0);
        this.mShenzhenConfig.mSensorRow = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_ROW, 0);
        this.mShenzhenConfig.mSensorCol = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_COL, 0);
        this.mShenzhenConfig.mSensorX = pref.getInt("sensor_x", ShenzhenConstants.DEFAULT_SENSOR_X);
        this.mShenzhenConfig.mSensorY = pref.getInt("sensor_y", ShenzhenConstants.DEFAULT_SENSOR_Y);
        this.mShenzhenConfig.mSensorWidth = pref.getInt("sensor_width", 173);
        this.mShenzhenConfig.mSensorHeight = pref.getInt("sensor_height", 173);
        this.mShenzhenConfig.mSensorLockAspectRatio = pref.getBoolean("lock_aspect_ratio", true);
        this.mShenzhenConfig.mSensorAspectRatioWidth = pref.getInt("aspect_ratio_width", 10);
        this.mShenzhenConfig.mSensorAspectRatioHeight = pref.getInt("aspect_ratio_height", 10);
        this.mShenzhenConfig.mSensorPreviewScaleRatio = pref.getInt("preview_scale_ratio", 150);
        this.mShenzhenConfig.mSensorAreaBackgroundColor = pref.getInt("sensor_area_background_color", -1);
        this.mShenzhenConfig.mTouchSensitive = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_TOUCH_SENSITIVE, 10);
        this.mShenzhenConfig.mIsSupportGradient = pref.getBoolean(ShenzhenConstants.SUPPORT_GRADIENT, false);
        this.mShenzhenConfig.mGradientColor = pref.getInt(ShenzhenConstants.SUPPORT_GRADIENT_COLOR, 0);
        this.mShenzhenConfig.mCircleRadius = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_RADIUS, 0);
        this.mShenzhenConfig.mCircleStokeWidth = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_STROKE_WIDTH, 0);
        this.mShenzhenConfig.mCircleColor = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_COLOR, 0);
        this.mShenzhenConfig.mAutoBackgroundColor = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_AUTO_COLOR, 0);
        this.mShenzhenConfig.mCircleRate = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_RATE, 0);
        this.mShenzhenConfig.mHighBrightness = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_IN_SYSTEM_H_BRIGHTNESS, 0);
        this.mShenzhenConfig.mMiddleBrightness = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_IN_SYSTEM_M_BRIGHTNESS, 0);
        this.mShenzhenConfig.mLowBrightness = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_IN_SYSTEM_L_BRIGHTNESS, 0);
        this.mShenzhenConfig.mEnrollMinTemplateCount = pref.getInt(ShenzhenConstants.PREFERENCE_KEY_ENROLLING_MIN_TEMPLATES, 20);
        Log.e(TAG, "loadShenzhenConfig mHighBrightness : " + this.mShenzhenConfig.mHighBrightness);
        Log.e(TAG, "loadShenzhenConfig mMiddleBrightness : " + this.mShenzhenConfig.mMiddleBrightness);
        Log.e(TAG, "loadShenzhenConfig mLowBrightness : " + this.mShenzhenConfig.mLowBrightness);
        Log.e(TAG, "loadShenzhenConfig mLibaryVersion : " + this.mShenzhenConfig.mLibaryVersion);
        Log.e(TAG, "loadShenzhenConfig mSensorX : " + this.mShenzhenConfig.mSensorY);
        Log.e(TAG, "loadShenzhenConfig mIsSupportGradient : " + pref.getBoolean(ShenzhenConstants.SUPPORT_GRADIENT, true));
        Log.e(TAG, "loadShenzhenConfig mGradientColor : " + pref.getInt(ShenzhenConstants.SUPPORT_GRADIENT_COLOR, 0));
        Log.e(TAG, "loadShenzhenConfig mCircleRadius : " + pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_STROKE_WIDTH, 0));
        Log.e(TAG, "loadShenzhenConfig mCircleStokeWidth : " + pref.getInt(ShenzhenConstants.SUPPORT_GRADIENT_COLOR, 0));
        Log.e(TAG, "loadShenzhenConfig mCircleColor : " + pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_COLOR, 0));
        Log.e(TAG, "loadShenzhenConfig mAutoBackgroundColor : " + pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_AUTO_COLOR, 0));
        Log.e(TAG, "loadShenzhenConfig mCircleRate : " + pref.getInt(ShenzhenConstants.PREFERENCE_KEY_SENSOR_CIRCLE_RATE, 0));
        Log.e(TAG, "loadShenzhenConfig mEnrollMinTemplateCount : " + pref.getInt(ShenzhenConstants.PREFERENCE_KEY_ENROLLING_MIN_TEMPLATES, 20));
    }

    public void openOrCloseHBMMode(int mode0) {
        if (mode0 == 1) {
            setHBMMode(true);
        } else if (mode0 == 0) {
            setHBMMode(false);
        }
    }

    public void setHBMMode(boolean open) {
        String content = "0x20000";
        if (!open) {
            content = "0xF0000";
        }
        try {
            Log.d(TAG, "setHBMMode , mode = " + open);
            FileOutputStream out = new FileOutputStream(new File("/sys/class/drm/card0-DSI-1/disp_param"));
            out.write(content.getBytes("ASCII"));
            out.close();
        } catch (Exception e) {
            Log.e(TAG, "write file failed!" + e);
        }
    }

    public void setScreenManualMode() {
        try {
            int mode = Settings.System.getInt(this.mContentResolver, "screen_brightness_mode");
            Log.d(TAG, "getCurrent mode = " + mode);
            if (mode == 1) {
                Settings.System.putInt(this.mContentResolver, "screen_brightness_mode", 0);
            }
        } catch (Settings.SettingNotFoundException e) {
            e.printStackTrace();
        }
    }

    public void setScreenBrightness(int value) {
        setScreenManualMode();
        Settings.System.putInt(this.mContentResolver, "screen_brightness", value);
    }

    public void setScreenBrightnessR15(int value) {
        try {
            Log.d(TAG, "setScreenBrightnessR15 , value = " + value);
            FileOutputStream out = new FileOutputStream(new File("/sys/class/backlight/panel/brightness"));
            out.write(String.valueOf(value).getBytes("ASCII"));
            out.close();
        } catch (Exception e) {
            Log.e(TAG, "write file failed!" + e);
        }
    }

    public int getScreenBrightness() {
        int value = Settings.System.getInt(this.mContentResolver, "screen_brightness", 255);
        Log.d(TAG, "getScreenBrightness ==" + value);
        return value;
    }

    public GFShenzhenConfig getShenzhenConfig() {
        Log.d(TAG, "getShenzhenConfig");
        loadShenzhenConfig();
        return new GFShenzhenConfig(this.mShenzhenConfig);
    }

    public void setShenzhenConfig(GFShenzhenConfig config) {
        Log.d(TAG, "setShenzhenConfig :mIsSupportGradient= " + config.mIsSupportGradient + " mGradientColor= " + config.mGradientColor);
        this.mShenzhenConfig = new GFShenzhenConfig(config);
        saveShenzhenConfig();
    }
}
