package com.google.common.eventbus;

import com.google.common.base.Preconditions;
import com.google.common.base.Throwables;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import com.google.common.collect.SetMultimap;
import com.google.common.reflect.TypeToken;
import com.google.common.util.concurrent.UncheckedExecutionException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.logging.Level;
import java.util.logging.Logger;

public class EventBus {
    private static final LoadingCache<Class<?>, Set<Class<?>>> flattenHierarchyCache = CacheBuilder.newBuilder().weakKeys().build(new CacheLoader<Class<?>, Set<Class<?>>>() {
        /* class com.google.common.eventbus.EventBus.AnonymousClass1 */

        public Set<Class<?>> load(Class<?> concreteClass) {
            return TypeToken.of((Class) concreteClass).getTypes().rawTypes();
        }
    });
    private final ThreadLocal<Queue<EventWithHandler>> eventsToDispatch;
    private final HandlerFindingStrategy finder;
    private final SetMultimap<Class<?>, EventHandler> handlersByType;
    private final ReadWriteLock handlersByTypeLock;
    private final ThreadLocal<Boolean> isDispatching;
    private final Logger logger;

    public EventBus() {
        this("default");
    }

    public EventBus(String identifier) {
        this.handlersByType = HashMultimap.create();
        this.handlersByTypeLock = new ReentrantReadWriteLock();
        this.finder = new AnnotatedHandlerFinder();
        this.eventsToDispatch = new ThreadLocal<Queue<EventWithHandler>>() {
            /* class com.google.common.eventbus.EventBus.AnonymousClass2 */

            /* access modifiers changed from: protected */
            @Override // java.lang.ThreadLocal
            public Queue<EventWithHandler> initialValue() {
                return new LinkedList();
            }
        };
        this.isDispatching = new ThreadLocal<Boolean>() {
            /* class com.google.common.eventbus.EventBus.AnonymousClass3 */

            /* access modifiers changed from: protected */
            @Override // java.lang.ThreadLocal
            public Boolean initialValue() {
                return false;
            }
        };
        this.logger = Logger.getLogger(EventBus.class.getName() + "." + ((String) Preconditions.checkNotNull(identifier)));
    }

    public void register(Object object) {
        Multimap<Class<?>, EventHandler> methodsInListener = this.finder.findAllHandlers(object);
        this.handlersByTypeLock.writeLock().lock();
        try {
            this.handlersByType.putAll(methodsInListener);
        } finally {
            this.handlersByTypeLock.writeLock().unlock();
        }
    }

    public void unregister(Object object) {
        for (Map.Entry<Class<?>, Collection<EventHandler>> entry : this.finder.findAllHandlers(object).asMap().entrySet()) {
            Class<?> eventType = entry.getKey();
            Collection<EventHandler> eventMethodsInListener = entry.getValue();
            this.handlersByTypeLock.writeLock().lock();
            try {
                Set<EventHandler> currentHandlers = this.handlersByType.get(eventType);
                if (currentHandlers.containsAll(eventMethodsInListener)) {
                    currentHandlers.removeAll(eventMethodsInListener);
                } else {
                    throw new IllegalArgumentException("missing event handler for an annotated method. Is " + object + " registered?");
                }
            } finally {
                this.handlersByTypeLock.writeLock().unlock();
            }
        }
    }

    public void post(Object event) {
        boolean dispatched = false;
        for (Class<?> eventType : flattenHierarchy(event.getClass())) {
            this.handlersByTypeLock.readLock().lock();
            try {
                Set<EventHandler> wrappers = this.handlersByType.get(eventType);
                if (!wrappers.isEmpty()) {
                    dispatched = true;
                    for (EventHandler wrapper : wrappers) {
                        enqueueEvent(event, wrapper);
                    }
                }
            } finally {
                this.handlersByTypeLock.readLock().unlock();
            }
        }
        if (!dispatched && !(event instanceof DeadEvent)) {
            post(new DeadEvent(this, event));
        }
        dispatchQueuedEvents();
    }

    /* access modifiers changed from: package-private */
    public void enqueueEvent(Object event, EventHandler handler) {
        this.eventsToDispatch.get().offer(new EventWithHandler(event, handler));
    }

    /* access modifiers changed from: package-private */
    public void dispatchQueuedEvents() {
        if (!this.isDispatching.get().booleanValue()) {
            this.isDispatching.set(true);
            try {
                Queue<EventWithHandler> events = this.eventsToDispatch.get();
                while (true) {
                    EventWithHandler eventWithHandler = events.poll();
                    if (eventWithHandler != null) {
                        dispatch(eventWithHandler.event, eventWithHandler.handler);
                    } else {
                        return;
                    }
                }
            } finally {
                this.isDispatching.remove();
                this.eventsToDispatch.remove();
            }
        }
    }

    /* access modifiers changed from: package-private */
    public void dispatch(Object event, EventHandler wrapper) {
        try {
            wrapper.handleEvent(event);
        } catch (InvocationTargetException e) {
            Logger logger2 = this.logger;
            Level level = Level.SEVERE;
            logger2.log(level, "Could not dispatch event: " + event + " to handler " + wrapper, (Throwable) e);
        }
    }

    /* access modifiers changed from: package-private */
    public Set<Class<?>> flattenHierarchy(Class<?> concreteClass) {
        try {
            return flattenHierarchyCache.getUnchecked(concreteClass);
        } catch (UncheckedExecutionException e) {
            throw Throwables.propagate(e.getCause());
        }
    }

    /* access modifiers changed from: package-private */
    public static class EventWithHandler {
        final Object event;
        final EventHandler handler;

        public EventWithHandler(Object event2, EventHandler handler2) {
            this.event = Preconditions.checkNotNull(event2);
            this.handler = (EventHandler) Preconditions.checkNotNull(handler2);
        }
    }
}
