package com.google.common.hash;

import com.google.common.base.Preconditions;
import com.google.common.hash.BloomFilter;
import com.google.common.math.LongMath;
import com.google.common.primitives.Ints;
import java.math.RoundingMode;
import java.util.Arrays;

enum BloomFilterStrategies implements BloomFilter.Strategy {
    MURMUR128_MITZ_32 {
        @Override // com.google.common.hash.BloomFilter.Strategy
        public <T> boolean put(T object, Funnel<? super T> funnel, int numHashFunctions, BitArray bits) {
            long hash64 = Hashing.murmur3_128().hashObject(object, funnel).asLong();
            int hash1 = (int) hash64;
            int hash2 = (int) (hash64 >>> 32);
            boolean bitsChanged = false;
            for (int i = 1; i <= numHashFunctions; i++) {
                int nextHash = (i * hash2) + hash1;
                if (nextHash < 0) {
                    nextHash = ~nextHash;
                }
                bitsChanged |= bits.set(nextHash % bits.bitSize());
            }
            return bitsChanged;
        }

        @Override // com.google.common.hash.BloomFilter.Strategy
        public <T> boolean mightContain(T object, Funnel<? super T> funnel, int numHashFunctions, BitArray bits) {
            long hash64 = Hashing.murmur3_128().hashObject(object, funnel).asLong();
            int hash1 = (int) hash64;
            int hash2 = (int) (hash64 >>> 32);
            for (int i = 1; i <= numHashFunctions; i++) {
                int nextHash = (i * hash2) + hash1;
                if (nextHash < 0) {
                    nextHash = ~nextHash;
                }
                if (!bits.get(nextHash % bits.bitSize())) {
                    return false;
                }
            }
            return true;
        }
    };

    /* access modifiers changed from: package-private */
    public static class BitArray {
        int bitCount;
        final long[] data;

        BitArray(long bits) {
            this(new long[Ints.checkedCast(LongMath.divide(bits, 64, RoundingMode.CEILING))]);
        }

        BitArray(long[] data2) {
            Preconditions.checkArgument(data2.length > 0, "data length is zero!");
            this.data = data2;
            int bitCount2 = 0;
            for (long value : data2) {
                bitCount2 += Long.bitCount(value);
            }
            this.bitCount = bitCount2;
        }

        /* access modifiers changed from: package-private */
        public boolean set(int index) {
            if (get(index)) {
                return false;
            }
            long[] jArr = this.data;
            int i = index >> 6;
            jArr[i] = jArr[i] | (1 << index);
            this.bitCount++;
            return true;
        }

        /* access modifiers changed from: package-private */
        public boolean get(int index) {
            return (this.data[index >> 6] & (1 << index)) != 0;
        }

        /* access modifiers changed from: package-private */
        public int bitSize() {
            return this.data.length * 64;
        }

        /* access modifiers changed from: package-private */
        public int bitCount() {
            return this.bitCount;
        }

        /* access modifiers changed from: package-private */
        public BitArray copy() {
            return new BitArray((long[]) this.data.clone());
        }

        /* access modifiers changed from: package-private */
        public void putAll(BitArray array) {
            long[] jArr = this.data;
            Preconditions.checkArgument(jArr.length == array.data.length, "BitArrays must be of equal length (%s != %s)", Integer.valueOf(jArr.length), Integer.valueOf(array.data.length));
            this.bitCount = 0;
            int i = 0;
            while (true) {
                long[] jArr2 = this.data;
                if (i < jArr2.length) {
                    jArr2[i] = jArr2[i] | array.data[i];
                    this.bitCount += Long.bitCount(jArr2[i]);
                    i++;
                } else {
                    return;
                }
            }
        }

        public boolean equals(Object o) {
            if (o instanceof BitArray) {
                return Arrays.equals(this.data, ((BitArray) o).data);
            }
            return false;
        }

        public int hashCode() {
            return Arrays.hashCode(this.data);
        }
    }
}
