package com.google.common.util.concurrent;

import com.google.common.base.Preconditions;
import com.google.common.collect.Queues;
import java.util.Iterator;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Executor;
import java.util.concurrent.locks.ReentrantLock;
import java.util.logging.Level;
import java.util.logging.Logger;

/* access modifiers changed from: package-private */
public final class ExecutionQueue {
    private static final Logger logger = Logger.getLogger(ExecutionQueue.class.getName());
    private final ReentrantLock lock = new ReentrantLock();
    private final ConcurrentLinkedQueue<RunnableExecutorPair> queuedListeners = Queues.newConcurrentLinkedQueue();

    ExecutionQueue() {
    }

    /* access modifiers changed from: package-private */
    public void add(Runnable runnable, Executor executor) {
        this.queuedListeners.add(new RunnableExecutorPair(runnable, executor));
    }

    /* access modifiers changed from: package-private */
    public void execute() {
        Iterator<RunnableExecutorPair> iterator = this.queuedListeners.iterator();
        while (iterator.hasNext()) {
            iterator.next().submit();
            iterator.remove();
        }
    }

    /* access modifiers changed from: private */
    public final class RunnableExecutorPair implements Runnable {
        private final Executor executor;
        private boolean hasBeenExecuted = false;
        private final Runnable runnable;

        RunnableExecutorPair(Runnable runnable2, Executor executor2) {
            this.runnable = (Runnable) Preconditions.checkNotNull(runnable2);
            this.executor = (Executor) Preconditions.checkNotNull(executor2);
        }

        /* access modifiers changed from: private */
        /* access modifiers changed from: public */
        private void submit() {
            ExecutionQueue.this.lock.lock();
            try {
                if (!this.hasBeenExecuted) {
                    try {
                        this.executor.execute(this);
                    } catch (Exception e) {
                        Logger logger = ExecutionQueue.logger;
                        Level level = Level.SEVERE;
                        logger.log(level, "Exception while executing listener " + this.runnable + " with executor " + this.executor, (Throwable) e);
                    }
                }
            } finally {
                if (ExecutionQueue.this.lock.isHeldByCurrentThread()) {
                    this.hasBeenExecuted = true;
                    ExecutionQueue.this.lock.unlock();
                }
            }
        }

        public final void run() {
            if (ExecutionQueue.this.lock.isHeldByCurrentThread()) {
                this.hasBeenExecuted = true;
                ExecutionQueue.this.lock.unlock();
            }
            this.runnable.run();
        }
    }
}
