package com.google.common.util.concurrent;

import com.google.common.base.Function;
import com.google.common.base.Objects;
import com.google.common.base.Preconditions;
import com.google.common.base.Predicates;
import com.google.common.base.Stopwatch;
import com.google.common.collect.Collections2;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableMultimap;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Ordering;
import com.google.common.util.concurrent.Monitor;
import com.google.common.util.concurrent.Service;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Executor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.inject.Inject;
import javax.inject.Singleton;

@Singleton
public final class ServiceManager {
    private static final Logger logger = Logger.getLogger(ServiceManager.class.getName());
    private final ImmutableMap<Service, ServiceListener> services;
    private final ServiceManagerState state;

    public static abstract class Listener {
        public void healthy() {
        }

        public void stopped() {
        }

        public void failure(Service service) {
        }
    }

    public ServiceManager(Iterable<? extends Service> services2) {
        ImmutableList<Service> copy = ImmutableList.copyOf(services2);
        if (copy.isEmpty()) {
            logger.log(Level.WARNING, "ServiceManager configured with no services.  Is your application configured properly?", (Throwable) new EmptyServiceManagerWarning());
            copy = ImmutableList.of(new NoOpService());
        }
        this.state = new ServiceManagerState(copy.size());
        ImmutableMap.Builder<Service, ServiceListener> builder = ImmutableMap.builder();
        Executor executor = MoreExecutors.sameThreadExecutor();
        Iterator i$ = copy.iterator();
        while (i$.hasNext()) {
            Service service = (Service) i$.next();
            ServiceListener listener = new ServiceListener(service, this.state);
            service.addListener(listener, executor);
            Preconditions.checkArgument(service.state() == Service.State.NEW, "Can only manage NEW services, %s", service);
            builder.put(service, listener);
        }
        this.services = builder.build();
    }

    @Inject
    ServiceManager(Set<Service> services2) {
        this((Iterable<? extends Service>) services2);
    }

    public void addListener(Listener listener, Executor executor) {
        this.state.addListener(listener, executor);
    }

    public void addListener(Listener listener) {
        this.state.addListener(listener, MoreExecutors.sameThreadExecutor());
    }

    public ServiceManager startAsync() {
        Iterator i$ = this.services.entrySet().iterator();
        while (i$.hasNext()) {
            Service service = ((Map.Entry) i$.next()).getKey();
            Service.State state2 = service.state();
            Preconditions.checkState(state2 == Service.State.NEW, "Service %s is %s, cannot start it.", service, state2);
        }
        Iterator i$2 = this.services.values().iterator();
        while (i$2.hasNext()) {
            ServiceListener service2 = (ServiceListener) i$2.next();
            try {
                service2.start();
            } catch (IllegalStateException e) {
                Logger logger2 = logger;
                Level level = Level.WARNING;
                logger2.log(level, "Unable to start Service " + service2.service, (Throwable) e);
            }
        }
        return this;
    }

    public void awaitHealthy() {
        this.state.awaitHealthy();
        Preconditions.checkState(isHealthy(), "Expected to be healthy after starting");
    }

    public void awaitHealthy(long timeout, TimeUnit unit) throws TimeoutException {
        if (this.state.awaitHealthy(timeout, unit)) {
            Preconditions.checkState(isHealthy(), "Expected to be healthy after starting");
            return;
        }
        throw new TimeoutException("Timeout waiting for the services to become healthy.");
    }

    public ServiceManager stopAsync() {
        Iterator i$ = this.services.keySet().iterator();
        while (i$.hasNext()) {
            ((Service) i$.next()).stop();
        }
        return this;
    }

    public void awaitStopped() {
        this.state.awaitStopped();
    }

    public void awaitStopped(long timeout, TimeUnit unit) throws TimeoutException {
        if (!this.state.awaitStopped(timeout, unit)) {
            throw new TimeoutException("Timeout waiting for the services to stop.");
        }
    }

    public boolean isHealthy() {
        Iterator i$ = this.services.keySet().iterator();
        while (i$.hasNext()) {
            if (!((Service) i$.next()).isRunning()) {
                return false;
            }
        }
        return true;
    }

    public ImmutableMultimap<Service.State, Service> servicesByState() {
        ImmutableMultimap.Builder<Service.State, Service> builder = ImmutableMultimap.builder();
        Iterator i$ = this.services.keySet().iterator();
        while (i$.hasNext()) {
            Service service = (Service) i$.next();
            if (!(service instanceof NoOpService)) {
                builder.put(service.state(), service);
            }
        }
        return builder.build();
    }

    public ImmutableMap<Service, Long> startupTimes() {
        List<Map.Entry<Service, Long>> loadTimes = Lists.newArrayListWithCapacity(this.services.size());
        Iterator i$ = this.services.entrySet().iterator();
        while (i$.hasNext()) {
            Map.Entry<Service, ServiceListener> entry = (Map.Entry) i$.next();
            Service service = entry.getKey();
            Service.State state2 = service.state();
            boolean z = false;
            boolean z2 = state2 != Service.State.NEW;
            if (state2 != Service.State.STARTING) {
                z = true;
            }
            if ((z2 & z) && (!(service instanceof NoOpService))) {
                loadTimes.add(Maps.immutableEntry(service, Long.valueOf(entry.getValue().startupTimeMillis())));
            }
        }
        Collections.sort(loadTimes, Ordering.natural().onResultOf(new Function<Map.Entry<Service, Long>, Long>() {
            /* class com.google.common.util.concurrent.ServiceManager.AnonymousClass1 */

            public Long apply(Map.Entry<Service, Long> input) {
                return input.getValue();
            }
        }));
        ImmutableMap.Builder<Service, Long> builder = ImmutableMap.builder();
        Iterator i$2 = loadTimes.iterator();
        while (i$2.hasNext()) {
            builder.put(i$2.next());
        }
        return builder.build();
    }

    public String toString() {
        return Objects.toStringHelper((Class<?>) ServiceManager.class).add("services", Collections2.filter(this.services.keySet(), Predicates.not(Predicates.instanceOf(NoOpService.class)))).toString();
    }

    /* access modifiers changed from: private */
    public static final class ServiceManagerState {
        final Monitor.Guard awaitHealthGuard;
        final List<ListenerExecutorPair> listeners = Lists.newArrayList();
        final Monitor monitor;
        final int numberOfServices;
        final ExecutionQueue queuedListeners = new ExecutionQueue();
        final Monitor.Guard stoppedGuard;
        int unstartedServices;
        int unstoppedServices;

        ServiceManagerState(int numberOfServices2) {
            Monitor monitor2 = new Monitor();
            this.monitor = monitor2;
            this.awaitHealthGuard = new Monitor.Guard(monitor2) {
                /* class com.google.common.util.concurrent.ServiceManager.ServiceManagerState.AnonymousClass1 */

                @Override // com.google.common.util.concurrent.Monitor.Guard
                public boolean isSatisfied() {
                    boolean z = true;
                    boolean z2 = ServiceManagerState.this.unstartedServices == 0;
                    if (ServiceManagerState.this.unstoppedServices == ServiceManagerState.this.numberOfServices) {
                        z = false;
                    }
                    return z2 | z;
                }
            };
            this.stoppedGuard = new Monitor.Guard(monitor2) {
                /* class com.google.common.util.concurrent.ServiceManager.ServiceManagerState.AnonymousClass2 */

                @Override // com.google.common.util.concurrent.Monitor.Guard
                public boolean isSatisfied() {
                    return ServiceManagerState.this.unstoppedServices == 0;
                }
            };
            this.numberOfServices = numberOfServices2;
            this.unstoppedServices = numberOfServices2;
            this.unstartedServices = numberOfServices2;
        }

        /* access modifiers changed from: package-private */
        public void addListener(Listener listener, Executor executor) {
            Preconditions.checkNotNull(listener, "listener");
            Preconditions.checkNotNull(executor, "executor");
            this.monitor.enter();
            try {
                if (this.unstartedServices > 0 || this.unstoppedServices > 0) {
                    this.listeners.add(new ListenerExecutorPair(listener, executor));
                }
            } finally {
                this.monitor.leave();
            }
        }

        /* access modifiers changed from: package-private */
        public void awaitHealthy() {
            this.monitor.enterWhenUninterruptibly(this.awaitHealthGuard);
            this.monitor.leave();
        }

        /* access modifiers changed from: package-private */
        public boolean awaitHealthy(long timeout, TimeUnit unit) {
            if (!this.monitor.enterWhenUninterruptibly(this.awaitHealthGuard, timeout, unit)) {
                return false;
            }
            this.monitor.leave();
            return true;
        }

        /* access modifiers changed from: package-private */
        public void awaitStopped() {
            this.monitor.enterWhenUninterruptibly(this.stoppedGuard);
            this.monitor.leave();
        }

        /* access modifiers changed from: package-private */
        public boolean awaitStopped(long timeout, TimeUnit unit) {
            if (!this.monitor.enterWhenUninterruptibly(this.stoppedGuard, timeout, unit)) {
                return false;
            }
            this.monitor.leave();
            return true;
        }

        /* access modifiers changed from: private */
        /* access modifiers changed from: public */
        private void serviceFinishedStarting(Service service, boolean currentlyHealthy) {
            Preconditions.checkState(this.unstartedServices > 0, "All services should have already finished starting but %s just finished.", service);
            int i = this.unstartedServices - 1;
            this.unstartedServices = i;
            if (currentlyHealthy && i == 0 && this.unstoppedServices == this.numberOfServices) {
                for (final ListenerExecutorPair pair : this.listeners) {
                    this.queuedListeners.add(new Runnable() {
                        /* class com.google.common.util.concurrent.ServiceManager.ServiceManagerState.AnonymousClass3 */

                        public void run() {
                            pair.listener.healthy();
                        }
                    }, pair.executor);
                }
            }
        }

        /* access modifiers changed from: private */
        /* access modifiers changed from: public */
        private void serviceTerminated(Service service) {
            serviceStopped(service);
        }

        /* access modifiers changed from: private */
        /* access modifiers changed from: public */
        private void serviceFailed(final Service service) {
            for (final ListenerExecutorPair pair : this.listeners) {
                this.queuedListeners.add(new Runnable() {
                    /* class com.google.common.util.concurrent.ServiceManager.ServiceManagerState.AnonymousClass4 */

                    public void run() {
                        pair.listener.failure(service);
                    }
                }, pair.executor);
            }
            serviceStopped(service);
        }

        private void serviceStopped(Service service) {
            Preconditions.checkState(this.unstoppedServices > 0, "All services should have already stopped but %s just stopped.", service);
            int i = this.unstoppedServices - 1;
            this.unstoppedServices = i;
            if (i == 0) {
                int i2 = this.unstartedServices;
                Preconditions.checkState(i2 == 0, "All services are stopped but %d services haven't finished starting", Integer.valueOf(i2));
                for (final ListenerExecutorPair pair : this.listeners) {
                    this.queuedListeners.add(new Runnable() {
                        /* class com.google.common.util.concurrent.ServiceManager.ServiceManagerState.AnonymousClass5 */

                        public void run() {
                            pair.listener.stopped();
                        }
                    }, pair.executor);
                }
                this.listeners.clear();
            }
        }

        /* access modifiers changed from: private */
        /* access modifiers changed from: public */
        private void executeListeners() {
            Preconditions.checkState(!this.monitor.isOccupiedByCurrentThread(), "It is incorrect to execute listeners with the monitor held.");
            this.queuedListeners.execute();
        }
    }

    /* access modifiers changed from: private */
    public static final class ServiceListener extends Service.Listener {
        final Service service;
        final ServiceManagerState state;
        final Stopwatch watch = Stopwatch.createUnstarted();

        ServiceListener(Service service2, ServiceManagerState state2) {
            this.service = service2;
            this.state = state2;
        }

        @Override // com.google.common.util.concurrent.Service.Listener
        public void starting() {
            startTimer();
        }

        @Override // com.google.common.util.concurrent.Service.Listener
        public void running() {
            this.state.monitor.enter();
            try {
                finishedStarting(true);
            } finally {
                this.state.monitor.leave();
                this.state.executeListeners();
            }
        }

        @Override // com.google.common.util.concurrent.Service.Listener
        public void stopping(Service.State from) {
            if (from == Service.State.STARTING) {
                this.state.monitor.enter();
                try {
                    finishedStarting(false);
                } finally {
                    this.state.monitor.leave();
                    this.state.executeListeners();
                }
            }
        }

        @Override // com.google.common.util.concurrent.Service.Listener
        public void terminated(Service.State from) {
            if (!(this.service instanceof NoOpService)) {
                ServiceManager.logger.log(Level.FINE, "Service {0} has terminated. Previous state was: {1}", new Object[]{this.service, from});
            }
            this.state.monitor.enter();
            try {
                if (from == Service.State.NEW) {
                    startTimer();
                    finishedStarting(false);
                }
                this.state.serviceTerminated(this.service);
            } finally {
                this.state.monitor.leave();
                this.state.executeListeners();
            }
        }

        @Override // com.google.common.util.concurrent.Service.Listener
        public void failed(Service.State from, Throwable failure) {
            Logger logger = ServiceManager.logger;
            Level level = Level.SEVERE;
            logger.log(level, "Service " + this.service + " has failed in the " + from + " state.", failure);
            this.state.monitor.enter();
            try {
                if (from == Service.State.STARTING) {
                    finishedStarting(false);
                }
                this.state.serviceFailed(this.service);
            } finally {
                this.state.monitor.leave();
                this.state.executeListeners();
            }
        }

        /* access modifiers changed from: package-private */
        public void finishedStarting(boolean currentlyHealthy) {
            synchronized (this.watch) {
                this.watch.stop();
                if (!(this.service instanceof NoOpService)) {
                    ServiceManager.logger.log(Level.FINE, "Started {0} in {1} ms.", new Object[]{this.service, Long.valueOf(startupTimeMillis())});
                }
            }
            this.state.serviceFinishedStarting(this.service, currentlyHealthy);
        }

        /* access modifiers changed from: package-private */
        public void start() {
            startTimer();
            this.service.startAsync();
        }

        /* access modifiers changed from: package-private */
        public void startTimer() {
            synchronized (this.watch) {
                if (!this.watch.isRunning()) {
                    this.watch.start();
                    if (!(this.service instanceof NoOpService)) {
                        ServiceManager.logger.log(Level.FINE, "Starting {0}.", this.service);
                    }
                }
            }
        }

        /* access modifiers changed from: package-private */
        public long startupTimeMillis() {
            long elapsed;
            synchronized (this.watch) {
                elapsed = this.watch.elapsed(TimeUnit.MILLISECONDS);
            }
            return elapsed;
        }
    }

    /* access modifiers changed from: private */
    public static final class ListenerExecutorPair {
        final Executor executor;
        final Listener listener;

        ListenerExecutorPair(Listener listener2, Executor executor2) {
            this.listener = listener2;
            this.executor = executor2;
        }
    }

    /* access modifiers changed from: private */
    public static final class NoOpService extends AbstractService {
        private NoOpService() {
        }

        /* access modifiers changed from: protected */
        @Override // com.google.common.util.concurrent.AbstractService
        public void doStart() {
            notifyStarted();
        }

        /* access modifiers changed from: protected */
        @Override // com.google.common.util.concurrent.AbstractService
        public void doStop() {
            notifyStopped();
        }
    }

    private static final class EmptyServiceManagerWarning extends Throwable {
        private EmptyServiceManagerWarning() {
        }
    }
}
