package com.google.zxing.oned;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.ChecksumException;
import com.google.zxing.DecodeHintType;
import com.google.zxing.FormatException;
import com.google.zxing.NotFoundException;
import com.google.zxing.Result;
import com.google.zxing.ResultPoint;
import com.google.zxing.common.BitArray;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public final class Code128Reader extends OneDReader {
    private static final int CODE_CODE_A = 101;
    private static final int CODE_CODE_B = 100;
    private static final int CODE_CODE_C = 99;
    private static final int CODE_FNC_1 = 102;
    private static final int CODE_FNC_2 = 97;
    private static final int CODE_FNC_3 = 96;
    private static final int CODE_FNC_4_A = 101;
    private static final int CODE_FNC_4_B = 100;
    static final int[][] CODE_PATTERNS = {new int[]{2, 1, 2, 2, 2, 2}, new int[]{2, 2, 2, 1, 2, 2}, new int[]{2, 2, 2, 2, 2, 1}, new int[]{1, 2, 1, 2, 2, 3}, new int[]{1, 2, 1, 3, 2, 2}, new int[]{1, 3, 1, 2, 2, 2}, new int[]{1, 2, 2, 2, 1, 3}, new int[]{1, 2, 2, 3, 1, 2}, new int[]{1, 3, 2, 2, 1, 2}, new int[]{2, 2, 1, 2, 1, 3}, new int[]{2, 2, 1, 3, 1, 2}, new int[]{2, 3, 1, 2, 1, 2}, new int[]{1, 1, 2, 2, 3, 2}, new int[]{1, 2, 2, 1, 3, 2}, new int[]{1, 2, 2, 2, 3, 1}, new int[]{1, 1, 3, 2, 2, 2}, new int[]{1, 2, 3, 1, 2, 2}, new int[]{1, 2, 3, 2, 2, 1}, new int[]{2, 2, 3, 2, 1, 1}, new int[]{2, 2, 1, 1, 3, 2}, new int[]{2, 2, 1, 2, 3, 1}, new int[]{2, 1, 3, 2, 1, 2}, new int[]{2, 2, 3, 1, 1, 2}, new int[]{3, 1, 2, 1, 3, 1}, new int[]{3, 1, 1, 2, 2, 2}, new int[]{3, 2, 1, 1, 2, 2}, new int[]{3, 2, 1, 2, 2, 1}, new int[]{3, 1, 2, 2, 1, 2}, new int[]{3, 2, 2, 1, 1, 2}, new int[]{3, 2, 2, 2, 1, 1}, new int[]{2, 1, 2, 1, 2, 3}, new int[]{2, 1, 2, 3, 2, 1}, new int[]{2, 3, 2, 1, 2, 1}, new int[]{1, 1, 1, 3, 2, 3}, new int[]{1, 3, 1, 1, 2, 3}, new int[]{1, 3, 1, 3, 2, 1}, new int[]{1, 1, 2, 3, 1, 3}, new int[]{1, 3, 2, 1, 1, 3}, new int[]{1, 3, 2, 3, 1, 1}, new int[]{2, 1, 1, 3, 1, 3}, new int[]{2, 3, 1, 1, 1, 3}, new int[]{2, 3, 1, 3, 1, 1}, new int[]{1, 1, 2, 1, 3, 3}, new int[]{1, 1, 2, 3, 3, 1}, new int[]{1, 3, 2, 1, 3, 1}, new int[]{1, 1, 3, 1, 2, 3}, new int[]{1, 1, 3, 3, 2, 1}, new int[]{1, 3, 3, 1, 2, 1}, new int[]{3, 1, 3, 1, 2, 1}, new int[]{2, 1, 1, 3, 3, 1}, new int[]{2, 3, 1, 1, 3, 1}, new int[]{2, 1, 3, 1, 1, 3}, new int[]{2, 1, 3, 3, 1, 1}, new int[]{2, 1, 3, 1, 3, 1}, new int[]{3, 1, 1, 1, 2, 3}, new int[]{3, 1, 1, 3, 2, 1}, new int[]{3, 3, 1, 1, 2, 1}, new int[]{3, 1, 2, 1, 1, 3}, new int[]{3, 1, 2, 3, 1, 1}, new int[]{3, 3, 2, 1, 1, 1}, new int[]{3, 1, 4, 1, 1, 1}, new int[]{2, 2, 1, 4, 1, 1}, new int[]{4, 3, 1, 1, 1, 1}, new int[]{1, 1, 1, 2, 2, 4}, new int[]{1, 1, 1, 4, 2, 2}, new int[]{1, 2, 1, 1, 2, 4}, new int[]{1, 2, 1, 4, 2, 1}, new int[]{1, 4, 1, 1, 2, 2}, new int[]{1, 4, 1, 2, 2, 1}, new int[]{1, 1, 2, 2, 1, 4}, new int[]{1, 1, 2, 4, 1, 2}, new int[]{1, 2, 2, 1, 1, 4}, new int[]{1, 2, 2, 4, 1, 1}, new int[]{1, 4, 2, 1, 1, 2}, new int[]{1, 4, 2, 2, 1, 1}, new int[]{2, 4, 1, 2, 1, 1}, new int[]{2, 2, 1, 1, 1, 4}, new int[]{4, 1, 3, 1, 1, 1}, new int[]{2, 4, 1, 1, 1, 2}, new int[]{1, 3, 4, 1, 1, 1}, new int[]{1, 1, 1, 2, 4, 2}, new int[]{1, 2, 1, 1, 4, 2}, new int[]{1, 2, 1, 2, 4, 1}, new int[]{1, 1, 4, 2, 1, 2}, new int[]{1, 2, 4, 1, 1, 2}, new int[]{1, 2, 4, 2, 1, 1}, new int[]{4, 1, 1, 2, 1, 2}, new int[]{4, 2, 1, 1, 1, 2}, new int[]{4, 2, 1, 2, 1, 1}, new int[]{2, 1, 2, 1, 4, 1}, new int[]{2, 1, 4, 1, 2, 1}, new int[]{4, 1, 2, 1, 2, 1}, new int[]{1, 1, 1, 1, 4, 3}, new int[]{1, 1, 1, 3, 4, 1}, new int[]{1, 3, 1, 1, 4, 1}, new int[]{1, 1, 4, 1, 1, 3}, new int[]{1, 1, 4, 3, 1, 1}, new int[]{4, 1, 1, 1, 1, 3}, new int[]{4, 1, 1, 3, 1, 1}, new int[]{1, 1, 3, 1, 4, 1}, new int[]{1, 1, 4, 1, 3, 1}, new int[]{3, 1, 1, 1, 4, 1}, new int[]{4, 1, 1, 1, 3, 1}, new int[]{2, 1, 1, 4, 1, 2}, new int[]{2, 1, 1, 2, 1, 4}, new int[]{2, 1, 1, 2, 3, 2}, new int[]{2, 3, 3, 1, 1, 1, 2}};
    private static final int CODE_SHIFT = 98;
    private static final int CODE_START_A = 103;
    private static final int CODE_START_B = 104;
    private static final int CODE_START_C = 105;
    private static final int CODE_STOP = 106;
    private static final float MAX_AVG_VARIANCE = 0.25f;
    private static final float MAX_INDIVIDUAL_VARIANCE = 0.7f;

    private static int[] findStartPattern(BitArray row) throws NotFoundException {
        int width = row.getSize();
        int rowOffset = row.getNextSet(0);
        int counterPosition = 0;
        int[] counters = new int[6];
        int patternStart = rowOffset;
        boolean isWhite = false;
        int patternLength = counters.length;
        for (int i = rowOffset; i < width; i++) {
            boolean z = true;
            if (row.get(i) ^ isWhite) {
                counters[counterPosition] = counters[counterPosition] + 1;
            } else {
                if (counterPosition == patternLength - 1) {
                    float bestVariance = MAX_AVG_VARIANCE;
                    int bestMatch = -1;
                    for (int startCode = 103; startCode <= 105; startCode++) {
                        float variance = patternMatchVariance(counters, CODE_PATTERNS[startCode], MAX_INDIVIDUAL_VARIANCE);
                        if (variance < bestVariance) {
                            bestVariance = variance;
                            bestMatch = startCode;
                        }
                    }
                    if (bestMatch < 0 || !row.isRange(Math.max(0, patternStart - ((i - patternStart) / 2)), patternStart, false)) {
                        patternStart += counters[0] + counters[1];
                        System.arraycopy(counters, 2, counters, 0, patternLength - 2);
                        counters[patternLength - 2] = 0;
                        counters[patternLength - 1] = 0;
                        counterPosition--;
                    } else {
                        return new int[]{patternStart, i, bestMatch};
                    }
                } else {
                    counterPosition++;
                }
                counters[counterPosition] = 1;
                if (isWhite) {
                    z = false;
                }
                isWhite = z;
            }
        }
        throw NotFoundException.getNotFoundInstance();
    }

    private static int decodeCode(BitArray row, int[] counters, int rowOffset) throws NotFoundException {
        recordPattern(row, rowOffset, counters);
        float bestVariance = MAX_AVG_VARIANCE;
        int bestMatch = -1;
        int d = 0;
        while (true) {
            int[][] iArr = CODE_PATTERNS;
            if (d >= iArr.length) {
                break;
            }
            float variance = patternMatchVariance(counters, iArr[d], MAX_INDIVIDUAL_VARIANCE);
            if (variance < bestVariance) {
                bestVariance = variance;
                bestMatch = d;
            }
            d++;
        }
        if (bestMatch >= 0) {
            return bestMatch;
        }
        throw NotFoundException.getNotFoundInstance();
    }

    /* JADX INFO: Multiple debug info for r2v2 byte[]: [D('lastPatternSize' int), D('rawBytes' byte[])] */
    /* JADX INFO: Can't fix incorrect switch cases order, some code will duplicate */
    /* JADX WARNING: Removed duplicated region for block: B:106:0x01bc A[FALL_THROUGH, PHI: r20 
      PHI: (r20v4 'lastCharacterWasPrintable' boolean) = (r20v2 'lastCharacterWasPrintable' boolean), (r20v5 'lastCharacterWasPrintable' boolean), (r20v5 'lastCharacterWasPrintable' boolean), (r20v5 'lastCharacterWasPrintable' boolean), (r20v5 'lastCharacterWasPrintable' boolean), (r20v2 'lastCharacterWasPrintable' boolean) binds: [B:25:0x0095, B:97:0x0199, B:99:0x01a0, B:103:0x01ac, B:102:0x01a8, B:93:0x018f] A[DONT_GENERATE, DONT_INLINE]] */
    @Override // com.google.zxing.oned.OneDReader
    public Result decodeRow(int rowNumber, BitArray row, Map<DecodeHintType, ?> hints) throws NotFoundException, FormatException, ChecksumException {
        int codeSet;
        boolean convertFNC1 = hints != null && hints.containsKey(DecodeHintType.ASSUME_GS1);
        int[] startPatternInfo = findStartPattern(row);
        int startCode = startPatternInfo[2];
        List<Byte> rawCodes = new ArrayList<>(20);
        rawCodes.add(Byte.valueOf((byte) startCode));
        switch (startCode) {
            case 103:
                codeSet = 101;
                break;
            case 104:
                codeSet = 100;
                break;
            case 105:
                codeSet = 99;
                break;
            default:
                throw FormatException.getFormatInstance();
        }
        boolean done = false;
        boolean isNextShifted = false;
        StringBuilder result = new StringBuilder(20);
        int lastStart = startPatternInfo[0];
        int nextStart = startPatternInfo[1];
        int[] counters = new int[6];
        int code = 0;
        int checksumTotal = startCode;
        int multiplier = 0;
        boolean lastCharacterWasPrintable = true;
        int lastCode = 0;
        boolean upperMode = false;
        boolean shiftUpperMode = false;
        while (!done) {
            isNextShifted = false;
            lastCode = code;
            int code2 = decodeCode(row, counters, nextStart);
            rawCodes.add(Byte.valueOf((byte) code2));
            if (code2 != 106) {
                lastCharacterWasPrintable = true;
            }
            if (code2 != 106) {
                multiplier++;
                checksumTotal += multiplier * code2;
            }
            lastStart = nextStart;
            for (int counter : counters) {
                nextStart += counter;
            }
            switch (code2) {
                case 103:
                case 104:
                case 105:
                    throw FormatException.getFormatInstance();
                default:
                    switch (codeSet) {
                        case 99:
                            if (code2 >= 100) {
                                if (code2 != 106) {
                                    lastCharacterWasPrintable = false;
                                }
                                switch (code2) {
                                    case 100:
                                        codeSet = 100;
                                        done = done;
                                        break;
                                    case 101:
                                        codeSet = 101;
                                        done = done;
                                        break;
                                    case 102:
                                        if (convertFNC1) {
                                            if (result.length() == 0) {
                                                result.append("]C1");
                                            } else {
                                                result.append((char) 29);
                                            }
                                        }
                                    case 103:
                                    case 104:
                                    case 105:
                                    default:
                                        done = done;
                                        break;
                                    case 106:
                                        done = true;
                                        break;
                                }
                            } else {
                                if (code2 < 10) {
                                    result.append('0');
                                }
                                result.append(code2);
                            }
                            done = done;
                        case 100:
                            if (code2 < 96) {
                                if (shiftUpperMode == upperMode) {
                                    result.append((char) (code2 + 32));
                                } else {
                                    result.append((char) (code2 + 32 + 128));
                                }
                                shiftUpperMode = false;
                                done = done;
                                break;
                            } else {
                                if (code2 != 106) {
                                    lastCharacterWasPrintable = false;
                                }
                                switch (code2) {
                                    case 96:
                                    case 97:
                                    case 103:
                                    case 104:
                                    case 105:
                                    default:
                                        done = done;
                                        break;
                                    case 98:
                                        isNextShifted = true;
                                        codeSet = 101;
                                        done = done;
                                        break;
                                    case 99:
                                        codeSet = 99;
                                        done = done;
                                        break;
                                    case 100:
                                        if (upperMode || !shiftUpperMode) {
                                            if (!upperMode || !shiftUpperMode) {
                                                shiftUpperMode = true;
                                                done = done;
                                                break;
                                            } else {
                                                upperMode = false;
                                                shiftUpperMode = false;
                                                done = done;
                                                break;
                                            }
                                        } else {
                                            upperMode = true;
                                            shiftUpperMode = false;
                                            done = done;
                                            break;
                                        }
                                    case 101:
                                        codeSet = 101;
                                        done = done;
                                        break;
                                    case 102:
                                        if (convertFNC1) {
                                            if (result.length() == 0) {
                                                result.append("]C1");
                                            } else {
                                                result.append((char) 29);
                                            }
                                        }
                                        done = done;
                                        break;
                                    case 106:
                                        done = true;
                                        break;
                                }
                            }
                        case 101:
                            if (code2 >= 64) {
                                if (code2 < 96) {
                                    if (shiftUpperMode == upperMode) {
                                        result.append((char) (code2 - 64));
                                    } else {
                                        result.append((char) (code2 + 64));
                                    }
                                    shiftUpperMode = false;
                                    done = done;
                                    break;
                                } else {
                                    if (code2 != 106) {
                                        lastCharacterWasPrintable = false;
                                    }
                                    switch (code2) {
                                        case 96:
                                        case 97:
                                        case 103:
                                        case 104:
                                        case 105:
                                        default:
                                            done = done;
                                            break;
                                        case 98:
                                            isNextShifted = true;
                                            codeSet = 100;
                                            done = done;
                                            break;
                                        case 99:
                                            codeSet = 99;
                                            done = done;
                                            break;
                                        case 100:
                                            codeSet = 100;
                                            done = done;
                                            break;
                                        case 101:
                                            if (upperMode || !shiftUpperMode) {
                                                if (!upperMode || !shiftUpperMode) {
                                                    shiftUpperMode = true;
                                                    done = done;
                                                    break;
                                                } else {
                                                    upperMode = false;
                                                    shiftUpperMode = false;
                                                    done = done;
                                                    break;
                                                }
                                            } else {
                                                upperMode = true;
                                                shiftUpperMode = false;
                                                done = done;
                                                break;
                                            }
                                        case 102:
                                            if (convertFNC1) {
                                                if (result.length() == 0) {
                                                    result.append("]C1");
                                                } else {
                                                    result.append((char) 29);
                                                }
                                            }
                                            done = done;
                                            break;
                                        case 106:
                                            done = true;
                                            break;
                                    }
                                }
                            } else {
                                if (shiftUpperMode == upperMode) {
                                    result.append((char) (code2 + 32));
                                } else {
                                    result.append((char) (code2 + 32 + 128));
                                }
                                shiftUpperMode = false;
                                done = done;
                                break;
                            }
                    }
                    if (isNextShifted) {
                        int i = 101;
                        if (codeSet == 101) {
                            i = 100;
                        }
                        codeSet = i;
                    }
                    code = code2;
                    startCode = startCode;
            }
        }
        int lastPatternSize = nextStart - lastStart;
        int nextStart2 = row.getNextUnset(nextStart);
        if (!row.isRange(nextStart2, Math.min(row.getSize(), ((nextStart2 - lastStart) / 2) + nextStart2), false)) {
            throw NotFoundException.getNotFoundInstance();
        } else if ((checksumTotal - (multiplier * lastCode)) % 103 == lastCode) {
            int resultLength = result.length();
            if (resultLength != 0) {
                if (resultLength > 0 && lastCharacterWasPrintable) {
                    if (codeSet == 99) {
                        result.delete(resultLength - 2, resultLength);
                    } else {
                        result.delete(resultLength - 1, resultLength);
                    }
                }
                float left = ((float) (startPatternInfo[1] + startPatternInfo[0])) / 2.0f;
                float right = ((float) lastStart) + (((float) lastPatternSize) / 2.0f);
                int rawCodesSize = rawCodes.size();
                byte[] rawBytes = new byte[rawCodesSize];
                for (int i2 = 0; i2 < rawCodesSize; i2++) {
                    rawBytes[i2] = rawCodes.get(i2).byteValue();
                }
                return new Result(result.toString(), rawBytes, new ResultPoint[]{new ResultPoint(left, (float) rowNumber), new ResultPoint(right, (float) rowNumber)}, BarcodeFormat.CODE_128);
            }
            throw NotFoundException.getNotFoundInstance();
        } else {
            throw ChecksumException.getChecksumInstance();
        }
    }
}
