package com.sec.factory.app.ui;

import android.app.Service;
import android.content.Intent;
import android.hardware.soundtrigger.SoundTrigger;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.AudioTrack;
import android.media.MediaPlayer;
import android.media.soundtrigger.SoundTriggerDetector;
import android.net.Uri;
import android.os.Binder;
import android.os.IBinder;
import android.util.Log;
import com.samsung.android.feature.SemFloatingFeature;
import com.samsung.android.util.SemLog;
import com.sec.factory.app.ui.SoundTriggerUtil;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.UUID;

public class AlwaysMicOnSoundTriggerService extends Service {
    private static final String TAG = "AlwaysMicOnSoundTriggerService";
    private final IBinder mBinder = new SoundTriggerTestBinder();
    private final Map<UUID, ModelInfo> mModelInfoMap = new HashMap();
    private SoundTriggerUtil mSoundTriggerUtil;
    private UserActivity mUserActivity;

    public static class ModelInfo {
        public boolean captureAudio;
        public int captureAudioMs;
        public AudioTrack captureAudioTrack;
        public SoundTriggerUtil.SoundTriggerDetectorProxy detector;
        public byte[] modelData;
        public UUID modelUuid;
        public String name;
        public String state;
        public MediaPlayer triggerAudioPlayer;
        public UUID vendorUuid;
    }

    public interface UserActivity {
        void addModel(UUID uuid, String str);

        void handleDetection(UUID uuid);
    }

    public synchronized int onStartCommand(Intent intent, int flags, int startId) {
        Log.i(TAG, "onStartCommand()");
        if (this.mModelInfoMap.isEmpty()) {
            this.mSoundTriggerUtil = new SoundTriggerUtil(this);
            loadModelsInDataDir();
        }
        return 1;
    }

    public void onCreate() {
        super.onCreate();
        Log.i(TAG, "onCreate()");
        try {
            getFilesDir().mkdir();
        } catch (Exception e) {
        }
    }

    public void onDestroy() {
        super.onDestroy();
        Log.i(TAG, "onDestroy()");
        stopAllRecognitions();
    }

    public void onTaskRemoved(Intent rootIntent) {
        super.onTaskRemoved(rootIntent);
        stopAllRecognitions();
        stopSelf();
    }

    public synchronized IBinder onBind(Intent intent) {
        Log.i(TAG, "onBind()");
        return this.mBinder;
    }

    public class SoundTriggerTestBinder extends Binder {
        public SoundTriggerTestBinder() {
        }

        /* access modifiers changed from: package-private */
        public AlwaysMicOnSoundTriggerService getService() {
            return AlwaysMicOnSoundTriggerService.this;
        }
    }

    public synchronized void setUserActivity(UserActivity activity) {
        this.mUserActivity = activity;
        if (activity != null) {
            for (Map.Entry<UUID, ModelInfo> entry : this.mModelInfoMap.entrySet()) {
                this.mUserActivity.addModel(entry.getKey(), entry.getValue().name);
            }
        }
    }

    private synchronized void stopAllRecognitions() {
        for (ModelInfo modelInfo : this.mModelInfoMap.values()) {
            if (modelInfo.detector != null) {
                Log.i(TAG, "Stopping recognition for " + modelInfo.name);
                try {
                    modelInfo.detector.stopRecognition();
                } catch (Exception e) {
                    Log.e(TAG, "Failed to stop recognition", e);
                }
            }
        }
    }

    private SoundTrigger.KeyphraseSoundModel createNewSoundModel(ModelInfo modelInfo) {
        return new SoundTrigger.KeyphraseSoundModel(modelInfo.modelUuid, modelInfo.vendorUuid, modelInfo.modelData, new SoundTrigger.Keyphrase[]{new SoundTrigger.Keyphrase(0, 1, new Locale("ko", "KR"), "higalaxy", new int[0])});
    }

    public synchronized void loadModel(UUID modelUuid) {
        ModelInfo modelInfo = this.mModelInfoMap.get(modelUuid);
        if (modelInfo == null) {
            postError("Could not find model for: " + modelUuid.toString());
            return;
        }
        postMessage("Loading model: " + modelInfo.name);
        SoundTrigger.KeyphraseSoundModel soundModel = createNewSoundModel(modelInfo);
        if (this.mSoundTriggerUtil.addOrUpdateSoundModel(soundModel)) {
            postToast("Successfully loaded " + modelInfo.name + ", UUID=" + soundModel.getUuid());
            setModelState(modelInfo, "Loaded");
            startRecognition(modelUuid);
        } else {
            postErrorToast("Failed to load " + modelInfo.name + ", UUID=" + soundModel.getUuid() + "!");
            setModelState(modelInfo, "Failed to load");
        }
    }

    public synchronized void unloadModel(UUID modelUuid) {
        ModelInfo modelInfo = this.mModelInfoMap.get(modelUuid);
        if (modelInfo == null) {
            postError("Could not find model for: " + modelUuid.toString());
            return;
        }
        postMessage("Unloading model: " + modelInfo.name);
        SoundTrigger.KeyphraseSoundModel soundModel = this.mSoundTriggerUtil.getSoundModel(modelUuid);
        if (soundModel == null) {
            postErrorToast("Sound model not found for " + modelInfo.name + "!");
            return;
        }
        modelInfo.detector = null;
        if (this.mSoundTriggerUtil.deleteSoundModel(modelUuid)) {
            postToast("Successfully unloaded " + modelInfo.name + ", UUID=" + soundModel.getUuid());
            setModelState(modelInfo, "Unloaded");
        } else {
            postErrorToast("Failed to unload " + modelInfo.name + ", UUID=" + soundModel.getUuid() + "!");
            setModelState(modelInfo, "Failed to unload");
        }
    }

    public synchronized void startRecognition(UUID modelUuid) {
        int i;
        ModelInfo modelInfo = this.mModelInfoMap.get(modelUuid);
        if (modelInfo == null) {
            postError("Could not find model for: " + modelUuid.toString());
            return;
        }
        if (modelInfo.detector == null) {
            postMessage("Creating SoundTriggerDetector for " + modelInfo.name);
            modelInfo.detector = this.mSoundTriggerUtil.createSoundTriggerDetector(modelUuid, new DetectorCallback(modelInfo));
        }
        postMessage("Starting recognition for " + modelInfo.name + ", UUID=" + modelInfo.modelUuid);
        SoundTriggerUtil.SoundTriggerDetectorProxy soundTriggerDetectorProxy = modelInfo.detector;
        if (modelInfo.captureAudio) {
            i = 1;
        } else {
            i = 2;
        }
        if (soundTriggerDetectorProxy.startRecognition(i)) {
            setModelState(modelInfo, "Started");
        } else {
            postErrorToast("Fast failure attempting to start recognition for " + modelInfo.name + ", UUID=" + modelInfo.modelUuid);
            setModelState(modelInfo, "Failed to start");
        }
    }

    public synchronized void stopRecognition(UUID modelUuid) {
        ModelInfo modelInfo = this.mModelInfoMap.get(modelUuid);
        if (modelInfo == null) {
            postError("Could not find model for: " + modelUuid.toString());
        } else if (modelInfo.detector == null) {
            postErrorToast("Stop called on null detector for " + modelInfo.name + ", UUID=" + modelInfo.modelUuid);
        } else {
            postMessage("Triggering stop recognition for " + modelInfo.name + ", UUID=" + modelInfo.modelUuid);
            if (modelInfo.detector.stopRecognition()) {
                setModelState(modelInfo, "Stopped");
                unloadModel(modelUuid);
            } else {
                postErrorToast("Fast failure attempting to stop recognition for " + modelInfo.name + ", UUID=" + modelInfo.modelUuid);
                setModelState(modelInfo, "Failed to stop");
            }
        }
    }

    private void loadModelsInDataDir() {
        Log.i(TAG, "loadModelsInDataDir()");
        boolean loadedModel = false;
        File[] listFiles = getFilesDir().listFiles();
        for (File file : listFiles) {
            if (file.getName().endsWith(".properties")) {
                try {
                    Properties properties = new Properties();
                    properties.load(new FileInputStream(file));
                    createModelInfo(properties);
                    loadedModel = true;
                } catch (Exception e) {
                    Log.e(TAG, "Failed to load properties file " + file.getName());
                }
            }
        }
        if (!loadedModel) {
            Properties dummyModelProperties = new Properties();
            String[] strArr = {"higalaxy"};
            for (int i = 0; i < 1; i++) {
                dummyModelProperties.setProperty("name", "Model " + strArr[i]);
                createModelInfo(dummyModelProperties);
            }
        }
    }

    private synchronized void createModelInfo(Properties properties) {
        try {
            ModelInfo modelInfo = new ModelInfo();
            if (properties.containsKey("name")) {
                modelInfo.name = properties.getProperty("name");
                if (properties.containsKey("modelUuid")) {
                    modelInfo.modelUuid = UUID.fromString(properties.getProperty("modelUuid"));
                } else {
                    modelInfo.modelUuid = this.mSoundTriggerUtil.getModuleProperties().getUuid();
                }
                if (properties.containsKey("vendorUuid")) {
                    modelInfo.vendorUuid = UUID.fromString(properties.getProperty("vendorUuid"));
                } else {
                    modelInfo.vendorUuid = this.mSoundTriggerUtil.getModuleProperties().getUuid();
                }
                if (properties.containsKey("triggerAudio")) {
                    modelInfo.triggerAudioPlayer = MediaPlayer.create(this, Uri.parse(getFilesDir().getPath() + "/" + properties.getProperty("triggerAudio")));
                    if (modelInfo.triggerAudioPlayer.getDuration() == 0) {
                        modelInfo.triggerAudioPlayer.release();
                        modelInfo.triggerAudioPlayer = null;
                    }
                }
                if (properties.containsKey("dataFile")) {
                    File modelDataFile = new File(getFilesDir().getPath() + "/" + properties.getProperty("dataFile"));
                    modelInfo.modelData = new byte[((int) modelDataFile.length())];
                    new FileInputStream(modelDataFile).read(modelInfo.modelData, 0, modelInfo.modelData.length);
                } else {
                    String dspBitrate = SemFloatingFeature.getInstance().getString("SEC_FLOATING_FEATURE_BIXBY_CONFIG_DSP_BITRATE", "32bit");
                    int customWakeup = SemFloatingFeature.getInstance().getInteger("SEC_FLOATING_FEATURE_BIXBY_SUPPORT_CUSTOM_WAKEUP", 0);
                    Log.d(TAG, "CONFIG_DSP_BITRATE : " + dspBitrate + " CUSTOM_WAKEUP : " + customWakeup);
                    if (customWakeup == 2) {
                        if ("32bit".equals(dspBitrate)) {
                            modelInfo.modelData = SoundTriggerEnrollmentUtil.readNewWakeupResource(this, "mobile_dsp_inv", "hibixby");
                        } else if ("24bit".equals(dspBitrate)) {
                            Log.d(TAG, "Not Defined for CONFIG_DSP_BITRATE - 24bit");
                        }
                        Log.d(TAG, "readNewWakeupResource mobile_dsp_inv");
                    } else {
                        if ("32bit".equals(dspBitrate)) {
                            modelInfo.modelData = SoundTriggerEnrollmentUtil.readWakeupResource(this, "higalaxy", "ko_kr");
                        } else if ("24bit".equals(dspBitrate)) {
                            modelInfo.modelData = SoundTriggerEnrollmentUtil.readWakeupResource(this, "higalaxy_halo", "ko_kr");
                        }
                        Log.d(TAG, "readNewWakeupResource higalaxy");
                    }
                    Log.d(TAG, "data size : " + modelInfo.modelData.length);
                    FileOutputStream out = null;
                    try {
                        out = openFileOutput("dsp_model", 0);
                        out.write(modelInfo.modelData, 0, modelInfo.modelData.length);
                    } catch (IOException e) {
                        SemLog.secE(TAG, e.getMessage());
                    }
                    if (out != null) {
                        out.flush();
                        out.close();
                    }
                }
                modelInfo.captureAudioMs = Integer.parseInt((String) properties.getOrDefault("captureAudioDurationMs", "5000"));
                this.mModelInfoMap.put(modelInfo.modelUuid, modelInfo);
                UserActivity userActivity = this.mUserActivity;
                if (userActivity != null) {
                    userActivity.addModel(modelInfo.modelUuid, modelInfo.name);
                }
            } else {
                throw new RuntimeException("must have a 'name' property");
            }
        } catch (IOException e2) {
            Log.e(TAG, "Error parsing properties for " + properties.getProperty("name"), e2);
        }
        return;
    }

    private class CaptureAudioRecorder implements Runnable {
        private final SoundTriggerDetector.EventPayload mEvent;
        private final ModelInfo mModelInfo;

        public CaptureAudioRecorder(ModelInfo modelInfo, SoundTriggerDetector.EventPayload event) {
            this.mModelInfo = modelInfo;
            this.mEvent = event;
        }

        /* JADX WARNING: Removed duplicated region for block: B:45:0x0164 A[SYNTHETIC] */
        public void run() {
            int bytesPerSample;
            String encoding;
            int bytesRequired;
            AudioFormat format = this.mEvent.getCaptureAudioFormat();
            if (format == null) {
                AlwaysMicOnSoundTriggerService.this.postErrorToast("No audio format in recognition event.");
                return;
            }
            AudioRecord audioRecord = null;
            AudioTrack playbackTrack = null;
            try {
                switch (format.getEncoding()) {
                    case 2:
                        bytesPerSample = 2;
                        encoding = "16bit";
                        break;
                    case 3:
                        bytesPerSample = 1;
                        encoding = "8bit";
                        break;
                    case 4:
                        bytesPerSample = 4;
                        encoding = "float";
                        break;
                    default:
                        throw new RuntimeException("Unhandled audio format in event");
                }
                int bytesRequired2 = (((format.getSampleRate() * format.getChannelCount()) * bytesPerSample) * this.mModelInfo.captureAudioMs) / 1000;
                int minBufferSize = AudioRecord.getMinBufferSize(format.getSampleRate(), format.getChannelMask(), format.getEncoding());
                if (minBufferSize > bytesRequired2) {
                    bytesRequired = minBufferSize;
                } else {
                    bytesRequired = bytesRequired2;
                }
                int channelConfig = 4;
                try {
                    if (format.getChannelCount() == 2) {
                        channelConfig = 12;
                    } else if (format.getChannelCount() >= 3) {
                        throw new RuntimeException("Too many channels in captured audio for playback");
                    }
                    playbackTrack = new AudioTrack(3, format.getSampleRate(), channelConfig, format.getEncoding(), bytesRequired, 0);
                } catch (Exception e) {
                    Log.e(AlwaysMicOnSoundTriggerService.TAG, "Exception creating playback track", e);
                    AlwaysMicOnSoundTriggerService.this.postErrorToast("Failed to create playback track: " + e.getMessage());
                }
                AudioRecord audioRecord2 = new AudioRecord(1999, 16000, 16, 2, bytesRequired);
                byte[] buffer = new byte[bytesRequired];
                FileOutputStream fos = null;
                try {
                    fos = new FileOutputStream(new File(AlwaysMicOnSoundTriggerService.this.getFilesDir() + File.separator + this.mModelInfo.name.replace(' ', '_') + "_capture_" + format.getChannelCount() + "ch_" + format.getSampleRate() + "hz_" + encoding + ".pcm"));
                } catch (IOException e2) {
                    Log.e(AlwaysMicOnSoundTriggerService.TAG, "Failed to open output for saving PCM data", e2);
                    AlwaysMicOnSoundTriggerService.this.postErrorToast("Failed to open output for saving PCM data: " + e2.getMessage());
                }
                AlwaysMicOnSoundTriggerService.this.setModelState(this.mModelInfo, "Recording");
                audioRecord2.startRecording();
                while (bytesRequired > 0) {
                    int bytesRead = audioRecord2.read(buffer, 0, buffer.length);
                    if (bytesRead == -1) {
                        audioRecord2.stop();
                        audioRecord2.release();
                        synchronized (AlwaysMicOnSoundTriggerService.this) {
                            if (this.mModelInfo.captureAudioTrack != null) {
                                this.mModelInfo.captureAudioTrack.release();
                            }
                            this.mModelInfo.captureAudioTrack = playbackTrack;
                        }
                        AlwaysMicOnSoundTriggerService.this.setModelState(this.mModelInfo, "Recording finished");
                        return;
                    }
                    if (fos != null) {
                        fos.write(buffer, 0, bytesRead);
                    }
                    if (playbackTrack != null) {
                        playbackTrack.write(buffer, 0, bytesRead);
                    }
                    bytesRequired -= bytesRead;
                }
                audioRecord2.stop();
                audioRecord2.release();
                synchronized (AlwaysMicOnSoundTriggerService.this) {
                }
            } catch (Exception e3) {
                try {
                    Log.e(AlwaysMicOnSoundTriggerService.TAG, "Error recording trigger audio", e3);
                    AlwaysMicOnSoundTriggerService.this.postErrorToast("Error recording trigger audio: " + e3.getMessage());
                    if (0 != 0) {
                        audioRecord.release();
                    }
                    synchronized (AlwaysMicOnSoundTriggerService.this) {
                        if (this.mModelInfo.captureAudioTrack != null) {
                            this.mModelInfo.captureAudioTrack.release();
                        }
                        this.mModelInfo.captureAudioTrack = null;
                    }
                } catch (Throwable th) {
                    if (0 != 0) {
                        audioRecord.release();
                    }
                    synchronized (AlwaysMicOnSoundTriggerService.this) {
                        if (this.mModelInfo.captureAudioTrack != null) {
                            this.mModelInfo.captureAudioTrack.release();
                        }
                        this.mModelInfo.captureAudioTrack = null;
                        AlwaysMicOnSoundTriggerService.this.setModelState(this.mModelInfo, "Recording finished");
                        throw th;
                    }
                }
            }
        }
    }

    /* access modifiers changed from: private */
    public class DetectorCallback extends SoundTriggerDetector.Callback {
        private final ModelInfo mModelInfo;

        public DetectorCallback(ModelInfo modelInfo) {
            this.mModelInfo = modelInfo;
        }

        @Override // android.media.soundtrigger.SoundTriggerDetector.Callback
        public void onAvailabilityChanged(int status) {
            AlwaysMicOnSoundTriggerService alwaysMicOnSoundTriggerService = AlwaysMicOnSoundTriggerService.this;
            alwaysMicOnSoundTriggerService.postMessage(this.mModelInfo.name + " availability changed to: " + status);
        }

        @Override // android.media.soundtrigger.SoundTriggerDetector.Callback
        public void onDetected(SoundTriggerDetector.EventPayload event) {
            AlwaysMicOnSoundTriggerService alwaysMicOnSoundTriggerService = AlwaysMicOnSoundTriggerService.this;
            alwaysMicOnSoundTriggerService.postMessage(this.mModelInfo.name + " onDetected(): " + AlwaysMicOnSoundTriggerService.this.eventPayloadToString(event));
            synchronized (AlwaysMicOnSoundTriggerService.this) {
                if (AlwaysMicOnSoundTriggerService.this.mUserActivity != null) {
                    AlwaysMicOnSoundTriggerService.this.mUserActivity.handleDetection(this.mModelInfo.modelUuid);
                }
                if (this.mModelInfo.captureAudio) {
                    new Thread(new CaptureAudioRecorder(this.mModelInfo, event)).start();
                }
            }
        }

        @Override // android.media.soundtrigger.SoundTriggerDetector.Callback
        public void onError() {
            AlwaysMicOnSoundTriggerService alwaysMicOnSoundTriggerService = AlwaysMicOnSoundTriggerService.this;
            alwaysMicOnSoundTriggerService.postMessage(this.mModelInfo.name + " onError()");
            AlwaysMicOnSoundTriggerService.this.setModelState(this.mModelInfo, "Error");
        }

        @Override // android.media.soundtrigger.SoundTriggerDetector.Callback
        public void onRecognitionPaused() {
            AlwaysMicOnSoundTriggerService alwaysMicOnSoundTriggerService = AlwaysMicOnSoundTriggerService.this;
            alwaysMicOnSoundTriggerService.postMessage(this.mModelInfo.name + " onRecognitionPaused()");
            AlwaysMicOnSoundTriggerService.this.setModelState(this.mModelInfo, "Paused");
        }

        @Override // android.media.soundtrigger.SoundTriggerDetector.Callback
        public void onRecognitionResumed() {
            AlwaysMicOnSoundTriggerService alwaysMicOnSoundTriggerService = AlwaysMicOnSoundTriggerService.this;
            alwaysMicOnSoundTriggerService.postMessage(this.mModelInfo.name + " onRecognitionResumed()");
            AlwaysMicOnSoundTriggerService.this.setModelState(this.mModelInfo, "Resumed");
        }
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private String eventPayloadToString(SoundTriggerDetector.EventPayload event) {
        if (event == null) {
            return "NullEventPayload";
        }
        AudioFormat format = event.getCaptureAudioFormat();
        StringBuilder sb = new StringBuilder();
        sb.append("EventPayload(");
        sb.append("AudioFormat: ");
        Object obj = "null";
        sb.append(format == null ? obj : format.toString());
        String result = sb.toString();
        byte[] triggerAudio = event.getTriggerAudio();
        StringBuilder sb2 = new StringBuilder();
        sb2.append(result);
        sb2.append("TriggerAudio: ");
        Object obj2 = obj;
        if (triggerAudio != null) {
            obj2 = Integer.valueOf(triggerAudio.length);
        }
        sb2.append(obj2);
        return (sb2.toString() + "CaptureSession: " + event.getCaptureSession()) + " )";
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private void postMessage(String msg) {
        showMessage(msg, 4, true);
    }

    private void postError(String msg) {
        showMessage(msg, 6, true);
    }

    private void postToast(String msg) {
        showMessage(msg, 4, true);
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private void postErrorToast(String msg) {
        showMessage(msg, 6, true);
    }

    private synchronized void showMessage(String msg, int logLevel, boolean showToast) {
        Log.println(logLevel, TAG, msg);
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private synchronized void setModelState(ModelInfo modelInfo, String state) {
        modelInfo.state = state;
    }
}
