package com.sec.factory.app.ui;

import android.content.Context;
import android.hardware.soundtrigger.SoundTrigger;
import android.hardware.soundtrigger.SoundTriggerModule;
import android.media.permission.Identity;
import android.media.soundtrigger.SoundTriggerDetector;
import android.os.Binder;
import android.util.Log;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class SoundTriggerUtil implements SoundTrigger.StatusListener {
    private static final String TAG = "SoundTriggerTestUtil";
    private final Context mContext;
    private final Map<UUID, SoundTriggerDetectorModule> mDetectorMap = new HashMap();
    private final Map<Integer, UUID> mHandleMap = new HashMap();
    private final Map<UUID, SoundTrigger.KeyphraseSoundModel> mModelMap = new HashMap();
    private SoundTriggerModule mModule;
    private final SoundTrigger.ModuleProperties mModuleProperties;
    private final Map<UUID, Integer> mUUIDHandleMap = new HashMap();

    public interface SoundTriggerDetectorProxy {
        boolean startRecognition(int i);

        boolean stopRecognition();
    }

    public SoundTriggerUtil(Context context) {
        this.mContext = context;
        ArrayList<SoundTrigger.ModuleProperties> modules = new ArrayList<>();
        SoundTrigger.listModules(modules);
        SoundTrigger.ModuleProperties moduleProperties = modules.get(0);
        this.mModuleProperties = moduleProperties;
        if (this.mModule == null) {
            Identity originatorIdentity = new Identity();
            originatorIdentity.pid = Binder.getCallingPid();
            originatorIdentity.uid = Binder.getCallingUid();
            SoundTriggerModule attachModuleAsOriginator = SoundTrigger.attachModuleAsOriginator(moduleProperties.getId(), this, null, originatorIdentity);
            this.mModule = attachModuleAsOriginator;
            if (attachModuleAsOriginator == null) {
                Log.e(TAG, "startRecognition cannot attach to sound trigger module");
            }
        }
    }

    public boolean addOrUpdateSoundModel(SoundTrigger.KeyphraseSoundModel keyphraseSoundModel) {
        if (this.mUUIDHandleMap.get(keyphraseSoundModel.getUuid()) != null && !deleteSoundModel(keyphraseSoundModel.getUuid())) {
            return false;
        }
        Log.i(TAG, "Loading sound model " + keyphraseSoundModel.getUuid());
        int[] handle = {-1};
        int status = this.mModule.loadSoundModel(keyphraseSoundModel, handle);
        if (status != 0) {
            Log.e(TAG, "loadSoundModel call failed with " + status);
            return false;
        } else if (handle[0] == -1) {
            Log.e(TAG, "Invalid model handle");
            return false;
        } else {
            this.mHandleMap.put(Integer.valueOf(handle[0]), keyphraseSoundModel.getUuid());
            this.mUUIDHandleMap.put(keyphraseSoundModel.getUuid(), Integer.valueOf(handle[0]));
            this.mModelMap.put(keyphraseSoundModel.getUuid(), keyphraseSoundModel);
            return true;
        }
    }

    public SoundTrigger.KeyphraseSoundModel getSoundModel(UUID modelId) {
        return this.mModelMap.get(modelId);
    }

    public boolean deleteSoundModel(UUID modelId) {
        if (this.mModelMap.get(modelId) == null) {
            return false;
        }
        int handle = this.mUUIDHandleMap.get(modelId).intValue();
        Log.i(TAG, "Unloaded existing model " + handle);
        int status = this.mModule.unloadSoundModel(handle);
        if (status != 0) {
            Log.e(TAG, "Failed to unload sound model " + status);
            return false;
        }
        this.mHandleMap.remove(Integer.valueOf(handle));
        this.mUUIDHandleMap.remove(modelId);
        this.mModelMap.remove(modelId);
        this.mDetectorMap.remove(modelId);
        return true;
    }

    public SoundTriggerDetectorProxy createSoundTriggerDetector(UUID modelId, SoundTriggerDetector.Callback callback) {
        if (this.mUUIDHandleMap.get(modelId) == null) {
            Log.e(TAG, "No model loaded for uuid " + modelId);
            return null;
        }
        SoundTriggerDetectorModule detector = new SoundTriggerDetectorModule(this.mUUIDHandleMap.get(modelId).intValue(), callback);
        this.mDetectorMap.put(modelId, detector);
        return detector;
    }

    @Override // android.hardware.soundtrigger.SoundTrigger.StatusListener
    public void onRecognition(SoundTrigger.RecognitionEvent recognitionEvent) {
        Log.e(TAG, "OnRecognitionEvent " + recognitionEvent.soundModelHandle);
        SoundTriggerDetectorModule detector = this.mDetectorMap.get(this.mHandleMap.get(Integer.valueOf(recognitionEvent.soundModelHandle)));
        if (detector == null) {
            return;
        }
        if (recognitionEvent == null) {
            Log.w(TAG, "Null recognition event!");
        } else if ((recognitionEvent instanceof SoundTrigger.KeyphraseRecognitionEvent) || (recognitionEvent instanceof SoundTrigger.GenericRecognitionEvent)) {
            switch (recognitionEvent.status) {
                case 0:
                    Log.d(TAG, "RECOGNITION_STATUS_SUCCESS");
                    detector.mCallback.onDetected(null);
                    return;
                case 1:
                    Log.d(TAG, "RECOGNITION_STATUS_ABORT");
                    return;
                case 2:
                    Log.d(TAG, "RECOGNITION_STATUS_FAILURE");
                    return;
                default:
                    return;
            }
        } else {
            Log.w(TAG, "Invalid recognition event type (not one of generic or keyphrase)!");
        }
    }

    @Override // android.hardware.soundtrigger.SoundTrigger.StatusListener
    public void onSoundModelUpdate(SoundTrigger.SoundModelEvent soundModelEvent) {
        Log.e(TAG, "OnSoundModelUpdate " + soundModelEvent.soundModelHandle);
    }

    @Override // android.hardware.soundtrigger.SoundTrigger.StatusListener
    public void onServiceStateChange(int i) {
        Log.e(TAG, "OnServiceStateChanged " + i);
        for (SoundTriggerDetectorModule detector : this.mDetectorMap.values()) {
            detector.mCallback.onAvailabilityChanged(i);
        }
    }

    @Override // android.hardware.soundtrigger.SoundTrigger.StatusListener
    public void onServiceDied() {
        Log.e(TAG, "OnServiceDied");
    }

    public SoundTrigger.ModuleProperties getModuleProperties() {
        return this.mModuleProperties;
    }

    /* access modifiers changed from: private */
    public class SoundTriggerDetectorModule implements SoundTriggerDetectorProxy {
        SoundTriggerDetector.Callback mCallback;
        int mHandle;

        SoundTriggerDetectorModule(int handle, SoundTriggerDetector.Callback callback) {
            this.mCallback = callback;
            this.mHandle = handle;
        }

        @Override // com.sec.factory.app.ui.SoundTriggerUtil.SoundTriggerDetectorProxy
        public boolean startRecognition(int flags) {
            Log.d(SoundTriggerUtil.TAG, "startRecognition()");
            SoundTrigger.KeyphraseRecognitionExtra[] KeyphraseRecognitionExtra = {new SoundTrigger.KeyphraseRecognitionExtra(0, 1, 60, new SoundTrigger.ConfidenceLevel[0])};
            byte[] extraData = SoundTriggerEnrollmentUtil.getExtraData();
            Log.d(SoundTriggerUtil.TAG, "extraData size: " + extraData.length);
            int status = SoundTriggerUtil.this.mModule.startRecognition(this.mHandle, new SoundTrigger.RecognitionConfig(false, true, KeyphraseRecognitionExtra, extraData));
            if (status == 0) {
                return true;
            }
            Log.e(SoundTriggerUtil.TAG, "Failed to start recognition " + status);
            return false;
        }

        @Override // com.sec.factory.app.ui.SoundTriggerUtil.SoundTriggerDetectorProxy
        public boolean stopRecognition() {
            Log.d(SoundTriggerUtil.TAG, "stopRecognition()");
            int status = SoundTriggerUtil.this.mModule.stopRecognition(this.mHandle);
            if (status == 0) {
                return true;
            }
            Log.e(SoundTriggerUtil.TAG, "Failed to stop recognition " + status);
            return false;
        }
    }

    private static class SoundTriggerDetectorActual implements SoundTriggerDetectorProxy {
        private SoundTriggerDetector mDetector;

        public SoundTriggerDetectorActual(SoundTriggerDetector detector) {
            this.mDetector = detector;
        }

        @Override // com.sec.factory.app.ui.SoundTriggerUtil.SoundTriggerDetectorProxy
        public boolean startRecognition(int flags) {
            return this.mDetector.startRecognition(flags);
        }

        @Override // com.sec.factory.app.ui.SoundTriggerUtil.SoundTriggerDetectorProxy
        public boolean stopRecognition() {
            return this.mDetector.stopRecognition();
        }
    }
}
