package org.apache.poi;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;
import org.apache.poi.hpsf.DocumentSummaryInformation;
import org.apache.poi.hpsf.MutablePropertySet;
import org.apache.poi.hpsf.PropertySet;
import org.apache.poi.hpsf.PropertySetFactory;
import org.apache.poi.hpsf.SummaryInformation;
import org.apache.poi.hpsf.WritingNotSupportedException;
import org.apache.poi.poifs.crypt.EncryptionInfo;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.DocumentInputStream;
import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
import org.apache.poi.poifs.filesystem.OPOIFSFileSystem;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.util.Internal;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public abstract class POIDocument implements Closeable {
    private static final POILogger logger = POILogFactory.getLogger(POIDocument.class);
    protected DirectoryNode directory;
    private DocumentSummaryInformation dsInf;
    private boolean initialized;
    private SummaryInformation sInf;

    public abstract void write() throws IOException;

    public abstract void write(File file) throws IOException;

    public abstract void write(OutputStream outputStream) throws IOException;

    protected POIDocument(DirectoryNode dir) {
        this.initialized = false;
        this.directory = dir;
    }

    protected POIDocument(OPOIFSFileSystem fs) {
        this(fs.getRoot());
    }

    protected POIDocument(NPOIFSFileSystem fs) {
        this(fs.getRoot());
    }

    protected POIDocument(POIFSFileSystem fs) {
        this(fs.getRoot());
    }

    public DocumentSummaryInformation getDocumentSummaryInformation() {
        if (!this.initialized) {
            readProperties();
        }
        return this.dsInf;
    }

    public SummaryInformation getSummaryInformation() {
        if (!this.initialized) {
            readProperties();
        }
        return this.sInf;
    }

    public void createInformationProperties() {
        if (!this.initialized) {
            readProperties();
        }
        if (this.sInf == null) {
            this.sInf = PropertySetFactory.newSummaryInformation();
        }
        if (this.dsInf == null) {
            this.dsInf = PropertySetFactory.newDocumentSummaryInformation();
        }
    }

    /* access modifiers changed from: protected */
    public void readProperties() {
        PropertySet ps = getPropertySet(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
        if (ps instanceof DocumentSummaryInformation) {
            this.dsInf = (DocumentSummaryInformation) ps;
        } else if (ps != null) {
            logger.log(5, "DocumentSummaryInformation property set came back with wrong class - ", ps.getClass());
        } else {
            logger.log(5, "DocumentSummaryInformation property set came back as null");
        }
        PropertySet ps2 = getPropertySet(SummaryInformation.DEFAULT_STREAM_NAME);
        if (ps2 instanceof SummaryInformation) {
            this.sInf = (SummaryInformation) ps2;
        } else if (ps2 != null) {
            logger.log(5, "SummaryInformation property set came back with wrong class - ", ps2.getClass());
        } else {
            logger.log(5, "SummaryInformation property set came back as null");
        }
        this.initialized = true;
    }

    /* access modifiers changed from: protected */
    public PropertySet getPropertySet(String setName) {
        return getPropertySet(setName, null);
    }

    /* access modifiers changed from: protected */
    public PropertySet getPropertySet(String setName, EncryptionInfo encryptionInfo) {
        DirectoryNode dirNode = this.directory;
        NPOIFSFileSystem encPoifs = null;
        if (encryptionInfo != null) {
            try {
                InputStream is = encryptionInfo.getDecryptor().getDataStream(this.directory);
                try {
                    encPoifs = new NPOIFSFileSystem(is);
                    dirNode = encPoifs.getRoot();
                } finally {
                    is.close();
                }
            } catch (Exception e) {
                POILogger pOILogger = logger;
                pOILogger.log(5, "Error " + "getting encrypted" + " property set with name " + setName, e);
                if (0 != 0) {
                    try {
                        encPoifs.close();
                    } catch (IOException e2) {
                        logger.log(5, "Error closing encrypted property poifs", e2);
                    }
                }
                return null;
            } catch (Throwable e3) {
                if (0 != 0) {
                    try {
                        encPoifs.close();
                    } catch (IOException e4) {
                        logger.log(5, "Error closing encrypted property poifs", e4);
                    }
                }
                throw e3;
            }
        }
        if (dirNode == null || !dirNode.hasEntry(setName)) {
            if (encPoifs != null) {
                try {
                    encPoifs.close();
                } catch (IOException e5) {
                    logger.log(5, "Error closing encrypted property poifs", e5);
                }
            }
            return null;
        }
        DocumentInputStream dis = dirNode.createDocumentInputStream(dirNode.getEntry(setName));
        try {
            PropertySet create = PropertySetFactory.create(dis);
            if (encPoifs != null) {
                try {
                    encPoifs.close();
                } catch (IOException e6) {
                    logger.log(5, "Error closing encrypted property poifs", e6);
                }
            }
            return create;
        } finally {
            dis.close();
        }
    }

    /* access modifiers changed from: protected */
    public void writeProperties() throws IOException {
        validateInPlaceWritePossible();
        writeProperties(this.directory.getFileSystem(), null);
    }

    /* access modifiers changed from: protected */
    public void writeProperties(NPOIFSFileSystem outFS) throws IOException {
        writeProperties(outFS, null);
    }

    /* access modifiers changed from: protected */
    public void writeProperties(NPOIFSFileSystem outFS, List<String> writtenEntries) throws IOException {
        SummaryInformation si = getSummaryInformation();
        if (si != null) {
            writePropertySet(SummaryInformation.DEFAULT_STREAM_NAME, si, outFS);
            if (writtenEntries != null) {
                writtenEntries.add(SummaryInformation.DEFAULT_STREAM_NAME);
            }
        }
        DocumentSummaryInformation dsi = getDocumentSummaryInformation();
        if (dsi != null) {
            writePropertySet(DocumentSummaryInformation.DEFAULT_STREAM_NAME, dsi, outFS);
            if (writtenEntries != null) {
                writtenEntries.add(DocumentSummaryInformation.DEFAULT_STREAM_NAME);
            }
        }
    }

    /* access modifiers changed from: protected */
    public void writePropertySet(String name, PropertySet set, NPOIFSFileSystem outFS) throws IOException {
        try {
            MutablePropertySet mSet = new MutablePropertySet(set);
            ByteArrayOutputStream bOut = new ByteArrayOutputStream();
            mSet.write(bOut);
            byte[] data = bOut.toByteArray();
            outFS.createOrUpdateDocument(new ByteArrayInputStream(data), name);
            POILogger pOILogger = logger;
            pOILogger.log(3, "Wrote property set " + name + " of size " + data.length);
        } catch (WritingNotSupportedException e) {
            POILogger pOILogger2 = logger;
            pOILogger2.log(7, "Couldn't write property set with name " + name + " as not supported by HPSF yet");
        }
    }

    /* access modifiers changed from: protected */
    public void validateInPlaceWritePossible() throws IllegalStateException {
        DirectoryNode directoryNode = this.directory;
        if (directoryNode == null) {
            throw new IllegalStateException("Newly created Document, cannot save in-place");
        } else if (directoryNode.getParent() != null) {
            throw new IllegalStateException("This is not the root Document, cannot save embedded resource in-place");
        } else if (this.directory.getFileSystem() == null || !this.directory.getFileSystem().isInPlaceWriteable()) {
            throw new IllegalStateException("Opened read-only or via an InputStream, a Writeable File is required");
        }
    }

    @Override // java.io.Closeable, java.lang.AutoCloseable
    public void close() throws IOException {
        DirectoryNode directoryNode = this.directory;
        if (directoryNode != null && directoryNode.getNFileSystem() != null) {
            this.directory.getNFileSystem().close();
            this.directory = null;
        }
    }

    @Internal
    public DirectoryNode getDirectory() {
        return this.directory;
    }
}
