package org.apache.poi.ddf;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import org.apache.poi.util.HexDump;
import org.apache.poi.util.LittleEndian;

public abstract class AbstractEscherOptRecord extends EscherRecord {
    protected List<EscherProperty> properties = new ArrayList();

    public void addEscherProperty(EscherProperty prop) {
        this.properties.add(prop);
    }

    @Override // org.apache.poi.ddf.EscherRecord
    public int fillFields(byte[] data, int offset, EscherRecordFactory recordFactory) {
        int bytesRemaining = readHeader(data, offset);
        short propertiesCount = readInstance(data, offset);
        this.properties = new EscherPropertyFactory().createProperties(data, offset + 8, propertiesCount);
        return bytesRemaining + 8;
    }

    public List<EscherProperty> getEscherProperties() {
        return this.properties;
    }

    public EscherProperty getEscherProperty(int index) {
        return this.properties.get(index);
    }

    private int getPropertiesSize() {
        int totalSize = 0;
        for (EscherProperty property : this.properties) {
            totalSize += property.getPropertySize();
        }
        return totalSize;
    }

    @Override // org.apache.poi.ddf.EscherRecord
    public int getRecordSize() {
        return getPropertiesSize() + 8;
    }

    public <T extends EscherProperty> T lookup(int propId) {
        Iterator i$ = this.properties.iterator();
        while (i$.hasNext()) {
            T t = (T) i$.next();
            if (t.getPropertyNumber() == propId) {
                return t;
            }
        }
        return null;
    }

    @Override // org.apache.poi.ddf.EscherRecord
    public int serialize(int offset, byte[] data, EscherSerializationListener listener) {
        listener.beforeRecordSerialize(offset, getRecordId(), this);
        LittleEndian.putShort(data, offset, getOptions());
        LittleEndian.putShort(data, offset + 2, getRecordId());
        LittleEndian.putInt(data, offset + 4, getPropertiesSize());
        int pos = offset + 8;
        for (EscherProperty property : this.properties) {
            pos += property.serializeSimplePart(data, pos);
        }
        for (EscherProperty property2 : this.properties) {
            pos += property2.serializeComplexPart(data, pos);
        }
        listener.afterRecordSerialize(pos, getRecordId(), pos - offset, this);
        return pos - offset;
    }

    public void sortProperties() {
        Collections.sort(this.properties, new Comparator<EscherProperty>() {
            /* class org.apache.poi.ddf.AbstractEscherOptRecord.AnonymousClass1 */

            public int compare(EscherProperty p1, EscherProperty p2) {
                short s1 = p1.getPropertyNumber();
                short s2 = p2.getPropertyNumber();
                if (s1 < s2) {
                    return -1;
                }
                return s1 == s2 ? 0 : 1;
            }
        });
    }

    public void setEscherProperty(EscherProperty value) {
        Iterator<EscherProperty> iterator = this.properties.iterator();
        while (iterator.hasNext()) {
            if (iterator.next().getId() == value.getId()) {
                iterator.remove();
            }
        }
        this.properties.add(value);
        sortProperties();
    }

    public void removeEscherProperty(int num) {
        Iterator<EscherProperty> iterator = getEscherProperties().iterator();
        while (iterator.hasNext()) {
            if (iterator.next().getPropertyNumber() == num) {
                iterator.remove();
            }
        }
    }

    public String toString() {
        String nl = System.getProperty("line.separator");
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(getClass().getName());
        stringBuilder.append(":");
        stringBuilder.append(nl);
        stringBuilder.append("  isContainer: ");
        stringBuilder.append(isContainerRecord());
        stringBuilder.append(nl);
        stringBuilder.append("  version: 0x");
        stringBuilder.append(HexDump.toHex(getVersion()));
        stringBuilder.append(nl);
        stringBuilder.append("  instance: 0x");
        stringBuilder.append(HexDump.toHex(getInstance()));
        stringBuilder.append(nl);
        stringBuilder.append("  recordId: 0x");
        stringBuilder.append(HexDump.toHex(getRecordId()));
        stringBuilder.append(nl);
        stringBuilder.append("  numchildren: ");
        stringBuilder.append(getChildRecords().size());
        stringBuilder.append(nl);
        stringBuilder.append("  properties:");
        stringBuilder.append(nl);
        for (EscherProperty property : this.properties) {
            stringBuilder.append("    ");
            stringBuilder.append(property.toString());
            stringBuilder.append(nl);
        }
        return stringBuilder.toString();
    }

    @Override // org.apache.poi.ddf.EscherRecord
    public String toXml(String tab) {
        StringBuilder builder = new StringBuilder();
        builder.append(tab);
        builder.append(formatXmlRecordHeader(getClass().getSimpleName(), HexDump.toHex(getRecordId()), HexDump.toHex(getVersion()), HexDump.toHex(getInstance())));
        for (EscherProperty property : getEscherProperties()) {
            builder.append(property.toXml(tab + "\t"));
        }
        builder.append(tab);
        builder.append("</");
        builder.append(getClass().getSimpleName());
        builder.append(">\n");
        return builder.toString();
    }
}
