package org.apache.poi.hpsf;

import android.support.v4.app.FragmentTransaction;
import android.support.v4.view.InputDeviceCompat;
import org.apache.poi.util.Internal;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

/* access modifiers changed from: package-private */
@Internal
public class TypedPropertyValue {
    private static final POILogger logger = POILogFactory.getLogger(TypedPropertyValue.class);
    private int _type;
    private Object _value;

    TypedPropertyValue() {
    }

    TypedPropertyValue(byte[] data, int startOffset) {
        read(data, startOffset);
    }

    TypedPropertyValue(int type, Object value) {
        this._type = type;
        this._value = value;
    }

    /* access modifiers changed from: package-private */
    public Object getValue() {
        return this._value;
    }

    /* access modifiers changed from: package-private */
    public int read(byte[] data, int startOffset) {
        this._type = LittleEndian.getShort(data, startOffset);
        int offset = startOffset + 2;
        short padding = LittleEndian.getShort(data, offset);
        int offset2 = offset + 2;
        if (padding != 0) {
            POILogger pOILogger = logger;
            pOILogger.log(5, "TypedPropertyValue padding at offset " + offset2 + " MUST be 0, but it's value is " + ((int) padding));
        }
        return (offset2 + readValue(data, offset2)) - startOffset;
    }

    /* access modifiers changed from: package-private */
    public int readValue(byte[] data, int offset) {
        int i = this._type;
        switch (i) {
            case 0:
            case 1:
                this._value = null;
                return 0;
            case 2:
            case 4:
                this._value = Short.valueOf(LittleEndian.getShort(data, offset));
                return 4;
            case 3:
            case 22:
                this._value = Integer.valueOf(LittleEndian.getInt(data, offset));
                return 4;
            case 5:
                this._value = Double.valueOf(LittleEndian.getDouble(data, offset));
                return 8;
            case 6:
                this._value = new Currency(data, offset);
                return 8;
            case 7:
                this._value = new Date(data, offset);
                return 8;
            case 8:
                CodePageString codePageString = new CodePageString(data, offset);
                this._value = codePageString;
                return codePageString.getSize();
            case 10:
            case 19:
            case 23:
                this._value = Long.valueOf(LittleEndian.getUInt(data, offset));
                return 4;
            case 11:
                this._value = new VariantBool(data, offset);
                return 2;
            case 14:
                this._value = new Decimal(data, offset);
                return 16;
            case 16:
                this._value = Byte.valueOf(data[offset]);
                return 1;
            case 17:
                this._value = Short.valueOf(LittleEndian.getUByte(data, offset));
                return 2;
            case 18:
                this._value = Integer.valueOf(LittleEndian.getUShort(data, offset));
                return 4;
            case 20:
                this._value = Long.valueOf(LittleEndian.getLong(data, offset));
                return 8;
            case 21:
                this._value = LittleEndian.getByteArray(data, offset, 8);
                return 8;
            case 30:
                CodePageString codePageString2 = new CodePageString(data, offset);
                this._value = codePageString2;
                return codePageString2.getSize();
            case 31:
                UnicodeString unicodeString = new UnicodeString(data, offset);
                this._value = unicodeString;
                return unicodeString.getSize();
            case 64:
                this._value = new Filetime(data, offset);
                return 8;
            case 65:
                Blob blob = new Blob(data, offset);
                this._value = blob;
                return blob.getSize();
            case 66:
            case 67:
            case 68:
            case 69:
                IndirectPropertyName indirectPropertyName = new IndirectPropertyName(data, offset);
                this._value = indirectPropertyName;
                return indirectPropertyName.getSize();
            case 70:
                Blob blob2 = new Blob(data, offset);
                this._value = blob2;
                return blob2.getSize();
            case 71:
                ClipboardData clipboardData = new ClipboardData(data, offset);
                this._value = clipboardData;
                return clipboardData.getSize();
            case 72:
                this._value = new GUID(data, offset);
                return 16;
            case 73:
                VersionedStream versionedStream = new VersionedStream(data, offset);
                this._value = versionedStream;
                return versionedStream.getSize();
            case InputDeviceCompat.SOURCE_TOUCHSCREEN:
            case FragmentTransaction.TRANSIT_FRAGMENT_FADE:
            case 4100:
            case 4101:
            case 4102:
            case 4103:
            case 4104:
            case 4106:
            case 4107:
            case 4108:
            case 4112:
            case 4113:
            case 4114:
            case 4115:
            case 4116:
            case 4117:
            case 4126:
            case 4127:
            case 4160:
            case 4167:
            case 4168:
                Vector vector = new Vector((short) (i & 4095));
                this._value = vector;
                return vector.read(data, offset);
            case 8194:
            case 8195:
            case 8196:
            case 8197:
            case 8198:
            case 8199:
            case 8200:
            case 8202:
            case 8203:
            case 8204:
            case 8206:
            case 8208:
            case 8209:
            case 8210:
            case 8211:
            case 8214:
            case 8215:
                Array array = new Array();
                this._value = array;
                return array.read(data, offset);
            default:
                throw new UnsupportedOperationException("Unknown (possibly, incorrect) TypedPropertyValue type: " + this._type);
        }
    }

    /* access modifiers changed from: package-private */
    public int readValuePadded(byte[] data, int offset) {
        int nonPadded = readValue(data, offset);
        return (nonPadded & 3) == 0 ? nonPadded : (4 - (nonPadded & 3)) + nonPadded;
    }
}
