package org.apache.poi.hssf.model;

import com.sec.factory.app.factorytest.FactoryTestManager;
import java.security.AccessControlException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import org.apache.poi.ddf.EscherBSERecord;
import org.apache.poi.ddf.EscherBoolProperty;
import org.apache.poi.ddf.EscherContainerRecord;
import org.apache.poi.ddf.EscherDgRecord;
import org.apache.poi.ddf.EscherDggRecord;
import org.apache.poi.ddf.EscherOptRecord;
import org.apache.poi.ddf.EscherProperties;
import org.apache.poi.ddf.EscherRGBProperty;
import org.apache.poi.ddf.EscherRecord;
import org.apache.poi.ddf.EscherSimpleProperty;
import org.apache.poi.ddf.EscherSpRecord;
import org.apache.poi.ddf.EscherSplitMenuColorsRecord;
import org.apache.poi.hssf.record.BOFRecord;
import org.apache.poi.hssf.record.BackupRecord;
import org.apache.poi.hssf.record.BookBoolRecord;
import org.apache.poi.hssf.record.BoundSheetRecord;
import org.apache.poi.hssf.record.CodepageRecord;
import org.apache.poi.hssf.record.CountryRecord;
import org.apache.poi.hssf.record.DSFRecord;
import org.apache.poi.hssf.record.DateWindow1904Record;
import org.apache.poi.hssf.record.DrawingGroupRecord;
import org.apache.poi.hssf.record.EOFRecord;
import org.apache.poi.hssf.record.EscherAggregate;
import org.apache.poi.hssf.record.ExtSSTRecord;
import org.apache.poi.hssf.record.ExtendedFormatRecord;
import org.apache.poi.hssf.record.FileSharingRecord;
import org.apache.poi.hssf.record.FnGroupCountRecord;
import org.apache.poi.hssf.record.FontRecord;
import org.apache.poi.hssf.record.FormatRecord;
import org.apache.poi.hssf.record.HideObjRecord;
import org.apache.poi.hssf.record.HyperlinkRecord;
import org.apache.poi.hssf.record.InterfaceEndRecord;
import org.apache.poi.hssf.record.InterfaceHdrRecord;
import org.apache.poi.hssf.record.MMSRecord;
import org.apache.poi.hssf.record.NameCommentRecord;
import org.apache.poi.hssf.record.NameRecord;
import org.apache.poi.hssf.record.PaletteRecord;
import org.apache.poi.hssf.record.PasswordRecord;
import org.apache.poi.hssf.record.PasswordRev4Record;
import org.apache.poi.hssf.record.PrecisionRecord;
import org.apache.poi.hssf.record.ProtectRecord;
import org.apache.poi.hssf.record.ProtectionRev4Record;
import org.apache.poi.hssf.record.RecalcIdRecord;
import org.apache.poi.hssf.record.Record;
import org.apache.poi.hssf.record.RefreshAllRecord;
import org.apache.poi.hssf.record.SSTRecord;
import org.apache.poi.hssf.record.StyleRecord;
import org.apache.poi.hssf.record.TabIdRecord;
import org.apache.poi.hssf.record.UseSelFSRecord;
import org.apache.poi.hssf.record.WindowOneRecord;
import org.apache.poi.hssf.record.WindowProtectRecord;
import org.apache.poi.hssf.record.WriteAccessRecord;
import org.apache.poi.hssf.record.WriteProtectRecord;
import org.apache.poi.hssf.record.common.UnicodeString;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFShape;
import org.apache.poi.poifs.crypt.CryptoFunctions;
import org.apache.poi.ss.formula.EvaluationWorkbook;
import org.apache.poi.ss.formula.FormulaShifter;
import org.apache.poi.ss.formula.ptg.Area3DPtg;
import org.apache.poi.ss.formula.ptg.NameXPtg;
import org.apache.poi.ss.formula.ptg.OperandPtg;
import org.apache.poi.ss.formula.ptg.Ptg;
import org.apache.poi.ss.formula.ptg.Ref3DPtg;
import org.apache.poi.ss.formula.udf.UDFFinder;
import org.apache.poi.ss.usermodel.BuiltinFormats;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.util.Internal;
import org.apache.poi.util.LocaleUtil;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

@Internal
public final class InternalWorkbook {
    private static final short CODEPAGE = 1200;
    private static final int DEBUG = 1;
    private static final int MAX_SENSITIVE_SHEET_NAME_LEN = 31;
    public static final String OLD_WORKBOOK_DIR_ENTRY_NAME = "Book";
    public static final String[] WORKBOOK_DIR_ENTRY_NAMES = {"Workbook", "WORKBOOK", "BOOK"};
    private static final POILogger log = POILogFactory.getLogger(InternalWorkbook.class);
    private final List<BoundSheetRecord> boundsheets = new ArrayList();
    private final Map<String, NameCommentRecord> commentRecords = new LinkedHashMap();
    private DrawingManager2 drawingManager;
    private List<EscherBSERecord> escherBSERecords = new ArrayList();
    private FileSharingRecord fileShare;
    private final List<FormatRecord> formats = new ArrayList();
    private final List<HyperlinkRecord> hyperlinks = new ArrayList();
    private LinkTable linkTable;
    private int maxformatid = -1;
    private int numfonts = 0;
    private int numxfs = 0;
    private final WorkbookRecordList records = new WorkbookRecordList();
    protected SSTRecord sst;
    private boolean uses1904datewindowing = false;
    private WindowOneRecord windowOne;
    private WriteAccessRecord writeAccess;
    private WriteProtectRecord writeProtect;

    private InternalWorkbook() {
    }

    public static InternalWorkbook createWorkbook(List<Record> recs) {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "Workbook (readfile) created with reclen=", Integer.valueOf(recs.size()));
        }
        InternalWorkbook retval = new InternalWorkbook();
        List<Record> records2 = new ArrayList<>(recs.size() / 3);
        retval.records.setRecords(records2);
        int k = 0;
        while (true) {
            if (k < recs.size()) {
                Record rec = recs.get(k);
                if (rec.getSid() == 10) {
                    records2.add(rec);
                    POILogger pOILogger2 = log;
                    if (pOILogger2.check(1)) {
                        pOILogger2.log(1, "found workbook eof record at " + k);
                    }
                } else {
                    switch (rec.getSid()) {
                        case 18:
                            POILogger pOILogger3 = log;
                            if (pOILogger3.check(1)) {
                                pOILogger3.log(1, "found protect record at " + k);
                            }
                            retval.records.setProtpos(k);
                            break;
                        case 23:
                            throw new RuntimeException("Extern sheet is part of LinkTable");
                        case 24:
                        case 430:
                            POILogger pOILogger4 = log;
                            if (pOILogger4.check(1)) {
                                pOILogger4.log(1, "found SupBook record at " + k);
                            }
                            LinkTable linkTable2 = new LinkTable(recs, k, retval.records, retval.commentRecords);
                            retval.linkTable = linkTable2;
                            k += linkTable2.getRecordCount() - 1;
                            continue;
                            k++;
                        case 34:
                            POILogger pOILogger5 = log;
                            if (pOILogger5.check(1)) {
                                pOILogger5.log(1, "found datewindow1904 record at " + k);
                            }
                            retval.uses1904datewindowing = ((DateWindow1904Record) rec).getWindowing() == 1;
                            break;
                        case 49:
                            POILogger pOILogger6 = log;
                            if (pOILogger6.check(1)) {
                                pOILogger6.log(1, "found font record at " + k);
                            }
                            retval.records.setFontpos(k);
                            retval.numfonts++;
                            break;
                        case 61:
                            POILogger pOILogger7 = log;
                            if (pOILogger7.check(1)) {
                                pOILogger7.log(1, "found WindowOneRecord at " + k);
                            }
                            retval.windowOne = (WindowOneRecord) rec;
                            break;
                        case 64:
                            POILogger pOILogger8 = log;
                            if (pOILogger8.check(1)) {
                                pOILogger8.log(1, "found backup record at " + k);
                            }
                            retval.records.setBackuppos(k);
                            break;
                        case 91:
                            POILogger pOILogger9 = log;
                            if (pOILogger9.check(1)) {
                                pOILogger9.log(1, "found FileSharing at " + k);
                            }
                            retval.fileShare = (FileSharingRecord) rec;
                            break;
                        case 92:
                            POILogger pOILogger10 = log;
                            if (pOILogger10.check(1)) {
                                pOILogger10.log(1, "found WriteAccess at " + k);
                            }
                            retval.writeAccess = (WriteAccessRecord) rec;
                            break;
                        case 133:
                            POILogger pOILogger11 = log;
                            if (pOILogger11.check(1)) {
                                pOILogger11.log(1, "found boundsheet record at " + k);
                            }
                            retval.boundsheets.add((BoundSheetRecord) rec);
                            retval.records.setBspos(k);
                            break;
                        case 134:
                            POILogger pOILogger12 = log;
                            if (pOILogger12.check(1)) {
                                pOILogger12.log(1, "found WriteProtect at " + k);
                            }
                            retval.writeProtect = (WriteProtectRecord) rec;
                            break;
                        case 146:
                            POILogger pOILogger13 = log;
                            if (pOILogger13.check(1)) {
                                pOILogger13.log(1, "found palette record at " + k);
                            }
                            retval.records.setPalettepos(k);
                            break;
                        case FactoryTestManager.TEST_LEAKAGE_CURRENT_CHECK_2:
                            POILogger pOILogger14 = log;
                            if (pOILogger14.check(1)) {
                                pOILogger14.log(1, "found XF record at " + k);
                            }
                            retval.records.setXfpos(k);
                            retval.numxfs++;
                            break;
                        case 252:
                            POILogger pOILogger15 = log;
                            if (pOILogger15.check(1)) {
                                pOILogger15.log(1, "found sst record at " + k);
                            }
                            retval.sst = (SSTRecord) rec;
                            break;
                        case 317:
                            POILogger pOILogger16 = log;
                            if (pOILogger16.check(1)) {
                                pOILogger16.log(1, "found tabid record at " + k);
                            }
                            retval.records.setTabpos(k);
                            break;
                        case FactoryTestManager.TEST_NFC_TYPE_B_START:
                            POILogger pOILogger17 = log;
                            if (pOILogger17.check(1)) {
                                pOILogger17.log(1, "found format record at " + k);
                            }
                            retval.formats.add((FormatRecord) rec);
                            retval.maxformatid = retval.maxformatid >= ((FormatRecord) rec).getIndexCode() ? retval.maxformatid : ((FormatRecord) rec).getIndexCode();
                            break;
                        case 2196:
                            NameCommentRecord ncr = (NameCommentRecord) rec;
                            POILogger pOILogger18 = log;
                            if (pOILogger18.check(1)) {
                                pOILogger18.log(1, "found NameComment at " + k);
                            }
                            retval.commentRecords.put(ncr.getNameText(), ncr);
                            break;
                        default:
                            POILogger pOILogger19 = log;
                            if (pOILogger19.check(1)) {
                                pOILogger19.log(1, "ignoring record (sid=" + ((int) rec.getSid()) + ") at " + k);
                                break;
                            }
                            break;
                    }
                    records2.add(rec);
                    k++;
                }
            }
        }
        while (k < recs.size()) {
            Record rec2 = recs.get(k);
            switch (rec2.getSid()) {
                case 440:
                    retval.hyperlinks.add((HyperlinkRecord) rec2);
                    break;
            }
            k++;
        }
        if (retval.windowOne == null) {
            retval.windowOne = createWindowOne();
        }
        POILogger pOILogger20 = log;
        if (pOILogger20.check(1)) {
            pOILogger20.log(1, "exit create workbook from existing file function");
        }
        return retval;
    }

    public static InternalWorkbook createWorkbook() {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "creating new workbook from scratch");
        }
        InternalWorkbook retval = new InternalWorkbook();
        List<Record> records2 = new ArrayList<>(30);
        retval.records.setRecords(records2);
        List<FormatRecord> formats2 = retval.formats;
        records2.add(createBOF());
        records2.add(new InterfaceHdrRecord(1200));
        records2.add(createMMS());
        records2.add(InterfaceEndRecord.instance);
        records2.add(createWriteAccess());
        records2.add(createCodepage());
        records2.add(createDSF());
        records2.add(createTabId());
        retval.records.setTabpos(records2.size() - 1);
        records2.add(createFnGroupCount());
        records2.add(createWindowProtect());
        records2.add(createProtect());
        retval.records.setProtpos(records2.size() - 1);
        records2.add(createPassword());
        records2.add(createProtectionRev4());
        records2.add(createPasswordRev4());
        WindowOneRecord createWindowOne = createWindowOne();
        retval.windowOne = createWindowOne;
        records2.add(createWindowOne);
        records2.add(createBackup());
        retval.records.setBackuppos(records2.size() - 1);
        records2.add(createHideObj());
        records2.add(createDateWindow1904());
        records2.add(createPrecision());
        records2.add(createRefreshAll());
        records2.add(createBookBool());
        records2.add(createFont());
        records2.add(createFont());
        records2.add(createFont());
        records2.add(createFont());
        retval.records.setFontpos(records2.size() - 1);
        retval.numfonts = 4;
        for (int i = 0; i <= 7; i++) {
            FormatRecord rec = createFormat(i);
            retval.maxformatid = retval.maxformatid >= rec.getIndexCode() ? retval.maxformatid : rec.getIndexCode();
            formats2.add(rec);
            records2.add(rec);
        }
        for (int k = 0; k < 21; k++) {
            records2.add(createExtendedFormat(k));
            retval.numxfs++;
        }
        retval.records.setXfpos(records2.size() - 1);
        for (int k2 = 0; k2 < 6; k2++) {
            records2.add(createStyle(k2));
        }
        records2.add(createUseSelFS());
        for (int k3 = 0; k3 < 1; k3++) {
            BoundSheetRecord bsr = createBoundSheet(k3);
            records2.add(bsr);
            retval.boundsheets.add(bsr);
            retval.records.setBspos(records2.size() - 1);
        }
        records2.add(createCountry());
        for (int k4 = 0; k4 < 1; k4++) {
            retval.getOrCreateLinkTable().checkExternSheet(k4);
        }
        SSTRecord sSTRecord = new SSTRecord();
        retval.sst = sSTRecord;
        records2.add(sSTRecord);
        records2.add(createExtendedSST());
        records2.add(EOFRecord.instance);
        POILogger pOILogger2 = log;
        if (pOILogger2.check(1)) {
            pOILogger2.log(1, "exit create new workbook from scratch");
        }
        return retval;
    }

    public NameRecord getSpecificBuiltinRecord(byte name, int sheetNumber) {
        return getOrCreateLinkTable().getSpecificBuiltinRecord(name, sheetNumber);
    }

    public void removeBuiltinRecord(byte name, int sheetIndex) {
        this.linkTable.removeBuiltinRecord(name, sheetIndex);
    }

    public int getNumRecords() {
        return this.records.size();
    }

    public FontRecord getFontRecordAt(int idx) {
        int index = idx;
        if (index > 4) {
            index--;
        }
        if (index <= this.numfonts - 1) {
            WorkbookRecordList workbookRecordList = this.records;
            return (FontRecord) workbookRecordList.get((workbookRecordList.getFontpos() - (this.numfonts - 1)) + index);
        }
        throw new ArrayIndexOutOfBoundsException("There are only " + this.numfonts + " font records, you asked for " + idx);
    }

    public int getFontIndex(FontRecord font) {
        for (int i = 0; i <= this.numfonts; i++) {
            WorkbookRecordList workbookRecordList = this.records;
            if (((FontRecord) workbookRecordList.get((workbookRecordList.getFontpos() - (this.numfonts - 1)) + i)) == font) {
                if (i > 3) {
                    return i + 1;
                } else {
                    return i;
                }
            }
        }
        throw new IllegalArgumentException("Could not find that font!");
    }

    public FontRecord createNewFont() {
        FontRecord rec = createFont();
        WorkbookRecordList workbookRecordList = this.records;
        workbookRecordList.add(workbookRecordList.getFontpos() + 1, rec);
        WorkbookRecordList workbookRecordList2 = this.records;
        workbookRecordList2.setFontpos(workbookRecordList2.getFontpos() + 1);
        this.numfonts++;
        return rec;
    }

    public void removeFontRecord(FontRecord rec) {
        this.records.remove(rec);
        this.numfonts--;
    }

    public int getNumberOfFontRecords() {
        return this.numfonts;
    }

    public void setSheetBof(int sheetIndex, int pos) {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "setting bof for sheetnum =", Integer.valueOf(sheetIndex), " at pos=", Integer.valueOf(pos));
        }
        checkSheets(sheetIndex);
        getBoundSheetRec(sheetIndex).setPositionOfBof(pos);
    }

    private BoundSheetRecord getBoundSheetRec(int sheetIndex) {
        return this.boundsheets.get(sheetIndex);
    }

    public BackupRecord getBackupRecord() {
        WorkbookRecordList workbookRecordList = this.records;
        return (BackupRecord) workbookRecordList.get(workbookRecordList.getBackuppos());
    }

    public void setSheetName(int sheetnum, String sheetname) {
        checkSheets(sheetnum);
        if (sheetname.length() > 31) {
            sheetname = sheetname.substring(0, 31);
        }
        this.boundsheets.get(sheetnum).setSheetname(sheetname);
    }

    public boolean doesContainsSheetName(String name, int excludeSheetIdx) {
        String aName = name;
        if (aName.length() > 31) {
            aName = aName.substring(0, 31);
        }
        for (int i = 0; i < this.boundsheets.size(); i++) {
            BoundSheetRecord boundSheetRecord = getBoundSheetRec(i);
            if (excludeSheetIdx != i) {
                String bName = boundSheetRecord.getSheetname();
                if (bName.length() > 31) {
                    bName = bName.substring(0, 31);
                }
                if (aName.equalsIgnoreCase(bName)) {
                    return true;
                }
            }
        }
        return false;
    }

    public void setSheetOrder(String sheetname, int pos) {
        int sheetNumber = getSheetIndex(sheetname);
        List<BoundSheetRecord> list = this.boundsheets;
        list.add(pos, list.remove(sheetNumber));
        int initialBspos = this.records.getBspos();
        int pos0 = initialBspos - (this.boundsheets.size() - 1);
        Record removed = this.records.get(pos0 + sheetNumber);
        this.records.remove(pos0 + sheetNumber);
        this.records.add(pos0 + pos, removed);
        this.records.setBspos(initialBspos);
    }

    public String getSheetName(int sheetIndex) {
        return getBoundSheetRec(sheetIndex).getSheetname();
    }

    public boolean isSheetHidden(int sheetnum) {
        return getBoundSheetRec(sheetnum).isHidden();
    }

    public boolean isSheetVeryHidden(int sheetnum) {
        return getBoundSheetRec(sheetnum).isVeryHidden();
    }

    public void setSheetHidden(int sheetnum, boolean hidden) {
        getBoundSheetRec(sheetnum).setHidden(hidden);
    }

    public void setSheetHidden(int sheetnum, int hidden) {
        BoundSheetRecord bsr = getBoundSheetRec(sheetnum);
        boolean h = false;
        boolean vh = false;
        if (hidden != 0) {
            if (hidden == 1) {
                h = true;
            } else if (hidden == 2) {
                vh = true;
            } else {
                throw new IllegalArgumentException("Invalid hidden flag " + hidden + " given, must be 0, 1 or 2");
            }
        }
        bsr.setHidden(h);
        bsr.setVeryHidden(vh);
    }

    public int getSheetIndex(String name) {
        for (int k = 0; k < this.boundsheets.size(); k++) {
            if (getSheetName(k).equalsIgnoreCase(name)) {
                return k;
            }
        }
        return -1;
    }

    private void checkSheets(int sheetnum) {
        if (this.boundsheets.size() > sheetnum) {
            return;
        }
        if (this.boundsheets.size() + 1 > sheetnum) {
            BoundSheetRecord bsr = createBoundSheet(sheetnum);
            WorkbookRecordList workbookRecordList = this.records;
            workbookRecordList.add(workbookRecordList.getBspos() + 1, bsr);
            WorkbookRecordList workbookRecordList2 = this.records;
            workbookRecordList2.setBspos(workbookRecordList2.getBspos() + 1);
            this.boundsheets.add(bsr);
            getOrCreateLinkTable().checkExternSheet(sheetnum);
            fixTabIdRecord();
            return;
        }
        throw new RuntimeException("Sheet number out of bounds!");
    }

    public void removeSheet(int sheetIndex) {
        if (this.boundsheets.size() > sheetIndex) {
            WorkbookRecordList workbookRecordList = this.records;
            workbookRecordList.remove((workbookRecordList.getBspos() - (this.boundsheets.size() - 1)) + sheetIndex);
            this.boundsheets.remove(sheetIndex);
            fixTabIdRecord();
        }
        int sheetNum1Based = sheetIndex + 1;
        for (int i = 0; i < getNumNames(); i++) {
            NameRecord nr = getNameRecord(i);
            if (nr.getSheetNumber() == sheetNum1Based) {
                nr.setSheetNumber(0);
            } else if (nr.getSheetNumber() > sheetNum1Based) {
                nr.setSheetNumber(nr.getSheetNumber() - 1);
            }
        }
        LinkTable linkTable2 = this.linkTable;
        if (linkTable2 != null) {
            linkTable2.removeSheet(sheetIndex);
        }
    }

    private void fixTabIdRecord() {
        WorkbookRecordList workbookRecordList = this.records;
        Record rec = workbookRecordList.get(workbookRecordList.getTabpos());
        if (this.records.getTabpos() > 0) {
            TabIdRecord tir = (TabIdRecord) rec;
            short[] tia = new short[this.boundsheets.size()];
            for (short k = 0; k < tia.length; k = (short) (k + 1)) {
                tia[k] = k;
            }
            tir.setTabIdArray(tia);
        }
    }

    public int getNumSheets() {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "getNumSheets=", Integer.valueOf(this.boundsheets.size()));
        }
        return this.boundsheets.size();
    }

    public int getNumExFormats() {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "getXF=", Integer.valueOf(this.numxfs));
        }
        return this.numxfs;
    }

    public ExtendedFormatRecord getExFormatAt(int index) {
        return (ExtendedFormatRecord) this.records.get((this.records.getXfpos() - (this.numxfs - 1)) + index);
    }

    public void removeExFormatRecord(ExtendedFormatRecord rec) {
        this.records.remove(rec);
        this.numxfs--;
    }

    public void removeExFormatRecord(int index) {
        this.records.remove((this.records.getXfpos() - (this.numxfs - 1)) + index);
        this.numxfs--;
    }

    public ExtendedFormatRecord createCellXF() {
        ExtendedFormatRecord xf = createExtendedFormat();
        WorkbookRecordList workbookRecordList = this.records;
        workbookRecordList.add(workbookRecordList.getXfpos() + 1, xf);
        WorkbookRecordList workbookRecordList2 = this.records;
        workbookRecordList2.setXfpos(workbookRecordList2.getXfpos() + 1);
        this.numxfs++;
        return xf;
    }

    public StyleRecord getStyleRecord(int xfIndex) {
        for (int i = this.records.getXfpos(); i < this.records.size(); i++) {
            Record r = this.records.get(i);
            if (!(r instanceof ExtendedFormatRecord) && (r instanceof StyleRecord)) {
                StyleRecord sr = (StyleRecord) r;
                if (sr.getXFIndex() == xfIndex) {
                    return sr;
                }
            }
        }
        return null;
    }

    public StyleRecord createStyleRecord(int xfIndex) {
        StyleRecord newSR = new StyleRecord();
        newSR.setXFIndex(xfIndex);
        int addAt = -1;
        for (int i = this.records.getXfpos(); i < this.records.size() && addAt == -1; i++) {
            Record r = this.records.get(i);
            if (!(r instanceof ExtendedFormatRecord) && !(r instanceof StyleRecord)) {
                addAt = i;
            }
        }
        if (addAt != -1) {
            this.records.add(addAt, newSR);
            return newSR;
        }
        throw new IllegalStateException("No XF Records found!");
    }

    public int addSSTString(UnicodeString string) {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "insert to sst string='", string);
        }
        if (this.sst == null) {
            insertSST();
        }
        return this.sst.addString(string);
    }

    public UnicodeString getSSTString(int str) {
        if (this.sst == null) {
            insertSST();
        }
        UnicodeString retval = this.sst.getString(str);
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "Returning SST for index=", Integer.valueOf(str), " String= ", retval);
        }
        return retval;
    }

    public void insertSST() {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "creating new SST via insertSST!");
        }
        this.sst = new SSTRecord();
        WorkbookRecordList workbookRecordList = this.records;
        workbookRecordList.add(workbookRecordList.size() - 1, createExtendedSST());
        WorkbookRecordList workbookRecordList2 = this.records;
        workbookRecordList2.add(workbookRecordList2.size() - 2, this.sst);
    }

    public int serialize(int offset, byte[] data) {
        POILogger pOILogger = log;
        if (pOILogger.check(1)) {
            pOILogger.log(1, "Serializing Workbook with offsets");
        }
        int pos = 0;
        SSTRecord sst2 = null;
        int sstPos = 0;
        boolean wroteBoundSheets = false;
        for (int k = 0; k < this.records.size(); k++) {
            Record record = this.records.get(k);
            int len = 0;
            if (record instanceof SSTRecord) {
                sst2 = (SSTRecord) record;
                sstPos = pos;
            }
            if (record.getSid() == 255 && sst2 != null) {
                record = sst2.createExtSSTRecord(sstPos + offset);
            }
            if (!(record instanceof BoundSheetRecord)) {
                len = record.serialize(pos + offset, data);
            } else if (!wroteBoundSheets) {
                for (int i = 0; i < this.boundsheets.size(); i++) {
                    len += getBoundSheetRec(i).serialize(pos + offset + len, data);
                }
                wroteBoundSheets = true;
            }
            pos += len;
        }
        POILogger pOILogger2 = log;
        if (pOILogger2.check(1)) {
            pOILogger2.log(1, "Exiting serialize workbook");
        }
        return pos;
    }

    public void preSerialize() {
        if (this.records.getTabpos() > 0) {
            WorkbookRecordList workbookRecordList = this.records;
            if (((TabIdRecord) workbookRecordList.get(workbookRecordList.getTabpos()))._tabids.length < this.boundsheets.size()) {
                fixTabIdRecord();
            }
        }
    }

    public int getSize() {
        int i;
        int retval = 0;
        SSTRecord sst2 = null;
        for (int k = 0; k < this.records.size(); k++) {
            Record record = this.records.get(k);
            if (record instanceof SSTRecord) {
                sst2 = (SSTRecord) record;
            }
            if (record.getSid() != 255 || sst2 == null) {
                i = record.getRecordSize();
            } else {
                i = sst2.calcExtSSTRecordSize();
            }
            retval += i;
        }
        return retval;
    }

    private static BOFRecord createBOF() {
        BOFRecord retval = new BOFRecord();
        retval.setVersion(1536);
        retval.setType(5);
        retval.setBuild(BOFRecord.BUILD);
        retval.setBuildYear(BOFRecord.BUILD_YEAR);
        retval.setHistoryBitMask(65);
        retval.setRequiredVersion(6);
        return retval;
    }

    private static MMSRecord createMMS() {
        MMSRecord retval = new MMSRecord();
        retval.setAddMenuCount((byte) 0);
        retval.setDelMenuCount((byte) 0);
        return retval;
    }

    private static WriteAccessRecord createWriteAccess() {
        WriteAccessRecord retval = new WriteAccessRecord();
        try {
            String username = System.getProperty("user.name");
            if (username == null) {
                username = "POI";
            }
            retval.setUsername(username);
        } catch (AccessControlException e) {
            retval.setUsername("POI");
        }
        return retval;
    }

    private static CodepageRecord createCodepage() {
        CodepageRecord retval = new CodepageRecord();
        retval.setCodepage(1200);
        return retval;
    }

    private static DSFRecord createDSF() {
        return new DSFRecord(false);
    }

    private static TabIdRecord createTabId() {
        return new TabIdRecord();
    }

    private static FnGroupCountRecord createFnGroupCount() {
        FnGroupCountRecord retval = new FnGroupCountRecord();
        retval.setCount(14);
        return retval;
    }

    private static WindowProtectRecord createWindowProtect() {
        return new WindowProtectRecord(false);
    }

    private static ProtectRecord createProtect() {
        return new ProtectRecord(false);
    }

    private static PasswordRecord createPassword() {
        return new PasswordRecord(0);
    }

    private static ProtectionRev4Record createProtectionRev4() {
        return new ProtectionRev4Record(false);
    }

    private static PasswordRev4Record createPasswordRev4() {
        return new PasswordRev4Record(0);
    }

    private static WindowOneRecord createWindowOne() {
        WindowOneRecord retval = new WindowOneRecord();
        retval.setHorizontalHold(360);
        retval.setVerticalHold(EscherProperties.BLIP__PICTURELINE);
        retval.setWidth(14940);
        retval.setHeight(9150);
        retval.setOptions(56);
        retval.setActiveSheetIndex(0);
        retval.setFirstVisibleTab(0);
        retval.setNumSelectedTabs(1);
        retval.setTabWidthRatio(600);
        return retval;
    }

    private static BackupRecord createBackup() {
        BackupRecord retval = new BackupRecord();
        retval.setBackup(0);
        return retval;
    }

    private static HideObjRecord createHideObj() {
        HideObjRecord retval = new HideObjRecord();
        retval.setHideObj(0);
        return retval;
    }

    private static DateWindow1904Record createDateWindow1904() {
        DateWindow1904Record retval = new DateWindow1904Record();
        retval.setWindowing(0);
        return retval;
    }

    private static PrecisionRecord createPrecision() {
        PrecisionRecord retval = new PrecisionRecord();
        retval.setFullPrecision(true);
        return retval;
    }

    private static RefreshAllRecord createRefreshAll() {
        return new RefreshAllRecord(false);
    }

    private static BookBoolRecord createBookBool() {
        BookBoolRecord retval = new BookBoolRecord();
        retval.setSaveLinkValues(0);
        return retval;
    }

    private static FontRecord createFont() {
        FontRecord retval = new FontRecord();
        retval.setFontHeight(EscherAggregate.ST_ACTIONBUTTONMOVIE);
        retval.setAttributes(0);
        retval.setColorPaletteIndex(Font.COLOR_NORMAL);
        retval.setBoldWeight(400);
        retval.setFontName(HSSFFont.FONT_ARIAL);
        return retval;
    }

    private static FormatRecord createFormat(int id) {
        switch (id) {
            case 0:
                return new FormatRecord(5, BuiltinFormats.getBuiltinFormat(5));
            case 1:
                return new FormatRecord(6, BuiltinFormats.getBuiltinFormat(6));
            case 2:
                return new FormatRecord(7, BuiltinFormats.getBuiltinFormat(7));
            case 3:
                return new FormatRecord(8, BuiltinFormats.getBuiltinFormat(8));
            case 4:
                return new FormatRecord(42, BuiltinFormats.getBuiltinFormat(42));
            case 5:
                return new FormatRecord(41, BuiltinFormats.getBuiltinFormat(41));
            case 6:
                return new FormatRecord(44, BuiltinFormats.getBuiltinFormat(44));
            case 7:
                return new FormatRecord(43, BuiltinFormats.getBuiltinFormat(43));
            default:
                throw new IllegalArgumentException("Unexpected id " + id);
        }
    }

    private static ExtendedFormatRecord createExtendedFormat(int id) {
        ExtendedFormatRecord retval = new ExtendedFormatRecord();
        switch (id) {
            case 0:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(0);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 1:
                retval.setFontIndex(1);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 2:
                retval.setFontIndex(1);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 3:
                retval.setFontIndex(2);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 4:
                retval.setFontIndex(2);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 5:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 6:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 7:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 8:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 9:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 10:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 11:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 12:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 13:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 14:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-3072);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 15:
                retval.setFontIndex(0);
                retval.setFormatIndex(0);
                retval.setCellOptions(1);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(0);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 16:
                retval.setFontIndex(1);
                retval.setFormatIndex(43);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-2048);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 17:
                retval.setFontIndex(1);
                retval.setFormatIndex(41);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-2048);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 18:
                retval.setFontIndex(1);
                retval.setFormatIndex(44);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-2048);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 19:
                retval.setFontIndex(1);
                retval.setFormatIndex(42);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-2048);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 20:
                retval.setFontIndex(1);
                retval.setFormatIndex(9);
                retval.setCellOptions(-11);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(-2048);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 21:
                retval.setFontIndex(5);
                retval.setFormatIndex(0);
                retval.setCellOptions(1);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(2048);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 22:
                retval.setFontIndex(6);
                retval.setFormatIndex(0);
                retval.setCellOptions(1);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(23552);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 23:
                retval.setFontIndex(0);
                retval.setFormatIndex(49);
                retval.setCellOptions(1);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(23552);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 24:
                retval.setFontIndex(0);
                retval.setFormatIndex(8);
                retval.setCellOptions(1);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(23552);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            case 25:
                retval.setFontIndex(6);
                retval.setFormatIndex(8);
                retval.setCellOptions(1);
                retval.setAlignmentOptions(32);
                retval.setIndentionOptions(23552);
                retval.setBorderOptions(0);
                retval.setPaletteOptions(0);
                retval.setAdtlPaletteOptions(0);
                retval.setFillPaletteOptions(8384);
                break;
            default:
                throw new IllegalStateException("Unrecognized format id: " + id);
        }
        return retval;
    }

    private static ExtendedFormatRecord createExtendedFormat() {
        ExtendedFormatRecord retval = new ExtendedFormatRecord();
        retval.setFontIndex(0);
        retval.setFormatIndex(0);
        retval.setCellOptions(1);
        retval.setAlignmentOptions(32);
        retval.setIndentionOptions(0);
        retval.setBorderOptions(0);
        retval.setPaletteOptions(0);
        retval.setAdtlPaletteOptions(0);
        retval.setFillPaletteOptions(8384);
        retval.setTopBorderPaletteIdx(8);
        retval.setBottomBorderPaletteIdx(8);
        retval.setLeftBorderPaletteIdx(8);
        retval.setRightBorderPaletteIdx(8);
        return retval;
    }

    private static StyleRecord createStyle(int id) {
        StyleRecord retval = new StyleRecord();
        switch (id) {
            case 0:
                retval.setXFIndex(16);
                retval.setBuiltinStyle(3);
                retval.setOutlineStyleLevel(-1);
                break;
            case 1:
                retval.setXFIndex(17);
                retval.setBuiltinStyle(6);
                retval.setOutlineStyleLevel(-1);
                break;
            case 2:
                retval.setXFIndex(18);
                retval.setBuiltinStyle(4);
                retval.setOutlineStyleLevel(-1);
                break;
            case 3:
                retval.setXFIndex(19);
                retval.setBuiltinStyle(7);
                retval.setOutlineStyleLevel(-1);
                break;
            case 4:
                retval.setXFIndex(0);
                retval.setBuiltinStyle(0);
                retval.setOutlineStyleLevel(-1);
                break;
            case 5:
                retval.setXFIndex(20);
                retval.setBuiltinStyle(5);
                retval.setOutlineStyleLevel(-1);
                break;
            default:
                throw new IllegalStateException("Unrecognized style id: " + id);
        }
        return retval;
    }

    private static PaletteRecord createPalette() {
        return new PaletteRecord();
    }

    private static UseSelFSRecord createUseSelFS() {
        return new UseSelFSRecord(false);
    }

    private static BoundSheetRecord createBoundSheet(int id) {
        return new BoundSheetRecord("Sheet" + (id + 1));
    }

    private static CountryRecord createCountry() {
        CountryRecord retval = new CountryRecord();
        retval.setDefaultCountry(1);
        if (LocaleUtil.getUserLocale().toString().equals("ru_RU")) {
            retval.setCurrentCountry(7);
        } else {
            retval.setCurrentCountry(1);
        }
        return retval;
    }

    private static ExtSSTRecord createExtendedSST() {
        ExtSSTRecord retval = new ExtSSTRecord();
        retval.setNumStringsPerBucket(8);
        return retval;
    }

    private LinkTable getOrCreateLinkTable() {
        if (this.linkTable == null) {
            this.linkTable = new LinkTable((short) getNumSheets(), this.records);
        }
        return this.linkTable;
    }

    public int linkExternalWorkbook(String name, Workbook externalWorkbook) {
        return getOrCreateLinkTable().linkExternalWorkbook(name, externalWorkbook);
    }

    public String findSheetFirstNameFromExternSheet(int externSheetIndex) {
        return findSheetNameFromIndex(this.linkTable.getFirstInternalSheetIndexForExtIndex(externSheetIndex));
    }

    public String findSheetLastNameFromExternSheet(int externSheetIndex) {
        return findSheetNameFromIndex(this.linkTable.getLastInternalSheetIndexForExtIndex(externSheetIndex));
    }

    private String findSheetNameFromIndex(int internalSheetIndex) {
        if (internalSheetIndex >= 0 && internalSheetIndex < this.boundsheets.size()) {
            return getSheetName(internalSheetIndex);
        }
        return "";
    }

    public EvaluationWorkbook.ExternalSheet getExternalSheet(int externSheetIndex) {
        String[] extNames = this.linkTable.getExternalBookAndSheetName(externSheetIndex);
        if (extNames == null) {
            return null;
        }
        if (extNames.length == 2) {
            return new EvaluationWorkbook.ExternalSheet(extNames[0], extNames[1]);
        }
        return new EvaluationWorkbook.ExternalSheetRange(extNames[0], extNames[1], extNames[2]);
    }

    public EvaluationWorkbook.ExternalName getExternalName(int externSheetIndex, int externNameIndex) {
        String nameName = this.linkTable.resolveNameXText(externSheetIndex, externNameIndex, this);
        if (nameName == null) {
            return null;
        }
        return new EvaluationWorkbook.ExternalName(nameName, externNameIndex, this.linkTable.resolveNameXIx(externSheetIndex, externNameIndex));
    }

    public int getFirstSheetIndexFromExternSheetIndex(int externSheetNumber) {
        return this.linkTable.getFirstInternalSheetIndexForExtIndex(externSheetNumber);
    }

    public int getLastSheetIndexFromExternSheetIndex(int externSheetNumber) {
        return this.linkTable.getLastInternalSheetIndexForExtIndex(externSheetNumber);
    }

    public short checkExternSheet(int sheetNumber) {
        return (short) getOrCreateLinkTable().checkExternSheet(sheetNumber);
    }

    public short checkExternSheet(int firstSheetNumber, int lastSheetNumber) {
        return (short) getOrCreateLinkTable().checkExternSheet(firstSheetNumber, lastSheetNumber);
    }

    public int getExternalSheetIndex(String workbookName, String sheetName) {
        return getOrCreateLinkTable().getExternalSheetIndex(workbookName, sheetName, sheetName);
    }

    public int getExternalSheetIndex(String workbookName, String firstSheetName, String lastSheetName) {
        return getOrCreateLinkTable().getExternalSheetIndex(workbookName, firstSheetName, lastSheetName);
    }

    public int getNumNames() {
        LinkTable linkTable2 = this.linkTable;
        if (linkTable2 == null) {
            return 0;
        }
        return linkTable2.getNumNames();
    }

    public NameRecord getNameRecord(int index) {
        return this.linkTable.getNameRecord(index);
    }

    public NameCommentRecord getNameCommentRecord(NameRecord nameRecord) {
        return this.commentRecords.get(nameRecord.getNameText());
    }

    public NameRecord createName() {
        return addName(new NameRecord());
    }

    public NameRecord addName(NameRecord name) {
        getOrCreateLinkTable().addName(name);
        return name;
    }

    public NameRecord createBuiltInName(byte builtInName, int sheetNumber) {
        if (sheetNumber < 0 || sheetNumber + 1 > 32767) {
            throw new IllegalArgumentException("Sheet number [" + sheetNumber + "]is not valid ");
        }
        NameRecord name = new NameRecord(builtInName, sheetNumber);
        if (!this.linkTable.nameAlreadyExists(name)) {
            addName(name);
            return name;
        }
        throw new RuntimeException("Builtin (" + ((int) builtInName) + ") already exists for sheet (" + sheetNumber + ")");
    }

    public void removeName(int nameIndex) {
        if (this.linkTable.getNumNames() > nameIndex) {
            this.records.remove(findFirstRecordLocBySid(24) + nameIndex);
            this.linkTable.removeName(nameIndex);
        }
    }

    public void updateNameCommentRecordCache(NameCommentRecord commentRecord) {
        if (this.commentRecords.containsValue(commentRecord)) {
            Iterator i$ = this.commentRecords.entrySet().iterator();
            while (true) {
                if (!i$.hasNext()) {
                    break;
                }
                Map.Entry<String, NameCommentRecord> entry = i$.next();
                if (entry.getValue().equals(commentRecord)) {
                    this.commentRecords.remove(entry.getKey());
                    break;
                }
            }
        }
        this.commentRecords.put(commentRecord.getNameText(), commentRecord);
    }

    public short getFormat(String format, boolean createIfNotFound) {
        for (FormatRecord r : this.formats) {
            if (r.getFormatString().equals(format)) {
                return (short) r.getIndexCode();
            }
        }
        if (createIfNotFound) {
            return (short) createFormat(format);
        }
        return -1;
    }

    public List<FormatRecord> getFormats() {
        return this.formats;
    }

    public int createFormat(String formatString) {
        int i = this.maxformatid;
        int i2 = 164;
        if (i >= 164) {
            i2 = i + 1;
        }
        this.maxformatid = i2;
        FormatRecord rec = new FormatRecord(i2, formatString);
        int pos = 0;
        while (pos < this.records.size() && this.records.get(pos).getSid() != 1054) {
            pos++;
        }
        int pos2 = pos + this.formats.size();
        this.formats.add(rec);
        this.records.add(pos2, rec);
        return this.maxformatid;
    }

    public Record findFirstRecordBySid(short sid) {
        Iterator i$ = this.records.iterator();
        while (i$.hasNext()) {
            Record record = i$.next();
            if (record.getSid() == sid) {
                return record;
            }
        }
        return null;
    }

    public int findFirstRecordLocBySid(short sid) {
        int index = 0;
        Iterator i$ = this.records.iterator();
        while (i$.hasNext()) {
            if (i$.next().getSid() == sid) {
                return index;
            }
            index++;
        }
        return -1;
    }

    public Record findNextRecordBySid(short sid, int pos) {
        int matches = 0;
        Iterator i$ = this.records.iterator();
        while (i$.hasNext()) {
            Record record = i$.next();
            if (record.getSid() == sid) {
                int matches2 = matches + 1;
                if (matches == pos) {
                    return record;
                }
                matches = matches2;
            }
        }
        return null;
    }

    public List<HyperlinkRecord> getHyperlinks() {
        return this.hyperlinks;
    }

    public List<Record> getRecords() {
        return this.records.getRecords();
    }

    public boolean isUsing1904DateWindowing() {
        return this.uses1904datewindowing;
    }

    public PaletteRecord getCustomPalette() {
        int palettePos = this.records.getPalettepos();
        if (palettePos != -1) {
            Record rec = this.records.get(palettePos);
            if (rec instanceof PaletteRecord) {
                return (PaletteRecord) rec;
            }
            throw new RuntimeException("InternalError: Expected PaletteRecord but got a '" + rec + "'");
        }
        PaletteRecord palette = createPalette();
        this.records.add(1, palette);
        this.records.setPalettepos(1);
        return palette;
    }

    public DrawingManager2 findDrawingGroup() {
        DrawingManager2 drawingManager2 = this.drawingManager;
        if (drawingManager2 != null) {
            return drawingManager2;
        }
        Iterator i$ = this.records.iterator();
        while (i$.hasNext()) {
            Record r = i$.next();
            if (r instanceof DrawingGroupRecord) {
                DrawingGroupRecord dg = (DrawingGroupRecord) r;
                dg.processChildRecords();
                EscherContainerRecord cr = dg.getEscherContainer();
                if (cr == null) {
                    continue;
                } else {
                    EscherDggRecord dgg = null;
                    EscherContainerRecord bStore = null;
                    Iterator<EscherRecord> it = cr.getChildIterator();
                    while (it.hasNext()) {
                        EscherRecord er = it.next();
                        if (er instanceof EscherDggRecord) {
                            dgg = (EscherDggRecord) er;
                        } else if (er.getRecordId() == -4095) {
                            bStore = (EscherContainerRecord) er;
                        }
                    }
                    if (dgg != null) {
                        this.drawingManager = new DrawingManager2(dgg);
                        if (bStore != null) {
                            for (EscherRecord bs : bStore.getChildRecords()) {
                                if (bs instanceof EscherBSERecord) {
                                    this.escherBSERecords.add((EscherBSERecord) bs);
                                }
                            }
                        }
                        return this.drawingManager;
                    }
                }
            }
        }
        int dgLoc = findFirstRecordLocBySid(DrawingGroupRecord.sid);
        if (dgLoc != -1) {
            EscherDggRecord dgg2 = null;
            EscherContainerRecord bStore2 = null;
            for (EscherRecord er2 : ((DrawingGroupRecord) this.records.get(dgLoc)).getEscherRecords()) {
                if (er2 instanceof EscherDggRecord) {
                    dgg2 = (EscherDggRecord) er2;
                } else if (er2.getRecordId() == -4095) {
                    bStore2 = (EscherContainerRecord) er2;
                }
            }
            if (dgg2 != null) {
                this.drawingManager = new DrawingManager2(dgg2);
                if (bStore2 != null) {
                    for (EscherRecord bs2 : bStore2.getChildRecords()) {
                        if (bs2 instanceof EscherBSERecord) {
                            this.escherBSERecords.add((EscherBSERecord) bs2);
                        }
                    }
                }
            }
        }
        return this.drawingManager;
    }

    public void createDrawingGroup() {
        if (this.drawingManager == null) {
            EscherContainerRecord dggContainer = new EscherContainerRecord();
            EscherDggRecord dgg = new EscherDggRecord();
            EscherOptRecord opt = new EscherOptRecord();
            EscherSplitMenuColorsRecord splitMenuColors = new EscherSplitMenuColorsRecord();
            dggContainer.setRecordId(EscherContainerRecord.DGG_CONTAINER);
            dggContainer.setOptions(15);
            dgg.setRecordId(EscherDggRecord.RECORD_ID);
            dgg.setOptions(0);
            dgg.setShapeIdMax(1024);
            dgg.setNumShapesSaved(0);
            dgg.setDrawingsSaved(0);
            dgg.setFileIdClusters(new EscherDggRecord.FileIdCluster[0]);
            this.drawingManager = new DrawingManager2(dgg);
            EscherContainerRecord bstoreContainer = null;
            if (this.escherBSERecords.size() > 0) {
                bstoreContainer = new EscherContainerRecord();
                bstoreContainer.setRecordId(EscherContainerRecord.BSTORE_CONTAINER);
                bstoreContainer.setOptions((short) (15 | (this.escherBSERecords.size() << 4)));
                for (EscherBSERecord escherRecord : this.escherBSERecords) {
                    bstoreContainer.addChildRecord(escherRecord);
                }
            }
            opt.setRecordId(EscherOptRecord.RECORD_ID);
            opt.setOptions(51);
            opt.addEscherProperty(new EscherBoolProperty(191, 524296));
            opt.addEscherProperty(new EscherRGBProperty(EscherProperties.FILL__FILLCOLOR, 134217793));
            opt.addEscherProperty(new EscherRGBProperty(EscherProperties.LINESTYLE__COLOR, HSSFShape.LINESTYLE__COLOR_DEFAULT));
            splitMenuColors.setRecordId(EscherSplitMenuColorsRecord.RECORD_ID);
            splitMenuColors.setOptions(64);
            splitMenuColors.setColor1(134217741);
            splitMenuColors.setColor2(134217740);
            splitMenuColors.setColor3(134217751);
            splitMenuColors.setColor4(268435703);
            dggContainer.addChildRecord(dgg);
            if (bstoreContainer != null) {
                dggContainer.addChildRecord(bstoreContainer);
            }
            dggContainer.addChildRecord(opt);
            dggContainer.addChildRecord(splitMenuColors);
            int dgLoc = findFirstRecordLocBySid(DrawingGroupRecord.sid);
            if (dgLoc == -1) {
                DrawingGroupRecord drawingGroup = new DrawingGroupRecord();
                drawingGroup.addEscherRecord(dggContainer);
                getRecords().add(findFirstRecordLocBySid(140) + 1, drawingGroup);
                return;
            }
            DrawingGroupRecord drawingGroup2 = new DrawingGroupRecord();
            drawingGroup2.addEscherRecord(dggContainer);
            getRecords().set(dgLoc, drawingGroup2);
        }
    }

    public WindowOneRecord getWindowOne() {
        return this.windowOne;
    }

    public EscherBSERecord getBSERecord(int pictureIndex) {
        return this.escherBSERecords.get(pictureIndex - 1);
    }

    public int addBSERecord(EscherBSERecord e) {
        EscherContainerRecord bstoreContainer;
        createDrawingGroup();
        this.escherBSERecords.add(e);
        EscherContainerRecord dggContainer = (EscherContainerRecord) ((DrawingGroupRecord) getRecords().get(findFirstRecordLocBySid(DrawingGroupRecord.sid))).getEscherRecord(0);
        if (dggContainer.getChild(1).getRecordId() == -4095) {
            bstoreContainer = (EscherContainerRecord) dggContainer.getChild(1);
        } else {
            EscherContainerRecord bstoreContainer2 = new EscherContainerRecord();
            bstoreContainer2.setRecordId(EscherContainerRecord.BSTORE_CONTAINER);
            List<EscherRecord> childRecords = dggContainer.getChildRecords();
            childRecords.add(1, bstoreContainer2);
            dggContainer.setChildRecords(childRecords);
            bstoreContainer = bstoreContainer2;
        }
        bstoreContainer.setOptions((short) ((this.escherBSERecords.size() << 4) | 15));
        bstoreContainer.addChildRecord(e);
        return this.escherBSERecords.size();
    }

    public DrawingManager2 getDrawingManager() {
        return this.drawingManager;
    }

    public WriteProtectRecord getWriteProtect() {
        if (this.writeProtect == null) {
            this.writeProtect = new WriteProtectRecord();
            int i = 0;
            while (i < this.records.size() && !(this.records.get(i) instanceof BOFRecord)) {
                i++;
            }
            this.records.add(i + 1, this.writeProtect);
        }
        return this.writeProtect;
    }

    public WriteAccessRecord getWriteAccess() {
        if (this.writeAccess == null) {
            this.writeAccess = createWriteAccess();
            int i = 0;
            while (i < this.records.size() && !(this.records.get(i) instanceof InterfaceEndRecord)) {
                i++;
            }
            this.records.add(i + 1, this.writeAccess);
        }
        return this.writeAccess;
    }

    public FileSharingRecord getFileSharing() {
        if (this.fileShare == null) {
            this.fileShare = new FileSharingRecord();
            int i = 0;
            while (i < this.records.size() && !(this.records.get(i) instanceof WriteAccessRecord)) {
                i++;
            }
            this.records.add(i + 1, this.fileShare);
        }
        return this.fileShare;
    }

    public boolean isWriteProtected() {
        if (this.fileShare != null && getFileSharing().getReadOnly() == 1) {
            return true;
        }
        return false;
    }

    public void writeProtectWorkbook(String password, String username) {
        FileSharingRecord frec = getFileSharing();
        WriteAccessRecord waccess = getWriteAccess();
        getWriteProtect();
        frec.setReadOnly(1);
        frec.setPassword((short) CryptoFunctions.createXorVerifier1(password));
        frec.setUsername(username);
        waccess.setUsername(username);
    }

    public void unwriteProtectWorkbook() {
        this.records.remove(this.fileShare);
        this.records.remove(this.writeProtect);
        this.fileShare = null;
        this.writeProtect = null;
    }

    public String resolveNameXText(int refIndex, int definedNameIndex) {
        return this.linkTable.resolveNameXText(refIndex, definedNameIndex, this);
    }

    public NameXPtg getNameXPtg(String name, int sheetRefIndex, UDFFinder udf) {
        LinkTable lnk = getOrCreateLinkTable();
        NameXPtg xptg = lnk.getNameXPtg(name, sheetRefIndex);
        if (xptg != null || udf.findFunction(name) == null) {
            return xptg;
        }
        return lnk.addNameXPtg(name);
    }

    public NameXPtg getNameXPtg(String name, UDFFinder udf) {
        return getNameXPtg(name, -1, udf);
    }

    public void cloneDrawings(InternalSheet sheet) {
        EscherAggregate agg;
        int aggLoc;
        InternalWorkbook internalWorkbook = this;
        findDrawingGroup();
        DrawingManager2 drawingManager2 = internalWorkbook.drawingManager;
        if (drawingManager2 != null) {
            int aggLoc2 = sheet.aggregateDrawingRecords(drawingManager2, false);
            if (aggLoc2 != -1) {
                EscherAggregate agg2 = (EscherAggregate) sheet.findFirstRecordBySid(EscherAggregate.sid);
                EscherContainerRecord escherContainer = agg2.getEscherContainer();
                if (escherContainer != null) {
                    EscherDggRecord dgg = internalWorkbook.drawingManager.getDgg();
                    int dgId = internalWorkbook.drawingManager.findNewDrawingGroupId();
                    dgg.addCluster(dgId, 0);
                    dgg.setDrawingsSaved(dgg.getDrawingsSaved() + 1);
                    EscherDgRecord dg = null;
                    Iterator<EscherRecord> it = escherContainer.getChildIterator();
                    while (it.hasNext()) {
                        EscherRecord er = it.next();
                        if (er instanceof EscherDgRecord) {
                            dg = (EscherDgRecord) er;
                            dg.setOptions((short) (dgId << 4));
                        } else if (er instanceof EscherContainerRecord) {
                            Iterator<EscherRecord> spIt = ((EscherContainerRecord) er).getChildRecords().iterator();
                            while (spIt.hasNext()) {
                                for (EscherRecord shapeChildRecord : ((EscherContainerRecord) spIt.next()).getChildRecords()) {
                                    int recordId = shapeChildRecord.getRecordId();
                                    if (recordId == -4086) {
                                        aggLoc = aggLoc2;
                                        agg = agg2;
                                        int shapeId = internalWorkbook.drawingManager.allocateShapeId((short) dgId, dg);
                                        dg.setNumShapes(dg.getNumShapes() - 1);
                                        ((EscherSpRecord) shapeChildRecord).setShapeId(shapeId);
                                    } else {
                                        aggLoc = aggLoc2;
                                        agg = agg2;
                                        if (recordId == -4085) {
                                            EscherSimpleProperty prop = (EscherSimpleProperty) ((EscherOptRecord) shapeChildRecord).lookup(260);
                                            if (prop != null) {
                                                EscherBSERecord bse = internalWorkbook.getBSERecord(prop.getPropertyValue());
                                                bse.setRef(bse.getRef() + 1);
                                            }
                                        }
                                    }
                                    internalWorkbook = this;
                                    aggLoc2 = aggLoc;
                                    agg2 = agg;
                                }
                                internalWorkbook = this;
                            }
                        }
                        internalWorkbook = this;
                        aggLoc2 = aggLoc2;
                        agg2 = agg2;
                    }
                }
            }
        }
    }

    /* JADX DEBUG: Multi-variable search result rejected for r4v3, resolved type: java.lang.Object[] */
    /* JADX WARN: Multi-variable type inference failed */
    public NameRecord cloneFilter(int filterDbNameIndex, int newSheetIndex) {
        NameRecord origNameRecord = getNameRecord(filterDbNameIndex);
        int newExtSheetIx = checkExternSheet(newSheetIndex);
        Ptg[] ptgs = origNameRecord.getNameDefinition();
        for (int i = 0; i < ptgs.length; i++) {
            Ptg ptg = ptgs[i];
            if (ptg instanceof Area3DPtg) {
                Area3DPtg a3p = (Area3DPtg) ((OperandPtg) ptg).copy();
                a3p.setExternSheetIndex(newExtSheetIx);
                ptgs[i] = a3p;
            } else if (ptg instanceof Ref3DPtg) {
                Ref3DPtg r3p = (Ref3DPtg) ((OperandPtg) ptg).copy();
                r3p.setExternSheetIndex(newExtSheetIx);
                ptgs[i] = r3p;
            }
        }
        NameRecord newNameRecord = createBuiltInName((byte) 13, newSheetIndex + 1);
        newNameRecord.setNameDefinition(ptgs);
        newNameRecord.setHidden(true);
        return newNameRecord;
    }

    public void updateNamesAfterCellShift(FormulaShifter shifter) {
        for (int i = 0; i < getNumNames(); i++) {
            NameRecord nr = getNameRecord(i);
            Ptg[] ptgs = nr.getNameDefinition();
            if (shifter.adjustFormula(ptgs, nr.getSheetNumber())) {
                nr.setNameDefinition(ptgs);
            }
        }
    }

    public RecalcIdRecord getRecalcId() {
        RecalcIdRecord record = (RecalcIdRecord) findFirstRecordBySid(449);
        if (record != null) {
            return record;
        }
        RecalcIdRecord record2 = new RecalcIdRecord();
        this.records.add(findFirstRecordLocBySid(140) + 1, record2);
        return record2;
    }

    public boolean changeExternalReference(String oldUrl, String newUrl) {
        return this.linkTable.changeExternalReference(oldUrl, newUrl);
    }
}
