package org.apache.poi.hssf.record;

import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.ss.util.CellRangeUtil;
import org.apache.poi.util.LittleEndianOutput;

public abstract class CFHeaderBase extends StandardRecord implements Cloneable {
    private int field_1_numcf;
    private int field_2_need_recalculation_and_id;
    private CellRangeAddress field_3_enclosing_cell_range;
    private CellRangeAddressList field_4_cell_ranges;

    @Override // org.apache.poi.hssf.record.Record, java.lang.Object
    public abstract CFHeaderBase clone();

    /* access modifiers changed from: protected */
    public abstract String getRecordName();

    protected CFHeaderBase() {
    }

    protected CFHeaderBase(CellRangeAddress[] regions, int nRules) {
        setCellRanges(CellRangeUtil.mergeCellRanges(regions));
        this.field_1_numcf = nRules;
    }

    /* access modifiers changed from: protected */
    public void createEmpty() {
        this.field_3_enclosing_cell_range = new CellRangeAddress(0, 0, 0, 0);
        this.field_4_cell_ranges = new CellRangeAddressList();
    }

    /* access modifiers changed from: protected */
    public void read(RecordInputStream in) {
        this.field_1_numcf = in.readShort();
        this.field_2_need_recalculation_and_id = in.readShort();
        this.field_3_enclosing_cell_range = new CellRangeAddress(in);
        this.field_4_cell_ranges = new CellRangeAddressList(in);
    }

    public int getNumberOfConditionalFormats() {
        return this.field_1_numcf;
    }

    public void setNumberOfConditionalFormats(int n) {
        this.field_1_numcf = n;
    }

    public boolean getNeedRecalculation() {
        return (this.field_2_need_recalculation_and_id & 1) == 1;
    }

    public void setNeedRecalculation(boolean b) {
        if (b != getNeedRecalculation()) {
            if (b) {
                this.field_2_need_recalculation_and_id++;
            } else {
                this.field_2_need_recalculation_and_id--;
            }
        }
    }

    public int getID() {
        return this.field_2_need_recalculation_and_id >> 1;
    }

    public void setID(int id) {
        boolean needsRecalc = getNeedRecalculation();
        int i = id << 1;
        this.field_2_need_recalculation_and_id = i;
        if (needsRecalc) {
            this.field_2_need_recalculation_and_id = i + 1;
        }
    }

    public CellRangeAddress getEnclosingCellRange() {
        return this.field_3_enclosing_cell_range;
    }

    public void setEnclosingCellRange(CellRangeAddress cr) {
        this.field_3_enclosing_cell_range = cr;
    }

    public void setCellRanges(CellRangeAddress[] cellRanges) {
        if (cellRanges != null) {
            CellRangeAddressList cral = new CellRangeAddressList();
            CellRangeAddress enclosingRange = null;
            for (CellRangeAddress cr : cellRanges) {
                enclosingRange = CellRangeUtil.createEnclosingCellRange(cr, enclosingRange);
                cral.addCellRangeAddress(cr);
            }
            this.field_3_enclosing_cell_range = enclosingRange;
            this.field_4_cell_ranges = cral;
            return;
        }
        throw new IllegalArgumentException("cellRanges must not be null");
    }

    public CellRangeAddress[] getCellRanges() {
        return this.field_4_cell_ranges.getCellRangeAddresses();
    }

    @Override // org.apache.poi.hssf.record.Record
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("[");
        buffer.append(getRecordName());
        buffer.append("]\n");
        buffer.append("\t.numCF             = ");
        buffer.append(getNumberOfConditionalFormats());
        buffer.append("\n");
        buffer.append("\t.needRecalc        = ");
        buffer.append(getNeedRecalculation());
        buffer.append("\n");
        buffer.append("\t.id                = ");
        buffer.append(getID());
        buffer.append("\n");
        buffer.append("\t.enclosingCellRange= ");
        buffer.append(getEnclosingCellRange());
        buffer.append("\n");
        buffer.append("\t.cfranges=[");
        int i = 0;
        while (i < this.field_4_cell_ranges.countRanges()) {
            buffer.append(i == 0 ? "" : ",");
            buffer.append(this.field_4_cell_ranges.getCellRangeAddress(i).toString());
            i++;
        }
        buffer.append("]\n");
        buffer.append("[/");
        buffer.append(getRecordName());
        buffer.append("]\n");
        return buffer.toString();
    }

    /* access modifiers changed from: protected */
    @Override // org.apache.poi.hssf.record.StandardRecord
    public int getDataSize() {
        return this.field_4_cell_ranges.getSize() + 12;
    }

    @Override // org.apache.poi.hssf.record.StandardRecord
    public void serialize(LittleEndianOutput out) {
        out.writeShort(this.field_1_numcf);
        out.writeShort(this.field_2_need_recalculation_and_id);
        this.field_3_enclosing_cell_range.serialize(out);
        this.field_4_cell_ranges.serialize(out);
    }

    /* access modifiers changed from: protected */
    public void copyTo(CFHeaderBase result) {
        result.field_1_numcf = this.field_1_numcf;
        result.field_2_need_recalculation_and_id = this.field_2_need_recalculation_and_id;
        result.field_3_enclosing_cell_range = this.field_3_enclosing_cell_range.copy();
        result.field_4_cell_ranges = this.field_4_cell_ranges.copy();
    }
}
