package org.apache.poi.hssf.record;

import java.util.Arrays;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.LittleEndianOutput;
import org.apache.poi.util.StringUtil;

public class DConRefRecord extends StandardRecord {
    public static final short sid = 81;
    private byte[] _unused;
    private int charCount;
    private int charType;
    private int firstCol;
    private int firstRow;
    private int lastCol;
    private int lastRow;
    private byte[] path;

    public DConRefRecord(byte[] data) {
        if (LittleEndian.getShort(data, 0) == 81) {
            int offset = 0 + 2 + 2;
            this.firstRow = LittleEndian.getUShort(data, offset);
            int offset2 = offset + 2;
            this.lastRow = LittleEndian.getUShort(data, offset2);
            int offset3 = offset2 + 2;
            this.firstCol = LittleEndian.getUByte(data, offset3);
            int offset4 = offset3 + 1;
            this.lastCol = LittleEndian.getUByte(data, offset4);
            int offset5 = offset4 + 1;
            int uShort = LittleEndian.getUShort(data, offset5);
            this.charCount = uShort;
            int offset6 = offset5 + 2;
            if (uShort >= 2) {
                short uByte = LittleEndian.getUByte(data, offset6);
                this.charType = uByte;
                int offset7 = offset6 + 1;
                int byteLength = this.charCount * ((uByte & 1) + 1);
                byte[] byteArray = LittleEndian.getByteArray(data, offset7, byteLength);
                this.path = byteArray;
                int offset8 = offset7 + byteLength;
                if (byteArray[0] == 2) {
                    this._unused = LittleEndian.getByteArray(data, offset8, this.charType + 1);
                    return;
                }
                return;
            }
            throw new RecordFormatException("Character count must be >= 2");
        }
        throw new RecordFormatException("incompatible sid.");
    }

    public DConRefRecord(RecordInputStream inStream) {
        if (inStream.getSid() == 81) {
            this.firstRow = inStream.readUShort();
            this.lastRow = inStream.readUShort();
            this.firstCol = inStream.readUByte();
            this.lastCol = inStream.readUByte();
            this.charCount = inStream.readUShort();
            int readUByte = inStream.readUByte() & 1;
            this.charType = readUByte;
            byte[] bArr = new byte[(this.charCount * (readUByte + 1))];
            this.path = bArr;
            inStream.readFully(bArr);
            if (this.path[0] == 2) {
                this._unused = inStream.readRemainder();
                return;
            }
            return;
        }
        throw new RecordFormatException("Wrong sid: " + ((int) inStream.getSid()));
    }

    /* access modifiers changed from: protected */
    @Override // org.apache.poi.hssf.record.StandardRecord
    public int getDataSize() {
        byte[] bArr = this.path;
        int sz = bArr.length + 9;
        if (bArr[0] == 2) {
            return sz + this._unused.length;
        }
        return sz;
    }

    /* access modifiers changed from: protected */
    @Override // org.apache.poi.hssf.record.StandardRecord
    public void serialize(LittleEndianOutput out) {
        out.writeShort(this.firstRow);
        out.writeShort(this.lastRow);
        out.writeByte(this.firstCol);
        out.writeByte(this.lastCol);
        out.writeShort(this.charCount);
        out.writeByte(this.charType);
        out.write(this.path);
        if (this.path[0] == 2) {
            out.write(this._unused);
        }
    }

    @Override // org.apache.poi.hssf.record.Record
    public short getSid() {
        return 81;
    }

    public int getFirstColumn() {
        return this.firstCol;
    }

    public int getFirstRow() {
        return this.firstRow;
    }

    public int getLastColumn() {
        return this.lastCol;
    }

    public int getLastRow() {
        return this.lastRow;
    }

    @Override // org.apache.poi.hssf.record.Record
    public String toString() {
        return "[DCONREF]\n" + "    .ref\n" + "        .firstrow   = " + this.firstRow + "\n" + "        .lastrow    = " + this.lastRow + "\n" + "        .firstcol   = " + this.firstCol + "\n" + "        .lastcol    = " + this.lastCol + "\n" + "    .cch            = " + this.charCount + "\n" + "    .stFile\n" + "        .h          = " + this.charType + "\n" + "        .rgb        = " + getReadablePath() + "\n" + "[/DCONREF]\n";
    }

    public byte[] getPath() {
        byte[] bArr = this.path;
        return Arrays.copyOf(bArr, bArr.length);
    }

    public String getReadablePath() {
        byte[] bArr;
        if (this.path == null) {
            return null;
        }
        int offset = 1;
        while (true) {
            bArr = this.path;
            if (bArr[offset] < 32 && offset < bArr.length) {
                offset++;
            }
        }
        return new String(Arrays.copyOfRange(bArr, offset, bArr.length), StringUtil.UTF8).replaceAll("\u0003", "/");
    }

    public boolean isExternalRef() {
        return this.path[0] == 1;
    }
}
