package org.apache.poi.hssf.record;

import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.util.HexDump;
import org.apache.poi.util.LittleEndianOutput;

public final class FilePassRecord extends StandardRecord implements Cloneable {
    static final /* synthetic */ boolean $assertionsDisabled = false;
    private static final int ENCRYPTION_OTHER = 1;
    private static final int ENCRYPTION_XOR = 0;
    public static final short sid = 47;
    private int _encryptionType;
    private KeyData _keyData;

    /* access modifiers changed from: private */
    public interface KeyData extends Cloneable {
        void appendToString(StringBuffer stringBuffer);

        @Override // java.lang.Object
        KeyData clone();

        int getDataSize();

        void read(RecordInputStream recordInputStream);

        void serialize(LittleEndianOutput littleEndianOutput);
    }

    public static final class Rc4KeyData implements KeyData, Cloneable {
        private static final int ENCRYPTION_OTHER_CAPI_2 = 2;
        private static final int ENCRYPTION_OTHER_CAPI_3 = 3;
        private static final int ENCRYPTION_OTHER_CAPI_4 = 4;
        private static final int ENCRYPTION_OTHER_RC4 = 1;
        private byte[] _encryptedVerifier;
        private byte[] _encryptedVerifierHash;
        private int _encryptionInfo;
        private int _minorVersionNo;
        private byte[] _salt;

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public void read(RecordInputStream in) {
            int readUShort = in.readUShort();
            this._encryptionInfo = readUShort;
            switch (readUShort) {
                case 1:
                    int readUShort2 = in.readUShort();
                    this._minorVersionNo = readUShort2;
                    if (readUShort2 == 1) {
                        this._salt = FilePassRecord.read(in, 16);
                        this._encryptedVerifier = FilePassRecord.read(in, 16);
                        this._encryptedVerifierHash = FilePassRecord.read(in, 16);
                        return;
                    }
                    throw new RecordFormatException("Unexpected VersionInfo number for RC4Header " + this._minorVersionNo);
                case 2:
                case 3:
                case 4:
                    throw new EncryptedDocumentException("HSSF does not currently support CryptoAPI encryption");
                default:
                    throw new RecordFormatException("Unknown encryption info " + this._encryptionInfo);
            }
        }

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public void serialize(LittleEndianOutput out) {
            out.writeShort(this._encryptionInfo);
            out.writeShort(this._minorVersionNo);
            out.write(this._salt);
            out.write(this._encryptedVerifier);
            out.write(this._encryptedVerifierHash);
        }

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public int getDataSize() {
            return 54;
        }

        public byte[] getSalt() {
            return (byte[]) this._salt.clone();
        }

        public void setSalt(byte[] salt) {
            this._salt = (byte[]) salt.clone();
        }

        public byte[] getEncryptedVerifier() {
            return (byte[]) this._encryptedVerifier.clone();
        }

        public void setEncryptedVerifier(byte[] encryptedVerifier) {
            this._encryptedVerifier = (byte[]) encryptedVerifier.clone();
        }

        public byte[] getEncryptedVerifierHash() {
            return (byte[]) this._encryptedVerifierHash.clone();
        }

        public void setEncryptedVerifierHash(byte[] encryptedVerifierHash) {
            this._encryptedVerifierHash = (byte[]) encryptedVerifierHash.clone();
        }

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public void appendToString(StringBuffer buffer) {
            buffer.append("    .rc4.info = ");
            buffer.append(HexDump.shortToHex(this._encryptionInfo));
            buffer.append("\n");
            buffer.append("    .rc4.ver  = ");
            buffer.append(HexDump.shortToHex(this._minorVersionNo));
            buffer.append("\n");
            buffer.append("    .rc4.salt = ");
            buffer.append(HexDump.toHex(this._salt));
            buffer.append("\n");
            buffer.append("    .rc4.verifier = ");
            buffer.append(HexDump.toHex(this._encryptedVerifier));
            buffer.append("\n");
            buffer.append("    .rc4.verifierHash = ");
            buffer.append(HexDump.toHex(this._encryptedVerifierHash));
            buffer.append("\n");
        }

        @Override // java.lang.Object, org.apache.poi.hssf.record.FilePassRecord.KeyData
        public Rc4KeyData clone() {
            Rc4KeyData other = new Rc4KeyData();
            other._salt = (byte[]) this._salt.clone();
            other._encryptedVerifier = (byte[]) this._encryptedVerifier.clone();
            other._encryptedVerifierHash = (byte[]) this._encryptedVerifierHash.clone();
            other._encryptionInfo = this._encryptionInfo;
            other._minorVersionNo = this._minorVersionNo;
            return other;
        }
    }

    public static final class XorKeyData implements KeyData, Cloneable {
        private int _key;
        private int _verifier;

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public void read(RecordInputStream in) {
            this._key = in.readUShort();
            this._verifier = in.readUShort();
        }

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public void serialize(LittleEndianOutput out) {
            out.writeShort(this._key);
            out.writeShort(this._verifier);
        }

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public int getDataSize() {
            return 6;
        }

        public int getKey() {
            return this._key;
        }

        public int getVerifier() {
            return this._verifier;
        }

        public void setKey(int key) {
            this._key = key;
        }

        public void setVerifier(int verifier) {
            this._verifier = verifier;
        }

        @Override // org.apache.poi.hssf.record.FilePassRecord.KeyData
        public void appendToString(StringBuffer buffer) {
            buffer.append("    .xor.key = ");
            buffer.append(HexDump.intToHex(this._key));
            buffer.append("\n");
            buffer.append("    .xor.verifier  = ");
            buffer.append(HexDump.intToHex(this._verifier));
            buffer.append("\n");
        }

        @Override // java.lang.Object, org.apache.poi.hssf.record.FilePassRecord.KeyData
        public XorKeyData clone() {
            XorKeyData other = new XorKeyData();
            other._key = this._key;
            other._verifier = this._verifier;
            return other;
        }
    }

    private FilePassRecord(FilePassRecord other) {
        this._encryptionType = other._encryptionType;
        this._keyData = other._keyData.clone();
    }

    public FilePassRecord(RecordInputStream in) {
        int readUShort = in.readUShort();
        this._encryptionType = readUShort;
        switch (readUShort) {
            case 0:
                this._keyData = new XorKeyData();
                break;
            case 1:
                this._keyData = new Rc4KeyData();
                break;
            default:
                throw new RecordFormatException("Unknown encryption type " + this._encryptionType);
        }
        this._keyData.read(in);
    }

    /* access modifiers changed from: private */
    public static byte[] read(RecordInputStream in, int size) {
        byte[] result = new byte[size];
        in.readFully(result);
        return result;
    }

    @Override // org.apache.poi.hssf.record.StandardRecord
    public void serialize(LittleEndianOutput out) {
        out.writeShort(this._encryptionType);
        this._keyData.serialize(out);
    }

    /* access modifiers changed from: protected */
    @Override // org.apache.poi.hssf.record.StandardRecord
    public int getDataSize() {
        return this._keyData.getDataSize();
    }

    public Rc4KeyData getRc4KeyData() {
        KeyData keyData = this._keyData;
        if (keyData instanceof Rc4KeyData) {
            return (Rc4KeyData) keyData;
        }
        return null;
    }

    public XorKeyData getXorKeyData() {
        KeyData keyData = this._keyData;
        if (keyData instanceof XorKeyData) {
            return (XorKeyData) keyData;
        }
        return null;
    }

    private Rc4KeyData checkRc4() {
        Rc4KeyData rc4 = getRc4KeyData();
        if (rc4 != null) {
            return rc4;
        }
        throw new RecordFormatException("file pass record doesn't contain a rc4 key.");
    }

    @Override // org.apache.poi.hssf.record.Record
    public short getSid() {
        return 47;
    }

    @Override // org.apache.poi.hssf.record.Record, java.lang.Object
    public FilePassRecord clone() {
        return new FilePassRecord(this);
    }

    @Override // org.apache.poi.hssf.record.Record
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("[FILEPASS]\n");
        buffer.append("    .type = ");
        buffer.append(HexDump.shortToHex(this._encryptionType));
        buffer.append("\n");
        this._keyData.appendToString(buffer);
        buffer.append("[/FILEPASS]\n");
        return buffer.toString();
    }
}
