package org.apache.poi.hssf.record.cf;

import org.apache.poi.ss.usermodel.IconMultiStateFormatting;
import org.apache.poi.util.BitField;
import org.apache.poi.util.BitFieldFactory;
import org.apache.poi.util.LittleEndianInput;
import org.apache.poi.util.LittleEndianOutput;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public final class IconMultiStateFormatting implements Cloneable {
    private static BitField iconOnly = BitFieldFactory.getInstance(1);
    private static POILogger log = POILogFactory.getLogger(IconMultiStateFormatting.class);
    private static BitField reversed = BitFieldFactory.getInstance(4);
    private IconMultiStateFormatting.IconSet iconSet;
    private byte options;
    private Threshold[] thresholds;

    public IconMultiStateFormatting() {
        IconMultiStateFormatting.IconSet iconSet2 = IconMultiStateFormatting.IconSet.GYR_3_TRAFFIC_LIGHTS;
        this.iconSet = iconSet2;
        this.options = 0;
        this.thresholds = new Threshold[iconSet2.num];
    }

    public IconMultiStateFormatting(LittleEndianInput in) {
        in.readShort();
        in.readByte();
        int num = in.readByte();
        IconMultiStateFormatting.IconSet byId = IconMultiStateFormatting.IconSet.byId(in.readByte());
        this.iconSet = byId;
        if (byId.num != num) {
            POILogger pOILogger = log;
            pOILogger.log(5, "Inconsistent Icon Set defintion, found " + this.iconSet + " but defined as " + num + " entries");
        }
        this.options = in.readByte();
        this.thresholds = new Threshold[this.iconSet.num];
        int i = 0;
        while (true) {
            Threshold[] thresholdArr = this.thresholds;
            if (i < thresholdArr.length) {
                thresholdArr[i] = new IconMultiStateThreshold(in);
                i++;
            } else {
                return;
            }
        }
    }

    public IconMultiStateFormatting.IconSet getIconSet() {
        return this.iconSet;
    }

    public void setIconSet(IconMultiStateFormatting.IconSet set) {
        this.iconSet = set;
    }

    public Threshold[] getThresholds() {
        return this.thresholds;
    }

    public void setThresholds(Threshold[] thresholds2) {
        this.thresholds = thresholds2 == null ? null : (Threshold[]) thresholds2.clone();
    }

    public boolean isIconOnly() {
        return getOptionFlag(iconOnly);
    }

    public void setIconOnly(boolean only) {
        setOptionFlag(only, iconOnly);
    }

    public boolean isReversed() {
        return getOptionFlag(reversed);
    }

    public void setReversed(boolean rev) {
        setOptionFlag(rev, reversed);
    }

    private boolean getOptionFlag(BitField field) {
        return field.getValue(this.options) != 0;
    }

    private void setOptionFlag(boolean option, BitField field) {
        this.options = field.setByteBoolean(this.options, option);
    }

    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("    [Icon Formatting]\n");
        buffer.append("          .icon_set = ");
        buffer.append(this.iconSet);
        buffer.append("\n");
        buffer.append("          .icon_only= ");
        buffer.append(isIconOnly());
        buffer.append("\n");
        buffer.append("          .reversed = ");
        buffer.append(isReversed());
        buffer.append("\n");
        for (Threshold t : this.thresholds) {
            buffer.append(t.toString());
        }
        buffer.append("    [/Icon Formatting]\n");
        return buffer.toString();
    }

    @Override // java.lang.Object
    public Object clone() {
        IconMultiStateFormatting rec = new IconMultiStateFormatting();
        rec.iconSet = this.iconSet;
        rec.options = this.options;
        Threshold[] thresholdArr = new Threshold[this.thresholds.length];
        rec.thresholds = thresholdArr;
        Threshold[] thresholdArr2 = this.thresholds;
        System.arraycopy(thresholdArr2, 0, thresholdArr, 0, thresholdArr2.length);
        return rec;
    }

    public int getDataLength() {
        int len = 6;
        for (Threshold t : this.thresholds) {
            len += t.getDataLength();
        }
        return len;
    }

    public void serialize(LittleEndianOutput out) {
        out.writeShort(0);
        out.writeByte(0);
        out.writeByte(this.iconSet.num);
        out.writeByte(this.iconSet.id);
        out.writeByte(this.options);
        for (Threshold t : this.thresholds) {
            t.serialize(out);
        }
    }
}
