package org.apache.poi.hssf.record.crypto;

import com.google.common.primitives.UnsignedBytes;
import java.io.InputStream;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.hssf.record.BiffHeaderInput;
import org.apache.poi.util.LittleEndianInput;
import org.apache.poi.util.LittleEndianInputStream;

public final class Biff8DecryptingStream implements BiffHeaderInput, LittleEndianInput {
    private final Biff8Cipher _cipher;
    private final LittleEndianInput _le;

    public Biff8DecryptingStream(InputStream in, int initialOffset, Biff8EncryptionKey key) {
        if (key instanceof Biff8RC4Key) {
            this._cipher = new Biff8RC4(initialOffset, (Biff8RC4Key) key);
        } else if (key instanceof Biff8XORKey) {
            this._cipher = new Biff8XOR(initialOffset, (Biff8XORKey) key);
        } else {
            throw new EncryptedDocumentException("Crypto API not supported yet.");
        }
        if (in instanceof LittleEndianInput) {
            this._le = (LittleEndianInput) in;
        } else {
            this._le = new LittleEndianInputStream(in);
        }
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.hssf.record.BiffHeaderInput
    public int available() {
        return this._le.available();
    }

    @Override // org.apache.poi.hssf.record.BiffHeaderInput
    public int readRecordSID() {
        int sid = this._le.readUShort();
        this._cipher.skipTwoBytes();
        this._cipher.startRecord(sid);
        return sid;
    }

    @Override // org.apache.poi.hssf.record.BiffHeaderInput
    public int readDataSize() {
        int dataSize = this._le.readUShort();
        this._cipher.skipTwoBytes();
        this._cipher.setNextRecordSize(dataSize);
        return dataSize;
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public double readDouble() {
        double result = Double.longBitsToDouble(readLong());
        if (!Double.isNaN(result)) {
            return result;
        }
        throw new RuntimeException("Did not expect to read NaN");
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public void readFully(byte[] buf) {
        readFully(buf, 0, buf.length);
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public void readFully(byte[] buf, int off, int len) {
        this._le.readFully(buf, off, len);
        this._cipher.xor(buf, off, len);
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public int readUByte() {
        return readByte() & UnsignedBytes.MAX_VALUE;
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public byte readByte() {
        return (byte) this._cipher.xorByte(this._le.readUByte());
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public int readUShort() {
        return readShort() & 65535;
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public short readShort() {
        return (short) this._cipher.xorShort(this._le.readUShort());
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public int readInt() {
        return this._cipher.xorInt(this._le.readInt());
    }

    @Override // org.apache.poi.util.LittleEndianInput
    public long readLong() {
        return this._cipher.xorLong(this._le.readLong());
    }
}
