package org.apache.poi.hssf.usermodel;

import org.apache.poi.hssf.model.HSSFFormulaParser;
import org.apache.poi.hssf.model.InternalWorkbook;
import org.apache.poi.hssf.record.NameCommentRecord;
import org.apache.poi.hssf.record.NameRecord;
import org.apache.poi.ss.formula.FormulaType;
import org.apache.poi.ss.formula.ptg.Ptg;
import org.apache.poi.ss.usermodel.Name;

public final class HSSFName implements Name {
    private HSSFWorkbook _book;
    private NameCommentRecord _commentRec;
    private NameRecord _definedNameRec;

    HSSFName(HSSFWorkbook book, NameRecord name) {
        this(book, name, null);
    }

    HSSFName(HSSFWorkbook book, NameRecord name, NameCommentRecord comment) {
        this._book = book;
        this._definedNameRec = name;
        this._commentRec = comment;
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public String getSheetName() {
        return this._book.getWorkbook().findSheetFirstNameFromExternSheet(this._definedNameRec.getExternSheetNumber());
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public String getNameName() {
        return this._definedNameRec.getNameText();
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public void setNameName(String nameName) {
        validateName(nameName);
        InternalWorkbook wb = this._book.getWorkbook();
        this._definedNameRec.setNameText(nameName);
        int sheetNumber = this._definedNameRec.getSheetNumber();
        for (int i = wb.getNumNames() - 1; i >= 0; i--) {
            NameRecord rec = wb.getNameRecord(i);
            if (rec != this._definedNameRec && rec.getNameText().equalsIgnoreCase(nameName) && sheetNumber == rec.getSheetNumber()) {
                StringBuilder sb = new StringBuilder();
                sb.append("The ");
                sb.append(sheetNumber == 0 ? "workbook" : "sheet");
                sb.append(" already contains this name: ");
                sb.append(nameName);
                String msg = sb.toString();
                this._definedNameRec.setNameText(nameName + "(2)");
                throw new IllegalArgumentException(msg);
            }
        }
        NameCommentRecord nameCommentRecord = this._commentRec;
        if (nameCommentRecord != null) {
            nameCommentRecord.setNameText(nameName);
            this._book.getWorkbook().updateNameCommentRecordCache(this._commentRec);
        }
    }

    private static void validateName(String name) {
        if (name.length() != 0) {
            char c = name.charAt(0);
            if (Character.isLetter(c) || "_".indexOf(c) != -1) {
                char[] arr$ = name.toCharArray();
                for (char ch : arr$) {
                    if (!(Character.isLetterOrDigit(ch) || "_\\".indexOf(ch) != -1)) {
                        throw new IllegalArgumentException("Invalid name: '" + name + "'");
                    }
                }
                return;
            }
            throw new IllegalArgumentException("Invalid name: '" + name + "': first character must be underscore or a letter");
        }
        throw new IllegalArgumentException("Name cannot be blank");
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public void setRefersToFormula(String formulaText) {
        this._definedNameRec.setNameDefinition(HSSFFormulaParser.parse(formulaText, this._book, FormulaType.NAMEDRANGE, getSheetIndex()));
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public String getRefersToFormula() {
        if (!this._definedNameRec.isFunctionName()) {
            Ptg[] ptgs = this._definedNameRec.getNameDefinition();
            if (ptgs.length < 1) {
                return null;
            }
            return HSSFFormulaParser.toFormulaString(this._book, ptgs);
        }
        throw new IllegalStateException("Only applicable to named ranges");
    }

    /* access modifiers changed from: package-private */
    public void setNameDefinition(Ptg[] ptgs) {
        this._definedNameRec.setNameDefinition(ptgs);
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public boolean isDeleted() {
        return Ptg.doesFormulaReferToDeletedCell(this._definedNameRec.getNameDefinition());
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public boolean isFunctionName() {
        return this._definedNameRec.isFunctionName();
    }

    public String toString() {
        StringBuffer sb = new StringBuffer(64);
        sb.append(getClass().getName());
        sb.append(" [");
        sb.append(this._definedNameRec.getNameText());
        sb.append("]");
        return sb.toString();
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public void setSheetIndex(int index) {
        String str;
        int lastSheetIx = this._book.getNumberOfSheets() - 1;
        if (index < -1 || index > lastSheetIx) {
            StringBuilder sb = new StringBuilder();
            sb.append("Sheet index (");
            sb.append(index);
            sb.append(") is out of range");
            if (lastSheetIx == -1) {
                str = "";
            } else {
                str = " (0.." + lastSheetIx + ")";
            }
            sb.append(str);
            throw new IllegalArgumentException(sb.toString());
        }
        this._definedNameRec.setSheetNumber(index + 1);
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public int getSheetIndex() {
        return this._definedNameRec.getSheetNumber() - 1;
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public String getComment() {
        NameCommentRecord nameCommentRecord = this._commentRec;
        if (nameCommentRecord == null || nameCommentRecord.getCommentText() == null || this._commentRec.getCommentText().length() <= 0) {
            return this._definedNameRec.getDescriptionText();
        }
        return this._commentRec.getCommentText();
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public void setComment(String comment) {
        this._definedNameRec.setDescriptionText(comment);
        NameCommentRecord nameCommentRecord = this._commentRec;
        if (nameCommentRecord != null) {
            nameCommentRecord.setCommentText(comment);
        }
    }

    @Override // org.apache.poi.ss.usermodel.Name
    public void setFunction(boolean value) {
        this._definedNameRec.setFunction(value);
    }
}
