package org.apache.poi.hssf.usermodel;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import org.apache.poi.ddf.EscherBoolProperty;
import org.apache.poi.ddf.EscherChildAnchorRecord;
import org.apache.poi.ddf.EscherClientAnchorRecord;
import org.apache.poi.ddf.EscherContainerRecord;
import org.apache.poi.ddf.EscherOptRecord;
import org.apache.poi.ddf.EscherProperties;
import org.apache.poi.ddf.EscherProperty;
import org.apache.poi.ddf.EscherRGBProperty;
import org.apache.poi.ddf.EscherSimpleProperty;
import org.apache.poi.ddf.EscherSpRecord;
import org.apache.poi.hssf.record.CommonObjectDataSubRecord;
import org.apache.poi.hssf.record.ObjRecord;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public abstract class HSSFShape {
    public static final int FILL__FILLCOLOR_DEFAULT = 134217737;
    public static final int LINESTYLE_DASHDOTDOTSYS = 4;
    public static final int LINESTYLE_DASHDOTGEL = 8;
    public static final int LINESTYLE_DASHDOTSYS = 3;
    public static final int LINESTYLE_DASHGEL = 6;
    public static final int LINESTYLE_DASHSYS = 1;
    public static final int LINESTYLE_DEFAULT = -1;
    public static final int LINESTYLE_DOTGEL = 5;
    public static final int LINESTYLE_DOTSYS = 2;
    public static final int LINESTYLE_LONGDASHDOTDOTGEL = 10;
    public static final int LINESTYLE_LONGDASHDOTGEL = 9;
    public static final int LINESTYLE_LONGDASHGEL = 7;
    public static final int LINESTYLE_NONE = -1;
    public static final int LINESTYLE_SOLID = 0;
    public static final int LINESTYLE__COLOR_DEFAULT = 134217792;
    public static final int LINEWIDTH_DEFAULT = 9525;
    public static final int LINEWIDTH_ONE_PT = 12700;
    private static final POILogger LOG = POILogFactory.getLogger(HSSFShape.class);
    public static final int NO_FILLHITTEST_FALSE = 65536;
    public static final int NO_FILLHITTEST_TRUE = 1114112;
    public static final boolean NO_FILL_DEFAULT = true;
    private final EscherContainerRecord _escherContainer;
    private final ObjRecord _objRecord;
    private final EscherOptRecord _optRecord;
    private HSSFPatriarch _patriarch;
    HSSFAnchor anchor;
    private HSSFShape parent;

    /* access modifiers changed from: package-private */
    public abstract void afterInsert(HSSFPatriarch hSSFPatriarch);

    /* access modifiers changed from: protected */
    public abstract void afterRemove(HSSFPatriarch hSSFPatriarch);

    /* access modifiers changed from: protected */
    public abstract HSSFShape cloneShape();

    /* access modifiers changed from: protected */
    public abstract ObjRecord createObjRecord();

    /* access modifiers changed from: protected */
    public abstract EscherContainerRecord createSpContainer();

    public HSSFShape(EscherContainerRecord spContainer, ObjRecord objRecord) {
        this._escherContainer = spContainer;
        this._objRecord = objRecord;
        this._optRecord = (EscherOptRecord) spContainer.getChildById(EscherOptRecord.RECORD_ID);
        this.anchor = HSSFAnchor.createAnchorFromEscher(spContainer);
    }

    public HSSFShape(HSSFShape parent2, HSSFAnchor anchor2) {
        this.parent = parent2;
        this.anchor = anchor2;
        EscherContainerRecord createSpContainer = createSpContainer();
        this._escherContainer = createSpContainer;
        this._optRecord = (EscherOptRecord) createSpContainer.getChildById(EscherOptRecord.RECORD_ID);
        this._objRecord = createObjRecord();
    }

    /* access modifiers changed from: package-private */
    public void setShapeId(int shapeId) {
        ((EscherSpRecord) this._escherContainer.getChildById(EscherSpRecord.RECORD_ID)).setShapeId(shapeId);
        ((CommonObjectDataSubRecord) this._objRecord.getSubRecords().get(0)).setObjectId((short) (shapeId % 1024));
    }

    /* access modifiers changed from: package-private */
    public int getShapeId() {
        return ((EscherSpRecord) this._escherContainer.getChildById(EscherSpRecord.RECORD_ID)).getShapeId();
    }

    /* access modifiers changed from: protected */
    public EscherContainerRecord getEscherContainer() {
        return this._escherContainer;
    }

    /* access modifiers changed from: protected */
    public ObjRecord getObjRecord() {
        return this._objRecord;
    }

    public EscherOptRecord getOptRecord() {
        return this._optRecord;
    }

    public HSSFShape getParent() {
        return this.parent;
    }

    public HSSFAnchor getAnchor() {
        return this.anchor;
    }

    public void setAnchor(HSSFAnchor anchor2) {
        int recordId = -1;
        if (this.parent == null) {
            if (!(anchor2 instanceof HSSFChildAnchor)) {
                EscherClientAnchorRecord anch = (EscherClientAnchorRecord) this._escherContainer.getChildById(EscherClientAnchorRecord.RECORD_ID);
                if (anch != null) {
                    for (int i = 0; i < this._escherContainer.getChildRecords().size(); i++) {
                        if (this._escherContainer.getChild(i).getRecordId() == -4080 && i != this._escherContainer.getChildRecords().size() - 1) {
                            recordId = this._escherContainer.getChild(i + 1).getRecordId();
                        }
                    }
                    this._escherContainer.removeChildRecord(anch);
                }
            } else {
                throw new IllegalArgumentException("Must use client anchors for shapes directly attached to sheet.");
            }
        } else if (!(anchor2 instanceof HSSFClientAnchor)) {
            EscherChildAnchorRecord anch2 = (EscherChildAnchorRecord) this._escherContainer.getChildById(EscherChildAnchorRecord.RECORD_ID);
            if (anch2 != null) {
                for (int i2 = 0; i2 < this._escherContainer.getChildRecords().size(); i2++) {
                    if (this._escherContainer.getChild(i2).getRecordId() == -4081 && i2 != this._escherContainer.getChildRecords().size() - 1) {
                        recordId = this._escherContainer.getChild(i2 + 1).getRecordId();
                    }
                }
                this._escherContainer.removeChildRecord(anch2);
            }
        } else {
            throw new IllegalArgumentException("Must use child anchors for shapes attached to groups.");
        }
        if (-1 == recordId) {
            this._escherContainer.addChildRecord(anchor2.getEscherAnchor());
        } else {
            this._escherContainer.addChildBefore(anchor2.getEscherAnchor(), recordId);
        }
        this.anchor = anchor2;
    }

    public int getLineStyleColor() {
        EscherRGBProperty rgbProperty = (EscherRGBProperty) this._optRecord.lookup(448);
        return rgbProperty == null ? LINESTYLE__COLOR_DEFAULT : rgbProperty.getRgbColor();
    }

    public void setLineStyleColor(int lineStyleColor) {
        setPropertyValue(new EscherRGBProperty(EscherProperties.LINESTYLE__COLOR, lineStyleColor));
    }

    public void setLineStyleColor(int red, int green, int blue) {
        setPropertyValue(new EscherRGBProperty(EscherProperties.LINESTYLE__COLOR, (blue << 16) | (green << 8) | red));
    }

    public int getFillColor() {
        EscherRGBProperty rgbProperty = (EscherRGBProperty) this._optRecord.lookup(385);
        return rgbProperty == null ? FILL__FILLCOLOR_DEFAULT : rgbProperty.getRgbColor();
    }

    public void setFillColor(int fillColor) {
        setPropertyValue(new EscherRGBProperty(EscherProperties.FILL__FILLCOLOR, fillColor));
    }

    public void setFillColor(int red, int green, int blue) {
        setPropertyValue(new EscherRGBProperty(EscherProperties.FILL__FILLCOLOR, (blue << 16) | (green << 8) | red));
    }

    public int getLineWidth() {
        EscherSimpleProperty property = (EscherSimpleProperty) this._optRecord.lookup(459);
        if (property == null) {
            return 9525;
        }
        return property.getPropertyValue();
    }

    public void setLineWidth(int lineWidth) {
        setPropertyValue(new EscherSimpleProperty(EscherProperties.LINESTYLE__LINEWIDTH, lineWidth));
    }

    public int getLineStyle() {
        EscherSimpleProperty property = (EscherSimpleProperty) this._optRecord.lookup(462);
        if (property == null) {
            return -1;
        }
        return property.getPropertyValue();
    }

    public void setLineStyle(int lineStyle) {
        setPropertyValue(new EscherSimpleProperty(EscherProperties.LINESTYLE__LINEDASHING, lineStyle));
        if (getLineStyle() != 0) {
            setPropertyValue(new EscherSimpleProperty(EscherProperties.LINESTYLE__LINEENDCAPSTYLE, 0));
            if (getLineStyle() == -1) {
                setPropertyValue(new EscherBoolProperty(EscherProperties.LINESTYLE__NOLINEDRAWDASH, 524288));
            } else {
                setPropertyValue(new EscherBoolProperty(EscherProperties.LINESTYLE__NOLINEDRAWDASH, 524296));
            }
        }
    }

    public boolean isNoFill() {
        EscherBoolProperty property = (EscherBoolProperty) this._optRecord.lookup(447);
        return property == null || property.getPropertyValue() == 1114112;
    }

    public void setNoFill(boolean noFill) {
        setPropertyValue(new EscherBoolProperty(EscherProperties.FILL__NOFILLHITTEST, noFill ? NO_FILLHITTEST_TRUE : 65536));
    }

    /* access modifiers changed from: protected */
    public void setPropertyValue(EscherProperty property) {
        this._optRecord.setEscherProperty(property);
    }

    public void setFlipVertical(boolean value) {
        EscherSpRecord sp = (EscherSpRecord) getEscherContainer().getChildById(EscherSpRecord.RECORD_ID);
        if (value) {
            sp.setFlags(sp.getFlags() | 128);
        } else {
            sp.setFlags(sp.getFlags() & 2147483519);
        }
    }

    public void setFlipHorizontal(boolean value) {
        EscherSpRecord sp = (EscherSpRecord) getEscherContainer().getChildById(EscherSpRecord.RECORD_ID);
        if (value) {
            sp.setFlags(sp.getFlags() | 64);
        } else {
            sp.setFlags(sp.getFlags() & 2147483583);
        }
    }

    public boolean isFlipVertical() {
        return (((EscherSpRecord) getEscherContainer().getChildById(EscherSpRecord.RECORD_ID)).getFlags() & 128) != 0;
    }

    public boolean isFlipHorizontal() {
        return (((EscherSpRecord) getEscherContainer().getChildById(EscherSpRecord.RECORD_ID)).getFlags() & 64) != 0;
    }

    public int getRotationDegree() {
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        EscherSimpleProperty property = (EscherSimpleProperty) getOptRecord().lookup(4);
        if (property == null) {
            return 0;
        }
        try {
            LittleEndian.putInt(property.getPropertyValue(), bos);
            return LittleEndian.getShort(bos.toByteArray(), 2);
        } catch (IOException e) {
            LOG.log(7, "can't determine rotation degree", e);
            return 0;
        }
    }

    public void setRotationDegree(short value) {
        setPropertyValue(new EscherSimpleProperty(4, value << 16));
    }

    public int countOfAllChildren() {
        return 1;
    }

    /* access modifiers changed from: protected */
    public void setPatriarch(HSSFPatriarch _patriarch2) {
        this._patriarch = _patriarch2;
    }

    public HSSFPatriarch getPatriarch() {
        return this._patriarch;
    }

    /* access modifiers changed from: protected */
    public void setParent(HSSFShape parent2) {
        this.parent = parent2;
    }
}
