package org.apache.poi.hssf.usermodel;

import com.sec.factory.app.factorytest.FactoryTestManager;
import com.sec.factory.modules.FactoryTestPhone;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.regex.Pattern;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.POIDocument;
import org.apache.poi.ddf.EscherBSERecord;
import org.apache.poi.ddf.EscherBitmapBlip;
import org.apache.poi.ddf.EscherBlipRecord;
import org.apache.poi.ddf.EscherMetafileBlip;
import org.apache.poi.ddf.EscherRecord;
import org.apache.poi.hpsf.ClassID;
import org.apache.poi.hssf.OldExcelFormatException;
import org.apache.poi.hssf.model.HSSFFormulaParser;
import org.apache.poi.hssf.model.InternalSheet;
import org.apache.poi.hssf.model.InternalWorkbook;
import org.apache.poi.hssf.model.RecordStream;
import org.apache.poi.hssf.record.AbstractEscherHolderRecord;
import org.apache.poi.hssf.record.DrawingGroupRecord;
import org.apache.poi.hssf.record.LabelRecord;
import org.apache.poi.hssf.record.LabelSSTRecord;
import org.apache.poi.hssf.record.NameRecord;
import org.apache.poi.hssf.record.RecalcIdRecord;
import org.apache.poi.hssf.record.Record;
import org.apache.poi.hssf.record.RecordFactory;
import org.apache.poi.hssf.record.UnknownRecord;
import org.apache.poi.hssf.record.aggregates.RecordAggregate;
import org.apache.poi.hssf.record.common.UnicodeString;
import org.apache.poi.hssf.util.CellReference;
import org.apache.poi.poifs.crypt.Decryptor;
import org.apache.poi.poifs.filesystem.DirectoryEntry;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.DocumentNode;
import org.apache.poi.poifs.filesystem.EntryUtils;
import org.apache.poi.poifs.filesystem.FilteringDirectoryNode;
import org.apache.poi.poifs.filesystem.NPOIFSDocument;
import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
import org.apache.poi.poifs.filesystem.Ole10Native;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.SpreadsheetVersion;
import org.apache.poi.ss.formula.FormulaShifter;
import org.apache.poi.ss.formula.FormulaType;
import org.apache.poi.ss.formula.SheetNameFormatter;
import org.apache.poi.ss.formula.udf.AggregatingUDFFinder;
import org.apache.poi.ss.formula.udf.IndexedUDFFinder;
import org.apache.poi.ss.formula.udf.UDFFinder;
import org.apache.poi.ss.usermodel.Name;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.WorkbookUtil;
import org.apache.poi.util.Configurator;
import org.apache.poi.util.HexDump;
import org.apache.poi.util.Internal;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public final class HSSFWorkbook extends POIDocument implements Workbook {
    private static final Pattern COMMA_PATTERN = Pattern.compile(",");
    private static final int DEBUG = 1;
    public static final int INITIAL_CAPACITY = Configurator.getIntValue("HSSFWorkbook.SheetInitialCapacity", 3);
    private static final int MAX_STYLES = 4030;
    private static POILogger log = POILogFactory.getLogger(HSSFWorkbook.class);
    protected List<HSSFSheet> _sheets;
    private UDFFinder _udfFinder;
    private Map<Short, HSSFFont> fonts;
    private HSSFDataFormat formatter;
    private Row.MissingCellPolicy missingCellPolicy;
    private ArrayList<HSSFName> names;
    private boolean preserveNodes;
    private InternalWorkbook workbook;

    public static HSSFWorkbook create(InternalWorkbook book) {
        return new HSSFWorkbook(book);
    }

    public HSSFWorkbook() {
        this(InternalWorkbook.createWorkbook());
    }

    private HSSFWorkbook(InternalWorkbook book) {
        super((DirectoryNode) null);
        this.missingCellPolicy = Row.MissingCellPolicy.RETURN_NULL_AND_BLANK;
        this._udfFinder = new IndexedUDFFinder(AggregatingUDFFinder.DEFAULT);
        this.workbook = book;
        int i = INITIAL_CAPACITY;
        this._sheets = new ArrayList(i);
        this.names = new ArrayList<>(i);
    }

    public HSSFWorkbook(POIFSFileSystem fs) throws IOException {
        this(fs, true);
    }

    public HSSFWorkbook(NPOIFSFileSystem fs) throws IOException {
        this(fs.getRoot(), true);
    }

    public HSSFWorkbook(POIFSFileSystem fs, boolean preserveNodes2) throws IOException {
        this(fs.getRoot(), fs, preserveNodes2);
    }

    public static String getWorkbookDirEntryName(DirectoryNode directory) {
        String[] arr$ = InternalWorkbook.WORKBOOK_DIR_ENTRY_NAMES;
        for (String wbName : arr$) {
            try {
                directory.getEntry(wbName);
                return wbName;
            } catch (FileNotFoundException e) {
            }
        }
        try {
            directory.getEntry(Decryptor.DEFAULT_POIFS_ENTRY);
            throw new EncryptedDocumentException("The supplied spreadsheet seems to be an Encrypted .xlsx file. It must be decrypted before use by XSSF, it cannot be used by HSSF");
        } catch (FileNotFoundException e2) {
            directory.getEntry(InternalWorkbook.OLD_WORKBOOK_DIR_ENTRY_NAME);
            throw new OldExcelFormatException("The supplied spreadsheet seems to be Excel 5.0/7.0 (BIFF5) format. POI only supports BIFF8 format (from Excel versions 97/2000/XP/2003)");
        } catch (FileNotFoundException e3) {
            throw new IllegalArgumentException("The supplied POIFSFileSystem does not contain a BIFF8 'Workbook' entry. Is it really an excel file?");
        }
    }

    public HSSFWorkbook(DirectoryNode directory, POIFSFileSystem fs, boolean preserveNodes2) throws IOException {
        this(directory, preserveNodes2);
    }

    public HSSFWorkbook(DirectoryNode directory, boolean preserveNodes2) throws IOException {
        super(directory);
        this.missingCellPolicy = Row.MissingCellPolicy.RETURN_NULL_AND_BLANK;
        this._udfFinder = new IndexedUDFFinder(AggregatingUDFFinder.DEFAULT);
        String workbookName = getWorkbookDirEntryName(directory);
        this.preserveNodes = preserveNodes2;
        if (!preserveNodes2) {
            this.directory = null;
        }
        int i = INITIAL_CAPACITY;
        this._sheets = new ArrayList(i);
        this.names = new ArrayList<>(i);
        List<Record> records = RecordFactory.createRecords(directory.createDocumentInputStream(workbookName));
        InternalWorkbook createWorkbook = InternalWorkbook.createWorkbook(records);
        this.workbook = createWorkbook;
        setPropertiesFromWorkbook(createWorkbook);
        int recOffset = this.workbook.getNumRecords();
        convertLabelRecords(records, recOffset);
        RecordStream rs = new RecordStream(records, recOffset);
        while (rs.hasNext()) {
            try {
                this._sheets.add(new HSSFSheet(this, InternalSheet.createSheet(rs)));
            } catch (InternalSheet.UnsupportedBOFType eb) {
                POILogger pOILogger = log;
                pOILogger.log(5, "Unsupported BOF found of type " + eb.getType());
            }
        }
        for (int i2 = 0; i2 < this.workbook.getNumNames(); i2++) {
            NameRecord nameRecord = this.workbook.getNameRecord(i2);
            this.names.add(new HSSFName(this, nameRecord, this.workbook.getNameCommentRecord(nameRecord)));
        }
    }

    public HSSFWorkbook(InputStream s) throws IOException {
        this(s, true);
    }

    public HSSFWorkbook(InputStream s, boolean preserveNodes2) throws IOException {
        this(new NPOIFSFileSystem(s).getRoot(), preserveNodes2);
    }

    private void setPropertiesFromWorkbook(InternalWorkbook book) {
        this.workbook = book;
    }

    private void convertLabelRecords(List<Record> records, int offset) {
        if (log.check(1)) {
            log.log(1, "convertLabelRecords called");
        }
        for (int k = offset; k < records.size(); k++) {
            Record rec = records.get(k);
            if (rec.getSid() == 516) {
                LabelRecord oldrec = (LabelRecord) rec;
                records.remove(k);
                LabelSSTRecord newrec = new LabelSSTRecord();
                int stringid = this.workbook.addSSTString(new UnicodeString(oldrec.getValue()));
                newrec.setRow(oldrec.getRow());
                newrec.setColumn(oldrec.getColumn());
                newrec.setXFIndex(oldrec.getXFIndex());
                newrec.setSSTIndex(stringid);
                records.add(k, newrec);
            }
        }
        if (log.check(1)) {
            log.log(1, "convertLabelRecords exit");
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public Row.MissingCellPolicy getMissingCellPolicy() {
        return this.missingCellPolicy;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setMissingCellPolicy(Row.MissingCellPolicy missingCellPolicy2) {
        this.missingCellPolicy = missingCellPolicy2;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setSheetOrder(String sheetname, int pos) {
        int oldSheetIndex = getSheetIndex(sheetname);
        List<HSSFSheet> list = this._sheets;
        list.add(pos, list.remove(oldSheetIndex));
        this.workbook.setSheetOrder(sheetname, pos);
        FormulaShifter shifter = FormulaShifter.createForSheetShift(oldSheetIndex, pos);
        for (HSSFSheet sheet : this._sheets) {
            sheet.getSheet().updateFormulasAfterCellShift(shifter, -1);
        }
        this.workbook.updateNamesAfterCellShift(shifter);
        int active = getActiveSheetIndex();
        if (active == oldSheetIndex) {
            setActiveSheet(pos);
        } else if (active >= oldSheetIndex || active >= pos) {
            if (active > oldSheetIndex && active > pos) {
                return;
            }
            if (pos > oldSheetIndex) {
                setActiveSheet(active - 1);
            } else {
                setActiveSheet(active + 1);
            }
        }
    }

    private void validateSheetIndex(int index) {
        int lastSheetIx = this._sheets.size() - 1;
        if (index < 0 || index > lastSheetIx) {
            String range = "(0.." + lastSheetIx + ")";
            if (lastSheetIx == -1) {
                range = "(no sheets)";
            }
            throw new IllegalArgumentException("Sheet index (" + index + ") is out of range " + range);
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setSelectedTab(int index) {
        validateSheetIndex(index);
        int nSheets = this._sheets.size();
        int i = 0;
        while (true) {
            boolean z = true;
            if (i < nSheets) {
                HSSFSheet sheetAt = getSheetAt(i);
                if (i != index) {
                    z = false;
                }
                sheetAt.setSelected(z);
                i++;
            } else {
                this.workbook.getWindowOne().setNumSelectedTabs(1);
                return;
            }
        }
    }

    public void setSelectedTabs(int[] indexes) {
        Collection<Integer> list = new ArrayList<>(indexes.length);
        for (int index : indexes) {
            list.add(Integer.valueOf(index));
        }
        setSelectedTabs(list);
    }

    public void setSelectedTabs(Collection<Integer> indexes) {
        for (Integer num : indexes) {
            validateSheetIndex(num.intValue());
        }
        Set<Integer> set = new HashSet<>(indexes);
        int nSheets = this._sheets.size();
        for (int i = 0; i < nSheets; i++) {
            getSheetAt(i).setSelected(set.contains(Integer.valueOf(i)));
        }
        this.workbook.getWindowOne().setNumSelectedTabs((short) set.size());
    }

    public Collection<Integer> getSelectedTabs() {
        Collection<Integer> indexes = new ArrayList<>();
        int nSheets = this._sheets.size();
        for (int i = 0; i < nSheets; i++) {
            if (getSheetAt(i).isSelected()) {
                indexes.add(Integer.valueOf(i));
            }
        }
        return Collections.unmodifiableCollection(indexes);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setActiveSheet(int index) {
        validateSheetIndex(index);
        int nSheets = this._sheets.size();
        int i = 0;
        while (i < nSheets) {
            getSheetAt(i).setActive(i == index);
            i++;
        }
        this.workbook.getWindowOne().setActiveSheetIndex(index);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getActiveSheetIndex() {
        return this.workbook.getWindowOne().getActiveSheetIndex();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setFirstVisibleTab(int index) {
        this.workbook.getWindowOne().setFirstVisibleTab(index);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getFirstVisibleTab() {
        return this.workbook.getWindowOne().getFirstVisibleTab();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setSheetName(int sheetIx, String name) {
        if (name == null) {
            throw new IllegalArgumentException("sheetName must not be null");
        } else if (!this.workbook.doesContainsSheetName(name, sheetIx)) {
            validateSheetIndex(sheetIx);
            this.workbook.setSheetName(sheetIx, name);
        } else {
            throw new IllegalArgumentException("The workbook already contains a sheet named '" + name + "'");
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public String getSheetName(int sheetIndex) {
        validateSheetIndex(sheetIndex);
        return this.workbook.getSheetName(sheetIndex);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public boolean isHidden() {
        return this.workbook.getWindowOne().getHidden();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setHidden(boolean hiddenFlag) {
        this.workbook.getWindowOne().setHidden(hiddenFlag);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public boolean isSheetHidden(int sheetIx) {
        validateSheetIndex(sheetIx);
        return this.workbook.isSheetHidden(sheetIx);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public boolean isSheetVeryHidden(int sheetIx) {
        validateSheetIndex(sheetIx);
        return this.workbook.isSheetVeryHidden(sheetIx);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setSheetHidden(int sheetIx, boolean hidden) {
        validateSheetIndex(sheetIx);
        this.workbook.setSheetHidden(sheetIx, hidden);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setSheetHidden(int sheetIx, int hidden) {
        validateSheetIndex(sheetIx);
        WorkbookUtil.validateSheetState(hidden);
        this.workbook.setSheetHidden(sheetIx, hidden);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getSheetIndex(String name) {
        return this.workbook.getSheetIndex(name);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getSheetIndex(Sheet sheet) {
        for (int i = 0; i < this._sheets.size(); i++) {
            if (this._sheets.get(i) == sheet) {
                return i;
            }
        }
        return -1;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFSheet createSheet() {
        HSSFSheet sheet = new HSSFSheet(this);
        this._sheets.add(sheet);
        boolean isOnlySheet = true;
        this.workbook.setSheetName(this._sheets.size() - 1, "Sheet" + (this._sheets.size() - 1));
        if (this._sheets.size() != 1) {
            isOnlySheet = false;
        }
        sheet.setSelected(isOnlySheet);
        sheet.setActive(isOnlySheet);
        return sheet;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFSheet cloneSheet(int sheetIndex) {
        validateSheetIndex(sheetIndex);
        String srcName = this.workbook.getSheetName(sheetIndex);
        HSSFSheet clonedSheet = this._sheets.get(sheetIndex).cloneSheet(this);
        clonedSheet.setSelected(false);
        clonedSheet.setActive(false);
        String name = getUniqueSheetName(srcName);
        int newSheetIndex = this._sheets.size();
        this._sheets.add(clonedSheet);
        this.workbook.setSheetName(newSheetIndex, name);
        int filterDbNameIndex = findExistingBuiltinNameRecordIdx(sheetIndex, (byte) 13);
        if (filterDbNameIndex != -1) {
            this.names.add(new HSSFName(this, this.workbook.cloneFilter(filterDbNameIndex, newSheetIndex)));
        }
        return clonedSheet;
    }

    /* JADX INFO: Multiple debug info for r5v0 int: [D('uniqueIndex' int), D('suffix' java.lang.String)] */
    private String getUniqueSheetName(String srcName) {
        String name;
        int uniqueIndex = 2;
        String baseName = srcName;
        int bracketPos = srcName.lastIndexOf(40);
        if (bracketPos > 0 && srcName.endsWith(")")) {
            try {
                uniqueIndex = Integer.parseInt(srcName.substring(bracketPos + 1, srcName.length() - ")".length()).trim()) + 1;
                baseName = srcName.substring(0, bracketPos).trim();
            } catch (NumberFormatException e) {
            }
        }
        while (true) {
            int uniqueIndex2 = uniqueIndex + 1;
            String index = Integer.toString(uniqueIndex);
            if (baseName.length() + index.length() + 2 < 31) {
                name = baseName + " (" + index + ")";
            } else {
                name = baseName.substring(0, (31 - index.length()) - 2) + "(" + index + ")";
            }
            if (this.workbook.getSheetIndex(name) == -1) {
                return name;
            }
            uniqueIndex = uniqueIndex2;
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFSheet createSheet(String sheetname) {
        if (sheetname == null) {
            throw new IllegalArgumentException("sheetName must not be null");
        } else if (!this.workbook.doesContainsSheetName(sheetname, this._sheets.size())) {
            HSSFSheet sheet = new HSSFSheet(this);
            this.workbook.setSheetName(this._sheets.size(), sheetname);
            this._sheets.add(sheet);
            boolean isOnlySheet = true;
            if (this._sheets.size() != 1) {
                isOnlySheet = false;
            }
            sheet.setSelected(isOnlySheet);
            sheet.setActive(isOnlySheet);
            return sheet;
        } else {
            throw new IllegalArgumentException("The workbook already contains a sheet named '" + sheetname + "'");
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public Iterator<Sheet> sheetIterator() {
        return new SheetIterator();
    }

    @Override // java.lang.Iterable
    public Iterator<Sheet> iterator() {
        return sheetIterator();
    }

    /* access modifiers changed from: private */
    public final class SheetIterator<T extends Sheet> implements Iterator<T> {
        private T cursor = null;
        private final Iterator<T> it;

        public SheetIterator() {
            this.it = HSSFWorkbook.this._sheets.iterator();
        }

        public boolean hasNext() {
            return this.it.hasNext();
        }

        @Override // java.util.Iterator
        public T next() throws NoSuchElementException {
            T next = this.it.next();
            this.cursor = next;
            return next;
        }

        public void remove() throws IllegalStateException {
            throw new UnsupportedOperationException("remove method not supported on HSSFWorkbook.iterator(). Use Sheet.removeSheetAt(int) instead.");
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getNumberOfSheets() {
        return this._sheets.size();
    }

    private HSSFSheet[] getSheets() {
        HSSFSheet[] result = new HSSFSheet[this._sheets.size()];
        this._sheets.toArray(result);
        return result;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFSheet getSheetAt(int index) {
        validateSheetIndex(index);
        return this._sheets.get(index);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFSheet getSheet(String name) {
        HSSFSheet retval = null;
        for (int k = 0; k < this._sheets.size(); k++) {
            if (this.workbook.getSheetName(k).equalsIgnoreCase(name)) {
                retval = this._sheets.get(k);
            }
        }
        return retval;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void removeSheetAt(int index) {
        validateSheetIndex(index);
        boolean wasSelected = getSheetAt(index).isSelected();
        this._sheets.remove(index);
        this.workbook.removeSheet(index);
        int nSheets = this._sheets.size();
        if (nSheets >= 1) {
            int newSheetIndex = index;
            if (newSheetIndex >= nSheets) {
                newSheetIndex = nSheets - 1;
            }
            if (wasSelected) {
                boolean someOtherSheetIsStillSelected = false;
                int i = 0;
                while (true) {
                    if (i >= nSheets) {
                        break;
                    } else if (getSheetAt(i).isSelected()) {
                        someOtherSheetIsStillSelected = true;
                        break;
                    } else {
                        i++;
                    }
                }
                if (!someOtherSheetIsStillSelected) {
                    setSelectedTab(newSheetIndex);
                }
            }
            int active = getActiveSheetIndex();
            if (active == index) {
                setActiveSheet(newSheetIndex);
            } else if (active > index) {
                setActiveSheet(active - 1);
            }
        }
    }

    public void setBackupFlag(boolean backupValue) {
        this.workbook.getBackupRecord().setBackup(backupValue ? (short) 1 : 0);
    }

    public boolean getBackupFlag() {
        return this.workbook.getBackupRecord().getBackup() != 0;
    }

    /* access modifiers changed from: package-private */
    public int findExistingBuiltinNameRecordIdx(int sheetIndex, byte builtinCode) {
        for (int defNameIndex = 0; defNameIndex < this.names.size(); defNameIndex++) {
            NameRecord r = this.workbook.getNameRecord(defNameIndex);
            if (r == null) {
                throw new RuntimeException("Unable to find all defined names to iterate over");
            } else if (r.isBuiltInName() && r.getBuiltInName() == builtinCode && r.getSheetNumber() - 1 == sheetIndex) {
                return defNameIndex;
            }
        }
        return -1;
    }

    /* access modifiers changed from: package-private */
    public HSSFName createBuiltInName(byte builtinCode, int sheetIndex) {
        HSSFName newName = new HSSFName(this, this.workbook.createBuiltInName(builtinCode, sheetIndex + 1), null);
        this.names.add(newName);
        return newName;
    }

    /* access modifiers changed from: package-private */
    public HSSFName getBuiltInName(byte builtinCode, int sheetIndex) {
        int index = findExistingBuiltinNameRecordIdx(sheetIndex, builtinCode);
        if (index < 0) {
            return null;
        }
        return this.names.get(index);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFFont createFont() {
        this.workbook.createNewFont();
        short fontindex = (short) (getNumberOfFonts() - 1);
        if (fontindex > 3) {
            fontindex = (short) (fontindex + 1);
        }
        if (fontindex != Short.MAX_VALUE) {
            return getFontAt(fontindex);
        }
        throw new IllegalArgumentException("Maximum number of fonts was exceeded");
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    @Deprecated
    public HSSFFont findFont(short boldWeight, short color, short fontHeight, String name, boolean italic, boolean strikeout, short typeOffset, byte underline) {
        short numberOfFonts = getNumberOfFonts();
        for (short i = 0; i <= numberOfFonts; i = (short) (i + 1)) {
            if (i != 4) {
                HSSFFont hssfFont = getFontAt(i);
                if (hssfFont.getBoldweight() == boldWeight && hssfFont.getColor() == color && hssfFont.getFontHeight() == fontHeight && hssfFont.getFontName().equals(name) && hssfFont.getItalic() == italic && hssfFont.getStrikeout() == strikeout && hssfFont.getTypeOffset() == typeOffset && hssfFont.getUnderline() == underline) {
                    return hssfFont;
                }
            }
        }
        return null;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFFont findFont(boolean bold, short color, short fontHeight, String name, boolean italic, boolean strikeout, short typeOffset, byte underline) {
        short numberOfFonts = getNumberOfFonts();
        for (short i = 0; i <= numberOfFonts; i = (short) (i + 1)) {
            if (i != 4) {
                HSSFFont hssfFont = getFontAt(i);
                if (hssfFont.getBold() == bold && hssfFont.getColor() == color && hssfFont.getFontHeight() == fontHeight && hssfFont.getFontName().equals(name) && hssfFont.getItalic() == italic && hssfFont.getStrikeout() == strikeout && hssfFont.getTypeOffset() == typeOffset && hssfFont.getUnderline() == underline) {
                    return hssfFont;
                }
            }
        }
        return null;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public short getNumberOfFonts() {
        return (short) this.workbook.getNumberOfFontRecords();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFFont getFontAt(short idx) {
        if (this.fonts == null) {
            this.fonts = new HashMap();
        }
        Short sIdx = Short.valueOf(idx);
        if (this.fonts.containsKey(sIdx)) {
            return this.fonts.get(sIdx);
        }
        HSSFFont retval = new HSSFFont(idx, this.workbook.getFontRecordAt(idx));
        this.fonts.put(sIdx, retval);
        return retval;
    }

    /* access modifiers changed from: protected */
    public void resetFontCache() {
        this.fonts = new HashMap();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFCellStyle createCellStyle() {
        if (this.workbook.getNumExFormats() != MAX_STYLES) {
            return new HSSFCellStyle((short) (getNumCellStyles() - 1), this.workbook.createCellXF(), this);
        }
        throw new IllegalStateException("The maximum number of cell styles was exceeded. You can define up to 4000 styles in a .xls workbook");
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getNumCellStyles() {
        return this.workbook.getNumExFormats();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFCellStyle getCellStyleAt(int idx) {
        return new HSSFCellStyle((short) idx, this.workbook.getExFormatAt(idx), this);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook, java.io.Closeable, org.apache.poi.POIDocument, java.lang.AutoCloseable
    public void close() throws IOException {
        super.close();
    }

    @Override // org.apache.poi.POIDocument
    public void write() throws IOException {
        validateInPlaceWritePossible();
        new NPOIFSDocument((DocumentNode) this.directory.getEntry(getWorkbookDirEntryName(this.directory))).replaceContents(new ByteArrayInputStream(getBytes()));
        writeProperties();
        this.directory.getFileSystem().writeFilesystem();
    }

    @Override // org.apache.poi.POIDocument
    public void write(File newFile) throws IOException {
        POIFSFileSystem fs = POIFSFileSystem.create(newFile);
        try {
            write(fs);
            fs.writeFilesystem();
        } finally {
            fs.close();
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook, org.apache.poi.POIDocument
    public void write(OutputStream stream) throws IOException {
        NPOIFSFileSystem fs = new NPOIFSFileSystem();
        try {
            write(fs);
            fs.writeFilesystem(stream);
        } finally {
            fs.close();
        }
    }

    private void write(NPOIFSFileSystem fs) throws IOException {
        List<String> excepts = new ArrayList<>(1);
        fs.createDocument(new ByteArrayInputStream(getBytes()), "Workbook");
        writeProperties(fs, excepts);
        if (this.preserveNodes) {
            excepts.addAll(Arrays.asList(InternalWorkbook.WORKBOOK_DIR_ENTRY_NAMES));
            EntryUtils.copyNodes(new FilteringDirectoryNode(this.directory, excepts), new FilteringDirectoryNode(fs.getRoot(), excepts));
            fs.getRoot().setStorageClsid(this.directory.getStorageClsid());
        }
    }

    /* access modifiers changed from: private */
    public static final class SheetRecordCollector implements RecordAggregate.RecordVisitor {
        private List<Record> _list = new ArrayList(128);
        private int _totalSize = 0;

        public int getTotalSize() {
            return this._totalSize;
        }

        @Override // org.apache.poi.hssf.record.aggregates.RecordAggregate.RecordVisitor
        public void visitRecord(Record r) {
            this._list.add(r);
            this._totalSize += r.getRecordSize();
        }

        public int serialize(int offset, byte[] data) {
            int result = 0;
            for (Record rec : this._list) {
                result += rec.serialize(offset + result, data);
            }
            return result;
        }
    }

    /* JADX INFO: Multiple debug info for r5v4 byte[]: [D('retval' byte[]), D('k' int)] */
    public byte[] getBytes() {
        if (log.check(1)) {
            log.log(1, "HSSFWorkbook.getBytes()");
        }
        HSSFSheet[] sheets = getSheets();
        int nSheets = sheets.length;
        this.workbook.preSerialize();
        for (HSSFSheet sheet : sheets) {
            sheet.getSheet().preSerialize();
            sheet.preSerialize();
        }
        int totalsize = this.workbook.getSize();
        SheetRecordCollector[] srCollectors = new SheetRecordCollector[nSheets];
        for (int k = 0; k < nSheets; k++) {
            this.workbook.setSheetBof(k, totalsize);
            SheetRecordCollector src = new SheetRecordCollector();
            sheets[k].getSheet().visitContainedRecords(src, totalsize);
            totalsize += src.getTotalSize();
            srCollectors[k] = src;
        }
        byte[] retval = new byte[totalsize];
        int pos = this.workbook.serialize(0, retval);
        for (int k2 = 0; k2 < nSheets; k2++) {
            SheetRecordCollector src2 = srCollectors[k2];
            int serializedSize = src2.serialize(pos, retval);
            if (serializedSize == src2.getTotalSize()) {
                pos += serializedSize;
            } else {
                throw new IllegalStateException("Actual serialized sheet size (" + serializedSize + ") differs from pre-calculated size (" + src2.getTotalSize() + ") for sheet (" + k2 + ")");
            }
        }
        return retval;
    }

    /* access modifiers changed from: package-private */
    public InternalWorkbook getWorkbook() {
        return this.workbook;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getNumberOfNames() {
        return this.names.size();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFName getName(String name) {
        int nameIndex = getNameIndex(name);
        if (nameIndex < 0) {
            return null;
        }
        return this.names.get(nameIndex);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public List<HSSFName> getNames(String name) {
        List<HSSFName> nameList = new ArrayList<>();
        Iterator i$ = this.names.iterator();
        while (i$.hasNext()) {
            HSSFName nr = i$.next();
            if (nr.getNameName().equals(name)) {
                nameList.add(nr);
            }
        }
        return Collections.unmodifiableList(nameList);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFName getNameAt(int nameIndex) {
        int nNames = this.names.size();
        if (nNames < 1) {
            throw new IllegalStateException("There are no defined names in this workbook");
        } else if (nameIndex >= 0 && nameIndex <= nNames) {
            return this.names.get(nameIndex);
        } else {
            StringBuilder sb = new StringBuilder();
            sb.append("Specified name index ");
            sb.append(nameIndex);
            sb.append(" is outside the allowable range (0..");
            sb.append(nNames - 1);
            sb.append(").");
            throw new IllegalArgumentException(sb.toString());
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public List<HSSFName> getAllNames() {
        return Collections.unmodifiableList(this.names);
    }

    public NameRecord getNameRecord(int nameIndex) {
        return getWorkbook().getNameRecord(nameIndex);
    }

    public String getNameName(int index) {
        return getNameAt(index).getNameName();
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setPrintArea(int sheetIndex, String reference) {
        NameRecord name = this.workbook.getSpecificBuiltinRecord((byte) 6, sheetIndex + 1);
        if (name == null) {
            name = this.workbook.createBuiltInName((byte) 6, sheetIndex + 1);
        }
        String[] parts = COMMA_PATTERN.split(reference);
        StringBuffer sb = new StringBuffer(32);
        for (int i = 0; i < parts.length; i++) {
            if (i > 0) {
                sb.append(",");
            }
            SheetNameFormatter.appendFormat(sb, getSheetName(sheetIndex));
            sb.append("!");
            sb.append(parts[i]);
        }
        name.setNameDefinition(HSSFFormulaParser.parse(sb.toString(), this, FormulaType.NAMEDRANGE, sheetIndex));
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setPrintArea(int sheetIndex, int startColumn, int endColumn, int startRow, int endRow) {
        String reference = new CellReference(startRow, startColumn, true, true).formatAsString();
        CellReference cell = new CellReference(endRow, endColumn, true, true);
        setPrintArea(sheetIndex, reference + ":" + cell.formatAsString());
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public String getPrintArea(int sheetIndex) {
        NameRecord name = this.workbook.getSpecificBuiltinRecord((byte) 6, sheetIndex + 1);
        if (name == null) {
            return null;
        }
        return HSSFFormulaParser.toFormulaString(this, name.getNameDefinition());
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void removePrintArea(int sheetIndex) {
        getWorkbook().removeBuiltinRecord((byte) 6, sheetIndex + 1);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFName createName() {
        HSSFName newName = new HSSFName(this, this.workbook.createName());
        this.names.add(newName);
        return newName;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int getNameIndex(String name) {
        for (int k = 0; k < this.names.size(); k++) {
            if (getNameName(k).equalsIgnoreCase(name)) {
                return k;
            }
        }
        return -1;
    }

    /* access modifiers changed from: package-private */
    public int getNameIndex(HSSFName name) {
        for (int k = 0; k < this.names.size(); k++) {
            if (name == this.names.get(k)) {
                return k;
            }
        }
        return -1;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void removeName(int index) {
        this.names.remove(index);
        this.workbook.removeName(index);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFDataFormat createDataFormat() {
        if (this.formatter == null) {
            this.formatter = new HSSFDataFormat(this.workbook);
        }
        return this.formatter;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void removeName(String name) {
        removeName(getNameIndex(name));
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void removeName(Name name) {
        removeName(getNameIndex((HSSFName) name));
    }

    public HSSFPalette getCustomPalette() {
        return new HSSFPalette(this.workbook.getCustomPalette());
    }

    public void insertChartRecord() {
        this.workbook.getRecords().add(this.workbook.findFirstRecordLocBySid(252), new UnknownRecord(235, new byte[]{15, 0, 0, -16, FactoryTestManager.TEST_IDCHECK, 0, 0, 0, 0, 0, 6, -16, 24, 0, 0, 0, 1, 8, 0, 0, 2, 0, 0, 0, 2, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 3, 0, 0, 0, FactoryTestManager.TEST_FM_INTENNA, 0, 11, -16, 18, 0, 0, 0, -65, 0, 8, 0, 8, 0, FactoryTestPhone.OEM_MISC_GET_RF_MIPI_INFO, 1, 9, 0, 0, 8, -64, 1, 64, 0, 0, 8, 64, 0, 30, -15, 16, 0, 0, 0, 13, 0, 0, 8, 12, 0, 0, 8, 23, 0, 0, 8, -9, 0, 0, 16}));
    }

    public void dumpDrawingGroupRecords(boolean fat) {
        DrawingGroupRecord r = (DrawingGroupRecord) this.workbook.findFirstRecordBySid(DrawingGroupRecord.sid);
        r.decode();
        List<EscherRecord> escherRecords = r.getEscherRecords();
        PrintWriter w = new PrintWriter(new OutputStreamWriter(System.out, Charset.defaultCharset()));
        for (EscherRecord escherRecord : escherRecords) {
            if (fat) {
                System.out.println(escherRecord.toString());
            } else {
                escherRecord.display(w, 0);
            }
        }
        w.flush();
    }

    /* access modifiers changed from: package-private */
    public void initDrawings() {
        if (this.workbook.findDrawingGroup() != null) {
            for (HSSFSheet sh : this._sheets) {
                sh.getDrawingPatriarch();
            }
            return;
        }
        this.workbook.createDrawingGroup();
    }

    /* JADX DEBUG: Multi-variable search result rejected for r2v16, resolved type: org.apache.poi.ddf.EscherBitmapBlip */
    /* JADX WARN: Multi-variable type inference failed */
    @Override // org.apache.poi.ss.usermodel.Workbook
    public int addPicture(byte[] pictureData, int format) {
        short escherTag;
        int blipSize;
        EscherBlipRecord blipRecord;
        initDrawings();
        byte[] uid = DigestUtils.md5(pictureData);
        switch (format) {
            case 3:
                if (LittleEndian.getInt(pictureData) == -1698247209) {
                    byte[] picDataNoHeader = new byte[(pictureData.length - 22)];
                    System.arraycopy(pictureData, 22, picDataNoHeader, 0, pictureData.length - 22);
                    pictureData = picDataNoHeader;
                }
            case 2:
                EscherMetafileBlip blipRecordMeta = new EscherMetafileBlip();
                blipRecord = blipRecordMeta;
                blipRecordMeta.setUID(uid);
                blipRecordMeta.setPictureData(pictureData);
                blipRecordMeta.setFilter((byte) -2);
                blipSize = blipRecordMeta.getCompressedSize() + 58;
                escherTag = 0;
                break;
            default:
                EscherBitmapBlip blipRecordBitmap = new EscherBitmapBlip();
                blipRecord = blipRecordBitmap;
                blipRecordBitmap.setUID(uid);
                blipRecordBitmap.setMarker((byte) -1);
                blipRecordBitmap.setPictureData(pictureData);
                blipSize = pictureData.length + 25;
                escherTag = 255;
                break;
        }
        blipRecord.setRecordId((short) (format - 4072));
        switch (format) {
            case 2:
                blipRecord.setOptions(HSSFPictureData.MSOBI_EMF);
                break;
            case 3:
                blipRecord.setOptions(HSSFPictureData.MSOBI_WMF);
                break;
            case 4:
                blipRecord.setOptions(HSSFPictureData.MSOBI_PICT);
                break;
            case 5:
                blipRecord.setOptions(HSSFPictureData.MSOBI_JPEG);
                break;
            case 6:
                blipRecord.setOptions(HSSFPictureData.MSOBI_PNG);
                break;
            case 7:
                blipRecord.setOptions(HSSFPictureData.MSOBI_DIB);
                break;
            default:
                throw new IllegalStateException("Unexpected picture format: " + format);
        }
        EscherBSERecord r = new EscherBSERecord();
        r.setRecordId(EscherBSERecord.RECORD_ID);
        r.setOptions((short) ((format << 4) | 2));
        r.setBlipTypeMacOS((byte) format);
        r.setBlipTypeWin32((byte) format);
        r.setUid(uid);
        r.setTag(escherTag);
        r.setSize(blipSize);
        r.setRef(0);
        r.setOffset(0);
        r.setBlipRecord(blipRecord);
        return this.workbook.addBSERecord(r);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public List<HSSFPictureData> getAllPictures() {
        List<HSSFPictureData> pictures = new ArrayList<>();
        for (Record r : this.workbook.getRecords()) {
            if (r instanceof AbstractEscherHolderRecord) {
                ((AbstractEscherHolderRecord) r).decode();
                searchForPictures(((AbstractEscherHolderRecord) r).getEscherRecords(), pictures);
            }
        }
        return Collections.unmodifiableList(pictures);
    }

    private void searchForPictures(List<EscherRecord> escherRecords, List<HSSFPictureData> pictures) {
        EscherBlipRecord blip;
        for (EscherRecord escherRecord : escherRecords) {
            if ((escherRecord instanceof EscherBSERecord) && (blip = ((EscherBSERecord) escherRecord).getBlipRecord()) != null) {
                pictures.add(new HSSFPictureData(blip));
            }
            searchForPictures(escherRecord.getChildRecords(), pictures);
        }
    }

    protected static Map<String, ClassID> getOleMap() {
        Map<String, ClassID> olemap = new HashMap<>();
        olemap.put("PowerPoint Document", ClassID.PPT_SHOW);
        for (String str : InternalWorkbook.WORKBOOK_DIR_ENTRY_NAMES) {
            olemap.put(str, ClassID.XLS_WORKBOOK);
        }
        return olemap;
    }

    public int addOlePackage(POIFSFileSystem poiData, String label, String fileName, String command) throws IOException {
        DirectoryNode root = poiData.getRoot();
        Iterator i$ = getOleMap().entrySet().iterator();
        while (true) {
            if (!i$.hasNext()) {
                break;
            }
            Map.Entry<String, ClassID> entry = i$.next();
            if (root.hasEntry(entry.getKey())) {
                root.setStorageClsid(entry.getValue());
                break;
            }
        }
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        poiData.writeFilesystem(bos);
        return addOlePackage(bos.toByteArray(), label, fileName, command);
    }

    public int addOlePackage(byte[] oleData, String label, String fileName, String command) throws IOException {
        if (this.directory == null) {
            this.directory = new NPOIFSFileSystem().getRoot();
            this.preserveNodes = true;
        }
        int storageId = 0;
        DirectoryEntry oleDir = null;
        do {
            StringBuilder sb = new StringBuilder();
            sb.append("MBD");
            storageId++;
            sb.append(HexDump.toHex(storageId));
            String storageStr = sb.toString();
            if (!this.directory.hasEntry(storageStr)) {
                oleDir = this.directory.createDirectory(storageStr);
                oleDir.setStorageClsid(ClassID.OLE10_PACKAGE);
                continue;
            }
        } while (oleDir == null);
        oleDir.createDocument("\u0001Ole", new ByteArrayInputStream(new byte[]{1, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}));
        Ole10Native oleNative = new Ole10Native(label, fileName, command, oleData);
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        oleNative.writeOut(bos);
        oleDir.createDocument(Ole10Native.OLE10_NATIVE, new ByteArrayInputStream(bos.toByteArray()));
        return storageId;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public int linkExternalWorkbook(String name, Workbook workbook2) {
        return this.workbook.linkExternalWorkbook(name, workbook2);
    }

    public boolean isWriteProtected() {
        return this.workbook.isWriteProtected();
    }

    public void writeProtectWorkbook(String password, String username) {
        this.workbook.writeProtectWorkbook(password, username);
    }

    public void unwriteProtectWorkbook() {
        this.workbook.unwriteProtectWorkbook();
    }

    public List<HSSFObjectData> getAllEmbeddedObjects() {
        List<HSSFObjectData> objects = new ArrayList<>();
        for (HSSFSheet sheet : this._sheets) {
            getAllEmbeddedObjects(sheet, objects);
        }
        return Collections.unmodifiableList(objects);
    }

    private void getAllEmbeddedObjects(HSSFSheet sheet, List<HSSFObjectData> objects) {
        HSSFPatriarch patriarch = sheet.getDrawingPatriarch();
        if (patriarch != null) {
            getAllEmbeddedObjects(patriarch, objects);
        }
    }

    private void getAllEmbeddedObjects(HSSFShapeContainer parent, List<HSSFObjectData> objects) {
        for (HSSFShape shape : parent.getChildren()) {
            if (shape instanceof HSSFObjectData) {
                objects.add((HSSFObjectData) shape);
            } else if (shape instanceof HSSFShapeContainer) {
                getAllEmbeddedObjects((HSSFShapeContainer) shape, objects);
            }
        }
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public HSSFCreationHelper getCreationHelper() {
        return new HSSFCreationHelper(this);
    }

    /* access modifiers changed from: package-private */
    public UDFFinder getUDFFinder() {
        return this._udfFinder;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void addToolPack(UDFFinder toopack) {
        ((AggregatingUDFFinder) this._udfFinder).add(toopack);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public void setForceFormulaRecalculation(boolean value) {
        getWorkbook().getRecalcId().setEngineId(0);
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public boolean getForceFormulaRecalculation() {
        RecalcIdRecord recalc = (RecalcIdRecord) getWorkbook().findFirstRecordBySid(449);
        return (recalc == null || recalc.getEngineId() == 0) ? false : true;
    }

    public boolean changeExternalReference(String oldUrl, String newUrl) {
        return this.workbook.changeExternalReference(oldUrl, newUrl);
    }

    public DirectoryNode getRootDirectory() {
        return this.directory;
    }

    @Internal
    public InternalWorkbook getInternalWorkbook() {
        return this.workbook;
    }

    @Override // org.apache.poi.ss.usermodel.Workbook
    public SpreadsheetVersion getSpreadsheetVersion() {
        return SpreadsheetVersion.EXCEL97;
    }
}
