package org.apache.poi.hssf.usermodel;

import java.awt.Font;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

/* access modifiers changed from: package-private */
public final class StaticFontMetrics {
    private static final POILogger LOGGER = POILogFactory.getLogger(StaticFontMetrics.class);
    private static final Map<String, FontDetails> fontDetailsMap = new HashMap();
    private static Properties fontMetricsProps;

    private StaticFontMetrics() {
    }

    public static synchronized FontDetails getFontDetails(Font font) {
        FontDetails fontDetails;
        synchronized (StaticFontMetrics.class) {
            if (fontMetricsProps == null) {
                try {
                    fontMetricsProps = loadMetrics();
                } catch (IOException e) {
                    throw new RuntimeException("Could not load font metrics", e);
                }
            }
            String fontName = font.getName();
            String fontStyle = "";
            if (font.isPlain()) {
                fontStyle = fontStyle + "plain";
            }
            if (font.isBold()) {
                fontStyle = fontStyle + "bold";
            }
            if (font.isItalic()) {
                fontStyle = fontStyle + "italic";
            }
            String fontHeight = FontDetails.buildFontHeightProperty(fontName);
            String styleHeight = FontDetails.buildFontHeightProperty(fontName + "." + fontStyle);
            if (fontMetricsProps.get(fontHeight) == null && fontMetricsProps.get(styleHeight) != null) {
                fontName = fontName + "." + fontStyle;
            }
            Map<String, FontDetails> map = fontDetailsMap;
            fontDetails = map.get(fontName);
            if (fontDetails == null) {
                fontDetails = FontDetails.create(fontName, fontMetricsProps);
                map.put(fontName, fontDetails);
            }
        }
        return fontDetails;
    }

    private static Properties loadMetrics() throws IOException {
        InputStream metricsIn;
        File propFile = null;
        try {
            String propFileName = System.getProperty("font.metrics.filename");
            if (propFileName != null) {
                propFile = new File(propFileName);
                if (!propFile.exists()) {
                    POILogger pOILogger = LOGGER;
                    pOILogger.log(5, "font_metrics.properties not found at path " + propFile.getAbsolutePath());
                    propFile = null;
                }
            }
        } catch (SecurityException e) {
            LOGGER.log(5, "Can't access font.metrics.filename system property", e);
        }
        InputStream metricsIn2 = null;
        if (propFile != null) {
            try {
                metricsIn = new FileInputStream(propFile);
            } catch (Throwable th) {
                if (0 != 0) {
                    metricsIn2.close();
                }
                throw th;
            }
        } else {
            metricsIn = FontDetails.class.getResourceAsStream("/font_metrics.properties");
            if (metricsIn == null) {
                throw new IOException("font_metrics.properties not found in classpath");
            }
        }
        Properties props = new Properties();
        props.load(metricsIn);
        if (metricsIn != null) {
            metricsIn.close();
        }
        return props;
    }
}
