package org.apache.poi.poifs.crypt;

import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import javax.crypto.Cipher;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.util.Internal;
import org.apache.poi.util.LittleEndianInput;
import org.apache.poi.util.LittleEndianInputStream;

@Internal
public abstract class ChunkedCipherInputStream extends LittleEndianInputStream {
    private byte[] _chunk;
    private Cipher _cipher;
    private int _lastIndex = 0;
    private long _pos = 0;
    private long _size;
    private final int chunkBits;
    private final int chunkMask;
    private final int chunkSize;

    /* access modifiers changed from: protected */
    public abstract Cipher initCipherForBlock(Cipher cipher, int i) throws GeneralSecurityException;

    public ChunkedCipherInputStream(LittleEndianInput stream, long size, int chunkSize2) throws GeneralSecurityException {
        super((InputStream) stream);
        this._size = size;
        this.chunkSize = chunkSize2;
        int i = chunkSize2 - 1;
        this.chunkMask = i;
        this.chunkBits = Integer.bitCount(i);
        this._cipher = initCipherForBlock(null, 0);
    }

    @Override // java.io.FilterInputStream, java.io.InputStream
    public int read() throws IOException {
        byte[] b = new byte[1];
        if (read(b) == 1) {
            return b[0];
        }
        return -1;
    }

    @Override // java.io.FilterInputStream, java.io.InputStream
    public int read(byte[] b, int off, int len) throws IOException {
        int total = 0;
        if (available() <= 0) {
            return -1;
        }
        while (len > 0) {
            if (this._chunk == null) {
                try {
                    this._chunk = nextChunk();
                } catch (GeneralSecurityException e) {
                    throw new EncryptedDocumentException(e.getMessage(), e);
                }
            }
            int count = (int) (((long) this.chunkSize) - (this._pos & ((long) this.chunkMask)));
            int avail = available();
            if (avail == 0) {
                return total;
            }
            int count2 = Math.min(avail, Math.min(count, len));
            System.arraycopy(this._chunk, (int) (this._pos & ((long) this.chunkMask)), b, off, count2);
            off += count2;
            len -= count2;
            long j = this._pos + ((long) count2);
            this._pos = j;
            if ((j & ((long) this.chunkMask)) == 0) {
                this._chunk = null;
            }
            total += count2;
        }
        return total;
    }

    @Override // java.io.FilterInputStream, java.io.InputStream
    public long skip(long n) throws IOException {
        long start = this._pos;
        long skip = Math.min((long) available(), n);
        long j = this._pos;
        if ((((j + skip) ^ start) & ((long) (~this.chunkMask))) != 0) {
            this._chunk = null;
        }
        this._pos = j + skip;
        return skip;
    }

    @Override // java.io.FilterInputStream, org.apache.poi.util.LittleEndianInputStream, org.apache.poi.util.LittleEndianInput, java.io.InputStream
    public int available() {
        return (int) (this._size - this._pos);
    }

    public boolean markSupported() {
        return false;
    }

    public synchronized void mark(int readlimit) {
        throw new UnsupportedOperationException();
    }

    @Override // java.io.FilterInputStream, java.io.InputStream
    public synchronized void reset() throws IOException {
        throw new UnsupportedOperationException();
    }

    private byte[] nextChunk() throws GeneralSecurityException, IOException {
        int index = (int) (this._pos >> this.chunkBits);
        initCipherForBlock(this._cipher, index);
        int i = this._lastIndex;
        if (i != index) {
            super.skip((long) ((index - i) << this.chunkBits));
        }
        byte[] block = new byte[Math.min(super.available(), this.chunkSize)];
        super.read(block, 0, block.length);
        this._lastIndex = index + 1;
        return this._cipher.doFinal(block);
    }
}
