package org.apache.poi.poifs.crypt;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.GeneralSecurityException;
import javax.crypto.Cipher;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.POIFSWriterEvent;
import org.apache.poi.poifs.filesystem.POIFSWriterListener;
import org.apache.poi.util.Internal;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;
import org.apache.poi.util.TempFile;

@Internal
public abstract class ChunkedCipherOutputStream extends FilterOutputStream {
    private static final POILogger logger = POILogFactory.getLogger(ChunkedCipherOutputStream.class);
    private final byte[] _chunk;
    private Cipher _cipher;
    private long _pos = 0;
    protected final int chunkBits;
    protected final int chunkMask;
    protected final int chunkSize;
    private final DirectoryNode dir;
    private final File fileOut;

    /* access modifiers changed from: protected */
    public abstract void calculateChecksum(File file, int i) throws GeneralSecurityException, IOException;

    /* access modifiers changed from: protected */
    public abstract void createEncryptionInfoEntry(DirectoryNode directoryNode, File file) throws IOException, GeneralSecurityException;

    /* access modifiers changed from: protected */
    public abstract Cipher initCipherForBlock(Cipher cipher, int i, boolean z) throws GeneralSecurityException;

    public ChunkedCipherOutputStream(DirectoryNode dir2, int chunkSize2) throws IOException, GeneralSecurityException {
        super(null);
        this.chunkSize = chunkSize2;
        int i = chunkSize2 - 1;
        this.chunkMask = i;
        this.chunkBits = Integer.bitCount(i);
        this._chunk = new byte[chunkSize2];
        File createTempFile = TempFile.createTempFile("encrypted_package", "crypt");
        this.fileOut = createTempFile;
        createTempFile.deleteOnExit();
        this.out = new FileOutputStream(createTempFile);
        this.dir = dir2;
        this._cipher = initCipherForBlock(null, 0, false);
    }

    @Override // java.io.OutputStream, java.io.FilterOutputStream
    public void write(int b) throws IOException {
        write(new byte[]{(byte) b});
    }

    @Override // java.io.OutputStream, java.io.FilterOutputStream
    public void write(byte[] b) throws IOException {
        write(b, 0, b.length);
    }

    @Override // java.io.OutputStream, java.io.FilterOutputStream
    public void write(byte[] b, int off, int len) throws IOException {
        if (len != 0) {
            if (len < 0 || b.length < off + len) {
                throw new IOException("not enough bytes in your input buffer");
            }
            while (len > 0) {
                int posInChunk = (int) (this._pos & ((long) this.chunkMask));
                int nextLen = Math.min(this.chunkSize - posInChunk, len);
                System.arraycopy(b, off, this._chunk, posInChunk, nextLen);
                long j = this._pos + ((long) nextLen);
                this._pos = j;
                off += nextLen;
                len -= nextLen;
                if ((j & ((long) this.chunkMask)) == 0) {
                    try {
                        writeChunk();
                    } catch (GeneralSecurityException e) {
                        throw new IOException(e);
                    }
                }
            }
        }
    }

    /* access modifiers changed from: protected */
    public void writeChunk() throws IOException, GeneralSecurityException {
        boolean lastChunk;
        long j = this._pos;
        int posInChunk = (int) (((long) this.chunkMask) & j);
        int index = (int) (j >> this.chunkBits);
        if (posInChunk == 0) {
            index--;
            posInChunk = this.chunkSize;
            lastChunk = false;
        } else {
            lastChunk = true;
        }
        Cipher initCipherForBlock = initCipherForBlock(this._cipher, index, lastChunk);
        this._cipher = initCipherForBlock;
        byte[] bArr = this._chunk;
        this.out.write(this._chunk, 0, initCipherForBlock.doFinal(bArr, 0, posInChunk, bArr));
    }

    @Override // java.io.OutputStream, java.io.Closeable, java.io.FilterOutputStream, java.lang.AutoCloseable
    public void close() throws IOException {
        try {
            writeChunk();
            super.close();
            calculateChecksum(this.fileOut, (int) this._pos);
            this.dir.createDocument(Decryptor.DEFAULT_POIFS_ENTRY, (int) (this.fileOut.length() + 8), new EncryptedPackageWriter());
            createEncryptionInfoEntry(this.dir, this.fileOut);
        } catch (GeneralSecurityException e) {
            throw new IOException(e);
        }
    }

    private class EncryptedPackageWriter implements POIFSWriterListener {
        private EncryptedPackageWriter() {
        }

        @Override // org.apache.poi.poifs.filesystem.POIFSWriterListener
        public void processPOIFSWriterEvent(POIFSWriterEvent event) {
            try {
                OutputStream os = event.getStream();
                byte[] buf = new byte[ChunkedCipherOutputStream.this.chunkSize];
                LittleEndian.putLong(buf, 0, ChunkedCipherOutputStream.this._pos);
                os.write(buf, 0, 8);
                FileInputStream fis = new FileInputStream(ChunkedCipherOutputStream.this.fileOut);
                while (true) {
                    int readBytes = fis.read(buf);
                    if (readBytes == -1) {
                        break;
                    }
                    os.write(buf, 0, readBytes);
                }
                fis.close();
                os.close();
                if (!ChunkedCipherOutputStream.this.fileOut.delete()) {
                    POILogger pOILogger = ChunkedCipherOutputStream.logger;
                    pOILogger.log(7, "Can't delete temporary encryption file: " + ChunkedCipherOutputStream.this.fileOut);
                }
            } catch (IOException e) {
                throw new EncryptedDocumentException(e);
            }
        }
    }
}
