package org.apache.poi.poifs.crypt;

import java.io.IOException;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
import org.apache.poi.poifs.filesystem.OPOIFSFileSystem;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.util.BitField;
import org.apache.poi.util.BitFieldFactory;
import org.apache.poi.util.LittleEndianInput;

public class EncryptionInfo {
    public static final BitField flagAES = BitFieldFactory.getInstance(32);
    public static final BitField flagCryptoAPI = BitFieldFactory.getInstance(4);
    public static final BitField flagDocProps = BitFieldFactory.getInstance(8);
    public static final BitField flagExternal = BitFieldFactory.getInstance(16);
    private final Decryptor decryptor;
    private final int encryptionFlags;
    private final Encryptor encryptor;
    private final EncryptionHeader header;
    private final EncryptionVerifier verifier;
    private final int versionMajor;
    private final int versionMinor;

    public EncryptionInfo(POIFSFileSystem fs) throws IOException {
        this(fs.getRoot());
    }

    public EncryptionInfo(OPOIFSFileSystem fs) throws IOException {
        this(fs.getRoot());
    }

    public EncryptionInfo(NPOIFSFileSystem fs) throws IOException {
        this(fs.getRoot());
    }

    public EncryptionInfo(DirectoryNode dir) throws IOException {
        this(dir.createDocumentInputStream("EncryptionInfo"), false);
    }

    public EncryptionInfo(LittleEndianInput dis, boolean isCryptoAPI) throws IOException {
        EncryptionMode encryptionMode;
        short readShort = dis.readShort();
        this.versionMajor = readShort;
        short readShort2 = dis.readShort();
        this.versionMinor = readShort2;
        if (!isCryptoAPI && readShort == EncryptionMode.binaryRC4.versionMajor && readShort2 == EncryptionMode.binaryRC4.versionMinor) {
            encryptionMode = EncryptionMode.binaryRC4;
            this.encryptionFlags = -1;
        } else if (!isCryptoAPI && readShort == EncryptionMode.agile.versionMajor && readShort2 == EncryptionMode.agile.versionMinor) {
            encryptionMode = EncryptionMode.agile;
            this.encryptionFlags = dis.readInt();
        } else if (!isCryptoAPI && 2 <= readShort && readShort <= 4 && readShort2 == EncryptionMode.standard.versionMinor) {
            encryptionMode = EncryptionMode.standard;
            this.encryptionFlags = dis.readInt();
        } else if (!isCryptoAPI || 2 > readShort || readShort > 4 || readShort2 != EncryptionMode.cryptoAPI.versionMinor) {
            int readInt = dis.readInt();
            this.encryptionFlags = readInt;
            throw new EncryptedDocumentException("Unknown encryption: version major: " + ((int) readShort) + " / version minor: " + ((int) readShort2) + " / fCrypto: " + flagCryptoAPI.isSet(readInt) + " / fExternal: " + flagExternal.isSet(readInt) + " / fDocProps: " + flagDocProps.isSet(readInt) + " / fAES: " + flagAES.isSet(readInt));
        } else {
            encryptionMode = EncryptionMode.cryptoAPI;
            this.encryptionFlags = dis.readInt();
        }
        try {
            EncryptionInfoBuilder eib = getBuilder(encryptionMode);
            eib.initialize(this, dis);
            this.header = eib.getHeader();
            this.verifier = eib.getVerifier();
            this.decryptor = eib.getDecryptor();
            this.encryptor = eib.getEncryptor();
        } catch (Exception e) {
            throw new IOException(e);
        }
    }

    public EncryptionInfo(EncryptionMode encryptionMode) {
        this(encryptionMode, null, null, -1, -1, null);
    }

    public EncryptionInfo(EncryptionMode encryptionMode, CipherAlgorithm cipherAlgorithm, HashAlgorithm hashAlgorithm, int keyBits, int blockSize, ChainingMode chainingMode) {
        this.versionMajor = encryptionMode.versionMajor;
        this.versionMinor = encryptionMode.versionMinor;
        this.encryptionFlags = encryptionMode.encryptionFlags;
        try {
            EncryptionInfoBuilder eib = getBuilder(encryptionMode);
            eib.initialize(this, cipherAlgorithm, hashAlgorithm, keyBits, blockSize, chainingMode);
            this.header = eib.getHeader();
            this.verifier = eib.getVerifier();
            this.decryptor = eib.getDecryptor();
            this.encryptor = eib.getEncryptor();
        } catch (Exception e) {
            throw new EncryptedDocumentException(e);
        }
    }

    protected static EncryptionInfoBuilder getBuilder(EncryptionMode encryptionMode) throws ClassNotFoundException, IllegalAccessException, InstantiationException {
        return (EncryptionInfoBuilder) Thread.currentThread().getContextClassLoader().loadClass(encryptionMode.builder).newInstance();
    }

    public int getVersionMajor() {
        return this.versionMajor;
    }

    public int getVersionMinor() {
        return this.versionMinor;
    }

    public int getEncryptionFlags() {
        return this.encryptionFlags;
    }

    public EncryptionHeader getHeader() {
        return this.header;
    }

    public EncryptionVerifier getVerifier() {
        return this.verifier;
    }

    public Decryptor getDecryptor() {
        return this.decryptor;
    }

    public Encryptor getEncryptor() {
        return this.encryptor;
    }
}
