package org.apache.poi.poifs.crypt.cryptoapi;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.GeneralSecurityException;
import java.security.MessageDigest;
import java.util.Arrays;
import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.ShortBufferException;
import javax.crypto.spec.SecretKeySpec;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.poifs.crypt.CryptoFunctions;
import org.apache.poi.poifs.crypt.Decryptor;
import org.apache.poi.poifs.crypt.EncryptionHeader;
import org.apache.poi.poifs.crypt.EncryptionInfoBuilder;
import org.apache.poi.poifs.crypt.EncryptionVerifier;
import org.apache.poi.poifs.crypt.HashAlgorithm;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.DocumentInputStream;
import org.apache.poi.poifs.filesystem.DocumentNode;
import org.apache.poi.poifs.filesystem.NPOIFSFileSystem;
import org.apache.poi.util.BitField;
import org.apache.poi.util.BitFieldFactory;
import org.apache.poi.util.BoundedInputStream;
import org.apache.poi.util.IOUtils;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.LittleEndianInputStream;
import org.apache.poi.util.StringUtil;

public class CryptoAPIDecryptor extends Decryptor {
    static final /* synthetic */ boolean $assertionsDisabled = false;
    private long _length = -1;

    private class SeekableByteArrayInputStream extends ByteArrayInputStream {
        Cipher cipher;
        byte[] oneByte = {0};

        public void seek(int newpos) {
            if (newpos <= this.count) {
                this.pos = newpos;
                this.mark = newpos;
                return;
            }
            throw new ArrayIndexOutOfBoundsException(newpos);
        }

        public void setBlock(int block) throws GeneralSecurityException {
            this.cipher = CryptoAPIDecryptor.this.initCipherForBlock(this.cipher, block);
        }

        @Override // java.io.InputStream
        public synchronized int read() {
            int ch = super.read();
            if (ch == -1) {
                return -1;
            }
            byte[] bArr = this.oneByte;
            bArr[0] = (byte) ch;
            try {
                this.cipher.update(bArr, 0, 1, bArr);
                return this.oneByte[0];
            } catch (ShortBufferException e) {
                throw new EncryptedDocumentException(e);
            }
        }

        @Override // java.io.InputStream
        public synchronized int read(byte[] b, int off, int len) {
            int readLen = super.read(b, off, len);
            if (readLen == -1) {
                return -1;
            }
            try {
                this.cipher.update(b, off, readLen, b, off);
                return readLen;
            } catch (ShortBufferException e) {
                throw new EncryptedDocumentException(e);
            }
        }

        public SeekableByteArrayInputStream(byte[] buf) throws GeneralSecurityException {
            super(buf);
            this.cipher = CryptoAPIDecryptor.this.initCipherForBlock(null, 0);
        }
    }

    static class StreamDescriptorEntry {
        static BitField flagStream = BitFieldFactory.getInstance(1);
        int block;
        int flags;
        int reserved2;
        String streamName;
        int streamOffset;
        int streamSize;

        StreamDescriptorEntry() {
        }
    }

    protected CryptoAPIDecryptor(CryptoAPIEncryptionInfoBuilder builder) {
        super(builder);
    }

    @Override // org.apache.poi.poifs.crypt.Decryptor
    public boolean verifyPassword(String password) {
        EncryptionVerifier ver = this.builder.getVerifier();
        SecretKey skey = generateSecretKey(password, ver);
        try {
            Cipher cipher = initCipherForBlock(null, 0, this.builder, skey, 2);
            byte[] encryptedVerifier = ver.getEncryptedVerifier();
            byte[] verifier = new byte[encryptedVerifier.length];
            cipher.update(encryptedVerifier, 0, encryptedVerifier.length, verifier);
            setVerifier(verifier);
            if (!Arrays.equals(CryptoFunctions.getMessageDigest(ver.getHashAlgorithm()).digest(verifier), cipher.doFinal(ver.getEncryptedVerifierHash()))) {
                return false;
            }
            setSecretKey(skey);
            return true;
        } catch (GeneralSecurityException e) {
            throw new EncryptedDocumentException(e);
        }
    }

    public Cipher initCipherForBlock(Cipher cipher, int block) throws GeneralSecurityException {
        return initCipherForBlock(cipher, block, this.builder, getSecretKey(), 2);
    }

    protected static Cipher initCipherForBlock(Cipher cipher, int block, EncryptionInfoBuilder builder, SecretKey skey, int encryptMode) throws GeneralSecurityException {
        HashAlgorithm hashAlgo = builder.getVerifier().getHashAlgorithm();
        byte[] blockKey = new byte[4];
        LittleEndian.putUInt(blockKey, 0, (long) block);
        MessageDigest hashAlg = CryptoFunctions.getMessageDigest(hashAlgo);
        hashAlg.update(skey.getEncoded());
        byte[] encKey = hashAlg.digest(blockKey);
        EncryptionHeader header = builder.getHeader();
        int keyBits = header.getKeySize();
        byte[] encKey2 = CryptoFunctions.getBlock0(encKey, keyBits / 8);
        if (keyBits == 40) {
            encKey2 = CryptoFunctions.getBlock0(encKey2, 16);
        }
        SecretKey key = new SecretKeySpec(encKey2, skey.getAlgorithm());
        if (cipher == null) {
            return CryptoFunctions.getCipher(key, header.getCipherAlgorithm(), null, null, encryptMode);
        }
        cipher.init(encryptMode, key);
        return cipher;
    }

    protected static SecretKey generateSecretKey(String password, EncryptionVerifier ver) {
        if (password.length() > 255) {
            password = password.substring(0, 255);
        }
        MessageDigest hashAlg = CryptoFunctions.getMessageDigest(ver.getHashAlgorithm());
        hashAlg.update(ver.getSalt());
        return new SecretKeySpec(hashAlg.digest(StringUtil.getToUnicodeLE(password)), ver.getCipherAlgorithm().jceId);
    }

    @Override // org.apache.poi.poifs.crypt.Decryptor
    public InputStream getDataStream(DirectoryNode dir) throws IOException, GeneralSecurityException {
        NPOIFSFileSystem fsOut = new NPOIFSFileSystem();
        DocumentNode es = (DocumentNode) dir.getEntry("EncryptedSummary");
        DocumentInputStream dis = dir.createDocumentInputStream(es);
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        IOUtils.copy(dis, bos);
        dis.close();
        SeekableByteArrayInputStream sbis = new SeekableByteArrayInputStream(bos.toByteArray());
        LittleEndianInputStream leis = new LittleEndianInputStream(sbis);
        leis.readUInt();
        sbis.skip((long) (((int) leis.readUInt()) - 8));
        sbis.setBlock(0);
        int encryptedStreamDescriptorCount = (int) leis.readUInt();
        StreamDescriptorEntry[] entries = new StreamDescriptorEntry[encryptedStreamDescriptorCount];
        for (int i = 0; i < encryptedStreamDescriptorCount; i++) {
            StreamDescriptorEntry entry = new StreamDescriptorEntry();
            entries[i] = entry;
            entry.streamOffset = (int) leis.readUInt();
            entry.streamSize = (int) leis.readUInt();
            entry.block = leis.readUShort();
            int nameSize = leis.readUByte();
            entry.flags = leis.readUByte();
            entry.reserved2 = leis.readInt();
            entry.streamName = StringUtil.readUnicodeLE(leis, nameSize);
            leis.readShort();
        }
        int len$ = entries.length;
        int i$ = 0;
        while (i$ < len$) {
            StreamDescriptorEntry entry2 = entries[i$];
            sbis.seek(entry2.streamOffset);
            sbis.setBlock(entry2.block);
            InputStream is = new BoundedInputStream(sbis, (long) entry2.streamSize);
            fsOut.createDocument(is, entry2.streamName);
            is.close();
            i$++;
            es = es;
            dis = dis;
        }
        leis.close();
        sbis.close();
        bos.reset();
        fsOut.writeFilesystem(bos);
        fsOut.close();
        this._length = (long) bos.size();
        return new ByteArrayInputStream(bos.toByteArray());
    }

    @Override // org.apache.poi.poifs.crypt.Decryptor
    public long getLength() {
        long j = this._length;
        if (j != -1) {
            return j;
        }
        throw new IllegalStateException("Decryptor.getDataStream() was not called");
    }
}
