package org.apache.poi.poifs.crypt.cryptoapi;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.security.GeneralSecurityException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import javax.crypto.Cipher;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.hpsf.DocumentSummaryInformation;
import org.apache.poi.hpsf.PropertySetFactory;
import org.apache.poi.hpsf.SummaryInformation;
import org.apache.poi.hpsf.WritingNotSupportedException;
import org.apache.poi.poifs.crypt.CryptoFunctions;
import org.apache.poi.poifs.crypt.DataSpaceMapUtils;
import org.apache.poi.poifs.crypt.EncryptionInfo;
import org.apache.poi.poifs.crypt.Encryptor;
import org.apache.poi.poifs.crypt.cryptoapi.CryptoAPIDecryptor;
import org.apache.poi.poifs.crypt.standard.EncryptionRecord;
import org.apache.poi.poifs.filesystem.DirectoryNode;
import org.apache.poi.poifs.filesystem.DocumentInputStream;
import org.apache.poi.util.IOUtils;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.LittleEndianByteArrayOutputStream;
import org.apache.poi.util.StringUtil;

public class CryptoAPIEncryptor extends Encryptor {
    static final /* synthetic */ boolean $assertionsDisabled = false;
    private final CryptoAPIEncryptionInfoBuilder builder;

    protected CryptoAPIEncryptor(CryptoAPIEncryptionInfoBuilder builder2) {
        this.builder = builder2;
    }

    @Override // org.apache.poi.poifs.crypt.Encryptor
    public void confirmPassword(String password) {
        Random r = new SecureRandom();
        byte[] salt = new byte[16];
        byte[] verifier = new byte[16];
        r.nextBytes(salt);
        r.nextBytes(verifier);
        confirmPassword(password, null, null, verifier, salt, null);
    }

    @Override // org.apache.poi.poifs.crypt.Encryptor
    public void confirmPassword(String password, byte[] keySpec, byte[] keySalt, byte[] verifier, byte[] verifierSalt, byte[] integritySalt) {
        CryptoAPIEncryptionVerifier ver = this.builder.getVerifier();
        ver.setSalt(verifierSalt);
        setSecretKey(CryptoAPIDecryptor.generateSecretKey(password, ver));
        try {
            Cipher cipher = initCipherForBlock(null, 0);
            byte[] encryptedVerifier = new byte[verifier.length];
            cipher.update(verifier, 0, verifier.length, encryptedVerifier);
            ver.setEncryptedVerifier(encryptedVerifier);
            ver.setEncryptedVerifierHash(cipher.doFinal(CryptoFunctions.getMessageDigest(ver.getHashAlgorithm()).digest(verifier)));
        } catch (GeneralSecurityException e) {
            throw new EncryptedDocumentException("Password confirmation failed", e);
        }
    }

    public Cipher initCipherForBlock(Cipher cipher, int block) throws GeneralSecurityException {
        return CryptoAPIDecryptor.initCipherForBlock(cipher, block, this.builder, getSecretKey(), 1);
    }

    @Override // org.apache.poi.poifs.crypt.Encryptor
    public OutputStream getDataStream(DirectoryNode dir) throws IOException, GeneralSecurityException {
        int i;
        CipherByteArrayOutputStream bos = new CipherByteArrayOutputStream();
        byte[] buf = new byte[8];
        int i2 = 0;
        bos.write(buf, 0, 8);
        String[] entryNames = {SummaryInformation.DEFAULT_STREAM_NAME, DocumentSummaryInformation.DEFAULT_STREAM_NAME};
        List<CryptoAPIDecryptor.StreamDescriptorEntry> descList = new ArrayList<>();
        int block = 0;
        int len$ = entryNames.length;
        int i$ = 0;
        while (true) {
            i = 1;
            if (i$ >= len$) {
                break;
            }
            String entryName = entryNames[i$];
            if (dir.hasEntry(entryName)) {
                CryptoAPIDecryptor.StreamDescriptorEntry descEntry = new CryptoAPIDecryptor.StreamDescriptorEntry();
                descEntry.block = block;
                descEntry.streamOffset = bos.size();
                descEntry.streamName = entryName;
                descEntry.flags = CryptoAPIDecryptor.StreamDescriptorEntry.flagStream.setValue(i2, 1);
                descEntry.reserved2 = i2;
                bos.setBlock(block);
                DocumentInputStream dis = dir.createDocumentInputStream(entryName);
                IOUtils.copy(dis, bos);
                dis.close();
                descEntry.streamSize = bos.size() - descEntry.streamOffset;
                descList.add(descEntry);
                dir.getEntry(entryName).delete();
                block++;
            }
            i$++;
            i2 = 0;
        }
        int streamDescriptorArrayOffset = bos.size();
        bos.setBlock(0);
        LittleEndian.putUInt(buf, 0, (long) descList.size());
        bos.write(buf, 0, 4);
        for (CryptoAPIDecryptor.StreamDescriptorEntry sde : descList) {
            LittleEndian.putUInt(buf, 0, (long) sde.streamOffset);
            bos.write(buf, 0, 4);
            LittleEndian.putUInt(buf, 0, (long) sde.streamSize);
            bos.write(buf, 0, 4);
            LittleEndian.putUShort(buf, 0, sde.block);
            bos.write(buf, 0, 2);
            LittleEndian.putUByte(buf, 0, (short) sde.streamName.length());
            bos.write(buf, 0, i);
            LittleEndian.putUByte(buf, 0, (short) sde.flags);
            bos.write(buf, 0, i);
            LittleEndian.putUInt(buf, 0, (long) sde.reserved2);
            bos.write(buf, 0, 4);
            byte[] nameBytes = StringUtil.getToUnicodeLE(sde.streamName);
            bos.write(nameBytes, 0, nameBytes.length);
            LittleEndian.putShort(buf, 0, 0);
            bos.write(buf, 0, 2);
            i = 1;
        }
        int savedSize = bos.size();
        LittleEndian.putUInt(buf, 0, (long) streamDescriptorArrayOffset);
        LittleEndian.putUInt(buf, 4, (long) (savedSize - streamDescriptorArrayOffset));
        bos.reset();
        bos.setBlock(0);
        bos.write(buf, 0, 8);
        bos.setSize(savedSize);
        dir.createDocument("EncryptedSummary", new ByteArrayInputStream(bos.getBuf(), 0, savedSize));
        try {
            PropertySetFactory.newDocumentSummaryInformation().write(dir, DocumentSummaryInformation.DEFAULT_STREAM_NAME);
            return bos;
        } catch (WritingNotSupportedException e) {
            throw new IOException(e);
        }
    }

    /* access modifiers changed from: protected */
    public int getKeySizeInBytes() {
        return this.builder.getHeader().getKeySize() / 8;
    }

    /* access modifiers changed from: protected */
    public void createEncryptionInfoEntry(DirectoryNode dir) throws IOException {
        DataSpaceMapUtils.addDefaultDataSpace(dir);
        final EncryptionInfo info = this.builder.getEncryptionInfo();
        final CryptoAPIEncryptionHeader header = this.builder.getHeader();
        final CryptoAPIEncryptionVerifier verifier = this.builder.getVerifier();
        DataSpaceMapUtils.createEncryptionEntry(dir, "EncryptionInfo", new EncryptionRecord() {
            /* class org.apache.poi.poifs.crypt.cryptoapi.CryptoAPIEncryptor.AnonymousClass1 */

            @Override // org.apache.poi.poifs.crypt.standard.EncryptionRecord
            public void write(LittleEndianByteArrayOutputStream bos) {
                bos.writeShort(info.getVersionMajor());
                bos.writeShort(info.getVersionMinor());
                header.write(bos);
                verifier.write(bos);
            }
        });
    }

    private class CipherByteArrayOutputStream extends ByteArrayOutputStream {
        Cipher cipher;
        byte[] oneByte = {0};

        public CipherByteArrayOutputStream() throws GeneralSecurityException {
            setBlock(0);
        }

        public byte[] getBuf() {
            return this.buf;
        }

        public void setSize(int count) {
            this.count = count;
        }

        public void setBlock(int block) throws GeneralSecurityException {
            this.cipher = CryptoAPIEncryptor.this.initCipherForBlock(this.cipher, block);
        }

        @Override // java.io.OutputStream, java.io.ByteArrayOutputStream
        public void write(int b) {
            try {
                byte[] bArr = this.oneByte;
                bArr[0] = (byte) b;
                this.cipher.update(bArr, 0, 1, bArr, 0);
                super.write(this.oneByte);
            } catch (Exception e) {
                throw new EncryptedDocumentException(e);
            }
        }

        @Override // java.io.OutputStream
        public void write(byte[] b, int off, int len) {
            try {
                this.cipher.update(b, off, len, b, off);
                super.write(b, off, len);
            } catch (Exception e) {
                throw new EncryptedDocumentException(e);
            }
        }
    }
}
