package org.apache.poi.poifs.crypt.standard;

import java.io.IOException;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.poifs.crypt.ChainingMode;
import org.apache.poi.poifs.crypt.CipherAlgorithm;
import org.apache.poi.poifs.crypt.EncryptionInfo;
import org.apache.poi.poifs.crypt.EncryptionInfoBuilder;
import org.apache.poi.poifs.crypt.HashAlgorithm;
import org.apache.poi.util.LittleEndianInput;

public class StandardEncryptionInfoBuilder implements EncryptionInfoBuilder {
    StandardDecryptor decryptor;
    StandardEncryptor encryptor;
    StandardEncryptionHeader header;
    EncryptionInfo info;
    StandardEncryptionVerifier verifier;

    @Override // org.apache.poi.poifs.crypt.EncryptionInfoBuilder
    public void initialize(EncryptionInfo info2, LittleEndianInput dis) throws IOException {
        this.info = info2;
        dis.readInt();
        this.header = new StandardEncryptionHeader(dis);
        this.verifier = new StandardEncryptionVerifier(dis, this.header);
        if (info2.getVersionMinor() != 2) {
            return;
        }
        if (info2.getVersionMajor() == 3 || info2.getVersionMajor() == 4) {
            this.decryptor = new StandardDecryptor(this);
        }
    }

    @Override // org.apache.poi.poifs.crypt.EncryptionInfoBuilder
    public void initialize(EncryptionInfo info2, CipherAlgorithm cipherAlgorithm, HashAlgorithm hashAlgorithm, int keyBits, int blockSize, ChainingMode chainingMode) {
        this.info = info2;
        if (cipherAlgorithm == null) {
            cipherAlgorithm = CipherAlgorithm.aes128;
        }
        if (cipherAlgorithm == CipherAlgorithm.aes128 || cipherAlgorithm == CipherAlgorithm.aes192 || cipherAlgorithm == CipherAlgorithm.aes256) {
            if (hashAlgorithm == null) {
                hashAlgorithm = HashAlgorithm.sha1;
            }
            if (hashAlgorithm == HashAlgorithm.sha1) {
                if (chainingMode == null) {
                    chainingMode = ChainingMode.ecb;
                }
                if (chainingMode == ChainingMode.ecb) {
                    if (keyBits == -1) {
                        keyBits = cipherAlgorithm.defaultKeySize;
                    }
                    if (blockSize == -1) {
                        blockSize = cipherAlgorithm.blockSize;
                    }
                    int[] arr$ = cipherAlgorithm.allowedKeySize;
                    int len$ = arr$.length;
                    boolean found = false;
                    for (int i$ = 0; i$ < len$; i$++) {
                        found |= arr$[i$] == keyBits;
                    }
                    if (found) {
                        this.header = new StandardEncryptionHeader(cipherAlgorithm, hashAlgorithm, keyBits, blockSize, chainingMode);
                        this.verifier = new StandardEncryptionVerifier(cipherAlgorithm, hashAlgorithm, keyBits, blockSize, chainingMode);
                        this.decryptor = new StandardDecryptor(this);
                        this.encryptor = new StandardEncryptor(this);
                        return;
                    }
                    throw new EncryptedDocumentException("KeySize " + keyBits + " not allowed for Cipher " + cipherAlgorithm.toString());
                }
                throw new EncryptedDocumentException("Standard encryption only supports ECB chaining.");
            }
            throw new EncryptedDocumentException("Standard encryption only supports SHA-1.");
        }
        throw new EncryptedDocumentException("Standard encryption only supports AES128/192/256.");
    }

    @Override // org.apache.poi.poifs.crypt.EncryptionInfoBuilder
    public StandardEncryptionHeader getHeader() {
        return this.header;
    }

    @Override // org.apache.poi.poifs.crypt.EncryptionInfoBuilder
    public StandardEncryptionVerifier getVerifier() {
        return this.verifier;
    }

    @Override // org.apache.poi.poifs.crypt.EncryptionInfoBuilder
    public StandardDecryptor getDecryptor() {
        return this.decryptor;
    }

    @Override // org.apache.poi.poifs.crypt.EncryptionInfoBuilder
    public StandardEncryptor getEncryptor() {
        return this.encryptor;
    }

    public EncryptionInfo getEncryptionInfo() {
        return this.info;
    }
}
