package org.apache.poi.poifs.filesystem;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.poi.util.Internal;

@Internal
public class EntryUtils {
    @Internal
    public static void copyNodeRecursively(Entry entry, DirectoryEntry target) throws IOException {
        if (entry.isDirectoryEntry()) {
            DirectoryEntry dirEntry = (DirectoryEntry) entry;
            DirectoryEntry newTarget = target.createDirectory(entry.getName());
            newTarget.setStorageClsid(dirEntry.getStorageClsid());
            Iterator<Entry> entries = dirEntry.getEntries();
            while (entries.hasNext()) {
                copyNodeRecursively(entries.next(), newTarget);
            }
            return;
        }
        DocumentEntry dentry = (DocumentEntry) entry;
        DocumentInputStream dstream = new DocumentInputStream(dentry);
        target.createDocument(dentry.getName(), dstream);
        dstream.close();
    }

    public static void copyNodes(DirectoryEntry sourceRoot, DirectoryEntry targetRoot) throws IOException {
        Iterator i$ = sourceRoot.iterator();
        while (i$.hasNext()) {
            copyNodeRecursively((Entry) i$.next(), targetRoot);
        }
    }

    public static void copyNodes(FilteringDirectoryNode filteredSource, FilteringDirectoryNode filteredTarget) throws IOException {
        copyNodes((DirectoryEntry) filteredSource, (DirectoryEntry) filteredTarget);
    }

    public static void copyNodes(DirectoryEntry sourceRoot, DirectoryEntry targetRoot, List<String> excepts) throws IOException {
        Iterator<Entry> entries = sourceRoot.getEntries();
        while (entries.hasNext()) {
            Entry entry = entries.next();
            if (!excepts.contains(entry.getName())) {
                copyNodeRecursively(entry, targetRoot);
            }
        }
    }

    public static void copyNodes(OPOIFSFileSystem source, OPOIFSFileSystem target) throws IOException {
        copyNodes(source.getRoot(), target.getRoot());
    }

    public static void copyNodes(NPOIFSFileSystem source, NPOIFSFileSystem target) throws IOException {
        copyNodes(source.getRoot(), target.getRoot());
    }

    public static void copyNodes(OPOIFSFileSystem source, OPOIFSFileSystem target, List<String> excepts) throws IOException {
        copyNodes(new FilteringDirectoryNode(source.getRoot(), excepts), new FilteringDirectoryNode(target.getRoot(), excepts));
    }

    public static void copyNodes(NPOIFSFileSystem source, NPOIFSFileSystem target, List<String> excepts) throws IOException {
        copyNodes(new FilteringDirectoryNode(source.getRoot(), excepts), new FilteringDirectoryNode(target.getRoot(), excepts));
    }

    public static boolean areDirectoriesIdentical(DirectoryEntry dirA, DirectoryEntry dirB) {
        boolean match;
        int size;
        if (!dirA.getName().equals(dirB.getName()) || dirA.getEntryCount() != dirB.getEntryCount()) {
            return false;
        }
        Map<String, Integer> aSizes = new HashMap<>();
        Iterator i$ = dirA.iterator();
        while (i$.hasNext()) {
            Entry a = (Entry) i$.next();
            String aName = a.getName();
            if (a.isDirectoryEntry()) {
                aSizes.put(aName, -12345);
            } else {
                aSizes.put(aName, Integer.valueOf(((DocumentNode) a).getSize()));
            }
        }
        Iterator i$2 = dirB.iterator();
        while (i$2.hasNext()) {
            Entry b = (Entry) i$2.next();
            String bName = b.getName();
            if (!aSizes.containsKey(bName)) {
                return false;
            }
            if (b.isDirectoryEntry()) {
                size = -12345;
            } else {
                size = ((DocumentNode) b).getSize();
            }
            if (size != aSizes.get(bName).intValue()) {
                return false;
            }
            aSizes.remove(bName);
        }
        if (!aSizes.isEmpty()) {
            return false;
        }
        Iterator i$3 = dirA.iterator();
        while (i$3.hasNext()) {
            Entry a2 = (Entry) i$3.next();
            try {
                Entry b2 = dirB.getEntry(a2.getName());
                if (a2.isDirectoryEntry()) {
                    match = areDirectoriesIdentical((DirectoryEntry) a2, (DirectoryEntry) b2);
                    continue;
                } else {
                    match = areDocumentsIdentical((DocumentEntry) a2, (DocumentEntry) b2);
                    continue;
                }
                if (!match) {
                    return false;
                }
            } catch (FileNotFoundException e) {
                return false;
            } catch (IOException e2) {
                return false;
            }
        }
        return true;
    }

    public static boolean areDocumentsIdentical(DocumentEntry docA, DocumentEntry docB) throws IOException {
        if (!docA.getName().equals(docB.getName()) || docA.getSize() != docB.getSize()) {
            return false;
        }
        boolean matches = true;
        DocumentInputStream inpA = null;
        DocumentInputStream inpB = null;
        try {
            DocumentInputStream inpA2 = new DocumentInputStream(docA);
            DocumentInputStream inpB2 = new DocumentInputStream(docB);
            while (true) {
                int readA = inpA2.read();
                int readB = inpB2.read();
                if (readA == readB) {
                    if (readA != -1) {
                        if (readB == -1) {
                            break;
                        }
                    } else {
                        break;
                    }
                } else {
                    matches = false;
                    break;
                }
            }
            inpA2.close();
            inpB2.close();
            return matches;
        } catch (Throwable th) {
            if (0 != 0) {
                inpA.close();
            }
            if (0 != 0) {
                inpB.close();
            }
            throw th;
        }
    }
}
