package org.apache.poi.poifs.filesystem;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Iterator;
import org.apache.poi.poifs.property.DocumentProperty;
import org.apache.poi.util.LittleEndian;

public final class NDocumentInputStream extends DocumentInputStream {
    private ByteBuffer _buffer;
    private boolean _closed;
    private int _current_block_count;
    private int _current_offset;
    private Iterator<ByteBuffer> _data;
    private NPOIFSDocument _document;
    private int _document_size;
    private int _marked_offset;
    private int _marked_offset_count;

    public NDocumentInputStream(DocumentEntry document) throws IOException {
        if (document instanceof DocumentNode) {
            this._current_offset = 0;
            this._current_block_count = 0;
            this._marked_offset = 0;
            this._marked_offset_count = 0;
            this._document_size = document.getSize();
            this._closed = false;
            DocumentNode doc = (DocumentNode) document;
            NPOIFSDocument nPOIFSDocument = new NPOIFSDocument((DocumentProperty) doc.getProperty(), ((DirectoryNode) doc.getParent()).getNFileSystem());
            this._document = nPOIFSDocument;
            this._data = nPOIFSDocument.getBlockIterator();
            return;
        }
        throw new IOException("Cannot open internal document storage, " + document + " not a Document Node");
    }

    public NDocumentInputStream(NPOIFSDocument document) {
        this._current_offset = 0;
        this._current_block_count = 0;
        this._marked_offset = 0;
        this._marked_offset_count = 0;
        this._document_size = document.getSize();
        this._closed = false;
        this._document = document;
        this._data = document.getBlockIterator();
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream, java.io.InputStream
    public int available() {
        if (!this._closed) {
            return this._document_size - this._current_offset;
        }
        throw new IllegalStateException("cannot perform requested operation on a closed stream");
    }

    @Override // java.io.Closeable, java.lang.AutoCloseable, org.apache.poi.poifs.filesystem.DocumentInputStream, java.io.InputStream
    public void close() {
        this._closed = true;
    }

    @Override // org.apache.poi.poifs.filesystem.DocumentInputStream
    public void mark(int ignoredReadlimit) {
        this._marked_offset = this._current_offset;
        this._marked_offset_count = Math.max(0, this._current_block_count - 1);
    }

    @Override // org.apache.poi.poifs.filesystem.DocumentInputStream, java.io.InputStream
    public int read() throws IOException {
        dieIfClosed();
        if (atEOD()) {
            return -1;
        }
        byte[] b = new byte[1];
        int result = read(b, 0, 1);
        if (result < 0) {
            return result;
        }
        if (b[0] < 0) {
            return b[0] + 256;
        }
        return b[0];
    }

    @Override // org.apache.poi.poifs.filesystem.DocumentInputStream, java.io.InputStream
    public int read(byte[] b, int off, int len) throws IOException {
        dieIfClosed();
        if (b == null) {
            throw new IllegalArgumentException("buffer must not be null");
        } else if (off < 0 || len < 0 || b.length < off + len) {
            throw new IndexOutOfBoundsException("can't read past buffer boundaries");
        } else if (len == 0) {
            return 0;
        } else {
            if (atEOD()) {
                return -1;
            }
            int limit = Math.min(available(), len);
            readFully(b, off, limit);
            return limit;
        }
    }

    @Override // org.apache.poi.poifs.filesystem.DocumentInputStream, java.io.InputStream
    public void reset() {
        int i;
        int i2;
        int i3 = this._marked_offset;
        if (i3 == 0 && (i2 = this._marked_offset_count) == 0) {
            this._current_block_count = i2;
            this._current_offset = i3;
            this._data = this._document.getBlockIterator();
            this._buffer = null;
            return;
        }
        this._data = this._document.getBlockIterator();
        this._current_offset = 0;
        int i4 = 0;
        while (true) {
            i = this._marked_offset_count;
            if (i4 >= i) {
                break;
            }
            ByteBuffer next = this._data.next();
            this._buffer = next;
            this._current_offset += next.remaining();
            i4++;
        }
        this._current_block_count = i;
        if (this._current_offset != this._marked_offset) {
            ByteBuffer next2 = this._data.next();
            this._buffer = next2;
            this._current_block_count++;
            next2.position(next2.position() + (this._marked_offset - this._current_offset));
        }
        this._current_offset = this._marked_offset;
    }

    @Override // org.apache.poi.poifs.filesystem.DocumentInputStream, java.io.InputStream
    public long skip(long n) throws IOException {
        dieIfClosed();
        if (n < 0) {
            return 0;
        }
        int i = this._current_offset;
        int new_offset = ((int) n) + i;
        if (new_offset < i) {
            new_offset = this._document_size;
        } else if (new_offset > this._document_size) {
            new_offset = this._document_size;
        }
        long rval = (long) (new_offset - i);
        readFully(new byte[((int) rval)]);
        return rval;
    }

    private void dieIfClosed() throws IOException {
        if (this._closed) {
            throw new IOException("cannot perform requested operation on a closed stream");
        }
    }

    private boolean atEOD() {
        return this._current_offset == this._document_size;
    }

    private void checkAvaliable(int requestedSize) {
        if (this._closed) {
            throw new IllegalStateException("cannot perform requested operation on a closed stream");
        } else if (requestedSize > this._document_size - this._current_offset) {
            throw new RuntimeException("Buffer underrun - requested " + requestedSize + " bytes but " + (this._document_size - this._current_offset) + " was available");
        }
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public void readFully(byte[] buf, int off, int len) {
        checkAvaliable(len);
        int read = 0;
        while (read < len) {
            ByteBuffer byteBuffer = this._buffer;
            if (byteBuffer == null || byteBuffer.remaining() == 0) {
                this._current_block_count++;
                this._buffer = this._data.next();
            }
            int limit = Math.min(len - read, this._buffer.remaining());
            this._buffer.get(buf, off + read, limit);
            this._current_offset += limit;
            read += limit;
        }
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public byte readByte() {
        return (byte) readUByte();
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public double readDouble() {
        return Double.longBitsToDouble(readLong());
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public long readLong() {
        checkAvaliable(8);
        byte[] data = new byte[8];
        readFully(data, 0, 8);
        return LittleEndian.getLong(data, 0);
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public short readShort() {
        checkAvaliable(2);
        byte[] data = new byte[2];
        readFully(data, 0, 2);
        return LittleEndian.getShort(data);
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public int readInt() {
        checkAvaliable(4);
        byte[] data = new byte[4];
        readFully(data, 0, 4);
        return LittleEndian.getInt(data);
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public int readUShort() {
        checkAvaliable(2);
        byte[] data = new byte[2];
        readFully(data, 0, 2);
        return LittleEndian.getUShort(data);
    }

    @Override // org.apache.poi.util.LittleEndianInput, org.apache.poi.poifs.filesystem.DocumentInputStream
    public int readUByte() {
        checkAvaliable(1);
        byte[] data = new byte[1];
        readFully(data, 0, 1);
        if (data[0] >= 0) {
            return data[0];
        }
        return data[0] + 256;
    }
}
