package org.apache.poi.poifs.filesystem;

import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.Iterator;
import org.apache.poi.poifs.filesystem.BlockStore;

public class NPOIFSStream implements Iterable<ByteBuffer> {
    private BlockStore blockStore;
    private OutputStream outStream;
    private int startBlock;

    public NPOIFSStream(BlockStore blockStore2, int startBlock2) {
        this.blockStore = blockStore2;
        this.startBlock = startBlock2;
    }

    public NPOIFSStream(BlockStore blockStore2) {
        this.blockStore = blockStore2;
        this.startBlock = -2;
    }

    public int getStartBlock() {
        return this.startBlock;
    }

    @Override // java.lang.Iterable
    public Iterator<ByteBuffer> iterator() {
        return getBlockIterator();
    }

    public Iterator<ByteBuffer> getBlockIterator() {
        int i = this.startBlock;
        if (i != -2) {
            return new StreamBlockByteBufferIterator(i);
        }
        throw new IllegalStateException("Can't read from a new stream before it has been written to");
    }

    public void updateContents(byte[] contents) throws IOException {
        OutputStream os = getOutputStream();
        os.write(contents);
        os.close();
    }

    public OutputStream getOutputStream() throws IOException {
        if (this.outStream == null) {
            this.outStream = new StreamBlockByteBuffer();
        }
        return this.outStream;
    }

    public void free() throws IOException {
        free(this.blockStore.getChainLoopDetector());
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private void free(BlockStore.ChainLoopDetector loopDetector) {
        int nextBlock = this.startBlock;
        while (nextBlock != -2) {
            loopDetector.claim(nextBlock);
            nextBlock = this.blockStore.getNextBlock(nextBlock);
            this.blockStore.setNextBlock(nextBlock, -1);
        }
        this.startBlock = -2;
    }

    /* access modifiers changed from: protected */
    public class StreamBlockByteBufferIterator implements Iterator<ByteBuffer> {
        private BlockStore.ChainLoopDetector loopDetector;
        private int nextBlock;

        protected StreamBlockByteBufferIterator(int firstBlock) {
            this.nextBlock = firstBlock;
            try {
                this.loopDetector = NPOIFSStream.this.blockStore.getChainLoopDetector();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        public boolean hasNext() {
            if (this.nextBlock == -2) {
                return false;
            }
            return true;
        }

        @Override // java.util.Iterator
        public ByteBuffer next() {
            int i = this.nextBlock;
            if (i != -2) {
                try {
                    this.loopDetector.claim(i);
                    ByteBuffer data = NPOIFSStream.this.blockStore.getBlockAt(this.nextBlock);
                    this.nextBlock = NPOIFSStream.this.blockStore.getNextBlock(this.nextBlock);
                    return data;
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            } else {
                throw new IndexOutOfBoundsException("Can't read past the end of the stream");
            }
        }

        public void remove() {
            throw new UnsupportedOperationException();
        }
    }

    /* access modifiers changed from: protected */
    public class StreamBlockByteBuffer extends OutputStream {
        ByteBuffer buffer;
        BlockStore.ChainLoopDetector loopDetector;
        int nextBlock;
        byte[] oneByte = new byte[1];
        int prevBlock;

        protected StreamBlockByteBuffer() throws IOException {
            this.loopDetector = NPOIFSStream.this.blockStore.getChainLoopDetector();
            this.prevBlock = -2;
            this.nextBlock = NPOIFSStream.this.startBlock;
        }

        /* access modifiers changed from: protected */
        public void createBlockIfNeeded() throws IOException {
            ByteBuffer byteBuffer = this.buffer;
            if (byteBuffer == null || !byteBuffer.hasRemaining()) {
                int thisBlock = this.nextBlock;
                if (thisBlock == -2) {
                    thisBlock = NPOIFSStream.this.blockStore.getFreeBlock();
                    this.loopDetector.claim(thisBlock);
                    this.nextBlock = -2;
                    if (this.prevBlock != -2) {
                        NPOIFSStream.this.blockStore.setNextBlock(this.prevBlock, thisBlock);
                    }
                    NPOIFSStream.this.blockStore.setNextBlock(thisBlock, -2);
                    if (NPOIFSStream.this.startBlock == -2) {
                        NPOIFSStream.this.startBlock = thisBlock;
                    }
                } else {
                    this.loopDetector.claim(thisBlock);
                    this.nextBlock = NPOIFSStream.this.blockStore.getNextBlock(thisBlock);
                }
                this.buffer = NPOIFSStream.this.blockStore.createBlockIfNeeded(thisBlock);
                this.prevBlock = thisBlock;
            }
        }

        @Override // java.io.OutputStream
        public void write(int b) throws IOException {
            byte[] bArr = this.oneByte;
            bArr[0] = (byte) (b & 255);
            write(bArr);
        }

        @Override // java.io.OutputStream
        public void write(byte[] b, int off, int len) throws IOException {
            if (off < 0 || off > b.length || len < 0 || off + len > b.length || off + len < 0) {
                throw new IndexOutOfBoundsException();
            } else if (len != 0) {
                do {
                    createBlockIfNeeded();
                    int writeBytes = Math.min(this.buffer.remaining(), len);
                    this.buffer.put(b, off, writeBytes);
                    off += writeBytes;
                    len -= writeBytes;
                } while (len > 0);
            }
        }

        @Override // java.io.OutputStream, java.io.Closeable, java.lang.AutoCloseable
        public void close() throws IOException {
            new NPOIFSStream(NPOIFSStream.this.blockStore, this.nextBlock).free(this.loopDetector);
            if (this.prevBlock != -2) {
                NPOIFSStream.this.blockStore.setNextBlock(this.prevBlock, -2);
            }
        }
    }
}
