package org.apache.poi.poifs.storage;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.Arrays;
import org.apache.poi.hssf.OldExcelFormatException;
import org.apache.poi.poifs.common.POIFSBigBlockSize;
import org.apache.poi.poifs.common.POIFSConstants;
import org.apache.poi.poifs.filesystem.NotOLE2FileException;
import org.apache.poi.poifs.filesystem.OfficeXmlFileException;
import org.apache.poi.util.HexDump;
import org.apache.poi.util.IOUtils;
import org.apache.poi.util.IntegerField;
import org.apache.poi.util.LittleEndian;
import org.apache.poi.util.LongField;
import org.apache.poi.util.ShortField;

public final class HeaderBlock implements HeaderBlockConstants {
    private static final byte[] MAGIC_BIFF2 = {9, 0, 4, 0, 0, 0, 112, 0};
    private static final byte[] MAGIC_BIFF3 = {9, 2, 6, 0, 0, 0, 112, 0};
    private static final byte[] MAGIC_BIFF4a = {9, 4, 6, 0, 0, 0, 112, 0};
    private static final byte[] MAGIC_BIFF4b = {9, 4, 6, 0, 0, 0, 0, 1};
    private static final byte _default_value = -1;
    private int _bat_count;
    private final byte[] _data;
    private int _property_start;
    private int _sbat_count;
    private int _sbat_start;
    private int _xbat_count;
    private int _xbat_start;
    private final POIFSBigBlockSize bigBlockSize;

    public HeaderBlock(InputStream stream) throws IOException {
        this(readFirst512(stream));
        if (this.bigBlockSize.getBigBlockSize() != 512) {
            IOUtils.readFully(stream, new byte[(this.bigBlockSize.getBigBlockSize() - 512)]);
        }
    }

    public HeaderBlock(ByteBuffer buffer) throws IOException {
        this(IOUtils.toByteArray(buffer, 512));
    }

    private HeaderBlock(byte[] data) throws IOException {
        byte[] bArr = (byte[]) data.clone();
        this._data = bArr;
        long signature = LittleEndian.getLong(bArr, 0);
        if (signature == HeaderBlockConstants._signature) {
            if (bArr[30] == 12) {
                this.bigBlockSize = POIFSConstants.LARGER_BIG_BLOCK_SIZE_DETAILS;
            } else if (bArr[30] == 9) {
                this.bigBlockSize = POIFSConstants.SMALLER_BIG_BLOCK_SIZE_DETAILS;
            } else {
                throw new IOException("Unsupported blocksize  (2^" + ((int) bArr[30]) + "). Expected 2^9 or 2^12.");
            }
            this._bat_count = new IntegerField(44, data).get();
            this._property_start = new IntegerField(48, bArr).get();
            this._sbat_start = new IntegerField(60, bArr).get();
            this._sbat_count = new IntegerField(64, bArr).get();
            this._xbat_start = new IntegerField(68, bArr).get();
            this._xbat_count = new IntegerField(72, bArr).get();
        } else if (cmp(POIFSConstants.OOXML_FILE_HEADER, data)) {
            throw new OfficeXmlFileException("The supplied data appears to be in the Office 2007+ XML. You are calling the part of POI that deals with OLE2 Office Documents. You need to call a different part of POI to process this data (eg XSSF instead of HSSF)");
        } else if (cmp(POIFSConstants.RAW_XML_FILE_HEADER, data)) {
            throw new NotOLE2FileException("The supplied data appears to be a raw XML file. Formats such as Office 2003 XML are not supported");
        } else if (cmp(MAGIC_BIFF2, data)) {
            throw new OldExcelFormatException("The supplied data appears to be in BIFF2 format. HSSF only supports the BIFF8 format, try OldExcelExtractor");
        } else if (cmp(MAGIC_BIFF3, data)) {
            throw new OldExcelFormatException("The supplied data appears to be in BIFF3 format. HSSF only supports the BIFF8 format, try OldExcelExtractor");
        } else if (cmp(MAGIC_BIFF4a, data) || cmp(MAGIC_BIFF4b, data)) {
            throw new OldExcelFormatException("The supplied data appears to be in BIFF4 format. HSSF only supports the BIFF8 format, try OldExcelExtractor");
        } else {
            throw new NotOLE2FileException("Invalid header signature; read " + HexDump.longToHex(signature) + ", expected " + HexDump.longToHex(HeaderBlockConstants._signature) + " - Your file appears " + "not to be a valid OLE2 document");
        }
    }

    public HeaderBlock(POIFSBigBlockSize bigBlockSize2) {
        this.bigBlockSize = bigBlockSize2;
        byte[] bArr = new byte[512];
        this._data = bArr;
        Arrays.fill(bArr, (byte) -1);
        new LongField(0, HeaderBlockConstants._signature, bArr);
        new IntegerField(8, 0, bArr);
        new IntegerField(12, 0, bArr);
        new IntegerField(16, 0, bArr);
        new IntegerField(20, 0, bArr);
        new ShortField(24, 59, bArr);
        new ShortField(26, 3, bArr);
        new ShortField(28, -2, bArr);
        new ShortField(30, bigBlockSize2.getHeaderValue(), bArr);
        new IntegerField(32, 6, bArr);
        new IntegerField(36, 0, bArr);
        new IntegerField(40, 0, bArr);
        new IntegerField(52, 0, bArr);
        new IntegerField(56, 4096, bArr);
        this._bat_count = 0;
        this._sbat_count = 0;
        this._xbat_count = 0;
        this._property_start = -2;
        this._sbat_start = -2;
        this._xbat_start = -2;
    }

    private static byte[] readFirst512(InputStream stream) throws IOException {
        byte[] data = new byte[512];
        int bsCount = IOUtils.readFully(stream, data);
        if (bsCount == 512) {
            return data;
        }
        throw alertShortRead(bsCount, 512);
    }

    private static IOException alertShortRead(int pRead, int expectedReadSize) {
        int read;
        if (pRead < 0) {
            read = 0;
        } else {
            read = pRead;
        }
        StringBuilder sb = new StringBuilder();
        sb.append(" byte");
        sb.append(read == 1 ? "" : "s");
        String type = sb.toString();
        return new IOException("Unable to read entire header; " + read + type + " read; expected " + expectedReadSize + " bytes");
    }

    public int getPropertyStart() {
        return this._property_start;
    }

    public void setPropertyStart(int startBlock) {
        this._property_start = startBlock;
    }

    public int getSBATStart() {
        return this._sbat_start;
    }

    public int getSBATCount() {
        return this._sbat_count;
    }

    public void setSBATStart(int startBlock) {
        this._sbat_start = startBlock;
    }

    public void setSBATBlockCount(int count) {
        this._sbat_count = count;
    }

    public int getBATCount() {
        return this._bat_count;
    }

    public void setBATCount(int count) {
        this._bat_count = count;
    }

    public int[] getBATArray() {
        int[] result = new int[Math.min(this._bat_count, 109)];
        int offset = 76;
        for (int j = 0; j < result.length; j++) {
            result[j] = LittleEndian.getInt(this._data, offset);
            offset += 4;
        }
        return result;
    }

    public void setBATArray(int[] bat_array) {
        int count = Math.min(bat_array.length, 109);
        int blank = 109 - count;
        int offset = 76;
        for (int i = 0; i < count; i++) {
            LittleEndian.putInt(this._data, offset, bat_array[i]);
            offset += 4;
        }
        for (int i2 = 0; i2 < blank; i2++) {
            LittleEndian.putInt(this._data, offset, -1);
            offset += 4;
        }
    }

    public int getXBATCount() {
        return this._xbat_count;
    }

    public void setXBATCount(int count) {
        this._xbat_count = count;
    }

    public int getXBATIndex() {
        return this._xbat_start;
    }

    public void setXBATStart(int startBlock) {
        this._xbat_start = startBlock;
    }

    public POIFSBigBlockSize getBigBlockSize() {
        return this.bigBlockSize;
    }

    /* access modifiers changed from: package-private */
    public void writeData(OutputStream stream) throws IOException {
        new IntegerField(44, this._bat_count, this._data);
        new IntegerField(48, this._property_start, this._data);
        new IntegerField(60, this._sbat_start, this._data);
        new IntegerField(64, this._sbat_count, this._data);
        new IntegerField(68, this._xbat_start, this._data);
        new IntegerField(72, this._xbat_count, this._data);
        stream.write(this._data, 0, 512);
        for (int i = 512; i < this.bigBlockSize.getBigBlockSize(); i++) {
            stream.write(0);
        }
    }

    /* JADX INFO: Multiple debug info for r0v2 byte: [D('i' int), D('d' byte)] */
    private static boolean cmp(byte[] magic, byte[] data) {
        int i = 0;
        int len$ = magic.length;
        int i$ = 0;
        while (i$ < len$) {
            byte m = magic[i$];
            int i2 = i + 1;
            byte d = data[i];
            if (d != m) {
                if (m != 112) {
                    return false;
                }
                if (!(d == 16 || d == 32 || d == 64)) {
                    return false;
                }
            }
            i$++;
            i = i2;
        }
        return true;
    }
}
