package org.apache.poi.sl.draw;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Path2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.EventFilter;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.events.XMLEvent;
import org.apache.poi.sl.draw.binding.CTCustomGeometry2D;
import org.apache.poi.sl.draw.geom.Context;
import org.apache.poi.sl.draw.geom.CustomGeometry;
import org.apache.poi.sl.draw.geom.Outline;
import org.apache.poi.sl.draw.geom.Path;
import org.apache.poi.sl.usermodel.LineDecoration;
import org.apache.poi.sl.usermodel.PaintStyle;
import org.apache.poi.sl.usermodel.SimpleShape;

public class DrawSimpleShape extends DrawShape {
    private static final double DECO_SIZE_POW = 1.5d;

    public DrawSimpleShape(SimpleShape<?, ?> shape) {
        super(shape);
    }

    @Override // org.apache.poi.sl.draw.DrawShape, org.apache.poi.sl.draw.Drawable
    public void draw(Graphics2D graphics) {
        DrawPaint drawPaint = DrawFactory.getInstance(graphics).getPaint(getShape());
        Paint fill = drawPaint.getPaint(graphics, getShape().getFillStyle().getPaint());
        Paint line = drawPaint.getPaint(graphics, getShape().getStrokeStyle().getPaint());
        BasicStroke stroke = getStroke();
        graphics.setStroke(stroke);
        Collection<Outline> elems = computeOutlines(graphics);
        drawShadow(graphics, elems, fill, line);
        if (fill != null) {
            graphics.setPaint(fill);
            for (Outline o : elems) {
                if (o.getPath().isFilled()) {
                    Shape s = o.getOutline();
                    graphics.setRenderingHint(Drawable.GRADIENT_SHAPE, s);
                    graphics.fill(s);
                }
            }
        }
        drawContent(graphics);
        if (line != null) {
            graphics.setPaint(line);
            graphics.setStroke(stroke);
            for (Outline o2 : elems) {
                if (o2.getPath().isStroked()) {
                    Shape s2 = o2.getOutline();
                    graphics.setRenderingHint(Drawable.GRADIENT_SHAPE, s2);
                    graphics.draw(s2);
                }
            }
        }
        drawDecoration(graphics, line, stroke);
    }

    /* access modifiers changed from: protected */
    public void drawDecoration(Graphics2D graphics, Paint line, BasicStroke stroke) {
        if (line != null) {
            graphics.setPaint(line);
            List<Outline> lst = new ArrayList<>();
            LineDecoration deco = getShape().getLineDecoration();
            Outline head = getHeadDecoration(graphics, deco, stroke);
            if (head != null) {
                lst.add(head);
            }
            Outline tail = getTailDecoration(graphics, deco, stroke);
            if (tail != null) {
                lst.add(tail);
            }
            for (Outline o : lst) {
                Shape s = o.getOutline();
                Path p = o.getPath();
                graphics.setRenderingHint(Drawable.GRADIENT_SHAPE, s);
                if (p.isFilled()) {
                    graphics.fill(s);
                }
                if (p.isStroked()) {
                    graphics.draw(s);
                }
            }
        }
    }

    /* access modifiers changed from: protected */
    public Outline getTailDecoration(Graphics2D graphics, LineDecoration deco, BasicStroke stroke) {
        Shape tailShape;
        AffineTransform at;
        if (deco == null || stroke == null) {
            return null;
        }
        LineDecoration.DecorationSize tailLength = deco.getTailLength();
        if (tailLength == null) {
            tailLength = LineDecoration.DecorationSize.MEDIUM;
        }
        LineDecoration.DecorationSize tailWidth = deco.getTailWidth();
        if (tailWidth == null) {
            tailWidth = LineDecoration.DecorationSize.MEDIUM;
        }
        double lineWidth = Math.max(2.5d, (double) stroke.getLineWidth());
        Rectangle2D anchor = getAnchor(graphics, getShape());
        double x2 = anchor.getX() + anchor.getWidth();
        double y2 = anchor.getY() + anchor.getHeight();
        double alpha = Math.atan(anchor.getHeight() / anchor.getWidth());
        AffineTransform at2 = new AffineTransform();
        Path p = null;
        double scaleY = Math.pow(DECO_SIZE_POW, (double) (tailWidth.ordinal() + 1));
        double scaleX = Math.pow(DECO_SIZE_POW, (double) (tailLength.ordinal() + 1));
        LineDecoration.DecorationShape tailShapeEnum = deco.getTailShape();
        if (tailShapeEnum == null) {
            return null;
        }
        switch (AnonymousClass2.$SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape[tailShapeEnum.ordinal()]) {
            case 1:
                Path p2 = new Path();
                Shape tailShape2 = new Ellipse2D.Double(0.0d, 0.0d, lineWidth * scaleX, lineWidth * scaleY);
                Rectangle2D bounds = tailShape2.getBounds2D();
                at2.translate(x2 - (bounds.getWidth() / 2.0d), y2 - (bounds.getHeight() / 2.0d));
                at = at2;
                at2.rotate(alpha, bounds.getX() + (bounds.getWidth() / 2.0d), bounds.getY() + (bounds.getHeight() / 2.0d));
                p = p2;
                tailShape = tailShape2;
                break;
            case 2:
            case 3:
                p = new Path(false, true);
                Shape arrow = new Path2D.Double();
                arrow.moveTo((-lineWidth) * scaleX, ((-lineWidth) * scaleY) / 2.0d);
                arrow.lineTo(0.0d, 0.0d);
                arrow.lineTo((-lineWidth) * scaleX, (lineWidth * scaleY) / 2.0d);
                tailShape = arrow;
                at2.translate(x2, y2);
                at2.rotate(alpha);
                at = at2;
                break;
            case 4:
                p = new Path();
                Shape triangle = new Path2D.Double();
                triangle.moveTo((-lineWidth) * scaleX, ((-lineWidth) * scaleY) / 2.0d);
                triangle.lineTo(0.0d, 0.0d);
                triangle.lineTo((-lineWidth) * scaleX, (lineWidth * scaleY) / 2.0d);
                triangle.closePath();
                tailShape = triangle;
                at2.translate(x2, y2);
                at2.rotate(alpha);
                at = at2;
                break;
            default:
                at = at2;
                tailShape = null;
                break;
        }
        if (tailShape != null) {
            tailShape = at.createTransformedShape(tailShape);
        }
        if (tailShape == null) {
            return null;
        }
        return new Outline(tailShape, p);
    }

    /* access modifiers changed from: package-private */
    /* renamed from: org.apache.poi.sl.draw.DrawSimpleShape$2  reason: invalid class name */
    public static /* synthetic */ class AnonymousClass2 {
        static final /* synthetic */ int[] $SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape;

        static {
            int[] iArr = new int[LineDecoration.DecorationShape.values().length];
            $SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape = iArr;
            try {
                iArr[LineDecoration.DecorationShape.OVAL.ordinal()] = 1;
            } catch (NoSuchFieldError e) {
            }
            try {
                $SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape[LineDecoration.DecorationShape.STEALTH.ordinal()] = 2;
            } catch (NoSuchFieldError e2) {
            }
            try {
                $SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape[LineDecoration.DecorationShape.ARROW.ordinal()] = 3;
            } catch (NoSuchFieldError e3) {
            }
            try {
                $SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape[LineDecoration.DecorationShape.TRIANGLE.ordinal()] = 4;
            } catch (NoSuchFieldError e4) {
            }
        }
    }

    /* access modifiers changed from: protected */
    public Outline getHeadDecoration(Graphics2D graphics, LineDecoration deco, BasicStroke stroke) {
        Shape headShape;
        AffineTransform at;
        if (deco == null || stroke == null) {
            return null;
        }
        LineDecoration.DecorationSize headLength = deco.getHeadLength();
        if (headLength == null) {
            headLength = LineDecoration.DecorationSize.MEDIUM;
        }
        LineDecoration.DecorationSize headWidth = deco.getHeadWidth();
        if (headWidth == null) {
            headWidth = LineDecoration.DecorationSize.MEDIUM;
        }
        double lineWidth = Math.max(2.5d, (double) stroke.getLineWidth());
        Rectangle2D anchor = getAnchor(graphics, getShape());
        double x1 = anchor.getX();
        double y1 = anchor.getY();
        double alpha = Math.atan(anchor.getHeight() / anchor.getWidth());
        AffineTransform at2 = new AffineTransform();
        Path p = null;
        double scaleY = Math.pow(DECO_SIZE_POW, (double) (headWidth.ordinal() + 1));
        double scaleX = Math.pow(DECO_SIZE_POW, (double) (headLength.ordinal() + 1));
        LineDecoration.DecorationShape headShapeEnum = deco.getHeadShape();
        if (headShapeEnum == null) {
            return null;
        }
        switch (AnonymousClass2.$SwitchMap$org$apache$poi$sl$usermodel$LineDecoration$DecorationShape[headShapeEnum.ordinal()]) {
            case 1:
                Path p2 = new Path();
                Shape headShape2 = new Ellipse2D.Double(0.0d, 0.0d, lineWidth * scaleX, lineWidth * scaleY);
                Rectangle2D bounds = headShape2.getBounds2D();
                at2.translate(x1 - (bounds.getWidth() / 2.0d), y1 - (bounds.getHeight() / 2.0d));
                at = at2;
                at2.rotate(alpha, bounds.getX() + (bounds.getWidth() / 2.0d), bounds.getY() + (bounds.getHeight() / 2.0d));
                p = p2;
                headShape = headShape2;
                break;
            case 2:
            case 3:
                p = new Path(false, true);
                Shape arrow = new Path2D.Double();
                arrow.moveTo(lineWidth * scaleX, ((-lineWidth) * scaleY) / 2.0d);
                arrow.lineTo(0.0d, 0.0d);
                arrow.lineTo(lineWidth * scaleX, (lineWidth * scaleY) / 2.0d);
                headShape = arrow;
                at2.translate(x1, y1);
                at2.rotate(alpha);
                at = at2;
                break;
            case 4:
                p = new Path();
                Shape triangle = new Path2D.Double();
                triangle.moveTo(lineWidth * scaleX, ((-lineWidth) * scaleY) / 2.0d);
                triangle.lineTo(0.0d, 0.0d);
                triangle.lineTo(lineWidth * scaleX, (lineWidth * scaleY) / 2.0d);
                triangle.closePath();
                headShape = triangle;
                at2.translate(x1, y1);
                at2.rotate(alpha);
                at = at2;
                break;
            default:
                at = at2;
                headShape = null;
                break;
        }
        if (headShape != null) {
            headShape = at.createTransformedShape(headShape);
        }
        if (headShape == null) {
            return null;
        }
        return new Outline(headShape, p);
    }

    public BasicStroke getStroke() {
        return getStroke(getShape().getStrokeStyle());
    }

    /* access modifiers changed from: protected */
    public void drawShadow(Graphics2D graphics, Collection<Outline> outlines, Paint fill, Paint line) {
        Shape s = getShape().getShadow();
        if (s == null) {
            return;
        }
        if (!(fill == null && line == null)) {
            PaintStyle.SolidPaint shadowPaint = s.getFillStyle();
            Color shadowColor = DrawPaint.applyColorTransform(shadowPaint.getSolidColor());
            double shapeRotation = getShape().getRotation();
            if (getShape().getFlipVertical()) {
                shapeRotation += 180.0d;
            }
            double angle = s.getAngle() - shapeRotation;
            double dist = s.getDistance();
            double dx = Math.cos(Math.toRadians(angle)) * dist;
            double dy = Math.sin(Math.toRadians(angle)) * dist;
            graphics.translate(dx, dy);
            for (Outline o : outlines) {
                Shape s2 = o.getOutline();
                Path p = o.getPath();
                graphics.setRenderingHint(Drawable.GRADIENT_SHAPE, s2);
                graphics.setPaint(shadowColor);
                if (fill != null && p.isFilled()) {
                    graphics.fill(s2);
                } else if (line != null && p.isStroked()) {
                    graphics.draw(s2);
                }
                s = s;
                shadowPaint = shadowPaint;
            }
            graphics.translate(-dx, -dy);
        }
    }

    protected static CustomGeometry getCustomGeometry(String name) {
        return getCustomGeometry(name, null);
    }

    protected static CustomGeometry getCustomGeometry(String name, Graphics2D graphics) {
        Map<String, CustomGeometry> presets = graphics == null ? null : (Map) graphics.getRenderingHint(Drawable.PRESET_GEOMETRY_CACHE);
        if (presets == null) {
            Map<String, CustomGeometry> presets2 = new HashMap<>();
            if (graphics != null) {
                graphics.setRenderingHint(Drawable.PRESET_GEOMETRY_CACHE, presets2);
            }
            InputStream presetIS = Drawable.class.getResourceAsStream("presetShapeDefinitions.xml");
            EventFilter startElementFilter = new EventFilter() {
                /* class org.apache.poi.sl.draw.DrawSimpleShape.AnonymousClass1 */

                public boolean accept(XMLEvent event) {
                    return event.isStartElement();
                }
            };
            try {
                XMLInputFactory staxFactory = XMLInputFactory.newInstance();
                XMLEventReader staxReader = staxFactory.createXMLEventReader(presetIS);
                XMLEventReader staxFiltRd = staxFactory.createFilteredReader(staxReader, startElementFilter);
                staxFiltRd.nextEvent();
                Unmarshaller unmarshaller = JAXBContext.newInstance("org.apache.poi.sl.draw.binding").createUnmarshaller();
                while (staxFiltRd.peek() != null) {
                    presets2.put(staxFiltRd.peek().getName().getLocalPart(), new CustomGeometry((CTCustomGeometry2D) unmarshaller.unmarshal(staxReader, CTCustomGeometry2D.class).getValue()));
                }
                staxFiltRd.close();
                staxReader.close();
                try {
                    presetIS.close();
                    presets = presets2;
                } catch (IOException e) {
                    throw new RuntimeException("Unable to load preset geometries.", e);
                }
            } catch (Exception e2) {
                throw new RuntimeException("Unable to load preset geometries.", e2);
            } catch (Throwable e3) {
                try {
                    presetIS.close();
                    throw e3;
                } catch (IOException e4) {
                    throw new RuntimeException("Unable to load preset geometries.", e4);
                }
            }
        }
        return presets.get(name);
    }

    /* access modifiers changed from: protected */
    public Collection<Outline> computeOutlines(Graphics2D graphics) {
        double scaleX;
        double scaleY;
        List<Outline> lst = new ArrayList<>();
        CustomGeometry geom = getShape().getGeometry();
        if (geom == null) {
            return lst;
        }
        Rectangle2D anchor = getAnchor(graphics, getShape());
        Iterator i$ = geom.iterator();
        while (i$.hasNext()) {
            Path p = i$.next();
            Shape gp = p.getPath(new Context(geom, new Rectangle2D.Double(0.0d, 0.0d, p.getW() == -1 ? anchor.getWidth() * 12700.0d : (double) p.getW(), p.getH() == -1 ? anchor.getHeight() * 12700.0d : (double) p.getH()), getShape()));
            AffineTransform at = new AffineTransform();
            at.translate(anchor.getX(), anchor.getY());
            if (p.getW() != -1) {
                scaleX = anchor.getWidth() / ((double) p.getW());
            } else {
                scaleX = 7.874015748031496E-5d;
            }
            if (p.getH() != -1) {
                scaleY = anchor.getHeight() / ((double) p.getH());
            } else {
                scaleY = 7.874015748031496E-5d;
            }
            at.scale(scaleX, scaleY);
            lst.add(new Outline(at.createTransformedShape(gp), p));
        }
        return lst;
    }

    /* access modifiers changed from: protected */
    @Override // org.apache.poi.sl.draw.DrawShape
    public SimpleShape<?, ?> getShape() {
        return (SimpleShape) this.shape;
    }
}
