package org.apache.poi.sl.draw;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.MultipleGradientPaint;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.IllegalPathStateException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.util.Hashtable;

/* access modifiers changed from: package-private */
public class PathGradientPaint implements Paint {
    protected final int capStyle;
    protected final Color[] colors;
    protected final float[] fractions;
    protected final int joinStyle;
    protected final int transparency;

    public PathGradientPaint(Color[] colors2, float[] fractions2) {
        this(colors2, fractions2, 1, 1);
    }

    public PathGradientPaint(Color[] colors2, float[] fractions2, int capStyle2, int joinStyle2) {
        int i;
        this.colors = (Color[]) colors2.clone();
        this.fractions = (float[]) fractions2.clone();
        this.capStyle = capStyle2;
        this.joinStyle = joinStyle2;
        int i2 = 1;
        int len$ = colors2.length;
        int i$ = 0;
        while (true) {
            i = 1;
            if (i$ >= len$) {
                break;
            }
            Color c = colors2[i$];
            if (c != null) {
                i2 = (i2 == 0 || c.getAlpha() != 255) ? 0 : i;
            }
            i$++;
        }
        this.transparency = i2 == 0 ? 3 : i;
    }

    public PaintContext createContext(ColorModel cm, Rectangle deviceBounds, Rectangle2D userBounds, AffineTransform transform, RenderingHints hints) {
        return new PathGradientContext(cm, deviceBounds, userBounds, transform, hints);
    }

    public int getTransparency() {
        return this.transparency;
    }

    class PathGradientContext implements PaintContext {
        protected final Rectangle deviceBounds;
        protected final int gradientSteps;
        protected final RenderingHints hints;
        protected final PaintContext pCtx;
        WritableRaster raster;
        protected final Shape shape;
        protected final Rectangle2D userBounds;
        protected final AffineTransform xform;

        public PathGradientContext(ColorModel cm, Rectangle deviceBounds2, Rectangle2D userBounds2, AffineTransform xform2, RenderingHints hints2) {
            Shape shape2 = (Shape) hints2.get(Drawable.GRADIENT_SHAPE);
            this.shape = shape2;
            if (shape2 != null) {
                this.deviceBounds = deviceBounds2;
                this.userBounds = userBounds2;
                this.xform = xform2;
                this.hints = hints2;
                int gradientSteps2 = getGradientSteps(shape2);
                this.gradientSteps = gradientSteps2;
                LinearGradientPaint gradientPaint = new LinearGradientPaint(new Point2D.Double(0.0d, 0.0d), new Point2D.Double((double) gradientSteps2, 0.0d), PathGradientPaint.this.fractions, PathGradientPaint.this.colors, MultipleGradientPaint.CycleMethod.NO_CYCLE, MultipleGradientPaint.ColorSpaceType.SRGB, new AffineTransform());
                Rectangle bounds = new Rectangle(0, 0, gradientSteps2, 1);
                this.pCtx = gradientPaint.createContext(cm, bounds, bounds, new AffineTransform(), hints2);
                return;
            }
            throw new IllegalPathStateException("PathGradientPaint needs a shape to be set via the rendering hint Drawable.GRADIANT_SHAPE.");
        }

        public void dispose() {
        }

        public ColorModel getColorModel() {
            return this.pCtx.getColorModel();
        }

        public Raster getRaster(int xOffset, int yOffset, int w, int h) {
            ColorModel cm = getColorModel();
            if (this.raster == null) {
                createRaster();
            }
            WritableRaster childRaster = cm.createCompatibleWritableRaster(w, h);
            Rectangle2D childRect = new Rectangle2D.Double((double) xOffset, (double) yOffset, (double) w, (double) h);
            if (!childRect.intersects(this.deviceBounds)) {
                return childRaster;
            }
            Rectangle2D destRect = new Rectangle2D.Double();
            Rectangle2D.intersect(childRect, this.deviceBounds, destRect);
            int dw = (int) destRect.getWidth();
            int dh = (int) destRect.getHeight();
            childRaster.setDataElements((int) (destRect.getX() - childRect.getX()), (int) (destRect.getY() - childRect.getY()), dw, dh, this.raster.getDataElements((int) (destRect.getX() - this.deviceBounds.getX()), (int) (destRect.getY() - this.deviceBounds.getY()), dw, dh, (Object) null));
            return childRaster;
        }

        /* access modifiers changed from: protected */
        public int getGradientSteps(Shape gradientShape) {
            Rectangle rect = gradientShape.getBounds();
            int lower = 1;
            int upper = (int) (Math.max(rect.getWidth(), rect.getHeight()) / 2.0d);
            while (lower < upper - 1) {
                int mid = ((upper - lower) / 2) + lower;
                if (new Area(new BasicStroke((float) mid, PathGradientPaint.this.capStyle, PathGradientPaint.this.joinStyle).createStrokedShape(gradientShape)).isSingular()) {
                    upper = mid;
                } else {
                    lower = mid;
                }
            }
            return upper;
        }

        /* access modifiers changed from: protected */
        public void createRaster() {
            ColorModel cm = getColorModel();
            this.raster = cm.createCompatibleWritableRaster((int) this.deviceBounds.getWidth(), (int) this.deviceBounds.getHeight());
            Graphics2D graphics = new BufferedImage(cm, this.raster, false, (Hashtable) null).createGraphics();
            graphics.setRenderingHints(this.hints);
            graphics.translate(-this.deviceBounds.getX(), -this.deviceBounds.getY());
            graphics.transform(this.xform);
            Raster img2 = this.pCtx.getRaster(0, 0, this.gradientSteps, 1);
            int[] rgb = new int[cm.getNumComponents()];
            for (int i = this.gradientSteps - 1; i >= 0; i--) {
                img2.getPixel(i, 0, rgb);
                Color c = new Color(rgb[0], rgb[1], rgb[2]);
                if (rgb.length == 4) {
                    graphics.setComposite(AlphaComposite.getInstance(2, ((float) rgb[3]) / 255.0f));
                }
                graphics.setStroke(new BasicStroke((float) (i + 1), PathGradientPaint.this.capStyle, PathGradientPaint.this.joinStyle));
                graphics.setColor(c);
                graphics.draw(this.shape);
            }
            graphics.dispose();
        }
    }
}
