package org.apache.poi.ss.format;

import java.util.ArrayList;
import java.util.Formatter;
import java.util.List;
import java.util.ListIterator;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.poi.ss.format.CellFormatPart;

public class CellElapsedFormatter extends CellFormatter {
    private static final double HOUR__FACTOR = 0.041666666666666664d;
    private static final double MIN__FACTOR = 6.944444444444444E-4d;
    private static final Pattern PERCENTS = Pattern.compile("%");
    private static final double SEC__FACTOR = 1.1574074074074073E-5d;
    private final String printfFmt;
    private final List<TimeSpec> specs;
    private TimeSpec topmost;

    /* access modifiers changed from: private */
    public static class TimeSpec {
        final double factor;
        final int len;
        double modBy = 0.0d;
        final int pos;
        final char type;

        public TimeSpec(char type2, int pos2, int len2, double factor2) {
            this.type = type2;
            this.pos = pos2;
            this.len = len2;
            this.factor = factor2;
        }

        public long valueFor(double elapsed) {
            double val;
            double d = this.modBy;
            if (d == 0.0d) {
                val = elapsed / this.factor;
            } else {
                val = (elapsed / this.factor) % d;
            }
            if (this.type == '0') {
                return Math.round(val);
            }
            return (long) val;
        }
    }

    private class ElapsedPartHandler implements CellFormatPart.PartHandler {
        private ElapsedPartHandler() {
        }

        @Override // org.apache.poi.ss.format.CellFormatPart.PartHandler
        public String handlePart(Matcher m, String part, CellFormatType type, StringBuffer desc) {
            int pos = desc.length();
            switch (part.charAt(0)) {
                case '\n':
                    return "%n";
                case '\"':
                    part = part.substring(1, part.length() - 1);
                    break;
                case '*':
                    if (part.length() > 1) {
                        part = CellFormatPart.expandChar(part);
                        break;
                    }
                    break;
                case '0':
                case 'h':
                case 'm':
                case 's':
                    String part2 = part.toLowerCase(Locale.ROOT);
                    CellElapsedFormatter.this.assignSpec(part2.charAt(0), pos, part2.length());
                    return part2;
                case '[':
                    if (part.length() >= 3) {
                        if (CellElapsedFormatter.this.topmost == null) {
                            String part3 = part.toLowerCase(Locale.ROOT);
                            int specLen = part3.length() - 2;
                            CellElapsedFormatter cellElapsedFormatter = CellElapsedFormatter.this;
                            cellElapsedFormatter.topmost = cellElapsedFormatter.assignSpec(part3.charAt(1), pos, specLen);
                            return part3.substring(1, specLen + 1);
                        }
                        throw new IllegalArgumentException("Duplicate '[' times in format");
                    }
                    break;
                case '\\':
                    part = part.substring(1);
                    break;
                case '_':
                    return null;
            }
            return CellElapsedFormatter.PERCENTS.matcher(part).replaceAll("%%");
        }
    }

    public CellElapsedFormatter(String pattern) {
        super(pattern);
        ArrayList arrayList = new ArrayList();
        this.specs = arrayList;
        StringBuffer desc = CellFormatPart.parseFormat(pattern, CellFormatType.ELAPSED, new ElapsedPartHandler());
        ListIterator<TimeSpec> it = arrayList.listIterator(arrayList.size());
        while (it.hasPrevious()) {
            TimeSpec spec = it.previous();
            int i = spec.pos;
            int i2 = spec.pos + spec.len;
            desc.replace(i, i2, "%0" + spec.len + "d");
            if (spec.type != this.topmost.type) {
                spec.modBy = modFor(spec.type, spec.len);
            }
        }
        this.printfFmt = desc.toString();
    }

    /* access modifiers changed from: private */
    /* access modifiers changed from: public */
    private TimeSpec assignSpec(char type, int pos, int len) {
        TimeSpec spec = new TimeSpec(type, pos, len, factorFor(type, len));
        this.specs.add(spec);
        return spec;
    }

    private static double factorFor(char type, int len) {
        switch (type) {
            case '0':
                return SEC__FACTOR / Math.pow(10.0d, (double) len);
            case 'h':
                return HOUR__FACTOR;
            case 'm':
                return MIN__FACTOR;
            case 's':
                return SEC__FACTOR;
            default:
                throw new IllegalArgumentException("Uknown elapsed time spec: " + type);
        }
    }

    private static double modFor(char type, int len) {
        switch (type) {
            case '0':
                return Math.pow(10.0d, (double) len);
            case 'h':
                return 24.0d;
            case 'm':
                return 60.0d;
            case 's':
                return 60.0d;
            default:
                throw new IllegalArgumentException("Uknown elapsed time spec: " + type);
        }
    }

    @Override // org.apache.poi.ss.format.CellFormatter
    public void formatValue(StringBuffer toAppendTo, Object value) {
        double elapsed = ((Number) value).doubleValue();
        if (elapsed < 0.0d) {
            toAppendTo.append('-');
            elapsed = -elapsed;
        }
        Object[] parts = new Long[this.specs.size()];
        for (int i = 0; i < this.specs.size(); i++) {
            parts[i] = Long.valueOf(this.specs.get(i).valueFor(elapsed));
        }
        Formatter formatter = new Formatter(toAppendTo, Locale.ROOT);
        try {
            formatter.format(this.printfFmt, parts);
        } finally {
            formatter.close();
        }
    }

    @Override // org.apache.poi.ss.format.CellFormatter
    public void simpleValue(StringBuffer toAppendTo, Object value) {
        formatValue(toAppendTo, value);
    }
}
