package org.apache.poi.ss.formula;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;
import org.apache.poi.ss.SpreadsheetVersion;
import org.apache.poi.ss.formula.constant.ErrorConstant;
import org.apache.poi.ss.formula.function.FunctionMetadata;
import org.apache.poi.ss.formula.function.FunctionMetadataRegistry;
import org.apache.poi.ss.formula.ptg.AbstractFunctionPtg;
import org.apache.poi.ss.formula.ptg.AddPtg;
import org.apache.poi.ss.formula.ptg.Area3DPxg;
import org.apache.poi.ss.formula.ptg.AreaPtg;
import org.apache.poi.ss.formula.ptg.ArrayPtg;
import org.apache.poi.ss.formula.ptg.AttrPtg;
import org.apache.poi.ss.formula.ptg.BoolPtg;
import org.apache.poi.ss.formula.ptg.ConcatPtg;
import org.apache.poi.ss.formula.ptg.DividePtg;
import org.apache.poi.ss.formula.ptg.EqualPtg;
import org.apache.poi.ss.formula.ptg.ErrPtg;
import org.apache.poi.ss.formula.ptg.FuncPtg;
import org.apache.poi.ss.formula.ptg.FuncVarPtg;
import org.apache.poi.ss.formula.ptg.GreaterEqualPtg;
import org.apache.poi.ss.formula.ptg.GreaterThanPtg;
import org.apache.poi.ss.formula.ptg.IntPtg;
import org.apache.poi.ss.formula.ptg.IntersectionPtg;
import org.apache.poi.ss.formula.ptg.LessEqualPtg;
import org.apache.poi.ss.formula.ptg.LessThanPtg;
import org.apache.poi.ss.formula.ptg.MemAreaPtg;
import org.apache.poi.ss.formula.ptg.MemFuncPtg;
import org.apache.poi.ss.formula.ptg.MissingArgPtg;
import org.apache.poi.ss.formula.ptg.MultiplyPtg;
import org.apache.poi.ss.formula.ptg.NamePtg;
import org.apache.poi.ss.formula.ptg.NameXPtg;
import org.apache.poi.ss.formula.ptg.NameXPxg;
import org.apache.poi.ss.formula.ptg.NotEqualPtg;
import org.apache.poi.ss.formula.ptg.NumberPtg;
import org.apache.poi.ss.formula.ptg.OperandPtg;
import org.apache.poi.ss.formula.ptg.OperationPtg;
import org.apache.poi.ss.formula.ptg.ParenthesisPtg;
import org.apache.poi.ss.formula.ptg.PercentPtg;
import org.apache.poi.ss.formula.ptg.PowerPtg;
import org.apache.poi.ss.formula.ptg.Ptg;
import org.apache.poi.ss.formula.ptg.RangePtg;
import org.apache.poi.ss.formula.ptg.RefPtg;
import org.apache.poi.ss.formula.ptg.StringPtg;
import org.apache.poi.ss.formula.ptg.SubtractPtg;
import org.apache.poi.ss.formula.ptg.UnaryMinusPtg;
import org.apache.poi.ss.formula.ptg.UnaryPlusPtg;
import org.apache.poi.ss.formula.ptg.UnionPtg;
import org.apache.poi.ss.formula.ptg.ValueOperatorPtg;
import org.apache.poi.ss.usermodel.FormulaError;
import org.apache.poi.ss.usermodel.Name;
import org.apache.poi.ss.usermodel.Table;
import org.apache.poi.ss.util.AreaReference;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.util.Internal;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

@Internal
public final class FormulaParser {
    private static final Pattern CELL_REF_PATTERN = Pattern.compile("(\\$?[A-Za-z]+)?(\\$?[0-9]+)?");
    private static final char CR = '\r';
    private static final char LF = '\n';
    private static final char TAB = '\t';
    private static final POILogger log = POILogFactory.getLogger(FormulaParser.class);
    private static final String specAll = "All";
    private static final String specData = "Data";
    private static final String specHeaders = "Headers";
    private static final String specThisRow = "This Row";
    private static final String specTotals = "Totals";
    private final FormulaParsingWorkbook _book;
    private final int _formulaLength;
    private final String _formulaString;
    private boolean _inIntersection = false;
    private int _pointer;
    private ParseNode _rootNode;
    private final int _rowIndex;
    private final int _sheetIndex;
    private final SpreadsheetVersion _ssVersion;
    private char look;

    private FormulaParser(String formula, FormulaParsingWorkbook book, int sheetIndex, int rowIndex) {
        this._formulaString = formula;
        this._pointer = 0;
        this._book = book;
        this._ssVersion = book == null ? SpreadsheetVersion.EXCEL97 : book.getSpreadsheetVersion();
        this._formulaLength = formula.length();
        this._sheetIndex = sheetIndex;
        this._rowIndex = rowIndex;
    }

    public static Ptg[] parse(String formula, FormulaParsingWorkbook workbook, FormulaType formulaType, int sheetIndex, int rowIndex) {
        FormulaParser fp = new FormulaParser(formula, workbook, sheetIndex, rowIndex);
        fp.parse();
        return fp.getRPNPtg(formulaType);
    }

    public static Ptg[] parse(String formula, FormulaParsingWorkbook workbook, FormulaType formulaType, int sheetIndex) {
        return parse(formula, workbook, formulaType, sheetIndex, -1);
    }

    public static Area3DPxg parseStructuredReference(String tableText, FormulaParsingWorkbook workbook, int rowIndex) {
        Ptg[] arr = parse(tableText, workbook, FormulaType.CELL, -1, rowIndex);
        if (arr.length == 1 && (arr[0] instanceof Area3DPxg)) {
            return (Area3DPxg) arr[0];
        }
        throw new IllegalStateException("Illegal structured reference");
    }

    private void GetChar() {
        if (!IsWhite(this.look)) {
            this._inIntersection = false;
        } else if (this.look == ' ') {
            this._inIntersection = true;
        }
        int i = this._pointer;
        int i2 = this._formulaLength;
        if (i <= i2) {
            if (i < i2) {
                this.look = this._formulaString.charAt(i);
            } else {
                this.look = 0;
                this._inIntersection = false;
            }
            this._pointer++;
            return;
        }
        throw new RuntimeException("too far");
    }

    private void resetPointer(int ptr) {
        this._pointer = ptr;
        if (ptr <= this._formulaLength) {
            this.look = this._formulaString.charAt(ptr - 1);
        } else {
            this.look = 0;
        }
    }

    private RuntimeException expected(String s) {
        String msg;
        if (this.look != '=' || this._formulaString.substring(0, this._pointer - 1).trim().length() >= 1) {
            msg = "Parse error near char " + (this._pointer - 1) + " '" + this.look + "'" + " in specified formula '" + this._formulaString + "'. Expected " + s;
        } else {
            msg = "The specified formula '" + this._formulaString + "' starts with an equals sign which is not allowed.";
        }
        return new FormulaParseException(msg);
    }

    private static boolean IsAlpha(char c) {
        return Character.isLetter(c) || c == '$' || c == '_';
    }

    private static boolean IsDigit(char c) {
        return Character.isDigit(c);
    }

    private static boolean IsWhite(char c) {
        return c == ' ' || c == '\t' || c == '\r' || c == '\n';
    }

    private void SkipWhite() {
        while (IsWhite(this.look)) {
            GetChar();
        }
    }

    private void Match(char x) {
        if (this.look == x) {
            GetChar();
            return;
        }
        throw expected("'" + x + "'");
    }

    private String GetNum() {
        StringBuffer value = new StringBuffer();
        while (IsDigit(this.look)) {
            value.append(this.look);
            GetChar();
        }
        if (value.length() == 0) {
            return null;
        }
        return value.toString();
    }

    private ParseNode parseRangeExpression() {
        ParseNode result = parseRangeable();
        boolean hasRange = false;
        while (this.look == ':') {
            int pos = this._pointer;
            GetChar();
            ParseNode nextPart = parseRangeable();
            checkValidRangeOperand("LHS", pos, result);
            checkValidRangeOperand("RHS", pos, nextPart);
            result = new ParseNode(RangePtg.instance, new ParseNode[]{result, nextPart});
            hasRange = true;
        }
        if (hasRange) {
            return augmentWithMemPtg(result);
        }
        return result;
    }

    private static ParseNode augmentWithMemPtg(ParseNode root) {
        Ptg memPtg;
        if (needsMemFunc(root)) {
            memPtg = new MemFuncPtg(root.getEncodedSize());
        } else {
            memPtg = new MemAreaPtg(root.getEncodedSize());
        }
        return new ParseNode(memPtg, root);
    }

    private static boolean needsMemFunc(ParseNode root) {
        Ptg token = root.getToken();
        if ((token instanceof AbstractFunctionPtg) || (token instanceof ExternSheetReferenceToken) || (token instanceof NamePtg) || (token instanceof NameXPtg)) {
            return true;
        }
        if ((token instanceof OperationPtg) || (token instanceof ParenthesisPtg)) {
            for (ParseNode child : root.getChildren()) {
                if (needsMemFunc(child)) {
                    return true;
                }
            }
            return false;
        } else if (!(token instanceof OperandPtg) && (token instanceof OperationPtg)) {
            return true;
        } else {
            return false;
        }
    }

    private static void checkValidRangeOperand(String sideName, int currentParsePosition, ParseNode pn) {
        if (!isValidRangeOperand(pn)) {
            throw new FormulaParseException("The " + sideName + " of the range operator ':' at position " + currentParsePosition + " is not a proper reference.");
        }
    }

    private static boolean isValidRangeOperand(ParseNode a) {
        Ptg tkn = a.getToken();
        if (tkn instanceof OperandPtg) {
            return true;
        }
        if (tkn instanceof AbstractFunctionPtg) {
            if (((AbstractFunctionPtg) tkn).getDefaultOperandClass() == 0) {
                return true;
            }
            return false;
        } else if (tkn instanceof ValueOperatorPtg) {
            return false;
        } else {
            if (tkn instanceof OperationPtg) {
                return true;
            }
            if (tkn instanceof ParenthesisPtg) {
                return isValidRangeOperand(a.getChildren()[0]);
            }
            if (tkn == ErrPtg.REF_INVALID) {
                return true;
            }
            return false;
        }
    }

    private ParseNode parseRangeable() {
        char c;
        SkipWhite();
        int savePointer = this._pointer;
        SheetIdentifier sheetIden = parseSheetName();
        if (sheetIden == null) {
            resetPointer(savePointer);
        } else {
            SkipWhite();
            savePointer = this._pointer;
        }
        SimpleRangePart part1 = parseSimpleRangePart();
        if (part1 != null) {
            boolean whiteAfterPart1 = IsWhite(this.look);
            if (whiteAfterPart1) {
                SkipWhite();
            }
            char c2 = this.look;
            if (c2 == ':') {
                int colonPos = this._pointer;
                GetChar();
                SkipWhite();
                SimpleRangePart part2 = parseSimpleRangePart();
                if (part2 != null && !part1.isCompatibleForArea(part2)) {
                    part2 = null;
                }
                if (part2 == null) {
                    resetPointer(colonPos);
                    if (!part1.isCell()) {
                        String prefix = "";
                        if (sheetIden != null) {
                            prefix = "'" + sheetIden.getSheetIdentifier().getName() + '!';
                        }
                        throw new FormulaParseException(prefix + part1.getRep() + "' is not a proper reference.");
                    }
                }
                return createAreaRefParseNode(sheetIden, part1, part2);
            } else if (c2 == '.') {
                GetChar();
                int dotCount = 1;
                while (true) {
                    c = this.look;
                    if (c != '.') {
                        break;
                    }
                    dotCount++;
                    GetChar();
                }
                boolean whiteBeforePart2 = IsWhite(c);
                SkipWhite();
                SimpleRangePart part22 = parseSimpleRangePart();
                String part1And2 = this._formulaString.substring(savePointer - 1, this._pointer - 1);
                if (part22 == null) {
                    if (sheetIden == null) {
                        return parseNonRange(savePointer);
                    }
                    throw new FormulaParseException("Complete area reference expected after sheet name at index " + this._pointer + ".");
                } else if (whiteAfterPart1 || whiteBeforePart2) {
                    if (!part1.isRowOrColumn() && !part22.isRowOrColumn()) {
                        return createAreaRefParseNode(sheetIden, part1, part22);
                    }
                    throw new FormulaParseException("Dotted range (full row or column) expression '" + part1And2 + "' must not contain whitespace.");
                } else if (dotCount == 1 && part1.isRow() && part22.isRow()) {
                    return parseNonRange(savePointer);
                } else {
                    if ((!part1.isRowOrColumn() && !part22.isRowOrColumn()) || dotCount == 2) {
                        return createAreaRefParseNode(sheetIden, part1, part22);
                    }
                    throw new FormulaParseException("Dotted range (full row or column) expression '" + part1And2 + "' must have exactly 2 dots.");
                }
            } else if (part1.isCell() && isValidCellReference(part1.getRep())) {
                return createAreaRefParseNode(sheetIden, part1, null);
            } else {
                if (sheetIden == null) {
                    return parseNonRange(savePointer);
                }
                throw new FormulaParseException("Second part of cell reference expected after sheet name at index " + this._pointer + ".");
            }
        } else if (sheetIden == null) {
            return parseNonRange(savePointer);
        } else {
            if (this.look == '#') {
                return new ParseNode(ErrPtg.valueOf(parseErrorLiteral()));
            }
            String name = parseAsName();
            if (name.length() != 0) {
                Ptg nameXPtg = this._book.getNameXPtg(name, sheetIden);
                if (nameXPtg != null) {
                    return new ParseNode(nameXPtg);
                }
                throw new FormulaParseException("Specified name '" + name + "' for sheet " + sheetIden.asFormulaString() + " not found");
            }
            throw new FormulaParseException("Cell reference or Named Range expected after sheet name at index " + this._pointer + ".");
        }
    }

    /* JADX INFO: Multiple debug info for r9v7 int: [D('startColumnName' java.lang.String), D('savePtr1' int)] */
    private ParseNode parseStructuredReference(String tableName) {
        boolean isThisRowSpec;
        boolean isDataSpec;
        boolean isHeadersSpec;
        boolean isAllSpec;
        boolean isTotalsSpec;
        boolean isThisRow;
        int startCol;
        String endColumnName;
        String startColumnName;
        int nColQuantifiers;
        if (this._ssVersion.equals(SpreadsheetVersion.EXCEL2007)) {
            Table tbl = this._book.getTable(tableName);
            if (tbl != null) {
                String sheetName = tbl.getSheetName();
                int startCol2 = tbl.getStartColIndex();
                int endCol = tbl.getEndColIndex();
                int startRow = tbl.getStartRowIndex();
                int endRow = tbl.getEndRowIndex();
                int savePtr0 = this._pointer;
                GetChar();
                boolean isTotalsSpec2 = false;
                boolean isThisRowSpec2 = false;
                boolean isDataSpec2 = false;
                boolean isHeadersSpec2 = false;
                boolean isAllSpec2 = false;
                int nSpecQuantifiers = 0;
                while (true) {
                    int savePtr1 = this._pointer;
                    boolean isTotalsSpec3 = isTotalsSpec2;
                    String specName = parseAsSpecialQuantifier();
                    isThisRowSpec = isThisRowSpec2;
                    isDataSpec = isDataSpec2;
                    isHeadersSpec = isHeadersSpec2;
                    isAllSpec = isAllSpec2;
                    if (specName != null) {
                        if (specName.equals(specAll)) {
                            isAllSpec = true;
                        } else if (specName.equals(specData)) {
                            isDataSpec = true;
                        } else if (specName.equals(specHeaders)) {
                            isHeadersSpec = true;
                        } else if (specName.equals(specThisRow)) {
                            isThisRowSpec = true;
                        } else if (specName.equals(specTotals)) {
                            isTotalsSpec3 = true;
                        } else {
                            throw new FormulaParseException("Unknown special quantifier " + specName);
                        }
                        int nSpecQuantifiers2 = nSpecQuantifiers + 1;
                        if (this.look != 44) {
                            isTotalsSpec = isTotalsSpec3;
                            nSpecQuantifiers = nSpecQuantifiers2;
                            break;
                        }
                        GetChar();
                        isTotalsSpec2 = isTotalsSpec3;
                        isThisRowSpec2 = isThisRowSpec;
                        isDataSpec2 = isDataSpec;
                        isHeadersSpec2 = isHeadersSpec;
                        isAllSpec2 = isAllSpec;
                        sheetName = sheetName;
                        nSpecQuantifiers = nSpecQuantifiers2;
                    } else {
                        resetPointer(savePtr1);
                        isTotalsSpec = isTotalsSpec3;
                        break;
                    }
                }
                SkipWhite();
                boolean isTotalsSpec4 = isTotalsSpec;
                if (this.look == '@') {
                    isThisRow = true;
                    GetChar();
                } else {
                    isThisRow = false;
                }
                int nColQuantifiers2 = 0;
                int savePtr12 = this._pointer;
                String startColumnName2 = parseAsColumnQuantifier();
                if (startColumnName2 == null) {
                    resetPointer(savePtr12);
                    startCol = startCol2;
                    endColumnName = null;
                } else {
                    nColQuantifiers2 = 0 + 1;
                    int savePtr13 = this.look;
                    startCol = startCol2;
                    if (savePtr13 == 44) {
                        throw new FormulaParseException("The formula " + this._formulaString + "is illegal: you should not use ',' with column quantifiers");
                    } else if (savePtr13 == 58) {
                        GetChar();
                        String endColumnName2 = parseAsColumnQuantifier();
                        nColQuantifiers2++;
                        if (endColumnName2 != null) {
                            endColumnName = endColumnName2;
                        } else {
                            throw new FormulaParseException("The formula " + this._formulaString + "is illegal: the string after ':' must be column quantifier");
                        }
                    } else {
                        endColumnName = null;
                    }
                }
                if (nColQuantifiers2 == 0 && nSpecQuantifiers == 0) {
                    resetPointer(savePtr0);
                    savePtr0 = this._pointer;
                    String startColumnName3 = parseAsColumnQuantifier();
                    if (startColumnName3 != null) {
                        nColQuantifiers = nColQuantifiers2 + 1;
                        startColumnName = startColumnName3;
                    } else {
                        resetPointer(savePtr0);
                        String name = parseAsSpecialQuantifier();
                        if (name != null) {
                            if (name.equals(specAll)) {
                                isAllSpec = true;
                            } else if (name.equals(specData)) {
                                isDataSpec = true;
                            } else if (name.equals(specHeaders)) {
                                isHeadersSpec = true;
                            } else if (name.equals(specThisRow)) {
                                isThisRowSpec = true;
                            } else if (name.equals(specTotals)) {
                                isTotalsSpec4 = true;
                            } else {
                                throw new FormulaParseException("Unknown special quantifier " + name);
                            }
                            nSpecQuantifiers++;
                            savePtr0 = savePtr0;
                            nColQuantifiers = nColQuantifiers2;
                            startColumnName = startColumnName3;
                        } else {
                            throw new FormulaParseException("The formula " + this._formulaString + " is illegal");
                        }
                    }
                } else {
                    Match(']');
                    nColQuantifiers = nColQuantifiers2;
                    startColumnName = startColumnName2;
                }
                if (isTotalsSpec4 && !tbl.isHasTotalsRow()) {
                    return new ParseNode(ErrPtg.REF_INVALID);
                }
                if (isThisRow || isThisRowSpec) {
                    int i = this._rowIndex;
                    if (i >= startRow) {
                        if (endRow < i) {
                        }
                    }
                    if (i >= 0) {
                        return new ParseNode(ErrPtg.VALUE_INVALID);
                    }
                    throw new FormulaParseException("Formula contained [#This Row] or [@] structured reference but this row < 0. Row index must be specified for row-referencing structured references.");
                }
                int actualStartRow = startRow;
                int actualEndRow = endRow;
                int actualStartCol = startCol;
                int actualEndCol = endCol;
                if (nSpecQuantifiers > 0) {
                    if (nSpecQuantifiers != 1 || !isAllSpec) {
                        if (!isDataSpec || !isHeadersSpec) {
                            if (isDataSpec && isTotalsSpec4) {
                                actualStartRow = startRow + 1;
                            } else if (nSpecQuantifiers == 1 && isDataSpec) {
                                actualStartRow = startRow + 1;
                                if (tbl.isHasTotalsRow()) {
                                    actualEndRow = endRow - 1;
                                }
                            } else if (nSpecQuantifiers == 1 && isHeadersSpec) {
                                actualEndRow = actualStartRow;
                            } else if (nSpecQuantifiers == 1 && isTotalsSpec4) {
                                actualStartRow = actualEndRow;
                            } else if ((nSpecQuantifiers != 1 || !isThisRowSpec) && !isThisRow) {
                                throw new FormulaParseException("The formula " + this._formulaString + " is illegal");
                            } else {
                                actualStartRow = this._rowIndex;
                                actualEndRow = this._rowIndex;
                            }
                        } else if (tbl.isHasTotalsRow()) {
                            actualEndRow = endRow - 1;
                        }
                    }
                } else if (isThisRow) {
                    actualStartRow = this._rowIndex;
                    actualEndRow = this._rowIndex;
                } else {
                    actualStartRow++;
                }
                if (nColQuantifiers == 2) {
                    if (startColumnName == null || endColumnName == null) {
                        throw new IllegalStateException("Fatal error");
                    }
                    int startIdx = tbl.findColumnIndex(startColumnName);
                    int endIdx = tbl.findColumnIndex(endColumnName);
                    if (startIdx == -1 || endIdx == -1) {
                        throw new FormulaParseException("One of the columns " + startColumnName + ", " + endColumnName + " doesn't exist in table " + tbl.getName());
                    }
                    actualStartCol = startCol + startIdx;
                    actualEndCol = startCol + endIdx;
                } else if (nColQuantifiers == 1 && !isThisRow) {
                    if (startColumnName != null) {
                        int idx = tbl.findColumnIndex(startColumnName);
                        if (idx != -1) {
                            actualStartCol = startCol + idx;
                            actualEndCol = actualStartCol;
                        } else {
                            throw new FormulaParseException("The column " + startColumnName + " doesn't exist in table " + tbl.getName());
                        }
                    } else {
                        throw new IllegalStateException("Fatal error");
                    }
                }
                return new ParseNode(this._book.get3DReferencePtg(new AreaReference(new CellReference(actualStartRow, actualStartCol), new CellReference(actualEndRow, actualEndCol)), new SheetIdentifier(null, new NameIdentifier(sheetName, true))));
            }
            throw new FormulaParseException("Illegal table name: '" + tableName + "'");
        }
        throw new FormulaParseException("Structured references work only on XSSF (Excel 2007+)!");
    }

    private String parseAsColumnQuantifier() {
        if (this.look != '[') {
            return null;
        }
        GetChar();
        char c = this.look;
        if (c == '#') {
            return null;
        }
        if (c == '@') {
            GetChar();
        }
        StringBuilder name = new StringBuilder();
        while (true) {
            char c2 = this.look;
            if (c2 != ']') {
                name.append(c2);
                GetChar();
            } else {
                Match(']');
                return name.toString();
            }
        }
    }

    private String parseAsSpecialQuantifier() {
        if (this.look != '[') {
            return null;
        }
        GetChar();
        if (this.look != '#') {
            return null;
        }
        GetChar();
        String name = parseAsName();
        if (name.equals("This")) {
            name = name + ' ' + parseAsName();
        }
        Match(']');
        return name;
    }

    private ParseNode parseNonRange(int savePointer) {
        resetPointer(savePointer);
        if (Character.isDigit(this.look)) {
            return new ParseNode(parseNumber());
        }
        if (this.look == '\"') {
            return new ParseNode(new StringPtg(parseStringLiteral()));
        }
        String name = parseAsName();
        char c = this.look;
        if (c == '(') {
            return function(name);
        }
        if (c == '[') {
            return parseStructuredReference(name);
        }
        if (name.equalsIgnoreCase("TRUE") || name.equalsIgnoreCase("FALSE")) {
            return new ParseNode(BoolPtg.valueOf(name.equalsIgnoreCase("TRUE")));
        }
        FormulaParsingWorkbook formulaParsingWorkbook = this._book;
        if (formulaParsingWorkbook != null) {
            EvaluationName evalName = formulaParsingWorkbook.getName(name, this._sheetIndex);
            if (evalName == null) {
                throw new FormulaParseException("Specified named range '" + name + "' does not exist in the current workbook.");
            } else if (evalName.isRange()) {
                return new ParseNode(evalName.createPtg());
            } else {
                throw new FormulaParseException("Specified name '" + name + "' is not a range as expected.");
            }
        } else {
            throw new IllegalStateException("Need book to evaluate name '" + name + "'");
        }
    }

    private String parseAsName() {
        char c;
        StringBuilder sb = new StringBuilder();
        if (Character.isLetter(this.look) || (c = this.look) == '_' || c == '\\') {
            while (isValidDefinedNameChar(this.look)) {
                sb.append(this.look);
                GetChar();
            }
            SkipWhite();
            return sb.toString();
        }
        throw expected("number, string, defined name, or data table");
    }

    private static boolean isValidDefinedNameChar(char ch) {
        if (Character.isLetterOrDigit(ch)) {
            return true;
        }
        switch (ch) {
            case '.':
            case '?':
            case '\\':
            case '_':
                return true;
            default:
                return false;
        }
    }

    private ParseNode createAreaRefParseNode(SheetIdentifier sheetIden, SimpleRangePart part1, SimpleRangePart part2) throws FormulaParseException {
        Ptg ptg;
        if (part2 == null) {
            CellReference cr = part1.getCellReference();
            if (sheetIden == null) {
                ptg = new RefPtg(cr);
            } else {
                ptg = this._book.get3DReferencePtg(cr, sheetIden);
            }
        } else {
            AreaReference areaRef = createAreaRef(part1, part2);
            if (sheetIden == null) {
                ptg = new AreaPtg(areaRef);
            } else {
                ptg = this._book.get3DReferencePtg(areaRef, sheetIden);
            }
        }
        return new ParseNode(ptg);
    }

    private AreaReference createAreaRef(SimpleRangePart part1, SimpleRangePart part2) {
        if (!part1.isCompatibleForArea(part2)) {
            throw new FormulaParseException("has incompatible parts: '" + part1.getRep() + "' and '" + part2.getRep() + "'.");
        } else if (part1.isRow()) {
            return AreaReference.getWholeRow(this._ssVersion, part1.getRep(), part2.getRep());
        } else {
            if (part1.isColumn()) {
                return AreaReference.getWholeColumn(this._ssVersion, part1.getRep(), part2.getRep());
            }
            return new AreaReference(part1.getCellReference(), part2.getCellReference());
        }
    }

    private SimpleRangePart parseSimpleRangePart() {
        int ptr = this._pointer - 1;
        boolean hasDigits = false;
        boolean hasLetters = false;
        while (ptr < this._formulaLength) {
            char ch = this._formulaString.charAt(ptr);
            if (!Character.isDigit(ch)) {
                if (!Character.isLetter(ch)) {
                    if (!(ch == '$' || ch == '_')) {
                        break;
                    }
                } else {
                    hasLetters = true;
                }
            } else {
                hasDigits = true;
            }
            ptr++;
        }
        int i = this._pointer;
        if (ptr <= i - 1) {
            return null;
        }
        String rep = this._formulaString.substring(i - 1, ptr);
        if (!CELL_REF_PATTERN.matcher(rep).matches()) {
            return null;
        }
        if (!hasLetters || !hasDigits) {
            if (hasLetters) {
                if (!CellReference.isColumnWithinRange(rep.replace("$", ""), this._ssVersion)) {
                    return null;
                }
            } else if (!hasDigits) {
                return null;
            } else {
                try {
                    int i2 = Integer.parseInt(rep.replace("$", ""));
                    if (i2 < 1 || i2 > this._ssVersion.getMaxRows()) {
                        return null;
                    }
                } catch (NumberFormatException e) {
                    return null;
                }
            }
        } else if (!isValidCellReference(rep)) {
            return null;
        }
        resetPointer(ptr + 1);
        return new SimpleRangePart(rep, hasLetters, hasDigits);
    }

    /* access modifiers changed from: private */
    public static final class SimpleRangePart {
        private final String _rep;
        private final Type _type;

        /* access modifiers changed from: private */
        public enum Type {
            CELL,
            ROW,
            COLUMN;

            public static Type get(boolean hasLetters, boolean hasDigits) {
                if (hasLetters) {
                    return hasDigits ? CELL : COLUMN;
                }
                if (hasDigits) {
                    return ROW;
                }
                throw new IllegalArgumentException("must have either letters or numbers");
            }
        }

        public SimpleRangePart(String rep, boolean hasLetters, boolean hasNumbers) {
            this._rep = rep;
            this._type = Type.get(hasLetters, hasNumbers);
        }

        public boolean isCell() {
            return this._type == Type.CELL;
        }

        public boolean isRowOrColumn() {
            return this._type != Type.CELL;
        }

        public CellReference getCellReference() {
            if (this._type == Type.CELL) {
                return new CellReference(this._rep);
            }
            throw new IllegalStateException("Not applicable to this type");
        }

        public boolean isColumn() {
            return this._type == Type.COLUMN;
        }

        public boolean isRow() {
            return this._type == Type.ROW;
        }

        public String getRep() {
            return this._rep;
        }

        public boolean isCompatibleForArea(SimpleRangePart part2) {
            return this._type == part2._type;
        }

        public String toString() {
            StringBuilder sb = new StringBuilder(64);
            sb.append(getClass().getName());
            sb.append(" [");
            sb.append(this._rep);
            sb.append("]");
            return sb.toString();
        }
    }

    private SheetIdentifier parseSheetName() {
        String bookName;
        if (this.look == '[') {
            StringBuilder sb = new StringBuilder();
            GetChar();
            while (true) {
                char c = this.look;
                if (c == ']') {
                    break;
                }
                sb.append(c);
                GetChar();
            }
            GetChar();
            bookName = sb.toString();
        } else {
            bookName = null;
        }
        char c2 = this.look;
        if (c2 == '\'') {
            StringBuffer sb2 = new StringBuffer();
            Match('\'');
            boolean done = this.look == '\'';
            while (!done) {
                sb2.append(this.look);
                GetChar();
                if (this.look == '\'') {
                    Match('\'');
                    done = this.look != '\'';
                }
            }
            NameIdentifier iden = new NameIdentifier(sb2.toString(), true);
            SkipWhite();
            char c3 = this.look;
            if (c3 == '!') {
                GetChar();
                return new SheetIdentifier(bookName, iden);
            } else if (c3 == ':') {
                return parseSheetRange(bookName, iden);
            } else {
                return null;
            }
        } else if (c2 == '_' || Character.isLetter(c2)) {
            StringBuilder sb3 = new StringBuilder();
            while (isUnquotedSheetNameChar(this.look)) {
                sb3.append(this.look);
                GetChar();
            }
            NameIdentifier iden2 = new NameIdentifier(sb3.toString(), false);
            SkipWhite();
            char c4 = this.look;
            if (c4 == '!') {
                GetChar();
                return new SheetIdentifier(bookName, iden2);
            } else if (c4 == ':') {
                return parseSheetRange(bookName, iden2);
            } else {
                return null;
            }
        } else if (this.look != '!' || bookName == null) {
            return null;
        } else {
            GetChar();
            return new SheetIdentifier(bookName, null);
        }
    }

    private SheetIdentifier parseSheetRange(String bookname, NameIdentifier sheet1Name) {
        GetChar();
        SheetIdentifier sheet2 = parseSheetName();
        if (sheet2 != null) {
            return new SheetRangeIdentifier(bookname, sheet1Name, sheet2.getSheetIdentifier());
        }
        return null;
    }

    private static boolean isUnquotedSheetNameChar(char ch) {
        if (Character.isLetterOrDigit(ch)) {
            return true;
        }
        switch (ch) {
            case '.':
            case '_':
                return true;
            default:
                return false;
        }
    }

    private boolean isValidCellReference(String str) {
        boolean result = true;
        boolean result2 = CellReference.classifyCellReference(str, this._ssVersion) == CellReference.NameType.CELL;
        if (!result2) {
            return result2;
        }
        if (!(FunctionMetadataRegistry.getFunctionByName(str.toUpperCase(Locale.ROOT)) != null)) {
            return result2;
        }
        int savePointer = this._pointer;
        resetPointer(this._pointer + str.length());
        SkipWhite();
        if (this.look == '(') {
            result = false;
        }
        resetPointer(savePointer);
        return result;
    }

    private ParseNode function(String name) {
        Ptg nameToken = null;
        if (!AbstractFunctionPtg.isBuiltInFunctionName(name)) {
            FormulaParsingWorkbook formulaParsingWorkbook = this._book;
            if (formulaParsingWorkbook != null) {
                EvaluationName hName = formulaParsingWorkbook.getName(name, this._sheetIndex);
                if (hName == null) {
                    nameToken = this._book.getNameXPtg(name, null);
                    if (nameToken == null) {
                        POILogger pOILogger = log;
                        if (pOILogger.check(5)) {
                            pOILogger.log(5, "FormulaParser.function: Name '" + name + "' is completely unknown in the current workbook.");
                        }
                        switch (AnonymousClass1.$SwitchMap$org$apache$poi$ss$SpreadsheetVersion[this._book.getSpreadsheetVersion().ordinal()]) {
                            case 1:
                                addName(name);
                                nameToken = this._book.getName(name, this._sheetIndex).createPtg();
                                break;
                            case 2:
                                nameToken = new NameXPxg(name);
                                break;
                            default:
                                throw new IllegalStateException("Unexpected spreadsheet version: " + this._book.getSpreadsheetVersion().name());
                        }
                    }
                } else if (hName.isFunctionName()) {
                    nameToken = hName.createPtg();
                } else {
                    throw new FormulaParseException("Attempt to use name '" + name + "' as a function, but defined name in workbook does not refer to a function");
                }
            } else {
                throw new IllegalStateException("Need book to evaluate name '" + name + "'");
            }
        }
        Match('(');
        ParseNode[] args = Arguments();
        Match(')');
        return getFunction(name, nameToken, args);
    }

    /* access modifiers changed from: package-private */
    /* renamed from: org.apache.poi.ss.formula.FormulaParser$1  reason: invalid class name */
    public static /* synthetic */ class AnonymousClass1 {
        static final /* synthetic */ int[] $SwitchMap$org$apache$poi$ss$SpreadsheetVersion;

        static {
            int[] iArr = new int[SpreadsheetVersion.values().length];
            $SwitchMap$org$apache$poi$ss$SpreadsheetVersion = iArr;
            try {
                iArr[SpreadsheetVersion.EXCEL97.ordinal()] = 1;
            } catch (NoSuchFieldError e) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$SpreadsheetVersion[SpreadsheetVersion.EXCEL2007.ordinal()] = 2;
            } catch (NoSuchFieldError e2) {
            }
        }
    }

    private final void addName(String functionName) {
        Name name = this._book.createName();
        name.setFunction(true);
        name.setNameName(functionName);
        name.setSheetIndex(this._sheetIndex);
    }

    private ParseNode getFunction(String name, Ptg namePtg, ParseNode[] args) {
        AbstractFunctionPtg retval;
        FunctionMetadata fm = FunctionMetadataRegistry.getFunctionByName(name.toUpperCase(Locale.ROOT));
        int numArgs = args.length;
        if (fm == null) {
            if (namePtg != null) {
                ParseNode[] allArgs = new ParseNode[(numArgs + 1)];
                allArgs[0] = new ParseNode(namePtg);
                System.arraycopy(args, 0, allArgs, 1, numArgs);
                return new ParseNode(FuncVarPtg.create(name, numArgs + 1), allArgs);
            }
            throw new IllegalStateException("NamePtg must be supplied for external functions");
        } else if (namePtg == null) {
            boolean isVarArgs = !fm.hasFixedArgsLength();
            int funcIx = fm.getIndex();
            if (funcIx == 4 && args.length == 1) {
                return new ParseNode(AttrPtg.getSumSingle(), args);
            }
            validateNumArgs(args.length, fm);
            if (isVarArgs) {
                retval = FuncVarPtg.create(name, numArgs);
            } else {
                retval = FuncPtg.create(funcIx);
            }
            return new ParseNode(retval, args);
        } else {
            throw new IllegalStateException("NamePtg no applicable to internal functions");
        }
    }

    private void validateNumArgs(int numArgs, FunctionMetadata fm) {
        int maxArgs;
        String msg;
        String msg2;
        if (numArgs < fm.getMinParams()) {
            String msg3 = "Too few arguments to function '" + fm.getName() + "'. ";
            if (fm.hasFixedArgsLength()) {
                msg2 = msg3 + "Expected " + fm.getMinParams();
            } else {
                msg2 = msg3 + "At least " + fm.getMinParams() + " were expected";
            }
            throw new FormulaParseException(msg2 + " but got " + numArgs + ".");
        }
        if (fm.hasUnlimitedVarags()) {
            FormulaParsingWorkbook formulaParsingWorkbook = this._book;
            if (formulaParsingWorkbook != null) {
                maxArgs = formulaParsingWorkbook.getSpreadsheetVersion().getMaxFunctionArgs();
            } else {
                maxArgs = fm.getMaxParams();
            }
        } else {
            maxArgs = fm.getMaxParams();
        }
        if (numArgs > maxArgs) {
            String msg4 = "Too many arguments to function '" + fm.getName() + "'. ";
            if (fm.hasFixedArgsLength()) {
                msg = msg4 + "Expected " + maxArgs;
            } else {
                msg = msg4 + "At most " + maxArgs + " were expected";
            }
            throw new FormulaParseException(msg + " but got " + numArgs + ".");
        }
    }

    private static boolean isArgumentDelimiter(char ch) {
        return ch == ',' || ch == ')';
    }

    private ParseNode[] Arguments() {
        List<ParseNode> temp = new ArrayList<>(2);
        SkipWhite();
        if (this.look == ')') {
            return ParseNode.EMPTY_ARRAY;
        }
        boolean missedPrevArg = true;
        while (true) {
            SkipWhite();
            if (isArgumentDelimiter(this.look)) {
                if (missedPrevArg) {
                    temp.add(new ParseNode(MissingArgPtg.instance));
                }
                if (this.look == ')') {
                    ParseNode[] result = new ParseNode[temp.size()];
                    temp.toArray(result);
                    return result;
                }
                Match(',');
                missedPrevArg = true;
            } else {
                temp.add(comparisonExpression());
                missedPrevArg = false;
                SkipWhite();
                if (!isArgumentDelimiter(this.look)) {
                    throw expected("',' or ')'");
                }
            }
        }
    }

    private ParseNode powerFactor() {
        ParseNode result = percentFactor();
        while (true) {
            SkipWhite();
            if (this.look != '^') {
                return result;
            }
            Match('^');
            result = new ParseNode(PowerPtg.instance, result, percentFactor());
        }
    }

    private ParseNode percentFactor() {
        ParseNode result = parseSimpleFactor();
        while (true) {
            SkipWhite();
            if (this.look != '%') {
                return result;
            }
            Match('%');
            result = new ParseNode(PercentPtg.instance, result);
        }
    }

    private ParseNode parseSimpleFactor() {
        char c;
        SkipWhite();
        char c2 = this.look;
        switch (c2) {
            case '\"':
                return new ParseNode(new StringPtg(parseStringLiteral()));
            case '#':
                return new ParseNode(ErrPtg.valueOf(parseErrorLiteral()));
            case '(':
                Match('(');
                ParseNode inside = unionExpression();
                Match(')');
                return new ParseNode(ParenthesisPtg.instance, inside);
            case '+':
                Match('+');
                return parseUnary(true);
            case '-':
                Match('-');
                return parseUnary(false);
            case '{':
                Match('{');
                ParseNode arrayNode = parseArray();
                Match('}');
                return arrayNode;
            default:
                if (IsAlpha(c2) || Character.isDigit(this.look) || (c = this.look) == '\'' || c == '[' || c == '_' || c == '\\') {
                    return parseRangeExpression();
                }
                if (c == '.') {
                    return new ParseNode(parseNumber());
                }
                throw expected("cell ref or constant literal");
        }
    }

    private ParseNode parseUnary(boolean isPlus) {
        boolean numberFollows = IsDigit(this.look) || this.look == '.';
        ParseNode factor = powerFactor();
        if (numberFollows) {
            Ptg token = factor.getToken();
            if (token instanceof NumberPtg) {
                if (isPlus) {
                    return factor;
                }
                return new ParseNode(new NumberPtg(-((NumberPtg) token).getValue()));
            } else if (token instanceof IntPtg) {
                if (isPlus) {
                    return factor;
                }
                return new ParseNode(new NumberPtg((double) (-((IntPtg) token).getValue())));
            }
        }
        return new ParseNode(isPlus ? UnaryPlusPtg.instance : UnaryMinusPtg.instance, factor);
    }

    private ParseNode parseArray() {
        List<Object[]> rowsData = new ArrayList<>();
        while (true) {
            rowsData.add(parseArrayRow());
            char c = this.look;
            if (c == '}') {
                Object[][] values2d = new Object[rowsData.size()][];
                rowsData.toArray(values2d);
                checkRowLengths(values2d, values2d[0].length);
                return new ParseNode(new ArrayPtg(values2d));
            } else if (c == ';') {
                Match(';');
            } else {
                throw expected("'}' or ';'");
            }
        }
    }

    private void checkRowLengths(Object[][] values2d, int nColumns) {
        for (int i = 0; i < values2d.length; i++) {
            int rowLen = values2d[i].length;
            if (rowLen != nColumns) {
                throw new FormulaParseException("Array row " + i + " has length " + rowLen + " but row 0 has length " + nColumns);
            }
        }
    }

    private Object[] parseArrayRow() {
        List<Object> temp = new ArrayList<>();
        while (true) {
            temp.add(parseArrayItem());
            SkipWhite();
            switch (this.look) {
                case ',':
                    Match(',');
                case ';':
                case '}':
                    Object[] result = new Object[temp.size()];
                    temp.toArray(result);
                    return result;
                default:
                    throw expected("'}' or ','");
            }
        }
    }

    private Object parseArrayItem() {
        SkipWhite();
        switch (this.look) {
            case '\"':
                return parseStringLiteral();
            case '#':
                return ErrorConstant.valueOf(parseErrorLiteral());
            case '-':
                Match('-');
                SkipWhite();
                return convertArrayNumber(parseNumber(), false);
            case 'F':
            case 'T':
            case 'f':
            case 't':
                return parseBooleanLiteral();
            default:
                return convertArrayNumber(parseNumber(), true);
        }
    }

    private Boolean parseBooleanLiteral() {
        String iden = parseUnquotedIdentifier();
        if ("TRUE".equalsIgnoreCase(iden)) {
            return Boolean.TRUE;
        }
        if ("FALSE".equalsIgnoreCase(iden)) {
            return Boolean.FALSE;
        }
        throw expected("'TRUE' or 'FALSE'");
    }

    private static Double convertArrayNumber(Ptg ptg, boolean isPositive) {
        double value;
        if (ptg instanceof IntPtg) {
            value = (double) ((IntPtg) ptg).getValue();
        } else if (ptg instanceof NumberPtg) {
            value = ((NumberPtg) ptg).getValue();
        } else {
            throw new RuntimeException("Unexpected ptg (" + ptg.getClass().getName() + ")");
        }
        if (!isPositive) {
            value = -value;
        }
        return new Double(value);
    }

    private Ptg parseNumber() {
        String number2 = null;
        String exponent = null;
        String number1 = GetNum();
        if (this.look == '.') {
            GetChar();
            number2 = GetNum();
        }
        if (this.look == 'E') {
            GetChar();
            String sign = "";
            char c = this.look;
            if (c == '+') {
                GetChar();
            } else if (c == '-') {
                GetChar();
                sign = "-";
            }
            String number = GetNum();
            if (number != null) {
                exponent = sign + number;
            } else {
                throw expected("Integer");
            }
        }
        if (number1 != null || number2 != null) {
            return getNumberPtgFromString(number1, number2, exponent);
        }
        throw expected("Integer");
    }

    private int parseErrorLiteral() {
        Match('#');
        String part1 = parseUnquotedIdentifier().toUpperCase(Locale.ROOT);
        if (part1 != null) {
            switch (part1.charAt(0)) {
                case 'D':
                    FormulaError fe = FormulaError.DIV0;
                    if (part1.equals("DIV")) {
                        Match('/');
                        Match('0');
                        Match('!');
                        return fe.getCode();
                    }
                    throw expected(fe.getString());
                case 'N':
                    FormulaError fe2 = FormulaError.NAME;
                    if (part1.equals(fe2.name())) {
                        Match('?');
                        return fe2.getCode();
                    }
                    FormulaError fe3 = FormulaError.NUM;
                    if (part1.equals(fe3.name())) {
                        Match('!');
                        return fe3.getCode();
                    }
                    FormulaError fe4 = FormulaError.NULL;
                    if (part1.equals(fe4.name())) {
                        Match('!');
                        return fe4.getCode();
                    }
                    FormulaError fe5 = FormulaError.NA;
                    if (part1.equals("N")) {
                        Match('/');
                        char c = this.look;
                        if (c == 'A' || c == 'a') {
                            Match(c);
                            return fe5.getCode();
                        }
                        throw expected(fe5.getString());
                    }
                    throw expected("#NAME?, #NUM!, #NULL! or #N/A");
                case 'R':
                    FormulaError fe6 = FormulaError.REF;
                    if (part1.equals(fe6.name())) {
                        Match('!');
                        return fe6.getCode();
                    }
                    throw expected(fe6.getString());
                case 'V':
                    FormulaError fe7 = FormulaError.VALUE;
                    if (part1.equals(fe7.name())) {
                        Match('!');
                        return fe7.getCode();
                    }
                    throw expected(fe7.getString());
                default:
                    throw expected("#VALUE!, #REF!, #DIV/0!, #NAME?, #NUM!, #NULL! or #N/A");
            }
        } else {
            throw expected("remainder of error constant literal");
        }
    }

    private String parseUnquotedIdentifier() {
        if (this.look != '\'') {
            StringBuilder sb = new StringBuilder();
            while (true) {
                if (!Character.isLetterOrDigit(this.look) && this.look != '.') {
                    break;
                }
                sb.append(this.look);
                GetChar();
            }
            if (sb.length() < 1) {
                return null;
            }
            return sb.toString();
        }
        throw expected("unquoted identifier");
    }

    private static Ptg getNumberPtgFromString(String number1, String number2, String exponent) {
        StringBuffer number = new StringBuffer();
        if (number2 == null) {
            number.append(number1);
            if (exponent != null) {
                number.append('E');
                number.append(exponent);
            }
            String numberStr = number.toString();
            try {
                int intVal = Integer.parseInt(numberStr);
                if (IntPtg.isInRange(intVal)) {
                    return new IntPtg(intVal);
                }
                return new NumberPtg(numberStr);
            } catch (NumberFormatException e) {
                return new NumberPtg(numberStr);
            }
        } else {
            if (number1 != null) {
                number.append(number1);
            }
            number.append('.');
            number.append(number2);
            if (exponent != null) {
                number.append('E');
                number.append(exponent);
            }
            return new NumberPtg(number.toString());
        }
    }

    private String parseStringLiteral() {
        Match('\"');
        StringBuffer token = new StringBuffer();
        while (true) {
            if (this.look == '\"') {
                GetChar();
                if (this.look != '\"') {
                    return token.toString();
                }
            }
            token.append(this.look);
            GetChar();
        }
    }

    private ParseNode Term() {
        Ptg operator;
        ParseNode result = powerFactor();
        while (true) {
            SkipWhite();
            switch (this.look) {
                case '*':
                    Match('*');
                    operator = MultiplyPtg.instance;
                    break;
                case '/':
                    Match('/');
                    operator = DividePtg.instance;
                    break;
                default:
                    return result;
            }
            result = new ParseNode(operator, result, powerFactor());
        }
    }

    private ParseNode unionExpression() {
        ParseNode result = intersectionExpression();
        boolean hasUnions = false;
        while (true) {
            SkipWhite();
            switch (this.look) {
                case ',':
                    GetChar();
                    hasUnions = true;
                    result = new ParseNode(UnionPtg.instance, result, intersectionExpression());
                default:
                    if (hasUnions) {
                        return augmentWithMemPtg(result);
                    }
                    return result;
            }
        }
    }

    private ParseNode intersectionExpression() {
        ParseNode result = comparisonExpression();
        boolean hasIntersections = false;
        while (true) {
            SkipWhite();
            if (!this._inIntersection) {
                break;
            }
            try {
                result = new ParseNode(IntersectionPtg.instance, result, comparisonExpression());
                hasIntersections = true;
            } catch (FormulaParseException e) {
                resetPointer(this._pointer);
            }
        }
        if (hasIntersections) {
            return augmentWithMemPtg(result);
        }
        return result;
    }

    private ParseNode comparisonExpression() {
        ParseNode result = concatExpression();
        while (true) {
            SkipWhite();
            switch (this.look) {
                case '<':
                case '=':
                case '>':
                    result = new ParseNode(getComparisonToken(), result, concatExpression());
                default:
                    return result;
            }
        }
    }

    private Ptg getComparisonToken() {
        char c = this.look;
        if (c == '=') {
            Match(c);
            return EqualPtg.instance;
        }
        boolean isGreater = c == '>';
        Match(c);
        if (!isGreater) {
            switch (this.look) {
                case '=':
                    Match('=');
                    return LessEqualPtg.instance;
                case '>':
                    Match('>');
                    return NotEqualPtg.instance;
                default:
                    return LessThanPtg.instance;
            }
        } else if (this.look != '=') {
            return GreaterThanPtg.instance;
        } else {
            Match('=');
            return GreaterEqualPtg.instance;
        }
    }

    private ParseNode concatExpression() {
        ParseNode result = additiveExpression();
        while (true) {
            SkipWhite();
            if (this.look != '&') {
                return result;
            }
            Match('&');
            result = new ParseNode(ConcatPtg.instance, result, additiveExpression());
        }
    }

    private ParseNode additiveExpression() {
        Ptg operator;
        ParseNode result = Term();
        while (true) {
            SkipWhite();
            switch (this.look) {
                case '+':
                    Match('+');
                    operator = AddPtg.instance;
                    break;
                case ',':
                default:
                    return result;
                case '-':
                    Match('-');
                    operator = SubtractPtg.instance;
                    break;
            }
            result = new ParseNode(operator, result, Term());
        }
    }

    private void parse() {
        this._pointer = 0;
        GetChar();
        this._rootNode = unionExpression();
        if (this._pointer <= this._formulaLength) {
            throw new FormulaParseException("Unused input [" + this._formulaString.substring(this._pointer - 1) + "] after attempting to parse the formula [" + this._formulaString + "]");
        }
    }

    private Ptg[] getRPNPtg(FormulaType formulaType) {
        new OperandClassTransformer(formulaType).transformFormula(this._rootNode);
        return ParseNode.toTokenArray(this._rootNode);
    }
}
