package org.apache.poi.ss.formula.functions;

import org.apache.poi.ss.formula.TwoDEval;
import org.apache.poi.ss.formula.eval.BoolEval;
import org.apache.poi.ss.formula.eval.ErrorEval;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.MissingArgEval;
import org.apache.poi.ss.formula.eval.OperandResolver;
import org.apache.poi.ss.formula.eval.RefEval;
import org.apache.poi.ss.formula.eval.ValueEval;

public abstract class BooleanFunction implements Function {
    public static final Function AND = new BooleanFunction() {
        /* class org.apache.poi.ss.formula.functions.BooleanFunction.AnonymousClass1 */

        /* access modifiers changed from: protected */
        @Override // org.apache.poi.ss.formula.functions.BooleanFunction
        public boolean getInitialResultValue() {
            return true;
        }

        /* access modifiers changed from: protected */
        @Override // org.apache.poi.ss.formula.functions.BooleanFunction
        public boolean partialEvaluate(boolean cumulativeResult, boolean currentValue) {
            return cumulativeResult && currentValue;
        }
    };
    public static final Function FALSE = new Fixed0ArgFunction() {
        /* class org.apache.poi.ss.formula.functions.BooleanFunction.AnonymousClass3 */

        @Override // org.apache.poi.ss.formula.functions.Function0Arg
        public ValueEval evaluate(int srcRowIndex, int srcColumnIndex) {
            return BoolEval.FALSE;
        }
    };
    public static final Function NOT = new Fixed1ArgFunction() {
        /* class org.apache.poi.ss.formula.functions.BooleanFunction.AnonymousClass5 */

        @Override // org.apache.poi.ss.formula.functions.Function1Arg
        public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval arg0) {
            try {
                boolean z = false;
                Boolean b = OperandResolver.coerceValueToBoolean(OperandResolver.getSingleValue(arg0, srcRowIndex, srcColumnIndex), false);
                if (!(b == null ? false : b.booleanValue())) {
                    z = true;
                }
                return BoolEval.valueOf(z);
            } catch (EvaluationException e) {
                return e.getErrorEval();
            }
        }
    };
    public static final Function OR = new BooleanFunction() {
        /* class org.apache.poi.ss.formula.functions.BooleanFunction.AnonymousClass2 */

        /* access modifiers changed from: protected */
        @Override // org.apache.poi.ss.formula.functions.BooleanFunction
        public boolean getInitialResultValue() {
            return false;
        }

        /* access modifiers changed from: protected */
        @Override // org.apache.poi.ss.formula.functions.BooleanFunction
        public boolean partialEvaluate(boolean cumulativeResult, boolean currentValue) {
            return cumulativeResult || currentValue;
        }
    };
    public static final Function TRUE = new Fixed0ArgFunction() {
        /* class org.apache.poi.ss.formula.functions.BooleanFunction.AnonymousClass4 */

        @Override // org.apache.poi.ss.formula.functions.Function0Arg
        public ValueEval evaluate(int srcRowIndex, int srcColumnIndex) {
            return BoolEval.TRUE;
        }
    };

    /* access modifiers changed from: protected */
    public abstract boolean getInitialResultValue();

    /* access modifiers changed from: protected */
    public abstract boolean partialEvaluate(boolean z, boolean z2);

    @Override // org.apache.poi.ss.formula.functions.Function
    public final ValueEval evaluate(ValueEval[] args, int srcRow, int srcCol) {
        if (args.length < 1) {
            return ErrorEval.VALUE_INVALID;
        }
        try {
            return BoolEval.valueOf(calculate(args));
        } catch (EvaluationException e) {
            return e.getErrorEval();
        }
    }

    private boolean calculate(ValueEval[] args) throws EvaluationException {
        Boolean tempVe;
        boolean result = getInitialResultValue();
        boolean atleastOneNonBlank = false;
        for (ValueEval arg : args) {
            if (arg instanceof TwoDEval) {
                TwoDEval ae = (TwoDEval) arg;
                int height = ae.getHeight();
                int width = ae.getWidth();
                for (int rrIx = 0; rrIx < height; rrIx++) {
                    for (int rcIx = 0; rcIx < width; rcIx++) {
                        Boolean tempVe2 = OperandResolver.coerceValueToBoolean(ae.getValue(rrIx, rcIx), true);
                        if (tempVe2 != null) {
                            result = partialEvaluate(result, tempVe2.booleanValue());
                            atleastOneNonBlank = true;
                        }
                    }
                }
            } else if (arg instanceof RefEval) {
                RefEval re = (RefEval) arg;
                int firstSheetIndex = re.getFirstSheetIndex();
                int lastSheetIndex = re.getLastSheetIndex();
                for (int sIx = firstSheetIndex; sIx <= lastSheetIndex; sIx++) {
                    Boolean tempVe3 = OperandResolver.coerceValueToBoolean(re.getInnerValueEval(sIx), true);
                    if (tempVe3 != null) {
                        result = partialEvaluate(result, tempVe3.booleanValue());
                        atleastOneNonBlank = true;
                    }
                }
            } else {
                if (arg == MissingArgEval.instance) {
                    tempVe = null;
                } else {
                    tempVe = OperandResolver.coerceValueToBoolean(arg, false);
                }
                if (tempVe != null) {
                    result = partialEvaluate(result, tempVe.booleanValue());
                    atleastOneNonBlank = true;
                }
            }
        }
        if (atleastOneNonBlank) {
            return result;
        }
        throw new EvaluationException(ErrorEval.VALUE_INVALID);
    }
}
