package org.apache.poi.ss.formula.functions;

import org.apache.poi.ss.formula.eval.AreaEval;
import org.apache.poi.ss.formula.eval.BlankEval;
import org.apache.poi.ss.formula.eval.ErrorEval;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.NotImplementedException;
import org.apache.poi.ss.formula.eval.NumericValueEval;
import org.apache.poi.ss.formula.eval.OperandResolver;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.StringValueEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.util.NumberComparer;

public final class DStarRunner implements Function3Arg {
    private final DStarAlgorithmEnum algoType;

    public enum DStarAlgorithmEnum {
        DGET,
        DMIN
    }

    /* access modifiers changed from: private */
    public enum operator {
        largerThan,
        largerEqualThan,
        smallerThan,
        smallerEqualThan,
        equal
    }

    public DStarRunner(DStarAlgorithmEnum algorithm) {
        this.algoType = algorithm;
    }

    @Override // org.apache.poi.ss.formula.functions.Function
    public final ValueEval evaluate(ValueEval[] args, int srcRowIndex, int srcColumnIndex) {
        if (args.length == 3) {
            return evaluate(srcRowIndex, srcColumnIndex, args[0], args[1], args[2]);
        }
        return ErrorEval.VALUE_INVALID;
    }

    @Override // org.apache.poi.ss.formula.functions.Function3Arg
    public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval database, ValueEval filterColumn, ValueEval conditionDatabase) {
        IDStarAlgorithm algorithm;
        if (!(database instanceof AreaEval) || !(conditionDatabase instanceof AreaEval)) {
            return ErrorEval.VALUE_INVALID;
        }
        AreaEval db = (AreaEval) database;
        AreaEval cdb = (AreaEval) conditionDatabase;
        try {
            try {
                int fc = getColumnForName(OperandResolver.getSingleValue(filterColumn, srcRowIndex, srcColumnIndex), db);
                if (fc == -1) {
                    return ErrorEval.VALUE_INVALID;
                }
                switch (AnonymousClass1.$SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$DStarAlgorithmEnum[this.algoType.ordinal()]) {
                    case 1:
                        algorithm = new DGet();
                        break;
                    case 2:
                        algorithm = new DMin();
                        break;
                    default:
                        throw new IllegalStateException("Unexpected algorithm type " + this.algoType + " encountered.");
                }
                int height = db.getHeight();
                for (int row = 1; row < height; row++) {
                    try {
                        if (fullfillsConditions(db, row, cdb) && !algorithm.processMatch(resolveReference(db, row, fc))) {
                            return algorithm.getResult();
                        }
                    } catch (EvaluationException e) {
                        return ErrorEval.VALUE_INVALID;
                    }
                }
                return algorithm.getResult();
            } catch (EvaluationException e2) {
                return ErrorEval.VALUE_INVALID;
            }
        } catch (EvaluationException e3) {
            return e3.getErrorEval();
        }
    }

    private static int getColumnForName(ValueEval nameValueEval, AreaEval db) throws EvaluationException {
        return getColumnForString(db, OperandResolver.coerceValueToString(nameValueEval));
    }

    private static int getColumnForString(AreaEval db, String name) throws EvaluationException {
        int width = db.getWidth();
        for (int column = 0; column < width; column++) {
            ValueEval columnNameValueEval = resolveReference(db, 0, column);
            if (!(columnNameValueEval instanceof BlankEval) && !(columnNameValueEval instanceof ErrorEval) && name.equals(OperandResolver.coerceValueToString(columnNameValueEval))) {
                return column;
            }
        }
        return -1;
    }

    private static boolean fullfillsConditions(AreaEval db, int row, AreaEval cdb) throws EvaluationException {
        int height = cdb.getHeight();
        for (int conditionRow = 1; conditionRow < height; conditionRow++) {
            boolean matches = true;
            int width = cdb.getWidth();
            int column = 0;
            while (true) {
                if (column >= width) {
                    break;
                }
                boolean columnCondition = true;
                ValueEval condition = resolveReference(cdb, conditionRow, column);
                if (!(condition instanceof BlankEval)) {
                    ValueEval targetHeader = resolveReference(cdb, 0, column);
                    if (targetHeader instanceof StringValueEval) {
                        if (getColumnForName(targetHeader, db) == -1) {
                            columnCondition = false;
                        }
                        if (columnCondition) {
                            if (!testNormalCondition(resolveReference(db, row, getColumnForName(targetHeader, db)), condition)) {
                                matches = false;
                                break;
                            }
                        } else if (OperandResolver.coerceValueToString(condition).isEmpty()) {
                            throw new EvaluationException(ErrorEval.VALUE_INVALID);
                        } else {
                            throw new NotImplementedException("D* function with formula conditions");
                        }
                    } else {
                        throw new EvaluationException(ErrorEval.VALUE_INVALID);
                    }
                }
                column++;
            }
            if (matches) {
                return true;
            }
        }
        return false;
    }

    private static boolean testNormalCondition(ValueEval value, ValueEval condition) throws EvaluationException {
        boolean itsANumber;
        if (condition instanceof StringEval) {
            String conditionString = ((StringEval) condition).getStringValue();
            if (conditionString.startsWith("<")) {
                String number = conditionString.substring(1);
                if (!number.startsWith("=")) {
                    return testNumericCondition(value, operator.smallerThan, number);
                }
                return testNumericCondition(value, operator.smallerEqualThan, number.substring(1));
            } else if (conditionString.startsWith(">")) {
                String number2 = conditionString.substring(1);
                if (!number2.startsWith("=")) {
                    return testNumericCondition(value, operator.largerThan, number2);
                }
                return testNumericCondition(value, operator.largerEqualThan, number2.substring(1));
            } else {
                String valueString = "";
                if (conditionString.startsWith("=")) {
                    String stringOrNumber = conditionString.substring(1);
                    if (stringOrNumber.isEmpty()) {
                        return value instanceof BlankEval;
                    }
                    try {
                        Integer.parseInt(stringOrNumber);
                        itsANumber = true;
                    } catch (NumberFormatException e) {
                        try {
                            Double.parseDouble(stringOrNumber);
                            itsANumber = true;
                        } catch (NumberFormatException e2) {
                            itsANumber = false;
                        }
                    }
                    if (itsANumber) {
                        return testNumericCondition(value, operator.equal, stringOrNumber);
                    }
                    if (!(value instanceof BlankEval)) {
                        valueString = OperandResolver.coerceValueToString(value);
                    }
                    return stringOrNumber.equals(valueString);
                } else if (conditionString.isEmpty()) {
                    return value instanceof StringEval;
                } else {
                    if (!(value instanceof BlankEval)) {
                        valueString = OperandResolver.coerceValueToString(value);
                    }
                    return valueString.startsWith(conditionString);
                }
            }
        } else if (condition instanceof NumericValueEval) {
            double conditionNumber = ((NumericValueEval) condition).getNumberValue();
            Double valueNumber = getNumberFromValueEval(value);
            if (valueNumber == null) {
                return false;
            }
            if (conditionNumber == valueNumber.doubleValue()) {
                return true;
            }
            return false;
        } else if (!(condition instanceof ErrorEval) || !(value instanceof ErrorEval)) {
            return false;
        } else {
            if (((ErrorEval) condition).getErrorCode() == ((ErrorEval) value).getErrorCode()) {
                return true;
            }
            return false;
        }
    }

    private static boolean testNumericCondition(ValueEval valueEval, operator op, String condition) throws EvaluationException {
        double conditionValue;
        if (!(valueEval instanceof NumericValueEval)) {
            return false;
        }
        double value = ((NumericValueEval) valueEval).getNumberValue();
        try {
            conditionValue = (double) Integer.parseInt(condition);
        } catch (NumberFormatException e) {
            try {
                conditionValue = Double.parseDouble(condition);
            } catch (NumberFormatException e2) {
                throw new EvaluationException(ErrorEval.VALUE_INVALID);
            }
        }
        int result = NumberComparer.compare(value, conditionValue);
        switch (AnonymousClass1.$SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator[op.ordinal()]) {
            case 1:
                if (result > 0) {
                    return true;
                }
                return false;
            case 2:
                if (result >= 0) {
                    return true;
                }
                return false;
            case 3:
                if (result < 0) {
                    return true;
                }
                return false;
            case 4:
                if (result <= 0) {
                    return true;
                }
                return false;
            case 5:
                if (result == 0) {
                    return true;
                }
                return false;
            default:
                return false;
        }
    }

    /* access modifiers changed from: package-private */
    /* renamed from: org.apache.poi.ss.formula.functions.DStarRunner$1  reason: invalid class name */
    public static /* synthetic */ class AnonymousClass1 {
        static final /* synthetic */ int[] $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$DStarAlgorithmEnum;
        static final /* synthetic */ int[] $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator;

        static {
            int[] iArr = new int[operator.values().length];
            $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator = iArr;
            try {
                iArr[operator.largerThan.ordinal()] = 1;
            } catch (NoSuchFieldError e) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator[operator.largerEqualThan.ordinal()] = 2;
            } catch (NoSuchFieldError e2) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator[operator.smallerThan.ordinal()] = 3;
            } catch (NoSuchFieldError e3) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator[operator.smallerEqualThan.ordinal()] = 4;
            } catch (NoSuchFieldError e4) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$operator[operator.equal.ordinal()] = 5;
            } catch (NoSuchFieldError e5) {
            }
            int[] iArr2 = new int[DStarAlgorithmEnum.values().length];
            $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$DStarAlgorithmEnum = iArr2;
            try {
                iArr2[DStarAlgorithmEnum.DGET.ordinal()] = 1;
            } catch (NoSuchFieldError e6) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$formula$functions$DStarRunner$DStarAlgorithmEnum[DStarAlgorithmEnum.DMIN.ordinal()] = 2;
            } catch (NoSuchFieldError e7) {
            }
        }
    }

    private static Double getNumberFromValueEval(ValueEval value) {
        if (value instanceof NumericValueEval) {
            return Double.valueOf(((NumericValueEval) value).getNumberValue());
        }
        if (!(value instanceof StringValueEval)) {
            return null;
        }
        try {
            return Double.valueOf(Double.parseDouble(((StringValueEval) value).getStringValue()));
        } catch (NumberFormatException e) {
            return null;
        }
    }

    private static ValueEval resolveReference(AreaEval db, int dbRow, int dbCol) {
        try {
            return OperandResolver.getSingleValue(db.getValue(dbRow, dbCol), db.getFirstRow() + dbRow, db.getFirstColumn() + dbCol);
        } catch (EvaluationException e) {
            return e.getErrorEval();
        }
    }
}
