package org.apache.poi.ss.usermodel;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DateFormat;
import java.text.DateFormatSymbols;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.poi.ss.format.CellFormat;
import org.apache.poi.ss.format.CellFormatResult;
import org.apache.poi.ss.util.NumberToTextConverter;
import org.apache.poi.util.LocaleUtil;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public class DataFormatter implements Observer {
    private static final Pattern alternateGrouping = Pattern.compile("([#0]([^.#0])[#0]{3})");
    private static final Pattern amPmPattern = Pattern.compile("((A|P)[M/P]*)", 2);
    private static final Pattern colorPattern = Pattern.compile("(\\[BLACK\\])|(\\[BLUE\\])|(\\[CYAN\\])|(\\[GREEN\\])|(\\[MAGENTA\\])|(\\[RED\\])|(\\[WHITE\\])|(\\[YELLOW\\])|(\\[COLOR\\s*\\d\\])|(\\[COLOR\\s*[0-5]\\d\\])", 2);
    private static final Pattern daysAsText = Pattern.compile("([d]{3,})", 2);
    private static final String defaultFractionFractionPartFormat = "#/##";
    private static final String defaultFractionWholePartFormat = "#";
    private static final Pattern fractionPattern = Pattern.compile("(?:([#\\d]+)\\s+)?(#+)\\s*\\/\\s*([#\\d]+)");
    private static final Pattern fractionStripper = Pattern.compile("(\"[^\"]*\")|([^ \\?#\\d\\/]+)");
    private static final String invalidDateTimeString;
    private static final Pattern localePatternGroup = Pattern.compile("(\\[\\$[^-\\]]*-[0-9A-Z]+\\])");
    private static POILogger logger = POILogFactory.getLogger(DataFormatter.class);
    private static final Pattern numPattern = Pattern.compile("[0#]+");
    private DateFormatSymbols dateSymbols;
    private DecimalFormatSymbols decimalSymbols;
    private DateFormat defaultDateformat;
    private Format defaultNumFormat;
    private final boolean emulateCSV;
    private final Map<String, Format> formats;
    private Format generalNumberFormat;
    private Locale locale;
    private final LocaleChangeObservable localeChangedObservable;
    private boolean localeIsAdapting;

    static {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < 255; i++) {
            buf.append('#');
        }
        invalidDateTimeString = buf.toString();
    }

    /* access modifiers changed from: private */
    public class LocaleChangeObservable extends Observable {
        private LocaleChangeObservable() {
        }

        /* synthetic */ LocaleChangeObservable(DataFormatter x0, AnonymousClass1 x1) {
            this();
        }

        /* access modifiers changed from: package-private */
        public void checkForLocaleChange() {
            checkForLocaleChange(LocaleUtil.getUserLocale());
        }

        /* access modifiers changed from: package-private */
        public void checkForLocaleChange(Locale newLocale) {
            if (DataFormatter.this.localeIsAdapting && !newLocale.equals(DataFormatter.this.locale)) {
                super.setChanged();
                notifyObservers(newLocale);
            }
        }
    }

    public DataFormatter() {
        this(false);
    }

    public DataFormatter(boolean emulateCSV2) {
        this(LocaleUtil.getUserLocale(), true, emulateCSV2);
    }

    public DataFormatter(Locale locale2) {
        this(locale2, false);
    }

    public DataFormatter(Locale locale2, boolean emulateCSV2) {
        this(locale2, false, emulateCSV2);
    }

    private DataFormatter(Locale locale2, boolean localeIsAdapting2, boolean emulateCSV2) {
        this.formats = new HashMap();
        LocaleChangeObservable localeChangeObservable = new LocaleChangeObservable(this, null);
        this.localeChangedObservable = localeChangeObservable;
        this.localeIsAdapting = true;
        localeChangeObservable.addObserver(this);
        localeChangeObservable.checkForLocaleChange(locale2);
        this.localeIsAdapting = localeIsAdapting2;
        this.emulateCSV = emulateCSV2;
    }

    private Format getFormat(Cell cell) {
        if (cell.getCellStyle() == null) {
            return null;
        }
        int formatIndex = cell.getCellStyle().getDataFormat();
        String formatStr = cell.getCellStyle().getDataFormatString();
        if (formatStr == null || formatStr.trim().length() == 0) {
            return null;
        }
        return getFormat(cell.getNumericCellValue(), formatIndex, formatStr);
    }

    private Format getFormat(double cellValue, int formatIndex, String formatStrIn) {
        this.localeChangedObservable.checkForLocaleChange();
        String formatStr = formatStrIn;
        if (formatStr.contains(";") && formatStr.indexOf(59) != formatStr.lastIndexOf(59)) {
            try {
                CellFormat cfmt = CellFormat.getInstance(formatStr);
                Object cellValueO = Double.valueOf(cellValue);
                if (DateUtil.isADateFormat(formatIndex, formatStr) && ((Double) cellValueO).doubleValue() != 0.0d) {
                    cellValueO = DateUtil.getJavaDate(cellValue);
                }
                return new CellFormatResultWrapper(this, cfmt.apply(cellValueO), null);
            } catch (Exception e) {
                POILogger pOILogger = logger;
                pOILogger.log(5, "Formatting failed for format " + formatStr + ", falling back", e);
            }
        }
        if (this.emulateCSV && cellValue == 0.0d && formatStr.contains(defaultFractionWholePartFormat) && !formatStr.contains("0")) {
            formatStr = formatStr.replaceAll(defaultFractionWholePartFormat, "");
        }
        Format format = this.formats.get(formatStr);
        if (format != null) {
            return format;
        }
        if ("General".equalsIgnoreCase(formatStr) || "@".equals(formatStr)) {
            return this.generalNumberFormat;
        }
        Format format2 = createFormat(cellValue, formatIndex, formatStr);
        this.formats.put(formatStr, format2);
        return format2;
    }

    public Format createFormat(Cell cell) {
        return createFormat(cell.getNumericCellValue(), cell.getCellStyle().getDataFormat(), cell.getCellStyle().getDataFormatString());
    }

    private Format createFormat(double cellValue, int formatIndex, String sFormat) {
        String colour;
        int at;
        this.localeChangedObservable.checkForLocaleChange();
        String formatStr = sFormat;
        Matcher colourM = colorPattern.matcher(formatStr);
        while (colourM.find() && (at = formatStr.indexOf((colour = colourM.group()))) != -1) {
            String nFormatStr = formatStr.substring(0, at) + formatStr.substring(colour.length() + at);
            if (nFormatStr.equals(formatStr)) {
                break;
            }
            formatStr = nFormatStr;
            colourM = colorPattern.matcher(formatStr);
        }
        Matcher m = localePatternGroup.matcher(formatStr);
        while (m.find()) {
            String match = m.group();
            String symbol = match.substring(match.indexOf(36) + 1, match.indexOf(45));
            if (symbol.indexOf(36) > -1) {
                symbol = symbol.substring(0, symbol.indexOf(36)) + '\\' + symbol.substring(symbol.indexOf(36), symbol.length());
            }
            formatStr = m.replaceAll(symbol);
            m = localePatternGroup.matcher(formatStr);
        }
        if (formatStr != null) {
            if (formatStr.trim().length() != 0) {
                if (!"General".equalsIgnoreCase(formatStr)) {
                    if (!"@".equals(formatStr)) {
                        if (DateUtil.isADateFormat(formatIndex, formatStr) && DateUtil.isValidExcelDate(cellValue)) {
                            return createDateFormat(formatStr, cellValue);
                        }
                        if (formatStr.contains("#/") || formatStr.contains("?/")) {
                            String[] chunks = formatStr.split(";");
                            int len$ = chunks.length;
                            int i$ = 0;
                            while (true) {
                                String wholePart = defaultFractionWholePartFormat;
                                if (i$ >= len$) {
                                    return new FractionFormat(wholePart, defaultFractionFractionPartFormat);
                                }
                                Matcher fractionMatcher = fractionPattern.matcher(fractionStripper.matcher(chunks[i$].replaceAll("\\?", wholePart)).replaceAll(" ").replaceAll(" +", " "));
                                if (fractionMatcher.find()) {
                                    if (fractionMatcher.group(1) == null) {
                                        wholePart = "";
                                    }
                                    return new FractionFormat(wholePart, fractionMatcher.group(3));
                                }
                                i$++;
                            }
                        } else if (numPattern.matcher(formatStr).find()) {
                            return createNumberFormat(formatStr, cellValue);
                        } else {
                            if (this.emulateCSV) {
                                return new ConstantStringFormat(cleanFormatForNumber(formatStr));
                            }
                            return null;
                        }
                    }
                }
                return this.generalNumberFormat;
            }
        }
        return getDefaultFormat(cellValue);
    }

    private Format createDateFormat(String pFormatStr, double cellValue) {
        String formatStr = pFormatStr.replaceAll("\\\\-", "-").replaceAll("\\\\,", ",").replaceAll("\\\\\\.", ".").replaceAll("\\\\ ", " ").replaceAll("\\\\/", "/").replaceAll(";@", "").replaceAll("\"/\"", "/").replace("\"\"", "'").replaceAll("\\\\T", "'T'");
        boolean hasAmPm = false;
        Matcher amPmMatcher = amPmPattern.matcher(formatStr);
        while (amPmMatcher.find()) {
            formatStr = amPmMatcher.replaceAll("@");
            hasAmPm = true;
            amPmMatcher = amPmPattern.matcher(formatStr);
        }
        String formatStr2 = formatStr.replaceAll("@", "a");
        Matcher dateMatcher = daysAsText.matcher(formatStr2);
        if (dateMatcher.find()) {
            formatStr2 = dateMatcher.replaceAll(dateMatcher.group(0).toUpperCase(Locale.ROOT).replaceAll("D", "E"));
        }
        StringBuilder sb = new StringBuilder();
        char[] chars = formatStr2.toCharArray();
        boolean mIsMonth = true;
        List<Integer> ms = new ArrayList<>();
        boolean isElapsed = false;
        int j = 0;
        while (j < chars.length) {
            char c = chars[j];
            if (c == '\'') {
                sb.append(c);
                while (true) {
                    j++;
                    if (j >= chars.length) {
                        break;
                    }
                    char c2 = chars[j];
                    sb.append(c2);
                    if (c2 == '\'') {
                        break;
                    }
                }
            } else if (c == '[' && !isElapsed) {
                isElapsed = true;
                mIsMonth = false;
                sb.append(c);
            } else if (c != ']' || !isElapsed) {
                char c3 = 'M';
                if (isElapsed) {
                    if (c == 'h' || c == 'H') {
                        sb.append('H');
                    } else if (c == 'm' || c == 'M') {
                        sb.append('m');
                    } else if (c == 's' || c == 'S') {
                        sb.append('s');
                    } else {
                        sb.append(c);
                    }
                } else if (c == 'h' || c == 'H') {
                    mIsMonth = false;
                    if (hasAmPm) {
                        sb.append('h');
                    } else {
                        sb.append('H');
                    }
                } else if (c == 'm' || c == 'M') {
                    if (mIsMonth) {
                        sb.append('M');
                        ms.add(Integer.valueOf(sb.length() - 1));
                    } else {
                        sb.append('m');
                    }
                } else if (c == 's' || c == 'S') {
                    sb.append('s');
                    for (Integer num : ms) {
                        int index = num.intValue();
                        if (sb.charAt(index) == c3) {
                            sb.replace(index, index + 1, "m");
                        }
                        c3 = 'M';
                    }
                    mIsMonth = true;
                    ms.clear();
                } else if (Character.isLetter(c)) {
                    mIsMonth = true;
                    ms.clear();
                    if (c == 'y' || c == 'Y') {
                        sb.append('y');
                    } else if (c == 'd' || c == 'D') {
                        sb.append('d');
                    } else {
                        sb.append(c);
                    }
                } else {
                    sb.append(c);
                }
            } else {
                isElapsed = false;
                sb.append(c);
            }
            j++;
        }
        try {
            return new ExcelStyleDateFormatter(sb.toString(), this.dateSymbols);
        } catch (IllegalArgumentException e) {
            return getDefaultFormat(cellValue);
        }
    }

    private String cleanFormatForNumber(String formatStr) {
        StringBuilder sb = new StringBuilder(formatStr);
        if (this.emulateCSV) {
            int i = 0;
            while (i < sb.length()) {
                char c = sb.charAt(i);
                if ((c == '_' || c == '*' || c == '?') && (i <= 0 || sb.charAt(i - 1) != '\\')) {
                    if (c == '?') {
                        sb.setCharAt(i, ' ');
                    } else if (i < sb.length() - 1) {
                        if (c == '_') {
                            sb.setCharAt(i + 1, ' ');
                        } else {
                            sb.deleteCharAt(i + 1);
                        }
                        sb.deleteCharAt(i);
                        i--;
                    }
                }
                i++;
            }
        } else {
            int i2 = 0;
            while (i2 < sb.length()) {
                char c2 = sb.charAt(i2);
                if ((c2 == '_' || c2 == '*') && (i2 <= 0 || sb.charAt(i2 - 1) != '\\')) {
                    if (i2 < sb.length() - 1) {
                        sb.deleteCharAt(i2 + 1);
                    }
                    sb.deleteCharAt(i2);
                    i2--;
                }
                i2++;
            }
        }
        int i3 = 0;
        while (i3 < sb.length()) {
            char c3 = sb.charAt(i3);
            if (c3 == '\\' || c3 == '\"') {
                sb.deleteCharAt(i3);
                i3--;
            } else if (c3 == '+' && i3 > 0 && sb.charAt(i3 - 1) == 'E') {
                sb.deleteCharAt(i3);
                i3--;
            }
            i3++;
        }
        return sb.toString();
    }

    private Format createNumberFormat(String formatStr, double cellValue) {
        char grouping;
        String format = cleanFormatForNumber(formatStr);
        DecimalFormatSymbols symbols = this.decimalSymbols;
        Matcher agm = alternateGrouping.matcher(format);
        if (agm.find() && (grouping = agm.group(2).charAt(0)) != ',') {
            symbols = DecimalFormatSymbols.getInstance(this.locale);
            symbols.setGroupingSeparator(grouping);
            String oldPart = agm.group(1);
            format = format.replace(oldPart, oldPart.replace(grouping, ','));
        }
        try {
            DecimalFormat df = new DecimalFormat(format, symbols);
            setExcelStyleRoundingMode(df);
            return df;
        } catch (IllegalArgumentException e) {
            return getDefaultFormat(cellValue);
        }
    }

    public Format getDefaultFormat(Cell cell) {
        return getDefaultFormat(cell.getNumericCellValue());
    }

    private Format getDefaultFormat(double cellValue) {
        this.localeChangedObservable.checkForLocaleChange();
        Format format = this.defaultNumFormat;
        if (format != null) {
            return format;
        }
        return this.generalNumberFormat;
    }

    private String performDateFormatting(Date d, Format dateFormat) {
        return (dateFormat != null ? dateFormat : this.defaultDateformat).format(d);
    }

    private String getFormattedDateString(Cell cell) {
        Format dateFormat = getFormat(cell);
        if (dateFormat instanceof ExcelStyleDateFormatter) {
            ((ExcelStyleDateFormatter) dateFormat).setDateToBeFormatted(cell.getNumericCellValue());
        }
        return performDateFormatting(cell.getDateCellValue(), dateFormat);
    }

    private String getFormattedNumberString(Cell cell) {
        Format numberFormat = getFormat(cell);
        double d = cell.getNumericCellValue();
        if (numberFormat == null) {
            return String.valueOf(d);
        }
        return numberFormat.format(new Double(d)).replaceFirst("E(\\d)", "E+$1");
    }

    public String formatRawCellContents(double value, int formatIndex, String formatString) {
        return formatRawCellContents(value, formatIndex, formatString, false);
    }

    public String formatRawCellContents(double value, int formatIndex, String formatString, boolean use1904Windowing) {
        String result;
        this.localeChangedObservable.checkForLocaleChange();
        if (DateUtil.isADateFormat(formatIndex, formatString)) {
            if (DateUtil.isValidExcelDate(value)) {
                Format dateFormat = getFormat(value, formatIndex, formatString);
                if (dateFormat instanceof ExcelStyleDateFormatter) {
                    ((ExcelStyleDateFormatter) dateFormat).setDateToBeFormatted(value);
                }
                return performDateFormatting(DateUtil.getJavaDate(value, use1904Windowing), dateFormat);
            } else if (this.emulateCSV) {
                return invalidDateTimeString;
            }
        }
        Format numberFormat = getFormat(value, formatIndex, formatString);
        if (numberFormat == null) {
            return String.valueOf(value);
        }
        String textValue = NumberToTextConverter.toText(value);
        if (textValue.indexOf(69) > -1) {
            result = numberFormat.format(new Double(value));
        } else {
            result = numberFormat.format(new BigDecimal(textValue));
        }
        if (result.indexOf(69) <= -1 || result.contains("E-")) {
            return result;
        }
        return result.replaceFirst("E", "E+");
    }

    public String formatCellValue(Cell cell) {
        return formatCellValue(cell, null);
    }

    public String formatCellValue(Cell cell, FormulaEvaluator evaluator) {
        this.localeChangedObservable.checkForLocaleChange();
        if (cell == null) {
            return "";
        }
        CellType cellType = cell.getCellTypeEnum();
        if (cellType == CellType.FORMULA) {
            if (evaluator == null) {
                return cell.getCellFormula();
            }
            cellType = evaluator.evaluateFormulaCellEnum(cell);
        }
        switch (AnonymousClass1.$SwitchMap$org$apache$poi$ss$usermodel$CellType[cellType.ordinal()]) {
            case 1:
                if (DateUtil.isCellDateFormatted(cell)) {
                    return getFormattedDateString(cell);
                }
                return getFormattedNumberString(cell);
            case 2:
                return cell.getRichStringCellValue().getString();
            case 3:
                return cell.getBooleanCellValue() ? "TRUE" : "FALSE";
            case 4:
                return "";
            case 5:
                return FormulaError.forInt(cell.getErrorCellValue()).getString();
            default:
                throw new RuntimeException("Unexpected celltype (" + cellType + ")");
        }
    }

    /* access modifiers changed from: package-private */
    /* renamed from: org.apache.poi.ss.usermodel.DataFormatter$1  reason: invalid class name */
    public static /* synthetic */ class AnonymousClass1 {
        static final /* synthetic */ int[] $SwitchMap$org$apache$poi$ss$usermodel$CellType;

        static {
            int[] iArr = new int[CellType.values().length];
            $SwitchMap$org$apache$poi$ss$usermodel$CellType = iArr;
            try {
                iArr[CellType.NUMERIC.ordinal()] = 1;
            } catch (NoSuchFieldError e) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$usermodel$CellType[CellType.STRING.ordinal()] = 2;
            } catch (NoSuchFieldError e2) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$usermodel$CellType[CellType.BOOLEAN.ordinal()] = 3;
            } catch (NoSuchFieldError e3) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$usermodel$CellType[CellType.BLANK.ordinal()] = 4;
            } catch (NoSuchFieldError e4) {
            }
            try {
                $SwitchMap$org$apache$poi$ss$usermodel$CellType[CellType.ERROR.ordinal()] = 5;
            } catch (NoSuchFieldError e5) {
            }
        }
    }

    public void setDefaultNumberFormat(Format format) {
        for (Map.Entry<String, Format> entry : this.formats.entrySet()) {
            if (entry.getValue() == this.generalNumberFormat) {
                entry.setValue(format);
            }
        }
        this.defaultNumFormat = format;
    }

    public void addFormat(String excelFormatStr, Format format) {
        this.formats.put(excelFormatStr, format);
    }

    /* access modifiers changed from: private */
    public static DecimalFormat createIntegerOnlyFormat(String fmt) {
        DecimalFormat result = new DecimalFormat(fmt, DecimalFormatSymbols.getInstance(Locale.ROOT));
        result.setParseIntegerOnly(true);
        return result;
    }

    public static void setExcelStyleRoundingMode(DecimalFormat format) {
        setExcelStyleRoundingMode(format, RoundingMode.HALF_UP);
    }

    public static void setExcelStyleRoundingMode(DecimalFormat format, RoundingMode roundingMode) {
        format.setRoundingMode(roundingMode);
    }

    public Observable getLocaleChangedObservable() {
        return this.localeChangedObservable;
    }

    public void update(Observable observable, Object localeObj) {
        if (localeObj instanceof Locale) {
            Locale newLocale = (Locale) localeObj;
            if (this.localeIsAdapting && !newLocale.equals(this.locale)) {
                this.locale = newLocale;
                this.dateSymbols = DateFormatSymbols.getInstance(newLocale);
                this.decimalSymbols = DecimalFormatSymbols.getInstance(this.locale);
                this.generalNumberFormat = new ExcelGeneralNumberFormat(this.locale);
                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", this.dateSymbols);
                this.defaultDateformat = simpleDateFormat;
                simpleDateFormat.setTimeZone(LocaleUtil.getUserTimeZone());
                this.formats.clear();
                Format zipFormat = ZipPlusFourFormat.instance;
                addFormat("00000\\-0000", zipFormat);
                addFormat("00000-0000", zipFormat);
                Format phoneFormat = PhoneFormat.instance;
                addFormat("[<=9999999]###\\-####;\\(###\\)\\ ###\\-####", phoneFormat);
                addFormat("[<=9999999]###-####;(###) ###-####", phoneFormat);
                addFormat("###\\-####;\\(###\\)\\ ###\\-####", phoneFormat);
                addFormat("###-####;(###) ###-####", phoneFormat);
                Format ssnFormat = SSNFormat.instance;
                addFormat("000\\-00\\-0000", ssnFormat);
                addFormat("000-00-0000", ssnFormat);
            }
        }
    }

    private static final class SSNFormat extends Format {
        private static final DecimalFormat df = DataFormatter.createIntegerOnlyFormat("000000000");
        public static final Format instance = new SSNFormat();

        private SSNFormat() {
        }

        public static String format(Number num) {
            String result = df.format(num);
            return result.substring(0, 3) + '-' + result.substring(3, 5) + '-' + result.substring(5, 9);
        }

        public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) {
            toAppendTo.append(format((Number) obj));
            return toAppendTo;
        }

        public Object parseObject(String source, ParsePosition pos) {
            return df.parseObject(source, pos);
        }
    }

    private static final class ZipPlusFourFormat extends Format {
        private static final DecimalFormat df = DataFormatter.createIntegerOnlyFormat("000000000");
        public static final Format instance = new ZipPlusFourFormat();

        private ZipPlusFourFormat() {
        }

        public static String format(Number num) {
            String result = df.format(num);
            return result.substring(0, 5) + '-' + result.substring(5, 9);
        }

        public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) {
            toAppendTo.append(format((Number) obj));
            return toAppendTo;
        }

        public Object parseObject(String source, ParsePosition pos) {
            return df.parseObject(source, pos);
        }
    }

    private static final class PhoneFormat extends Format {
        private static final DecimalFormat df = DataFormatter.createIntegerOnlyFormat("##########");
        public static final Format instance = new PhoneFormat();

        private PhoneFormat() {
        }

        public static String format(Number num) {
            String result = df.format(num);
            StringBuilder sb = new StringBuilder();
            int len = result.length();
            if (len <= 4) {
                return result;
            }
            String seg3 = result.substring(len - 4, len);
            String seg2 = result.substring(Math.max(0, len - 7), len - 4);
            String seg1 = result.substring(Math.max(0, len - 10), Math.max(0, len - 7));
            if (seg1.trim().length() > 0) {
                sb.append('(');
                sb.append(seg1);
                sb.append(") ");
            }
            if (seg2.trim().length() > 0) {
                sb.append(seg2);
                sb.append('-');
            }
            sb.append(seg3);
            return sb.toString();
        }

        public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) {
            toAppendTo.append(format((Number) obj));
            return toAppendTo;
        }

        public Object parseObject(String source, ParsePosition pos) {
            return df.parseObject(source, pos);
        }
    }

    /* access modifiers changed from: private */
    public static final class ConstantStringFormat extends Format {
        private static final DecimalFormat df = DataFormatter.createIntegerOnlyFormat("##########");
        private final String str;

        public ConstantStringFormat(String s) {
            this.str = s;
        }

        public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) {
            toAppendTo.append(this.str);
            return toAppendTo;
        }

        public Object parseObject(String source, ParsePosition pos) {
            return df.parseObject(source, pos);
        }
    }

    /* access modifiers changed from: private */
    public final class CellFormatResultWrapper extends Format {
        private final CellFormatResult result;

        /* synthetic */ CellFormatResultWrapper(DataFormatter x0, CellFormatResult x1, AnonymousClass1 x2) {
            this(x1);
        }

        private CellFormatResultWrapper(CellFormatResult result2) {
            this.result = result2;
        }

        public StringBuffer format(Object obj, StringBuffer toAppendTo, FieldPosition pos) {
            if (DataFormatter.this.emulateCSV) {
                toAppendTo.append(this.result.text);
                return toAppendTo;
            }
            toAppendTo.append(this.result.text.trim());
            return toAppendTo;
        }

        public Object parseObject(String source, ParsePosition pos) {
            return null;
        }
    }
}
