package org.apache.poi.ss.usermodel.charts;

import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.CellValue;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;

public class DataSources {
    private DataSources() {
    }

    public static <T> ChartDataSource<T> fromArray(T[] elements) {
        return new ArrayDataSource(elements);
    }

    public static ChartDataSource<Number> fromNumericCellRange(Sheet sheet, CellRangeAddress cellRangeAddress) {
        return new AbstractCellRangeDataSource<Number>(sheet, cellRangeAddress) {
            /* class org.apache.poi.ss.usermodel.charts.DataSources.AnonymousClass1 */

            @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
            public Number getPointAt(int index) {
                CellValue cellValue = getCellValueAt(index);
                if (cellValue == null || cellValue.getCellTypeEnum() != CellType.NUMERIC) {
                    return null;
                }
                return Double.valueOf(cellValue.getNumberValue());
            }

            @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
            public boolean isNumeric() {
                return true;
            }
        };
    }

    public static ChartDataSource<String> fromStringCellRange(Sheet sheet, CellRangeAddress cellRangeAddress) {
        return new AbstractCellRangeDataSource<String>(sheet, cellRangeAddress) {
            /* class org.apache.poi.ss.usermodel.charts.DataSources.AnonymousClass2 */

            @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
            public String getPointAt(int index) {
                CellValue cellValue = getCellValueAt(index);
                if (cellValue == null || cellValue.getCellTypeEnum() != CellType.STRING) {
                    return null;
                }
                return cellValue.getStringValue();
            }

            @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
            public boolean isNumeric() {
                return false;
            }
        };
    }

    private static class ArrayDataSource<T> implements ChartDataSource<T> {
        private final T[] elements;

        public ArrayDataSource(T[] elements2) {
            this.elements = (T[]) ((Object[]) elements2.clone());
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public int getPointCount() {
            return this.elements.length;
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public T getPointAt(int index) {
            return this.elements[index];
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public boolean isReference() {
            return false;
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public boolean isNumeric() {
            return Number.class.isAssignableFrom(this.elements.getClass().getComponentType());
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public String getFormulaString() {
            throw new UnsupportedOperationException("Literal data source can not be expressed by reference.");
        }
    }

    private static abstract class AbstractCellRangeDataSource<T> implements ChartDataSource<T> {
        private final CellRangeAddress cellRangeAddress;
        private FormulaEvaluator evaluator;
        private final int numOfCells;
        private final Sheet sheet;

        protected AbstractCellRangeDataSource(Sheet sheet2, CellRangeAddress cellRangeAddress2) {
            this.sheet = sheet2;
            CellRangeAddress copy = cellRangeAddress2.copy();
            this.cellRangeAddress = copy;
            this.numOfCells = copy.getNumberOfCells();
            this.evaluator = sheet2.getWorkbook().getCreationHelper().createFormulaEvaluator();
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public int getPointCount() {
            return this.numOfCells;
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public boolean isReference() {
            return true;
        }

        @Override // org.apache.poi.ss.usermodel.charts.ChartDataSource
        public String getFormulaString() {
            return this.cellRangeAddress.formatAsString(this.sheet.getSheetName(), true);
        }

        /* access modifiers changed from: protected */
        public CellValue getCellValueAt(int index) {
            if (index < 0 || index >= this.numOfCells) {
                StringBuilder sb = new StringBuilder();
                sb.append("Index must be between 0 and ");
                sb.append(this.numOfCells - 1);
                sb.append(" (inclusive), given: ");
                sb.append(index);
                throw new IndexOutOfBoundsException(sb.toString());
            }
            int firstRow = this.cellRangeAddress.getFirstRow();
            int firstCol = this.cellRangeAddress.getFirstColumn();
            int width = (this.cellRangeAddress.getLastColumn() - firstCol) + 1;
            int cellIndex = (index % width) + firstCol;
            Row row = this.sheet.getRow((index / width) + firstRow);
            if (row == null) {
                return null;
            }
            return this.evaluator.evaluate(row.getCell(cellIndex));
        }
    }
}
