package org.apache.poi.hssf.record.cf;

import org.apache.poi.hssf.record.common.ExtendedColor;
import org.apache.poi.util.BitField;
import org.apache.poi.util.BitFieldFactory;
import org.apache.poi.util.LittleEndianInput;
import org.apache.poi.util.LittleEndianOutput;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public final class ColorGradientFormatting implements Cloneable {
    private static BitField background = BitFieldFactory.getInstance(2);
    private static BitField clamp = BitFieldFactory.getInstance(1);
    private static POILogger log = POILogFactory.getLogger(ColorGradientFormatting.class);
    private ExtendedColor[] colors;
    private byte options;
    private ColorGradientThreshold[] thresholds;

    public ColorGradientFormatting() {
        this.options = 0;
        this.options = 3;
        this.thresholds = new ColorGradientThreshold[3];
        this.colors = new ExtendedColor[3];
    }

    public ColorGradientFormatting(LittleEndianInput in) {
        this.options = 0;
        in.readShort();
        in.readByte();
        int numI = in.readByte();
        int numG = in.readByte();
        if (numI != numG) {
            POILogger pOILogger = log;
            pOILogger.log(5, "Inconsistent Color Gradient defintion, found " + numI + " vs " + numG + " entries");
        }
        this.options = in.readByte();
        this.thresholds = new ColorGradientThreshold[numI];
        int i = 0;
        while (true) {
            ColorGradientThreshold[] colorGradientThresholdArr = this.thresholds;
            if (i >= colorGradientThresholdArr.length) {
                break;
            }
            colorGradientThresholdArr[i] = new ColorGradientThreshold(in);
            i++;
        }
        this.colors = new ExtendedColor[numG];
        for (int i2 = 0; i2 < this.colors.length; i2++) {
            in.readDouble();
            this.colors[i2] = new ExtendedColor(in);
        }
    }

    public int getNumControlPoints() {
        return this.thresholds.length;
    }

    public void setNumControlPoints(int num) {
        ColorGradientThreshold[] colorGradientThresholdArr = this.thresholds;
        if (num != colorGradientThresholdArr.length) {
            ColorGradientThreshold[] nt = new ColorGradientThreshold[num];
            ExtendedColor[] nc = new ExtendedColor[num];
            int copy = Math.min(colorGradientThresholdArr.length, num);
            System.arraycopy(this.thresholds, 0, nt, 0, copy);
            System.arraycopy(this.colors, 0, nc, 0, copy);
            this.thresholds = nt;
            this.colors = nc;
            updateThresholdPositions();
        }
    }

    public ColorGradientThreshold[] getThresholds() {
        return this.thresholds;
    }

    public void setThresholds(ColorGradientThreshold[] thresholds2) {
        this.thresholds = thresholds2 == null ? null : (ColorGradientThreshold[]) thresholds2.clone();
        updateThresholdPositions();
    }

    public ExtendedColor[] getColors() {
        return this.colors;
    }

    public void setColors(ExtendedColor[] colors2) {
        this.colors = colors2 == null ? null : (ExtendedColor[]) colors2.clone();
    }

    public boolean isClampToCurve() {
        return getOptionFlag(clamp);
    }

    public boolean isAppliesToBackground() {
        return getOptionFlag(background);
    }

    private boolean getOptionFlag(BitField field) {
        return field.getValue(this.options) != 0;
    }

    private void updateThresholdPositions() {
        double step = 1.0d / ((double) (this.thresholds.length - 1));
        int i = 0;
        while (true) {
            ColorGradientThreshold[] colorGradientThresholdArr = this.thresholds;
            if (i < colorGradientThresholdArr.length) {
                colorGradientThresholdArr[i].setPosition(((double) i) * step);
                i++;
            } else {
                return;
            }
        }
    }

    /* JADX INFO: Multiple debug info for r1v5 org.apache.poi.hssf.record.common.ExtendedColor[]: [D('arr$' org.apache.poi.hssf.record.cf.ColorGradientThreshold[]), D('arr$' org.apache.poi.hssf.record.common.ExtendedColor[])] */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("    [Color Gradient Formatting]\n");
        buffer.append("          .clamp     = ");
        buffer.append(isClampToCurve());
        buffer.append("\n");
        buffer.append("          .background= ");
        buffer.append(isAppliesToBackground());
        buffer.append("\n");
        for (Threshold t : this.thresholds) {
            buffer.append(t.toString());
        }
        for (ExtendedColor c : this.colors) {
            buffer.append(c.toString());
        }
        buffer.append("    [/Color Gradient Formatting]\n");
        return buffer.toString();
    }

    @Override // java.lang.Object
    public Object clone() {
        ColorGradientFormatting rec = new ColorGradientFormatting();
        rec.options = this.options;
        ColorGradientThreshold[] colorGradientThresholdArr = new ColorGradientThreshold[this.thresholds.length];
        rec.thresholds = colorGradientThresholdArr;
        rec.colors = new ExtendedColor[this.colors.length];
        ColorGradientThreshold[] colorGradientThresholdArr2 = this.thresholds;
        System.arraycopy(colorGradientThresholdArr2, 0, colorGradientThresholdArr, 0, colorGradientThresholdArr2.length);
        ExtendedColor[] extendedColorArr = this.colors;
        System.arraycopy(extendedColorArr, 0, rec.colors, 0, extendedColorArr.length);
        return rec;
    }

    /* JADX INFO: Multiple debug info for r1v1 org.apache.poi.hssf.record.common.ExtendedColor[]: [D('arr$' org.apache.poi.hssf.record.cf.ColorGradientThreshold[]), D('arr$' org.apache.poi.hssf.record.common.ExtendedColor[])] */
    public int getDataLength() {
        int len = 6;
        for (Threshold t : this.thresholds) {
            len += t.getDataLength();
        }
        for (ExtendedColor c : this.colors) {
            len = len + c.getDataLength() + 8;
        }
        return len;
    }

    public void serialize(LittleEndianOutput out) {
        out.writeShort(0);
        out.writeByte(0);
        out.writeByte(this.thresholds.length);
        out.writeByte(this.thresholds.length);
        out.writeByte(this.options);
        for (ColorGradientThreshold t : this.thresholds) {
            t.serialize(out);
        }
        double step = 1.0d / ((double) (this.colors.length - 1));
        for (int i = 0; i < this.colors.length; i++) {
            out.writeDouble(((double) i) * step);
            this.colors[i].serialize(out);
        }
    }
}
