package org.apache.poi.sl.draw;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.image.ImageObserver;
import java.awt.image.RescaleOp;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import javax.imageio.ImageIO;
import javax.imageio.ImageReadParam;
import javax.imageio.ImageReader;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.stream.ImageInputStream;
import javax.imageio.stream.MemoryCacheImageInputStream;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;

public class BitmapImageRenderer implements ImageRenderer {
    private static final POILogger LOG = POILogFactory.getLogger(ImageRenderer.class);
    protected BufferedImage img;

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public void loadImage(InputStream data, String contentType) throws IOException {
        this.img = readImage(data, contentType);
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public void loadImage(byte[] data, String contentType) throws IOException {
        this.img = readImage(new ByteArrayInputStream(data), contentType);
    }

    /* JADX INFO: finally extract failed */
    private static BufferedImage readImage(InputStream data, String contentType) throws IOException {
        IOException lastException = null;
        BufferedImage img2 = null;
        ImageInputStream iis = new MemoryCacheImageInputStream(data);
        try {
            ImageInputStream iis2 = new MemoryCacheImageInputStream(data);
            iis2.mark();
            Iterator<ImageReader> iter = ImageIO.getImageReaders(iis2);
            while (img2 == null && iter.hasNext()) {
                ImageReader reader = iter.next();
                ImageReadParam param = reader.getDefaultReadParam();
                int mode = 0;
                while (img2 == null && mode < 2) {
                    iis2.reset();
                    iis2.mark();
                    if (mode == 1) {
                        Iterator<ImageTypeSpecifier> imageTypes = reader.getImageTypes(0);
                        while (true) {
                            if (imageTypes.hasNext()) {
                                ImageTypeSpecifier imageTypeSpecifier = imageTypes.next();
                                if (imageTypeSpecifier.getBufferedImageType() == 10) {
                                    param.setDestinationType(imageTypeSpecifier);
                                    break;
                                }
                            }
                        }
                    }
                    try {
                        reader.setInput(iis2, false, true);
                        img2 = reader.read(0, param);
                    } catch (IOException e) {
                        lastException = e;
                    } catch (RuntimeException e2) {
                        StringBuilder sb = new StringBuilder();
                        sb.append("ImageIO runtime exception - ");
                        sb.append(mode == 0 ? "normal" : "fallback");
                        lastException = new IOException(sb.toString(), e2);
                    }
                    mode++;
                }
                reader.dispose();
            }
            iis2.close();
            if (img2 == null) {
                if (lastException == null) {
                    POILogger pOILogger = LOG;
                    pOILogger.log(5, "Content-type: " + contentType + " is not support. Image ignored.");
                    return null;
                }
                throw lastException;
            } else if (img2.getType() == 2) {
                return img2;
            } else {
                BufferedImage argbImg = new BufferedImage(img2.getWidth(), img2.getHeight(), 2);
                Graphics g = argbImg.getGraphics();
                g.drawImage(img2, 0, 0, (ImageObserver) null);
                g.dispose();
                return argbImg;
            }
        } catch (Throwable th) {
            iis.close();
            throw th;
        }
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public BufferedImage getImage() {
        return this.img;
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public BufferedImage getImage(Dimension dim) {
        double w_old = (double) this.img.getWidth();
        double h_old = (double) this.img.getHeight();
        BufferedImage scaled = new BufferedImage((int) w_old, (int) h_old, 2);
        double w_new = dim.getWidth();
        double h_new = dim.getHeight();
        AffineTransform at = new AffineTransform();
        at.scale(w_new / w_old, h_new / h_old);
        new AffineTransformOp(at, 2).filter(this.img, scaled);
        return scaled;
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public Dimension getDimension() {
        return this.img == null ? new Dimension(0, 0) : new Dimension(this.img.getWidth(), this.img.getHeight());
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public void setAlpha(double alpha) {
        if (this.img != null) {
            Dimension dim = getDimension();
            BufferedImage newImg = new BufferedImage((int) dim.getWidth(), (int) dim.getHeight(), 2);
            Graphics2D g = newImg.createGraphics();
            g.drawImage(this.img, new RescaleOp(new float[]{1.0f, 1.0f, 1.0f, (float) alpha}, new float[]{0.0f, 0.0f, 0.0f, 0.0f}, (RenderingHints) null), 0, 0);
            g.dispose();
            this.img = newImg;
        }
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public boolean drawImage(Graphics2D graphics, Rectangle2D anchor) {
        return drawImage(graphics, anchor, null);
    }

    @Override // org.apache.poi.sl.draw.ImageRenderer
    public boolean drawImage(Graphics2D graphics, Rectangle2D anchor, Insets clip) {
        Insets clip2;
        if (this.img == null) {
            return false;
        }
        boolean isClipped = true;
        if (clip == null) {
            isClipped = false;
            clip2 = new Insets(0, 0, 0, 0);
        } else {
            clip2 = clip;
        }
        int iw = this.img.getWidth();
        int ih = this.img.getHeight();
        double sx = anchor.getWidth() / (((double) iw) * (((double) ((100000 - clip2.left) - clip2.right)) / 100000.0d));
        double sy = anchor.getHeight() / (((double) ih) * (((double) ((100000 - clip2.top) - clip2.bottom)) / 100000.0d));
        AffineTransform at = new AffineTransform(sx, 0.0d, 0.0d, sy, anchor.getX() - (((((double) iw) * sx) * ((double) clip2.left)) / 100000.0d), anchor.getY() - (((((double) ih) * sy) * ((double) clip2.top)) / 100000.0d));
        Shape clipOld = graphics.getClip();
        if (isClipped) {
            graphics.clip(anchor.getBounds2D());
        }
        graphics.drawRenderedImage(this.img, at);
        graphics.setClip(clipOld);
        return true;
    }
}
