package org.apache.poi.sl.draw;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Iterator;
import org.apache.poi.sl.usermodel.Insets2D;
import org.apache.poi.sl.usermodel.PlaceableShape;
import org.apache.poi.sl.usermodel.ShapeContainer;
import org.apache.poi.sl.usermodel.TextParagraph;
import org.apache.poi.sl.usermodel.TextRun;
import org.apache.poi.sl.usermodel.TextShape;

public class DrawTextShape extends DrawSimpleShape {
    public DrawTextShape(TextShape<?, ?> shape) {
        super(shape);
    }

    /* JADX INFO: Multiple debug info for r0v9 double: [D('dx' double), D('w' double)] */
    @Override // org.apache.poi.sl.draw.DrawShape, org.apache.poi.sl.draw.Drawable
    public void drawContent(Graphics2D graphics) {
        double x;
        double y;
        DrawFactory.getInstance(graphics).fixFonts(graphics);
        TextShape<?, ?> s = getShape();
        Rectangle2D anchor = DrawShape.getAnchor(graphics, s);
        Insets2D insets = s.getInsets();
        double x2 = anchor.getX() + insets.left;
        double y2 = anchor.getY();
        AffineTransform tx = graphics.getTransform();
        boolean vertFlip = s.getFlipVertical();
        boolean horzFlip = s.getFlipHorizontal();
        ShapeContainer<?, ?> sc = s.getParent();
        while (sc instanceof PlaceableShape) {
            PlaceableShape<?, ?> ps = (PlaceableShape) sc;
            vertFlip ^= ps.getFlipVertical();
            horzFlip ^= ps.getFlipHorizontal();
            sc = ps.getParent();
        }
        if (horzFlip ^ vertFlip) {
            double ax = anchor.getX();
            double ay = anchor.getY();
            graphics.translate(ax + anchor.getWidth(), ay);
            x = x2;
            graphics.scale(-1.0d, 1.0d);
            graphics.translate(-ax, -ay);
        } else {
            x = x2;
        }
        Double textRot = s.getTextRotation();
        if (!(textRot == null || textRot.doubleValue() == 0.0d)) {
            double cx = anchor.getCenterX();
            double cy = anchor.getCenterY();
            graphics.translate(cx, cy);
            graphics.rotate(Math.toRadians(textRot.doubleValue()));
            graphics.translate(-cx, -cy);
        }
        switch (s.getVerticalAlignment()) {
            case BOTTOM:
                y = y2 + ((anchor.getHeight() - getTextHeight(graphics)) - insets.bottom);
                break;
            case MIDDLE:
                y = y2 + insets.top + ((((anchor.getHeight() - getTextHeight(graphics)) - insets.top) - insets.bottom) / 2.0d);
                break;
            default:
                y = y2 + insets.top;
                break;
        }
        TextShape.TextDirection textDir = s.getTextDirection();
        if (textDir == TextShape.TextDirection.VERTICAL || textDir == TextShape.TextDirection.VERTICAL_270) {
            double deg = textDir == TextShape.TextDirection.VERTICAL ? 90.0d : 270.0d;
            double cx2 = anchor.getCenterX();
            double cy2 = anchor.getCenterY();
            graphics.translate(cx2, cy2);
            graphics.rotate(Math.toRadians(deg));
            graphics.translate(-cx2, -cy2);
            double w = (anchor.getWidth() - anchor.getHeight()) / 2.0d;
            graphics.translate(w, -w);
        }
        drawParagraphs(graphics, x, y);
        graphics.setTransform(tx);
    }

    public double drawParagraphs(Graphics2D graphics, double x, double y) {
        DrawFactory fact;
        Graphics2D graphics2D = graphics;
        DrawFactory fact2 = DrawFactory.getInstance(graphics);
        Iterator<? extends TextParagraph<?, ?, ? extends TextRun>> paragraphs = getShape().iterator();
        boolean isFirstLine = true;
        int autoNbrIdx = 0;
        double y2 = y;
        while (paragraphs.hasNext()) {
            T p = paragraphs.next();
            DrawTextParagraph dp = fact2.getDrawable((TextParagraph<?, ?, ?>) p);
            TextParagraph.BulletStyle bs = p.getBulletStyle();
            if (bs == null || bs.getAutoNumberingScheme() == null) {
                autoNbrIdx = -1;
            } else {
                Integer startAt = bs.getAutoNumberingStartAt();
                if (startAt == null) {
                    startAt = 1;
                }
                if (startAt.intValue() > autoNbrIdx) {
                    autoNbrIdx = startAt.intValue();
                }
            }
            dp.setAutoNumberingIdx(autoNbrIdx);
            dp.breakText(graphics2D);
            if (!isFirstLine) {
                Double spaceBefore = p.getSpaceBefore();
                if (spaceBefore == null) {
                    spaceBefore = Double.valueOf(0.0d);
                }
                if (spaceBefore.doubleValue() > 0.0d) {
                    y2 += spaceBefore.doubleValue() * 0.01d * ((double) dp.getFirstLineHeight());
                } else {
                    y2 += -spaceBefore.doubleValue();
                }
            }
            isFirstLine = false;
            dp.setPosition(x, y2);
            dp.draw(graphics2D);
            y2 += dp.getY();
            if (paragraphs.hasNext()) {
                Double spaceAfter = p.getSpaceAfter();
                if (spaceAfter == null) {
                    spaceAfter = Double.valueOf(0.0d);
                }
                if (spaceAfter.doubleValue() > 0.0d) {
                    fact = fact2;
                    y2 += spaceAfter.doubleValue() * 0.01d * ((double) dp.getLastLineHeight());
                } else {
                    fact = fact2;
                    y2 += -spaceAfter.doubleValue();
                }
            } else {
                fact = fact2;
            }
            autoNbrIdx++;
            graphics2D = graphics;
            fact2 = fact;
        }
        return y2 - y;
    }

    public double getTextHeight() {
        return getTextHeight(null);
    }

    /* access modifiers changed from: protected */
    public double getTextHeight(Graphics2D oldGraphics) {
        Graphics2D graphics = new BufferedImage(1, 1, 1).createGraphics();
        if (oldGraphics != null) {
            graphics.addRenderingHints(oldGraphics.getRenderingHints());
            graphics.setTransform(oldGraphics.getTransform());
        }
        DrawFactory.getInstance(graphics).fixFonts(graphics);
        return drawParagraphs(graphics, 0.0d, 0.0d);
    }

    /* access modifiers changed from: protected */
    @Override // org.apache.poi.sl.draw.DrawShape, org.apache.poi.sl.draw.DrawSimpleShape, org.apache.poi.sl.draw.DrawSimpleShape
    public TextShape<?, ?> getShape() {
        return (TextShape) this.shape;
    }
}
