package org.apache.poi.ss.util;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.stream.ImageInputStream;
import org.apache.poi.hssf.usermodel.HSSFClientAnchor;
import org.apache.poi.ss.usermodel.ClientAnchor;
import org.apache.poi.ss.usermodel.Picture;
import org.apache.poi.ss.usermodel.PictureData;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.util.POILogFactory;
import org.apache.poi.util.POILogger;
import org.apache.poi.util.Units;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class ImageUtils {
    public static final int PIXEL_DPI = 96;
    private static final POILogger logger = POILogFactory.getLogger(ImageUtils.class);

    /* JADX INFO: finally extract failed */
    public static Dimension getImageDimension(InputStream is, int type) {
        Dimension size = new Dimension();
        switch (type) {
            case 5:
            case 6:
            case 7:
                try {
                    ImageInputStream iis = ImageIO.createImageInputStream(is);
                    try {
                        ImageReader r = ImageIO.getImageReaders(iis).next();
                        try {
                            r.setInput(iis);
                            BufferedImage img = r.read(0);
                            int[] dpi = getResolution(r);
                            if (dpi[0] == 0) {
                                dpi[0] = 96;
                            }
                            if (dpi[1] == 0) {
                                dpi[1] = 96;
                            }
                            size.width = (img.getWidth() * 96) / dpi[0];
                            size.height = (img.getHeight() * 96) / dpi[1];
                            r.dispose();
                            break;
                        } catch (Throwable th) {
                            r.dispose();
                            throw th;
                        }
                    } finally {
                        iis.close();
                    }
                } catch (IOException e) {
                    logger.log(5, e);
                    break;
                }
            default:
                logger.log(5, "Only JPEG, PNG and DIB pictures can be automatically sized");
                break;
        }
        return size;
    }

    public static int[] getResolution(ImageReader r) throws IOException {
        int hdpi = 96;
        int vdpi = 96;
        Element node = (Element) r.getImageMetadata(0).getAsTree("javax_imageio_1.0");
        NodeList lst = node.getElementsByTagName("HorizontalPixelSize");
        if (lst != null && lst.getLength() == 1) {
            hdpi = (int) (25.4d / ((double) Float.parseFloat(((Element) lst.item(0)).getAttribute("value"))));
        }
        NodeList lst2 = node.getElementsByTagName("VerticalPixelSize");
        if (lst2 != null && lst2.getLength() == 1) {
            vdpi = (int) (25.4d / ((double) Float.parseFloat(((Element) lst2.item(0)).getAttribute("value"))));
        }
        return new int[]{hdpi, vdpi};
    }

    public static Dimension setPreferredSize(Picture picture, double scaleX, double scaleY) {
        double w;
        int col2;
        double h;
        int row2;
        ClientAnchor anchor = picture.getClientAnchor();
        boolean isHSSF = anchor instanceof HSSFClientAnchor;
        PictureData data = picture.getPictureData();
        Sheet sheet = picture.getSheet();
        Dimension imgSize = getImageDimension(new ByteArrayInputStream(data.getData()), data.getPictureType());
        Dimension anchorSize = getDimensionFromAnchor(picture);
        double scaledWidth = scaleX == Double.MAX_VALUE ? imgSize.getWidth() : (anchorSize.getWidth() / 9525.0d) * scaleX;
        double scaledHeight = scaleY == Double.MAX_VALUE ? imgSize.getHeight() : (anchorSize.getHeight() / 9525.0d) * scaleY;
        int col22 = anchor.getCol1();
        int dx2 = 0;
        int col23 = col22 + 1;
        double w2 = (double) sheet.getColumnWidthInPixels(col22);
        if (isHSSF) {
            w = w2 * (1.0d - (((double) anchor.getDx1()) / 1024.0d));
        } else {
            w = w2 - (((double) anchor.getDx1()) / 9525.0d);
        }
        while (true) {
            col2 = col23;
            if (w >= scaledWidth) {
                break;
            }
            col23 = col2 + 1;
            w += (double) sheet.getColumnWidthInPixels(col2);
        }
        if (w > scaledWidth) {
            col2--;
            double cw = (double) sheet.getColumnWidthInPixels(col2);
            double delta = w - scaledWidth;
            if (isHSSF) {
                dx2 = (int) (((cw - delta) / cw) * 1024.0d);
            } else {
                dx2 = (int) ((cw - delta) * 9525.0d);
            }
            if (dx2 < 0) {
                dx2 = 0;
            }
        }
        anchor.setCol2(col2);
        anchor.setDx2(dx2);
        int row22 = anchor.getRow1();
        int dy2 = 0;
        int row23 = row22 + 1;
        double h2 = getRowHeightInPixels(sheet, row22);
        if (isHSSF) {
            h = h2 * (1.0d - (((double) anchor.getDy1()) / 256.0d));
        } else {
            h = h2 - (((double) anchor.getDy1()) / 9525.0d);
        }
        while (true) {
            row2 = row23;
            if (h >= scaledHeight) {
                break;
            }
            row23 = row2 + 1;
            h += getRowHeightInPixels(sheet, row2);
        }
        if (h > scaledHeight) {
            row2--;
            double ch = getRowHeightInPixels(sheet, row2);
            double delta2 = h - scaledHeight;
            if (isHSSF) {
                dy2 = (int) (((ch - delta2) / ch) * 256.0d);
            } else {
                dy2 = (int) ((ch - delta2) * 9525.0d);
            }
            if (dy2 < 0) {
                dy2 = 0;
            }
        }
        anchor.setRow2(row2);
        anchor.setDy2(dy2);
        return new Dimension((int) Math.round(scaledWidth * 9525.0d), (int) Math.round(9525.0d * scaledHeight));
    }

    public static Dimension getDimensionFromAnchor(Picture picture) {
        double w;
        double w2;
        double h;
        double d;
        double h2;
        ClientAnchor anchor = picture.getClientAnchor();
        boolean isHSSF = anchor instanceof HSSFClientAnchor;
        Sheet sheet = picture.getSheet();
        int col2 = anchor.getCol1();
        int col22 = col2 + 1;
        double w3 = (double) sheet.getColumnWidthInPixels(col2);
        if (isHSSF) {
            w = w3 * (1.0d - (((double) anchor.getDx1()) / 1024.0d));
        } else {
            w = w3 - (((double) anchor.getDx1()) / 9525.0d);
        }
        while (col22 < anchor.getCol2()) {
            w += (double) sheet.getColumnWidthInPixels(col22);
            col22++;
        }
        if (isHSSF) {
            w2 = w + (((double) (sheet.getColumnWidthInPixels(col22) * ((float) anchor.getDx2()))) / 1024.0d);
        } else {
            w2 = w + (((double) anchor.getDx2()) / 9525.0d);
        }
        int row2 = anchor.getRow1();
        int row22 = row2 + 1;
        double h3 = getRowHeightInPixels(sheet, row2);
        if (isHSSF) {
            h = h3 * (1.0d - (((double) anchor.getDy1()) / 256.0d));
        } else {
            h = h3 - (((double) anchor.getDy1()) / 9525.0d);
        }
        while (row22 < anchor.getRow2()) {
            h += getRowHeightInPixels(sheet, row22);
            row22++;
        }
        if (isHSSF) {
            h2 = h + ((getRowHeightInPixels(sheet, row22) * ((double) anchor.getDy2())) / 256.0d);
            d = 9525.0d;
        } else {
            d = 9525.0d;
            h2 = h + (((double) anchor.getDy2()) / 9525.0d);
        }
        return new Dimension((int) Math.rint(w2 * d), (int) Math.rint(h2 * d));
    }

    private static double getRowHeightInPixels(Sheet sheet, int rowNum) {
        Row r = sheet.getRow(rowNum);
        return ((double) Units.toEMU((double) (r == null ? sheet.getDefaultRowHeightInPoints() : r.getHeightInPoints()))) / 9525.0d;
    }
}
